use signalTypes;
use decoder;
use costasLoop;

package carrierRecovery is

begin logic

facet automaticGainControl(IResampled,QResampled::in bitvector12;
modRate:: in integer;clk::in bit;gainControlBit:: out bit;Iagc,Qagc::
out bitvector12) is
        
        agc(inSignal::bitvector12;rate::integer)::bitvector12;
        getTotalPower(p1,p2::real)::real is p1+p2;
        threshold::real;//constant
        sigPowerI,sigPowerQ,totalPower::real;

begin state_based

        
        agcI: (%clk) =>
                       (Iagc'=agc(IResampled,modRate)) and
                       (Qagc'=agc(QResampled,modRate));

        Ipower:(sigPowerI'=getSignalPower(modRate,IResampled));

        Qpower:(sigPowerQ'=getSignalPower(modRate,QResampled));

        power:(totalPower' = getTotalPower(sigPowerI,sigPowerQ));
        
        compare:(gainControlBit' =(%(totalPower >= threshold)));

end automaticGainControl;


facet vco(clk::in bit;modType,modRate::in integer;freqDev::in
real;Ivco,Qvco::out bitvector12) is
        
        soqpskIvco(s1::real;rate::integer)::bitvector12;
        soqpskQvco(s1::real;rate::integer)::bitvector12;
        sbpskIvco(s1::real;rate::integer)::bitvector12;
        sbpskQvco(s1::real;rate::integer)::bitvector12;
        fskIvco(s1::real;rate::integer)::bitvector12;
        fskQvco(s1::real;rate::integer)::bitvector12;
        bpskIvco(s1::real;rate::integer)::bitvector12;
        bpskQvco(s1::real;rate::integer)::bitvector12;
        deqpskIvco(s1::real;rate::integer)::bitvector12;
        deqpskQvco(s1::real;rate::integer)::bitvector12;
begin state_based
                       
        ISoqpskVco:(modType = 0) and (%clk) =>
              (Ivco' = soqpskIvco(freqDev,modRate))and
              (Qvco' = soqpskQvco(freqDev,modRate));
        ISbpskVco:(modType = 1) and (%clk) =>
                (Ivco' = sbpskIvco(freqDev,modRate)) and
                (Qvco' = sbpskQvco(freqDev,modRate)); 
        IFskVco:(modType = 2) and (%clk)  =>
                (Ivco' = fskIvco(freqDev,modRate)) and
                (Qvco' = fskQvco(freqDev,modRate)); 
          IBpskVco:(modType = 3) and (%clk)=>
                (Ivco' = bpskIvco(freqDev,modRate)) and
                (Qvco' = bpskQvco(freqDev,modRate));   

end vco;

facet costasLoopStruct(Iagc,Qagc,Ivco,Qvco::in bitvector12;clk::in
bit;accessType,modType,modRate::in integer;phaseLock::out bit;freqDev::out 
real;IDataSamples,QDataSamples::out bitvector12) is
                
        Ibb,Qbb,Imf,Qmf,phaseError::bitvector12;   
                
begin state_based

        quad:quadMixer(Iagc,Qagc,Ivco,Qvco,clk,modRate,Ibb,Qbb);   

        Mfilter:matchedFilter(Ibb,Qbb,clk,modType,modRate,Imf,Qmf);

        lockdetect:lockDetect(Imf,Qmf,clk,modRate,phaseLock);

        loop:loopError(Imf,Qmf,clk,modType,modRate,phaseError);   

        loopfilter:loopFilter(clk,modType,modRate,phaseError,freqDev);

	main:initPhase(Ibb,Qbb,clk,modType,modRate,accessType,IDataSamples,
                                QDataSamples);

end costasLoopStruct;

export all;
end carrierRecovery;
