use signalTypes;
use decoder;
use carrierRecovery;
package components is

begin logic

facet decimator(I,Q::in bitvector12;clk::in bit;
modRate::in integer;IDecimated,QDecimated::out bitvector12) is

	 count,maximum::integer;
begin state_based

	
      maxiDeci: (%clk) =>(maximum '= getMaxDecimateRate(modRate));

      getCount:(%clk)=>(count'=increment_count(count,maximum));

      decimate:(count =maximum - 1) => (IDecimated' =I) and
                                       (QDecimated' =Q);

end decimator;


facet resampler(I,Q:: in bitvector12;clk::in bit;modRate:: in  integer;
Iresampled,Qresampled::out bitvector12) is

        count,maximum::integer;

begin state_based

        getMax:maximum'=getMaxResampleRate(modRate);

        getcount:(%clk) => (count' = increment_count(count,maximum));

        resample:(count = maximum - 1)=> (Iresampled'=I) and (Qresampled' =Q);

end resampler;

facet bitSynchronizer(I,Q::in bitvector12;clk::in bit;modRate::in
integer;ISoftDecision,QSoftDecision::out bitvector3;erasure::out bit)
is

        sync(input::bitvector12;rate::integer)::bitvector3;
        getErasure(Iin,Qin::bitvector12;rate::integer)::bit;

begin state_based

        Ibit:(%clk) =>
                 (ISoftDecision' = sync(I,modRate)) and
                 (QSoftDecision' = sync(Q,modRate));

        era:(%clk)=>
		(erasure'=getErasure(I,Q,modRate));

end bitSynchronizer;

facet carrierRecoveryStruct(IResampled,QResampled::in bitvector12;clk::in
bit;modType,accessType,modRate::in integer;IDataSamples,QDataSamples::out 
bitvector12;agcBit,phaseLock::out bit) is

      Iagc,Qagc,Ivco,Qvco::bitvector12;
      freqDev::real;

begin state_based

	agc:automaticGainControl(IResampled,QResampled,modRate,clk,agcBit,Iagc,
                                Qagc);

        loop:costasLoopStruct(Iagc,Qagc,Ivco,Qvco,clk,accessType,modType,
	               modRate,phaseLock,freqDev,IDataSamples,QDataSamples);
        
	vcoOper:vco(clk,modType,modRate,freqDev,Ivco,Qvco);

end carrierRecoveryStruct;

facet errorCorrectionDecoder(I,Q::in bitvector3;erasure::in bit;clk::in bit;
modRate,fecrate::in integer;Idata,Qdata::out bit) is

        correctionAndDecode(s1::bitvector3;flag::bit;rate::integer;
                                kval,modRate::real)::bit is
	if (s1=[0;;0;;0])or(s1=[0;;0;;1])or(s1=[0;;1;;0])or(s1=[0;;1;;1])
        	then 1
        	else 0	
	endif;

	k::real;
	fec::real;

begin state_based

	getK:(%clk) => (k' = getValueK(fecrate));

	getFEC:(%clk)=>(fec' = getValueFECrate(fecrate));

        getIdata:(%clk) =>
		(Idata'=correctionAndDecode(I,erasure,modRate,k,fec));

        getQdata:(%clk) =>
		(Qdata'=correctionAndDecode(Q,erasure,modRate,k,fec));

end errorCorrectionDecoder;



facet uniqueWordDetector(IData,QData::in bit;modRate,burstType,uniqueWdType::in
 integer;clk::in bit;uniqueWdDetected::out bit) is

        Ibuffer,Qbuffer::bitvector;
        shift(data::bit;buffer::bitvector)::bitvector ;
        compare(buffer1,buffer2::bitvector;tocompare::integer)::bit;

begin state_based

        comparing:uniqueWdDetected' =
				compare(Ibuffer,Qbuffer,uniqueWdType);

        getIbuffer:Ibuffer'=shift(IData,Ibuffer);

        getQbuffer:Qbuffer'=shift(QData,Qbuffer);

end uniqueWordDetector;

export all;
end ;
