package springPackage is 

begin logic

  facet spring ( d :: real;  	// wire diameter
	       OD :: real;	// outside diameter of spring
	       ID :: real;	// inside diameter of spring
	       G :: real;	// Modulus of Rigidity of wire
	       N :: real;	// total number of coils
	       p :: real	// pitch
	     ) is


   pi :: real is 3.1415926452645;

   D :: real is (OD + ID) / 2.0;	// mean diameter of spring
   C :: real is D / d;		// spring constant

   n :: real;			// number of active coils
   H :: real;			// fully compressed height

   R :: real;			// Force/length (lb/inch)
   P :: real;			// force to fully deflect spring

   l :: real;			// maximum spring displacement
   Lm :: real;			// minimum free length
   L :: real;			// desired free length

   Tu :: real;			// ultimate tensile strength of wire
   Ty :: real;			// tensile yield strength
   Sy :: real;			// shear yield strength
   S :: real;			// uncorrected shear stress
   k :: real;			// correction factor
   Sk :: real;			// corrected shear factor

   export R, L;

begin logic

   C1: (OD - ID) / 2.0 = d;	// dependencies between diameter parameters

   C2: C >= 4.0 and C =< 20.0;	// empirical constraints on spring constant

   F1: n = (G * d^4.0) / (8.0 * R * D^3.0);
   C3: n >= 3.0;			// empirical constraint on n

   F2: N = n + 2.0;		// for closed and ground spring
   
   F3: H = d * N;

   F4: Lm = H + l;
   F5: L = H + (l / 0.8);

   F6: p = (L - 2.0*d) / n;
   C4: p =< D;			// pitch can't exceed coil diameter

   F7: Tu = 200000.0 * D^(-0.14);
   F8: Ty = 0.75 * Tu;
   F9: Sy = Ty * 0.577;

   F10: P = R * l;

   F11: S = (8.0 * P * D) / (pi * d^3.0);
   F12: k = (4.0*C - 1.0) / (4.0*C - 4.0) + 0.615 / C;
   F13: Sk = S * k;

   C5: Sk < Sy;			// shear force < wire shear yield strength  

end spring;

end;
