// library rosetta;
package lang is
begin null

  domain logic is
    universal :: type;
    univ :: subtype(universal);
    item, constructed, function, element, set, array, facet :: subtype(univ);

    // added by ckong
    itemlist, expression, sequence :: subtype(univ);
    if_term :: subtype(expression);

    domain, package, component, interaction :: subtype(facet);

    character, enumeration, number :: subtype(element);
    string :: array(character);
    label::type;
    //label :: subtype(constructed) is 
    //data make_label(labelName :: string) :: isLabel;

    boolean :: subtype(number) is enumeration[true | false];
    complex :: subtype(number);
    real :: subtype(complex);
    rational :: subtype(real);
    integer :: subtype(rational);
    natural :: subtype(integer);


    bit :: subtype(natural) is enumeration[0 | 1];
    // added by ckong
    bitvector :: subtype(array(bit));

    j :: complex;
    im(x :: complex) :: real;
    rl(x :: complex) :: real;
    fl(x :: real) :: integer;
    abs(x :: real) :: real;
    sqrt(x :: real) :: complex;
    sqr(x :: real) :: real;
    log(x :: real) :: real;


    //sel :: <* (params :: set(univ); pred(x :: set(univ)) :: boolean) 
    //:: set(univ) *>;

    __=__ :: <* (lhs, rhs :: univ) :: boolean *>;
    __/=__ :: <* (lhs, rhs :: univ) :: boolean *>;
    __==__ :: <* (lhs, rhs :: univ) :: boolean *>;


    #__ :: <* (rhs :: set(univ)) :: natural *>;
    __++__ :: <* (lhs, rhs :: set(univ)) :: set(univ) *>;
    __**__ :: <* (lhs, rhs :: set(univ)) :: set(univ) *>;
    __--__ :: <* (lhs, rhs :: set(univ)) :: set(univ) *>;
    __in__ :: <* (lhs :: univ; rhs :: set(univ)) :: boolean *>;
    $__ :: <* (rhs :: array(univ)) :: natural *>;
    __;;__ :: <* (lhs, rhs :: array(univ)) :: array(univ) *>;
    subst(a :: array(univ); i :: natural; v :: univ) :: array(univ); // polymorphic


    %__ :: <* (rhs :: bit) :: natural *>;

    __<__ :: <* (lhs, rhs :: number) :: boolean *>;
    __>__ :: <* (lhs, rhs :: number) :: boolean *>;
    __>=__ :: <* (lhs, rhs :: number) :: boolean *>;
    __=<__ :: <* (lhs, rhs :: number) :: boolean *>;
    __>=__ :: <* (lhs, rhs :: number) :: boolean *>;
    __max__ :: <* (lhs, rhs :: number) :: number *>;
    __min__ :: <* (lhs, rhs :: number) :: number *>;

    __+__ :: <* (lhs, rhs :: complex) :: complex *>;
    __-__ :: <* (lhs, rhs :: complex) :: complex *>;
    __/__ :: <* (lhs, rhs :: complex) :: complex *>;
//    __div__ :: <* (lhs, rhs :: complex) :: complex *>;
    __*__ :: <* (lhs, rhs :: complex) :: complex *>;
    __^__ :: <* (lhs, rhs :: complex) :: complex *>;

    not__ :: <* (rhs :: boolean) :: boolean *>;
    __implies__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __=>__ :: <* (lhs, rhs :: boolean) :: boolean *>; 
    __nmax__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __nmin__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __and__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __nand__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __or__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __nor__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __xor__ :: <* (lhs, rhs :: boolean) :: boolean *>;
    __xnor__ :: <* (lhs, rhs :: boolean) :: boolean *>;

    // added by ckong
    posreal :: subtype(real) is sel(x::real | x >= 0);
    tl(seq :: sequence(univ)) :: sequence(univ);
    nil :: sequence(univ);

    export all;
  begin null
  end domain logic;


  domain state_based_semantics is
    States :: subtype(univ);
    s :: States;
    next :: <* (x :: States) :: States *>;
    meta_getItem :: <* (lhs :: label; rhs :: States) :: item *>;
    meta_getType :: <* (lhs :: label; rhs :: item) :: type *>;
    __@__ :: <* (lhs :: label; rhs :: States) :: item *> is 
       <* (lhs :: label; rhs :: States) :: item is meta_getItem(lhs, rhs) *>;
    __' :: <* (lhs :: label) :: item *> is 
       <* (lhs :: label) :: item is lhs@next(s) *>;

    input(x :: label) :: boolean;
    output(x :: label) :: boolean;
    inout(x :: label) :: boolean;
    design(x :: label) :: boolean;

    export all;
  begin logic
  end domain state_based_semantics;


  facet reflect is
   meta_items(f::facet)::itemlist;
   meta_params(f::facet)::itemlist;
   meta_if(pred,thenC,elseC::expression)::if_term;
   meta_getOperators(f::facet)::sequence(label);
   meta_getInstantiatedFacets(f::facet)::set(facet);
   meta_equal(x,y::univ)::expression;
   export all;
  begin logic
  end facet reflect;

  
  domain discrete(f::facet) is
  begin state_based_semantics 
    d1: forall(s::States; n::natural| 
          exists(fnc::<*(st::States)::natural*>;
                 fncInv::<*(nt::natural)::States*> |
           (fnc(s) = n) and (fncInv(n) = s) and 
           (fncInv(fnc(s)) = s) and 
           (fnc(fncInv(n)) = n)));
  end domain discrete;
  

  domain infinite_state(f::facet) is
    __>__(s1::States; s2::States) :: boolean;
  begin discrete
    l1: forall(s1::States | next(s1) > s1);
    l2: forall(s1::States | - (s1 < s1));
    l3: forall(s1::States; s2::States | s1 < s2 => -(s2 < s1));
    l4: forall(s1::States; s2::States; s3::States | s1 < s2 and s2 < s3 
                                                  => (s1 < s3));
  end domain infinite_state;


  domain finite_state(f::facet) is
  begin state_based_semantics
    l1: #States in natural;
  end domain finite_state;

  domain discrete_time(f::facet) is
    Time::subtype(natural());
    t::Time;
    delta::natural -- 0;
    next(t::Time)::Time is t + delta;
  begin infinite_state
    l1: Time = States and t = s;
  end domain discrete_time;


//  domain continuous_state(f::facet) is
//    deriv(x::real)::real is deriv(x, s);
//    antideriv(x::real)::real is antideriv(x, s, 0);
//    integ(x, u, l::real)::real is integ(x, s, u, l);
//  begin state_based_semantics
//    t1: forall(s::States; x::label | 
//                           forall (d::meta_getType(x, meta_items(f)) |
//                           forall(e::T | abs(x@(t+e)-x') < d )));
//    t2: forall(s::States; x::label | 
//               deriv(x@t) = lim ((x@next(s) - x@s) / ( next(s) - s), 
//                                 next(s) - s, 0));
//  end domain continuous_state;


//  domain continuous_time(f::facet) is
//    Time::subtype(real);
//    t::Time;
//  begin continuous_state
//    t1: Time = States and t = s;
//  end domain continuous_time;


//  domain signal_based_semantics(f::facet) is
//    Values:: type;
//   Tags:: type;
//    Event:: type is data
//      event(value:: Values, tag:: Tags):: eventp;
//    Signals:: subtype(set(Event));
//    // n-tuple of signals
//    Sn:: sequence(Signals);
//    N:: natural;
//    Processes::subtype(set(Sn));
//    lambda:: Signals is {};
//    bigLambda:: Sn is nil;
//    union(s1::Sn; s2::Sn)::Sn;
//    satisfy(s1::Sn; p:: Processes):: boolean is (s1 in p);
//    connection:: Processes;
//
//    __<=__(t1::Tags; t2::Tags):: boolean;
//    __<__(t1::Tags; t2::Tags):: boolean;
//    __<__(e1:: Event; e2:: Event):: boolean;
//
//  begin logic
//    // Sn represents an n-tuple of signals
//    s1: #Sn = N;  
//    // Union of any tuple with empty tuple is the tuple
//    s2: forall(seq::Sn| union(seq, bigLambda) = seq);
//    // A connection is a process where at least two signals are identical
//    // ~ is an operator that converts a tuple to a set
//    s3: #connection > #(~connection);
//
//    reflexive: forall(t1::Tags| t1 <= t1);
//    transitive: forall(t1::Tags; t2::Tags; t3:: Tags | 
//                       ((t1 <= t2) and (t2 <= t3)) => (t1 <= t3));
//    antisymmetric: forall(t1:: Tags; t2:: Tags | 
//                         ((t1 <= t2) and (t2 <= t1)) => (t1 = t2)); 
//    irreflexive: forall(t1::Tags; t2:: Tags | 
//                       ((t1 <= t2) and (t1 /= t2)) => (t1 < t2));
//    event: forall(e1::Event; e2:: Event | 
//                 ((e1 < e2) => (tag(e1) < tag(e2))) and
//                 ((e1 < e2) <= (tag(e1) < tag(e2))));
//  end domain signal_based_semantics;


//  domain timed(f::facet) is
//  begin signal_based_semantics
    // Tags set is a totally ordered set
//    t1: forall(t1::Tags; t2::Tags | (t1 /= t2) => ((t1 < t2) or (t2 < t1)));
//  end domain timed;


//  domain metric_time(f::facet) is
//    __+__(t1::Tags; t2::Tags):: Tags;
//    __-__(t1::Tags; t2::Tags):: Tags;
//    -__(t1::Tags):: Tags;
//    zero:: Tags;
//  begin timed
//    m1: forall(t1::Tags | t1 + zero = t1);
//    m2: forall(t1::Tags | t1 - t1 = 0);
//    m3: forall(t1::Tags | t1 + (-t1) = 0);
//  end timed;


//  domain sequential(f::facet) is
//  begin signal_based
    // A sequential process has only one signal
//    s1: forall(p1::Processes | #p1 = 1);
//    s2: forall(t1::Tags; t2::Tags | (t1 /= t2) => ((t1 < t2) or (t2 < t1)));
//  end domain sequential;

//  domain csp(f::facet) is
//    rendez_vous(s1::Signals; p1::Processes; p2::Processes):: boolean;
//  begin signal_based_semantics 
//    c1: forall(p1::Processes | #p1 = 1);
//    c2: N = 1;
//    c3: forall(p1::Processes; seq::Sn; sig::Signals; ev1,ev2::Event | 
//         ((seq in p1) and (sig = seq(0)) and (ev1 in sig) and (ev2 in sig) 
//         and (tag(ev1) /= tag(ev2)))
//         => ((tag(ev1) > tag(ev2)) or (tag(ev2) > tag(ev1))));
//    c4: exists(e1::Event; e2::Event; e3::Event; 
//               s1::Signals; s2::Signals; s3::Signals;
//               p1::Processes; p2::Processes |
//        ((s1 in p1) and (s2 in p2) 
//         and (e1 in s1) and (e2 in s2) and (e3 in s3) 
//         and (tag(e1) = tag(e2)) and (tag(e2) = tag(e3)))
//        = rendez_vous(s3, p1, p2));
//    c5: forall(t1,t2::Tags | t1 <= t2 or t2 <= t1);
//  end domain csp;


  export all;
end package lang;



