//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ArrayValue.java,v 1.3 2002/07/16 07:15:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ArrayValue.java,v $
//  Revision 1.3  2002/07/16 07:15:47  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:00:20  kimmell
//  1. Removed instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:51  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:15  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.8.2.4  2002/01/04 12:30:46  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.8.2.3  2001/11/24 06:44:44  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.8.2.2  2001/11/14 19:17:17  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.8.2.1  2001/11/07 22:41:02  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.8  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.7  2001/02/01 22:34:29  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.6  2000/11/13 21:49:10  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.5  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.4  2000/09/18 20:41:02  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.3  2000/08/30 20:39:10  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/15 20:26:53  ckong
//  *** empty log message ***
//
//  Revision 1.1  2000/08/11 16:28:24  cvs
//  Replacing Array.java, Set.java, and Sequence.java with files named
//  xxxValue.java. All three are a bit more fleshed out as well, with
//  accessors. Purpose of replacement: To distinguish these classes
//  from those of same name to appear (soon) as subclasses of
//  Type. -David 
//
//  Revision 1.2  2000/08/10 01:04:11  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:06  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:44:14  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ArrayValue</B> object. An instance of this
 *  class indicates an array has been constructed (eg. "[a, b, 5]").
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class ArrayValue extends CompositeValue  {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /** The primary constructor.
   */
  public ArrayValue()
  {
  }

  /** The secondary constructor.
   * @param elementsIn The list of the elements of the array.
   */
  public ArrayValue(ExpressionList elementsIn) {
    super(elementsIn);
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ArrayValue(Map settings)
    {
    super(settings);
    }




  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString() {
    return "[" + contentString(", ") + "]";
  }


  /** To get the string required to define this array.
   * @param indent The current string to use as a logical block indentor.
   * @return String The declaration of this array.
   */
  public String publishROM(String indent) {
    return "[" + contentPublish(", ", indent) + "]";
  }
 

}


