//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Steve sparker@averstar.com
//
//  $Id: Domain.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Domain.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//
//============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/**
 * <b>Domain</b>:  Represtents the type of facet know as a 
 * domain.
 *
 * @author <i>Steve Parker</i> 
 * (<a href="mailto:sparker@averstar.com">sparker@averstar.com</a>).
 */
public class Domain extends Facet {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /**
   * Primary constructor.
   */
  public Domain() {
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Domain(Map settings)
    {
    super(settings);
    }



  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
  { return "domain" + partialString(); }


  /** Method to get the infix string declaration of the object
   * @param myLabel The name of this syntactical shorthand object.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  public String publishROM(Label myLabel, String indent)
  { return publishUses(indent) + "domain " + myLabel + partialPublish(indent) + " domain " + myLabel; }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent)
  { return "domain" + partialPublish(indent) + " domain"; }


}
