//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Srinivas akki@ittc.ukans.edu
//
// 
//  $Id: ExpressionList.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ExpressionList.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:04:28  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/04/17 19:55:04  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:17  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.2.2.3  2002/01/04 12:30:48  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.2.2.2  2001/11/14 19:17:19  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.2.2.1  2001/11/07 22:41:03  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.2  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods getType  and alreadyDeclared are also added to get a type that has been previously declared.
//  --srinivas
//
//  Revision 1.1  2000/10/30 22:32:51  cvs
//  The ExpressionList class is added
//  --srinivas
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ExpressionList</B> object. The contents
 *  of this list are guarenteed to be Expressions.
 * @author <I> Cindy Kong and Srinivas Akkipeddi </I>
 * (ckong@ittc.ukans.edu, akki@ittc.ukans.edu)
 */
public class ExpressionList extends BaseObjectList
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /** The primary Constructor
   */
  public ExpressionList()
    { }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ExpressionList(Map settings)
    { super(settings); }


  /** To add an Expression to the list.
   * @param expr The Expression to add.
   * @return void
   */
  public void add(Expression expr)
    { addObject(expr); }


  /** To add an Expression to the list in a specific position.
   * @param pos The position in the list to add the Expression to.
   * @param expr The Expression to add.
   * @return void
   */
  public void add(int pos, Expression expr)
    { addObject(pos, expr); }


  /** To add lots of Expressions to the list.
   * @param expressions The ExpressionList to add
   * @return void
   */
  public void add(ExpressionList expressions)
    {
    int pos;

    if (expressions == null)
      { return; }

    for (pos = 0; pos < expressions.size(); pos++)
      { add(expressions.get(pos)); }
    }



  /** To remove an Expression from the list by position.
   * @param int The position to remove the Expression from.
   * @return Expression The Expression removed from that position.
   */
  public Expression remove(int pos)
    { return (Expression)removeObject(pos); }


  /** To retrieve the first Expression in the list.
   * @return Expression The first Expression.
   */
  public Expression firstElement()
    { return (Expression)getObjectAt(0); }


  /** To retrieve the last Expression in the list.
   * @return Expression The last Expression in the list.
   */
  public Expression lastElement()
    { return (Expression)getObjectAt(size()-1); }


  /** Method to retrieve an Expression from the list by position.
   * @param pos The position to retreive the Expression from.
   * @return Expression The Expression found at that position.
   */
  public Expression get(int pos)
    { return (Expression)getObjectAt(pos); }


  /** Method to retreive all the Expressions from the list.
   * @return Expression[] The Expressions by position.
   */
  public Expression[] toArray()
    {
    int pos;
    BaseObject[] base;
    Expression[] answer;

    base = toObjectArray();
    answer = new Expression[base.length];
    for (pos = 0; pos < base.length; pos++)
      { answer[pos] = (Expression)base[pos]; }
    return answer;
    }

  }


