//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Facet.java,v 1.7 2002/11/20 17:14:45 kimmell Exp $
//
//  Revision History:
//  ----------------
//  $Log: Facet.java,v $
//  Revision 1.7  2002/11/20 17:14:45  kimmell
//  Removed an unnecessary log message from publishInterface.
//
//  Revision 1.6  2002/11/19 16:43:27  kimmell
//  1. Modified the partialPublish so that it does not include any terms
//     beginning with "__#".
//
//  Revision 1.5  2002/11/19 16:27:24  kimmell
//  1. Modified publishInterface so that it *always* prints parens, even
//     if there are no parameters.
//
//  Revision 1.4  2002/11/18 17:58:55  kimmell
//  1. Moved printing of domain from after "begin" statement to after
//     interface.
//  -kimmell
//
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:04:57  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:56  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:17  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.13.2.5  2002/01/04 12:30:48  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.13.2.4  2001/11/24 06:44:46  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.13.2.3  2001/11/14 19:17:19  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.13.2.2  2001/11/07 22:41:03  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.16  2001/10/09 23:39:09  cvs
//  Relabeling facet instantiation now works. A case was simply missing
//  in the labelLookup function - there was no lookup for labels in
//  already defined terms.
//  -- Cindy
//
//  Revision 1.15  2001/10/01 18:50:18  cvs
//  I added a couple of predefined operators that were missing in
//  universe of ROM. Tabulation is set to two. 
//  -- Cindy
//
//  Revision 1.14  2001/09/27 22:31:58  cvs
//  Had Some problems with the previous commit.Cooresponding changes
//  were made and now the parser supports
//  interactions,domains,enumerations. 
//   --Murthy
//
//  Revision 1.13  2001/08/28 23:30:26  cvs
//  Added the implementation of Domains and interactions
//     --Murthy
//
//  Revision 1.12  2001/03/26 19:01:42  cvs
//  Completed the implementation for the change in export clauses.
//  -- Cindy
//
//  Revision 1.11  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.9  2001/02/07 20:52:20  cvs
//  There was an error causing an infinite loop when doing label lookup in
//  use clauses. Modifications were made in labelLookup methods to remove
//  error.
//  -- Cindy
//
//  Revision 1.8  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.7  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.4  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.3  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.2  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:52  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;
import java.util.Iterator;


/** This represents the <B>Facet</B> object. This is one of the fundamental
 *  building blocks of the Rosetta language. All models basically consist
 *  of a set of Facets whose collective properties describe the system
 *  being modelled.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Facet extends Expression {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "UseClauses", "Domain", "InterfaceItems", "DeclaredItems", "TermItems", "ExportItems", "ExportAll" };


  private ExpressionList useClauses, exportList;
  private Expression domainRef;
  private ItemList interfaceItems, declaredItems, termItems;
  private boolean isExportAll;


  /** The primary Constructor
   */
  public Facet() {
    interfaceItems = new ItemList();
    declaredItems = new ItemList();
    termItems = new ItemList();
    exportList = new ExpressionList();
    useClauses = new ExpressionList();
    domainRef = null;
    isExportAll = false;
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Facet(Map settings)
    {
    super(settings);

    useClauses = (ExpressionList)getSetting(Facet.keySet[0]);
    domainRef = (Expression)getSetting(Facet.keySet[1]);
    interfaceItems = (ItemList)getSetting(Facet.keySet[2]);
    declaredItems = (ItemList)getSetting(Facet.keySet[3]);
    termItems = (ItemList)getSetting(Facet.keySet[4]);
    exportList = (ExpressionList)getSetting(Facet.keySet[5]);
    if (getSetting(Facet.keySet[6]) == null)
      { isExportAll = false; }
    else
      { isExportAll = ((Boolean)getSetting(Facet.keySet[6])).booleanValue(); }

    addChildList(useClauses);
    addChildList(interfaceItems);
    addChildObject(domainRef);
    addChildList(declaredItems);
    addChildList(termItems);
    addChildList(exportList);
    }



  /** To set the domain
   * @param domainIn The Label corresponding to the domain
   * @return void
   */
  public void setDomain(Expression domainIn) {
    removeChildObject(domainRef);
    domainRef = domainIn;
    addChildObject(domainRef);
  }

  /** To get the domain
   * @return domain
   */
  public Expression getDomain() {
    return domainRef;
  }


  /** Method for adding another Use clause.
   * @param use The new use clause.
   * @return void
   */
  public void addUseClause(Expression use)
    {
    if (useClauses == null)
      { useClauses = new ExpressionList(); }
    useClauses.add(use);
    addChildObject(use);
    }


  /** Method for setting all the Use clauses at once.
   * @param uses The new use clauses.
   * @return void
   */
  public void setUseClauses(ExpressionList uses)
    {
    removeChildList(useClauses);
    useClauses = uses;
    addChildList(useClauses);
    }


  /** Method for retreiving all the Use clauses.
   * @return ExpressionList The use clauses of this package.
   */
  public ExpressionList getUseClauses()
    { return useClauses; }


  /** To add an interface
   * @param interfaceItemIn The Item to be added
   * @return void
   */
  public void addInterfaceItem(Item interfaceItem) {
    if (interfaceItems == null)
      { interfaceItems = new ItemList(); }
    interfaceItems.add(interfaceItem);
    addChildObject(interfaceItem);
  }


  /** To set the interface list
   * @param listIn, ItemList to be set
   * @return void
   */
  public void setInterfaceItems(ItemList listIn) {
    removeChildList(interfaceItems);
    interfaceItems = listIn;
    addChildList(interfaceItems);
  }

  /** To get the interface list
   * @return The ItemList containing the interfaces
   */
  public ItemList getInterfaceItems() {
    return interfaceItems;
  }

  /** To add an item
   * @param declaredItemIn The declared item to be added
   * @return void
   */
  public void addDeclaredItem(Item declaredItemIn) {
    if (declaredItems == null)
      { declaredItems = new ItemList(); }
    declaredItems.add(declaredItemIn);
    addChildObject(declaredItemIn);
  }

  /** To set the item list
   * @param listIn, list of items to be set
   * @return void
   */
  public void setDeclaredItems(ItemList listIn) {
    removeChildList(declaredItems);
    declaredItems = listIn;
    addChildList(declaredItems);
  }

  /** To get the item list
   * @return The ItemList containing declared items
   */
  public ItemList getDeclaredItems() {
    return declaredItems;
  }

  /** To add a term
   * @param termIn The term to be added
   * @return void
   */
  public void addTermItem(Item termIn) {
    if (termItems == null)
      { termItems = new ItemList(); }
    termItems.add(termIn);
    addChildObject(termIn);
  }
	
  /** To set the term list
   * @param listIn, the term list to be set
   * @return void
   */
  public void setTermItems(ItemList listIn) {
    removeChildList(termItems);
    termItems = listIn;
    addChildList(termItems);
  }

  /** To get the term list
   * @return ItemList containing terms
   */
  public ItemList getTermItems() {
    return termItems;
  }





  /** To set export all status
   * @param exportAll export status to be set
   * @return void
   */
  public void setExportAll(boolean exportAll) {
    isExportAll = exportAll;
  }

  /** To get export all status
   * @return boolean export status
   * @return void
   */
  public boolean getExportAll() {
    return isExportAll;
  }

  /** To add to the export list
   * @param exportIn The export label to be added
   * @return void
   */
  public void addExportItem(Expression exportIn) {
    if (exportList == null)
      { exportList = new ExpressionList(); }
    exportList.add(exportIn);
    addChildObject(exportIn);
  }

  /** To set export list
   * @param listIn, list of export names to be set
   * @return void
   */
  public void setExportItems(ExpressionList listIn) {
    removeChildList(exportList);
    exportList = listIn;
    addChildList(exportList);
  }

  /** To get export list
   * @return exportList
   */
  public ExpressionList getExportItems() {
    return exportList;
  }




  /** Method to make it easier for subclasses to pretty-print.
   * @return String representing the string of this object.
   */
  protected String partialString()
    {
	int pos;
	String answer;

	answer = ":: " + domainRef + " is (";
	if ((interfaceItems != null) && (!interfaceItems.isEmpty()))
	  {
	  answer = answer + interfaceItems.get(0);
	  for (pos = 1; pos < interfaceItems.size(); pos++)
	    { answer = answer + "; " + interfaceItems.get(pos); }
	  }
	answer = answer + ") ->";
        if ((useClauses != null) && (!useClauses.isEmpty()))
          {
          answer = answer + useClauses.get(0);
          for (pos = 1; pos < useClauses.size(); pos++)
            { answer = answer + ", " + useClauses.get(pos); }
          }
        answer = answer + "-> {";
	if ((declaredItems != null) && (!declaredItems.isEmpty()))
	  {
	  answer = answer + declaredItems.get(0);
	  for (pos = 1; pos < declaredItems.size(); pos++)
	    { answer = answer + "; " + declaredItems.get(pos); }
	  }
	answer = answer + "} [";
	if ((termItems != null) && (!termItems.isEmpty()))
	  {
	  answer = answer + termItems.get(0);
	  for (pos = 1; pos < termItems.size(); pos++)
	    { answer = answer + "; " + termItems.get(pos); }
	  }
	answer = answer + "] -> {";
	if ((exportList != null) && (!exportList.isEmpty()))
	  {
	  answer = answer + exportList.get(0);
	  for (pos = 1; pos < exportList.size(); pos++)
	    { answer = answer + "; " + exportList.get(pos); }
	  }
	answer = answer + "}";
	return answer;
	}

  /** To get the string representation of the object
   * @return String representing the string of this object.
   */
  public String toString()
    { return "facet" + partialString(); }








  /** Method for accessing the domain of this facet for printing.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the domain declaration of this object.
   */
  protected String publishDomain(String indent)
    {
    if (domainRef != null)
      { return domainRef.publishROM(indent + defaultPublishROMIndent); }
    else
      { return "NULL"; }
    }


  /** Method for accessing the 'context' of this facet for printing.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the context declaration of this object.
   */
  protected String publishUses(String indent)
    {
    int pos, deIndent;
    String answer;

    if (indent.length() < 2)
      { deIndent = 0; }
    else
      { deIndent = 2; }

    if ((useClauses != null) && (!useClauses.isEmpty()))
      {
      answer = "use " + useClauses.get(0).publishROM(indent + defaultPublishROMIndent);
      for (pos = 1; pos < useClauses.size(); pos++)
        { answer = answer + ", " + useClauses.get(pos).publishROM(indent + defaultPublishROMIndent); }
      answer = answer + ";\n" + indent.substring(0, indent.length()-deIndent);
      return answer;
      }
    else
      { return ""; }
    }


  /** Method for accessing the interface of this facet for printing.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the interface declaration of this object.
   */
  protected String publishInterface(String indent)
  {
    // System.out.println("Called publishInterface");
    int pos;
    String answer = "(";

    if ((interfaceItems != null) && (!interfaceItems.isEmpty()))
      {
      answer = "(" + interfaceItems.get(0).publishROM(indent + defaultPublishROMIndent);
      for (pos = 1; pos < interfaceItems.size(); pos++)
        { answer = answer + "; " + interfaceItems.get(pos).publishROM(indent + defaultPublishROMIndent); }
      answer = answer + ")";
      return answer;
      }
    else
      { return answer + ")"; }
    }


  /** Method for accessing the declarations of this facet for printing.
   * @param indent The current string to use as a logical block indentor.
   * @param eol The string to use as an end-of-line feed.
   * @return String representing the declarations of this object.
   */
  protected String publishDeclarations(String indent, String eol)
    {
    int pos;
    String answer;

    if (declaredItems != null)
      {
      answer = "";
      for (pos = 0; pos < declaredItems.size(); pos++)
        { answer = answer + declaredItems.get(pos).publishROM(indent + defaultPublishROMIndent) + ";" + eol + indent; }
      return answer;
      }
    else
      { return ""; }
    }


  /** Method for accessing the exports for this facet for printing.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the export declaration of this object.
   */
  protected String publishExports(String indent)
    {
    int pos;
    String answer;

    if (isExportAll)
      { return "export ALL;\n" + indent; }

    if ((exportList != null) && (!exportList.isEmpty()))
      {
      answer = "export " + exportList.get(0);
      for (pos = 1; pos < exportList.size(); pos++)
        { answer = answer + ", " + exportList.get(pos); }
      answer = answer + ";\n" + indent;
      return answer;
      }
    else
      { return ""; }
    }




  /** Method to make it easier to pretty-print.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  protected String partialPublish(String indent)
    {
    int pos, deIndent;
    String answer;

    if (indent.length() < 2)
      { deIndent = 0; }
    else
      { deIndent = 2; }

    answer = publishInterface(indent);
    answer = answer + "::" + publishDomain("") + " is\n" + indent;
    answer = answer + publishDeclarations(indent, "\n") + publishExports(indent);
    answer = answer.substring(0, answer.length()-deIndent); // + "begin " + publishDomain(indent);
    answer = answer + "\n" + indent;
    if (termItems != null) {
      for (Iterator termIter = termItems.iterator(); termIter.hasNext();){
	Item term = (Item) termIter.next();
	if (! term.getLabel().toString().startsWith("__#")) {
	  answer = answer +  term.publishTerm(indent + defaultPublishROMIndent) + ";\n" + indent;
	} // end of if (! term.getLabel().toString().startsWith("__#"))
	 
      } // end of for (Iterator termIter = termItems.iterator(); termIter.hasNext();) 
    }
    answer = answer.substring(0, answer.length()-deIndent) + "end";
    return answer;
    }


  /** Method to get the infix string declaration of the object
   * @param myLabel The name of this syntactical shorthand object.
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  public String publishROM(Label myLabel, String indent)
    { return publishUses(indent) + "facet " + myLabel + partialPublish(indent) + " facet " + myLabel; }



  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object.
   */
  public String publishROM(String indent)
    { return "facet" + partialPublish(indent) + " facet"; }


}
