//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: ItemList.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ItemList.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:08:46  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/04/17 19:55:05  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:19  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.1.2.1  2001/11/14 19:17:19  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.1  2000/08/09 23:03:09  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from List.java, made Universe a subclass of BaseObject. --David
//
//  Revision 1.1  2000/08/09 04:29:53  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>ItemList</B> object. The contents of
 *  this list are guarenteed to be Items.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class ItemList extends BaseObjectList {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /** The primary Constructor
   */
  public ItemList()
    { }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public ItemList(Map settings)
    { super(settings); }


  /** To add an Item to the list.
   * @param item The Item to add
   * @return void
   */
  public void add(Item item)
    { addObject(item); }

  /** To add an Item to the list in a specific position.
   * @param pos The position in the list to add the Item to.
   * @param item The Item to add.
   * @return void
   */
  public void add(int pos, Item item)
    { addObject(pos, item); }


  /** To add lots of Items to the list.
   * @param items The ItemList to add
   * @return void
   */
  public void add(ItemList items)
    {
    int pos;

    if (items == null)
      { return; }

    for (pos = 0; pos < items.size(); pos++)
      { add(items.get(pos)); }
    }



  /** To remove an Item from the list by position.
   * @param int The position to remove the Item from.
   * @return Item The Item removed from that position.
   */
  public Item remove(int pos)
    { return (Item)removeObject(pos); }


  /** To retrieve the first Item in the list.
   * @return Item The first Item.
   */
  public Item firstElement()
    { return (Item)getObjectAt(0); }


  /** To retrieve the last Item in the list.
   * @return Item The last Item in the list
   */
  public Item lastItem()
    { return (Item)getObjectAt(size()-1); }


  /** Method to retreive an Item from the list by position.
   * @param pos The position to retreive the Item from.
   * @return Item The Item found at that position.
   */
  public Item get(int pos)
    { return (Item)getObjectAt(pos); }


  /** Method to retreive all the Items from the list.
   * @return Item[] The Items by position.
   */
  public Item[] toArray()
    {
    int pos;
    BaseObject[] base;
    Item[] answer;

    base = toObjectArray();
    answer = new Item[base.length];
    for (pos = 0; pos < base.length; pos++)
      { answer[pos] = (Item)base[pos]; }
    return answer;
    }


  }

