//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: LabelList.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: LabelList.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:09:11  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/04/17 19:55:06  cvs
//  Modified ROM structural elements, such as Item and Label, to inherit from Expression. This allows the meta-functions to operate along the same lines as any other function.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:19  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.2.2.2  2002/01/04 12:30:49  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.2.2.1  2001/11/14 19:17:20  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.2  2001/03/26 19:01:42  cvs
//  Completed the implementation for the change in export clauses.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:11  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>LabelList</B> object. The contents
 *  of this list are guarenteed to be Labels.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class LabelList extends BaseObjectList
  {
  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };



  /** The primary Constructor
   */
  public LabelList()
    { }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public LabelList(Map settings)
    { super(settings); }


  /** To add a Label to the list.
   * @param lbl The Label to add.
   * @return void
   */
  public void add(Label lbl)
    { addObject(lbl); }


  /** To add a Label to the list in a specific position.
   * @param pos The position in the list to add the Label to.
   * @param lbl The Label to add.
   * @return void
   */
  public void add(int pos, Label lbl)
    { addObject(pos, lbl); }


  /** To remove a Label from the list by position.
   * @param pos The position to remove the Label from.
   * @return Label The Label removed from that position.
   */
  public Label remove(int pos)
    { return (Label)removeObject(pos); }


  /** To retrieve the first Label in the list.
   * @return Label The first Label.
   */
  public Label firstElement()
    { return (Label)getObjectAt(0); }


  /** To retrieve the last Label in the list.
   * @return Label The last Label in the list.
   */
  public Label lastElement()
    { return (Label)getObjectAt(size()-1); }


  /** To retrieve a Label from the list by position.
   * @param pos The position to retreive the Label from.
   * @return Label The Label found at that position.
   */
  public Label get(int pos)
    { return (Label)getObjectAt(pos); }


  /** Method to retreive all the Labels from the list.
   * @return Label[] The Labels by position.
   */
  public Label[] toArray()
    {
    int pos;
    BaseObject[] base;
    Label[] answer;

    base = toObjectArray();
    answer = new Label[base.length];
    for (pos = 0; pos < base.length; pos++)
      { answer[pos] = (Label)base[pos]; }
    return answer;
    }


  /** Method to check if two lists equal by positional label equals tests.
   * @param lhs The list to check equality against.
   * @param rhs The list to check for equality against the first list.
   * @return boolean Do the lists have exactly the same label contents?
   */
  public static boolean equal(LabelList lhs, LabelList rhs)
    {
    int pos;

    if ((lhs == null) || (rhs == null))
      { return (lhs == rhs); }
    if (lhs.size() != rhs.size())
      { return false; }
    for (pos = 0; pos < lhs.size(); pos++)
      {
      if (lhs.get(pos) == rhs.get(pos))
        { continue; }
      if (((lhs.get(pos) == null) && (rhs.get(pos) != null))
          || !lhs.get(pos).equals(rhs.get(pos)))
        { return false; }
      }
    return true;
    }


}


