//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Literal.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Literal.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:09:50  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:59  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:20  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.4.2.1  2001/11/14 19:17:20  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.4  2000/12/03 23:59:46  cvs
//  Added the new types of array, set and sequence. The methods getType  and alreadyDeclared are also added to get a type that has been previously declared.
//  --srinivas
//
//  Revision 1.3  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.2  2000/08/10 01:04:12  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:12  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:44:15  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>Literal</B> object. An instance of this
 *  class indicates a primitive sematically irreducible value.
 *  Any primitive whose value is defined by the language spec should be
 *  encapsulated by one of these (such as integers, true/false, etc...).
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public abstract class Literal extends Expression  {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };


  /** The primary constructor.
   */
  public Literal() {
  }


  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public Literal(Map settings)
    {
    super(settings);
    }
 
}

		

