//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: NumberLiteral.java,v 1.1 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: NumberLiteral.java,v $
//  Revision 1.1  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 20:57:40  kimmell
//  1. Removed instantiate and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:51:55  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:17  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.7.2.4  2002/01/04 12:30:47  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.7.2.3  2001/11/24 06:44:45  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.7.2.2  2001/11/14 19:17:18  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.7.2.1  2001/11/07 22:41:03  cvs
//  Added treenode functionality to the ROM.
//  -Joseph
//
//  Revision 1.7  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.5  2001/02/01 22:34:29  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.4  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.3  2000/08/17 20:48:22  cvs
//  Added some members to the literal classes.
//  Modified the lookup algorithm in Universe to remove an error.
//  -- Cindy
//
//  Revision 1.2  2000/08/10 01:04:12  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:07  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:44:15  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>NumberLiteral</B> object. An instance of this
 *  class indicates a literal Number in the ROM. Note that in Rosetta
 *  the boolean type is also treated as a number.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class NumberLiteral extends Literal {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { "Data" };



  Number numericData;

  /** The primary constructor
   * @param numericIn Numeric value of the data
   */
  public NumberLiteral(Number numericIn) {
    numericData = numericIn;
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public NumberLiteral(Map settings)
    {
    super(settings);

    numericData = (Number)getSetting(NumberLiteral.keySet[0]);
    }



  /** To get the numeric value
   * @return Number
   */
  public Number getData(){
    return numericData;
  }

  /** To set the numeric value
   * @param num The new Number value of this Literal.
   * @return void
   */
  public void setData(Number num){
    numericData = num;
  }




  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString() {
    return "" + numericData;
  }


  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent) {
    return toString();
  }

}


