//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: SetValue.java,v 1.3 2002/07/16 07:15:48 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SetValue.java,v $
//  Revision 1.3  2002/07/16 07:15:48  cvs
//  1. Renamed DataLiteral to NumberLiteral.
//  2. Added a Map-based constructor to the ROM classes.
//  3. Removed some old ROM classes that were not used anymore.
//  4. Cleaned up a few inconsistancies in the class inheritance tree.
//
//  Revision 1.2  2002/07/08 21:10:25  kimmell
//  1. Remove instantiate, duplicate, and matches methods.
//  2. Removed implementation of Visitable interface.
//  -kimmell
//
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.2  2002/03/12 22:52:01  cvs
//  Made several modifications and cleanups to ROM.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/17 00:22:21  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.6.2.3  2002/01/04 12:30:50  cvs
//  Updated ROM with 'instantiation' interface implementation.
//
//  Revision 1.6.2.2  2001/11/24 06:44:48  cvs
//  Fixed the constructed types. A big upgrade on all pretty-printers so now
//  regression testing works. Implicit operators (array references, non-direct
//  function calls, lists of function calls, etc...) now supported.
//
//  Revision 1.6.2.1  2001/11/14 19:17:21  cvs
//  Updated the ROM classes to implements Types as Expressions. Fixed up the
//  commenting in several classes. Also added classes needed for changes
//  to the grammar. Lastly composite values now have a common sub-class.
//
//  Revision 1.6  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.5  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some accessor functions are also added
//  --srinivas
//
//  Revision 1.4  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.3  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all other classes affected by this change. -David
//
//  Revision 1.2  2000/08/16 18:44:39  cvs
//  Testing CVS.
//  -- Cindy
//
//  Revision 1.1  2000/08/11 16:28:25  cvs
//  Replacing Array.java, Set.java, and Sequence.java with files named
//  xxxValue.java. All three are a bit more fleshed out as well, with
//  accessors. Purpose of replacement: To distinguish these classes
//  from those of same name to appear (soon) as subclasses of
//  Type. -David 
//
//  Revision 1.2  2000/08/10 01:04:13  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:12  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 15:43:11  cvs
//  Just adding some class skeltons for various subclasses of Expression
//
//
//=============================================================================
package org.sldl.rosetta.ROM;

import java.util.Map;


/** This represents the <B>SetValue</B> object. An instance of this
 *  class indicates an actual set has been constructed.
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class SetValue extends CompositeValue {

  /** The <code>keySet</code> array lists a collection of keys
   *  correspond to a description of the internal state of an
   *  instance of this class when considered in concert with all
   *  other <code>keySet</code> arrays in its superclasses.
   */
  public static final String[] keySet = { };



  /** The primary constructor
   * @param elementsIn The list of elements in set
   */
  public SetValue(ExpressionList elementsIn) {
    super(elementsIn);
  }

  /** The secondary constructor
   */
  public SetValue() {
  }

  /** Map Constructor.
   * @param settings The Map containing the default settings of this object.
   */
  public SetValue(Map settings)
    {
    super(settings);
    }



  /** To get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString() {
    return "{" + contentString(", ") + "}";
  }
  
  
  /** To get the string declaration of the object
   * @param indent The current string to use as a logical block indentor.
   * @return String representing the declaration of this object
   */
  public String publishROM(String indent) {
    return "{" + contentPublish(", ", indent) + "}";
  }
  
}


