//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: ComplexNumber.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ComplexNumber.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:22  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/04 12:40:48  cvs
//  Updated support for the number types Rosetta supports.
//
//
//=============================================================================
package org.sldl.rosetta.lang;

/** This represents a <B>Complex</B> number. A complex number
 *  is nothing more than a wrapper for two other numbers, one
 *  representing the 'real' part, the other the 'imaginary'.
 * @author <I>Joseph Kuehn</I> (josephk@cs.adelaide.edu.au)
 */
public final class ComplexNumber extends Number
  {
  /** Method to generate a complex number that matches a given string.
   * @param image The string to convert to a complex number.
   * @throws NumberFormatException If the string is illegal.
   */
  public static ComplexNumber valueOf(String image) throws NumberFormatException
    {
    return null;
    }





  private Number realPart, imPart;

  /** The primary Constructor.
   * @param real The real part of the complex number.
   * @param im The imaginary part of the complex number.
   */
  public ComplexNumber(Number real, Number im)
    {
    realPart = real;
    imPart = im;
    }


  /** Method to retreive the real part of the number.
   * @return Number The real part.
   */
  public Number getReal()
    { return realPart; }

  /** Method to retreive the imaginary part of the number.
   * @return Number The imaginary part.
   */
  public Number getIm()
    { return imPart; }


  /** Method to generate a string for the complex number.
   * @return String The string representation of the complex number.
   */
  public String toString()
    { return (realPart + "j" + imPart); }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public byte byteValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.byteValue();
    }

  /** Returns the value of the specified number as a double.
   * @return double The double value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public double doubleValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.doubleValue();
    }

  /** Returns the value of the specified number as a float.
   * @return float The float value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public float floatValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.floatValue();
    }

  /** Returns the value of the specified number as a int.
   * @return int The int value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public int intValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.intValue();
    }

  /** Returns the value of the specified number as a long.
   * @return long The long value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public long longValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.longValue();
    }

  /** Returns the value of the specified number as a short.
   * @return short The short value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public short shortValue()
    {
    if (imPart.doubleValue() != 0.0)
      { throw new NumberFormatException("The imaginary part is non-zero."); }
    return realPart.shortValue();
    }
  }
