//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: NumberParser.java,v 1.1 2002/11/22 03:03:05 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: NumberParser.java,v $
//  Revision 1.1  2002/11/22 03:03:05  josephk
//  Added the default NumberParser to the rosetta.lang directory.
//
//  Revision 1.3  2002/09/12 05:45:39  josephk
//  Moved the files across to the "ROM" implementation.
//
//  Revision 1.2  2002/05/30 12:53:35  josephk
//  Minor clean fix for useability.
//
//  Revision 1.1  2002/04/22 07:53:39  josephk
//  Added the NumberParser to org.sldl.rosetta.lang for the basic number types.
//
//
//=============================================================================
package org.sldl.rosetta.lang;

import java.io.Serializable;
import org.sldl.rosetta.lang.logic.RationalNumber;
import org.sldl.rosetta.lang.logic.RadixNumber;
import org.sldl.rosetta.lang.logic.ComplexNumber;
import org.sldl.rosetta.lang.logic.BooleanNumber;


/** Standard static class to provide methods for parsing
 *  the literal number types provided by Rosetta.
 */
public class NumberParser implements Serializable
  {
  public static final String[] keySet = { };


  /** Method to get the String representation of a number.
   * @param num The number to convert into a string form.
   * @return String The printed form of the number.
   */
  public static String toString(Number num)
    { return "" + num; }


  /** Method to attempt to parse a Rosetta number from a given string.
   * @param image The string to be parsed into a number.
   * @return Number The result of parsing the string.
   * @throws NumberFormatException If no number can be formed from that string.
   */
  public static Number valueOf(String image) throws NumberFormatException
    {
    // Integer needs to go first.
    try
      { return Integer.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Double needs to go second.
    try
      { return Double.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a couple of '\\'.
    try
      { return RadixNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a 'j' and possibly a '+', '*', etc...
    try
      { return ComplexNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a '/'.
    try
      { return RationalNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    return BooleanNumber.valueOf(image);
    }

  }

