//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Joseph josephk@cs.adelaide.edu.au
//
//  $Id: RadixNumber.java,v 1.1.1.1 2002/07/01 15:47:35 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: RadixNumber.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:35  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:22  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.1  2002/01/04 12:40:48  cvs
//  Updated support for the number types Rosetta supports.
//
//
//=============================================================================
package org.sldl.rosetta.lang;

/** This represents a <B>RadixNumber</B> number. A radix-number
 *  consists of a string of digits with an optional single decimal
 *  point in it somewhere, an integer radix that defines
 *  the interpretation of those digits and an integer exponent that
 *  allows for a shift of the decimal point.
 * @author <I>Joseph Kuehn</I> (josephk@cs.adelaide.edu.au)
 */
public final class RadixNumber extends Number
  {
  /** Method to generate a radix-number that matches a given string.
   * @param image The string to convert to a radix-number.
   * @throws NumberFormatException If the string is illegal.
   */
  public static RadixNumber valueOf(String image) throws NumberFormatException
    {
    return null;
    }





  private int radix, exponent;
  private String digitString;

  /** The primary Constructor.
   * @param rad The radix of the digits.
   * @param digits The string that represents the number.
   * @param exp The exponent of the number.
   * @throws NumberFormatException If the radix is less than 2 or the
   *                               digits have more than one decimal point.
   */
  public RadixNumber(int rad, String digits, int exp) throws NumberFormatException
    {
    int deciPos;

    radix = rad;
    if (radix < 2)
      { throw new NumberFormatException("A radix of less than 2 cannot form a legal number."); }
    digitString = digits;
    deciPos = digitString.indexOf('.');
    if (deciPos != -1)
      {
      if (deciPos != digitString.substring(deciPos+1, digitString.length()).indexOf('.'))
        { throw new NumberFormatException("A radix-number cannot have more than one decimal point."); }
      }
    exponent = exp;
    }


  /** Method to generate a string for the rational.
   * @return String The string representation of the rational.
   */
  public String toString()
    {
    if (exponent != 0)
      { return (radix + "\\" + digitString + "\\e" + exponent); }
    else
      { return (radix + "\\" + digitString + "\\"); }
    }


  /** Method to determine if the value of the specified number is an integer.
   * @return boolean Do the digits represent an Integer?
   */
  public boolean isInteger()
    { return (digitString.indexOf('.') == -1); }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this rational.
   */
  public byte byteValue()
    { return (byte)0; }


  /** Returns the value of the specified number as a double.
   * @return double The double value of this rational.
   */
  public double doubleValue()
    { return 0.0; }


  /** Returns the value of the specified number as a float.
   * @return float The float value of this rational.
   */
  public float floatValue()
    { return (new Double(0.0)).floatValue(); }


  /** Returns the value of the specified number as a int.
   * @return int The int value of this rational.
   */
  public int intValue()
    { return 0; }


  /** Returns the value of the specified number as a long.
   * @return long The long value of this rational.
   */
  public long longValue()
    { return 0; }


  /** Returns the value of the specified number as a short.
   * @return short The short value of this rational.
   */
  public short shortValue()
    { return 0; }


  }
