//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PrimIntegerEquals.java,v 1.2 2002/11/15 17:36:28 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: PrimIntegerEquals.java,v $
// Revision 1.2  2002/11/15 17:36:28  kimmell
// Updated primitives functions so that the type of the primitive is no longer an instance of FunctionType. FunctionType is no longer present in the ROM, and a Function should be used instead. Currently, the code for setting the type is just commented out. See Bug 260 for the status of this issue.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:35  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.2  2002/04/17 19:48:48  cvs
// Added Primitive functions for operations on Reals.
// -kimmell
//
// Revision 1.1.2.1  2002/03/21 20:25:53  cvs
// Added some functions for dealing with primitive integer arithmetic.
// Made changes to BooleanValue and IntValue fixing a bug in the creation of the values.
// -kimmell
//
package org.sldl.rosetta.lang.functions;

import org.sldl.rosetta.parser.primitive.PrimitiveFunction;
import org.sldl.rosetta.tools.evaluator.Evaluator;
import java.util.List;

import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;

import org.sldl.rosetta.ROM.Function;



/**
 * PrimIntegerEquals.java
 *
 *
 * Created: Tue Mar 19 16:38:39 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

import org.sldl.rosetta.lang.types.IntType;
import org.sldl.rosetta.lang.values.IntValue;
import org.sldl.rosetta.lang.values.BooleanValue;
import org.sldl.rosetta.lang.types.BooleanType;



public class PrimIntegerEquals extends PrimitiveFunction  {
  Function myType;

  public PrimIntegerEquals (){
    /*
    ExpressionList type = new ExpressionList();
    type.add(new IntType());
    type.add(new IntType());
    type.add(new BooleanType());
    myType = new FunctionType(type);
    */
  }
  // implementation of org.sldl.rosetta.parser.primitive.PrimitiveFunction interface

  /**
   *
   * @return <description>
   */
  public String getName()
  {
    // TODO: implement this org.sldl.rosetta.parser.primitive.PrimitiveFunction method
    return "lang.logic.=";
  }

  /**
   *
   * @return <description>
   */
  public Expression getType()
  {
    // TODO: implement this
    // org.sldl.rosetta.parser.primitive.PrimitiveFunction method
    return myType;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object invoke(List params, Evaluator eval)
  {
    try {
      Object arg1 = params.get(0);
      Object arg2 = params.get(1);
      IntValue val1 = (IntValue) eval.traverse(arg1);
      IntValue val2 = (IntValue) eval.traverse(arg2);
      Integer int1 = (Integer) val1.getPrimitiveValue();
      Integer int2 = (Integer) val2.getPrimitiveValue();
      BooleanValue result = new BooleanValue(int1.intValue() ==
					     int2.intValue());
      return result;
    
    } catch (Exception e) {
      System.out.println("Evaluation Exception");
      return null;
    } // end of catch
  }

}// PrimIntegerEquals
