//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PrimTail.java,v 1.2 2002/11/15 17:36:28 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: PrimTail.java,v $
// Revision 1.2  2002/11/15 17:36:28  kimmell
// Updated primitives functions so that the type of the primitive is no longer an instance of FunctionType. FunctionType is no longer present in the ROM, and a Function should be used instead. Currently, the code for setting the type is just commented out. See Bug 260 for the status of this issue.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:35  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.1  2002/05/30 18:59:56  cvs
// Added the following primitive functions:
// 1. null?
// 2. head
// 3. tail
// --morel
//
//
//============================================================================

// Java Package
package org.sldl.rosetta.lang.functions;

// Parser Imports
import org.sldl.rosetta.parser.primitive.PrimitiveFunction;

// Tools Imports
import org.sldl.rosetta.tools.evaluator.Evaluator;

// Java Imports
import java.util.List;

// ROM Imports
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Function;

// Lang Imports
import org.sldl.rosetta.lang.types.IntType;
import org.sldl.rosetta.lang.values.IntValue;
import org.sldl.rosetta.lang.values.BooleanValue;
import org.sldl.rosetta.lang.types.BooleanType;


/** This represents the <B>Tail Function</B>.
 * @author <a href="mailto:morel@ittc.ku.edu">Brandon Morel</a>
 * @version
 */
public class PrimTail extends PrimitiveFunction {


  /** This is the main constructor
   */
  public PrimTail() {    
    myName = "lang.logic.tail";
    /*
    ExpressionList type = new ExpressionList();
    myType = new FunctionType(type);
    */
  }


  /** This is the invoke routine
   * @param params a List of parameters
   * @param eval the Evaluator
   * @return Object version of the equals application expression
   */
  public Object invoke(List params, Evaluator eval)
  {
    try {

      ExpressionList arg1 = (ExpressionList)params.get(0);
      if (arg1 == null)
	return null;
      else {
	ExpressionList returnList = new ExpressionList();
	for (int pos = 1; pos < arg1.size(); pos++) { 
	  returnList.add(arg1.get(pos));
	}
	return returnList;
      }

    } catch (Exception e) {
      System.out.println("Primitive Tail Function Exception");
      return (Expression)null;
    }
  }
}
