//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: MetaEqual.java,v 1.2 2002/11/15 17:36:28 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: MetaEqual.java,v $
// Revision 1.2  2002/11/15 17:36:28  kimmell
// Updated primitives functions so that the type of the primitive is no longer an instance of FunctionType. FunctionType is no longer present in the ROM, and a Function should be used instead. Currently, the code for setting the type is just commented out. See Bug 260 for the status of this issue.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:35  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.3  2002/05/24 21:39:01  cvs
// Fixed bug in meta_equal function again.
// --morel
//
// Revision 1.1.2.2  2002/05/24 19:30:40  cvs
// Fixed meta_equal type bug.
// --morel
//
// Revision 1.1.2.1  2002/05/24 17:36:19  cvs
// Added meta_equal and meta_getOperators meta functions.
// --morel
//
//
//============================================================================

// Java Package
package org.sldl.rosetta.lang.functions.meta;

// Import Java Classes
import java.util.List;

// Import ROM Classes
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
// import org.sldl.rosetta.ROM.FunctionType;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.ItemReference;

// Import Primitive Base Classes
import org.sldl.rosetta.parser.primitive.PrimitiveFunction;

// Import Primitive Classes
import org.sldl.rosetta.lang.types.meta.FacetType;

// Import Tools Classes
import org.sldl.rosetta.tools.evaluator.Evaluator;


/** This represents the <B>Equal Meta Function</B>.
 * @author <a href="mailto:morel@ittc.ku.edu">Brandon Morel</a>
 * @version
 */
public class MetaEqual extends PrimitiveFunction {


  /** This is the main constructor
   */
  public MetaEqual() {

    // Init meta_equal
    myName = "lang.reflect.meta_equal";
    /*
    ExpressionList type = new ExpressionList();
    myType = new FunctionType(type);
    */
  }

    
  /** This is the invoke routine
   * @param params a List of parameters
   * @param eval the Evaluator
   * @return Object version of the equals application expression
   */
  public Object invoke(List params, Evaluator eval) {

    // Create an equals application
    Application eqApp = new Application(new ItemReference("="), 
					(ExpressionList)params, 
					Application.BINARY);
    return (Expression)eqApp;
  }
  
}





