//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: MetaGetOperators.java,v 1.2 2002/11/15 17:36:28 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: MetaGetOperators.java,v $
// Revision 1.2  2002/11/15 17:36:28  kimmell
// Updated primitives functions so that the type of the primitive is no longer an instance of FunctionType. FunctionType is no longer present in the ROM, and a Function should be used instead. Currently, the code for setting the type is just commented out. See Bug 260 for the status of this issue.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:35  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.3  2002/06/06 19:16:11  kimmell
// Fixed a bug that occured when trying to collect operators from something
// other than applications.
// -kimmell
//
// Revision 1.1.2.2  2002/06/05 16:35:55  kimmell
// Fixed a bug which threw a NullPointerException due to the result list not being initialized.
// -kimmell
//
// Revision 1.1.2.1  2002/05/24 17:36:19  cvs
// Added meta_equal and meta_getOperators meta functions.
// --morel
//
//
//
//============================================================================

// Java Package
package org.sldl.rosetta.lang.functions.meta;

// Import Java Classes
import java.util.List;
import java.util.Iterator;

// Import ROM Classes
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.IfExpression;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.Label;

// Import Primitive Base Classes
import org.sldl.rosetta.parser.primitive.PrimitiveFunction;

// Import Primitive Classes
import org.sldl.rosetta.lang.types.meta.FacetType;

// Import Tools Classes
import org.sldl.rosetta.tools.evaluator.Evaluator;


/** This represents the <B>Get Operators Meta Function</B>.
 * @author <a href="mailto:morel@ittc.ku.edu">Brandon Morel</a>
 * @version
 */
public class MetaGetOperators extends PrimitiveFunction {


  /** This is the main constructor
   */
  public MetaGetOperators() {

    // Init meta_operators
    myName = "lang.reflect.meta_getOperators";
    ExpressionList type = new ExpressionList();
    type.add(new ItemReference("Expression"));
    type.add(new ItemReference("ExpressionList"));
    myType = new Function(type);
  }


  /** This is the get operators routine
   * @param value the term Expression
   * @return an ExpressionList of operators
   */
  public ExpressionList getOps(Expression value) {

    // Local Variables
    ExpressionList opList = new ExpressionList();
    ExpressionList recurList = null;
    Application thisApp = null;
    ItemReference iRef = null;
    Item referredItem = null;
    Expression referredValue = null;

    // If value is not null
    if(value != null) {
      // System.out.println("Value is of type " + value.getClass().getName());
      // If an instance of application
      if(value instanceof Application) {
	thisApp = (Application)value;
	if(thisApp != null) {
	  
	  // A normal function
	  if(thisApp.getApplicationType() == Application.NORMAL) {
	    // System.out.println("GetOps: Application is " + thisApp);
	    Expression funct = thisApp.getFunction();
	    if(funct != null) {
	      
	      // Get all operators in the function
	      if(thisApp.getFunction() instanceof ItemReference) {
		iRef = (ItemReference)thisApp.getFunction();
		if(iRef != null) {
		  referredItem = iRef.getItemReferred();
		  if(referredItem != null) {
		    /*
		    System.out.println(iRef + 
				       "'s itemReferred is " +
				       referredItem);
		    */
		    referredValue = referredItem.getValue();
		    
		    // System.out.println("Type of Referred Value: " + referredValue.getClass().getName());
		      recurList = getOps(referredValue);
		    if(recurList != null)
		      opList.add(recurList);  
		  } else {
		    
		    // System.out.println("ItemReference of " + thisApp + " is null");
		  } // end of else
		  
		}
	      } else {
		// System.out.println("Function type: " + thisApp.getFunction().getClass().getName());
	      } // end of else
	      
	    }
  	  }

	  // Else add function operator if binary, prefix, or postfix
	  else 
	    // System.out.println("Application is " + thisApp);
	    try {
	      //System.out.println(thisApp.getFunction());
	      opList.add(new Label(thisApp.getFunction().toString()));
	    } catch (Exception e) {
	      System.out.println("Could not get function for " +
				 thisApp);
	      System.out.println(e);
	    } // end of try-catch
	  
	  //	    opList.add(new Label(thisApp.getFunction().toString()));
	  
	  // Check the list of function paramaters
	  ExpressionList params = thisApp.getParameters();
	  Iterator paramIter = params.iterator(); 
	  while(paramIter.hasNext()) {
	    recurList = getOps((Expression)paramIter.next());
	    if(recurList != null)
	      opList.add(recurList);
	  }
	}
      } else if (value instanceof Function) {
	Function thisFun = (Function) value;
	return getOps(thisFun.getExpression());
	
      } else if (value instanceof IfExpression) {
	IfExpression ifExpr = (IfExpression) value;
	Expression ifs = ifExpr.getIfValue();
	recurList = getOps(ifs);
	if (recurList != null) {
	  opList.add(recurList);
	} // end of if (recurList != nul)

	ifs = ifExpr.getThenValue();
	recurList = getOps(ifs);
	if (recurList != null) {
	  opList.add(recurList);
	} // end of if (recurList != nul)
	
	recurList = getOps((Expression)ifExpr.getElseValue());
	if (recurList != null) {
	  opList.add(recurList);
	} // end of if (recurList != null)
	
      } // end of else
      


    }

    // Return the list of operators
    return opList;
  }
  

  /** This is the invoke routine
   * @param params a List of parameters
   * @param eval the Evaluator
   * @return Object version of the list of operators
   */
  public Object invoke(List params, Evaluator eval) {

    // Local Variables
    Item arg1 = (Item)params.get(0);
    Facet facet = (Facet)arg1.getValue();
    ItemList terms = facet.getTermItems();
    ExpressionList returnList = new ExpressionList();
    ExpressionList getOpList = null;
    Item termItem = null;
    Expression termValue = null;

    // For each term, get operators
    Iterator termIter = terms.iterator(); 
    while(termIter.hasNext()) {
      termItem = (Item)termIter.next();
      termValue = termItem.getValue();

      // Get ops on the value of the term
      getOpList = getOps(termValue);
      if(getOpList != null)
	returnList.add(getOpList);  
    } 

    // Return the list of operators
    return returnList;
  }
  
}
