//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: BooleanNumber.java,v 1.1 2002/07/15 04:33:08 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BooleanNumber.java,v $
//  Revision 1.1  2002/07/15 04:33:08  cvs
//  Added the language-defined number implementations to the lang.logic package.
//
//
//=============================================================================
package org.sldl.rosetta.lang.logic;

/** This represents a <B>BooleanNumber</B> number. A boolean number
 *  is nothing more than a wrapper for the Java 'boolean' value, however
 *  we provide an extension of the Java Number class so that we can utilise
 *  the fact that booleans are part of Rosetta's number space.
 */
public final class BooleanNumber extends Number
  {
  /** Method to generate a boolean number that matches a given string.
   * @param image The string to convert to a boolean number.
   * @throws NumberFormatException If the string is illegal.
   */
  public static BooleanNumber valueOf(String image) throws NumberFormatException
    { return new BooleanNumber(Boolean.valueOf(image).booleanValue()); }




  private boolean myValue;

  /** The primary Constructor.
   * @param val The value of this boolean number.
   */
  public BooleanNumber(boolean val)
    { myValue = val; }

  /** Method to generate a string for the complex number.
   * @return String The string representation of the complex number.
   */
  public String toString()
    { return ("" + myValue); }


  /** Method to find the boolean value of this number.
   * @return boolean The value of this boolean number.
   */
  public boolean booleanValue()
    { return myValue; }




  /** Returns the value of the specified number as a byte.
   * @return byte The byte value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public byte byteValue()
    {
    if (myValue)
      { return Byte.MAX_VALUE; }
    else
      { return Byte.MIN_VALUE; }
    }

  /** Returns the value of the specified number as a double.
   * @return double The double value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public double doubleValue()
    {
    if (myValue)
      { return Double.MAX_VALUE; }
    else
      { return Double.MIN_VALUE; }
    }

  /** Returns the value of the specified number as a float.
   * @return float The float value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public float floatValue()
    {
    if (myValue)
      { return Float.MAX_VALUE; }
    else
      { return Float.MIN_VALUE; }
    }

  /** Returns the value of the specified number as a int.
   * @return int The int value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public int intValue()
    {
    if (myValue)
      { return Integer.MAX_VALUE; }
    else
      { return Integer.MIN_VALUE; }
    }

  /** Returns the value of the specified number as a long.
   * @return long The long value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public long longValue()
    {
    if (myValue)
      { return Long.MAX_VALUE; }
    else
      { return Long.MIN_VALUE; }
    }

  /** Returns the value of the specified number as a short.
   * @return short The short value of this complex number.
   * @throws NumberFormatException If the imaginary part is non-zero.
   */
  public short shortValue()
    {
    if (myValue)
      { return Short.MAX_VALUE; }
    else
      { return Short.MIN_VALUE; }
    }


  }


