//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: NumberParser.java,v 1.1 2002/07/15 04:33:08 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: NumberParser.java,v $
//  Revision 1.1  2002/07/15 04:33:08  cvs
//  Added the language-defined number implementations to the lang.logic package.
//
//
//=============================================================================
package org.sldl.rosetta.lang.logic;

import java.io.Serializable;


/** Standard static class to provide methods for parsing
 *  the literal number types provided by Rosetta.
 */
public class NumberParser implements Serializable
  {
  /** Method to get the String representation of a number.
   * @param num The number to convert into a string form.
   * @return String The printed form of the number.
   */
  public static String toString(Number num)
    { return "" + num; }


  /** Method to attempt to parse a Rosetta number from a given string.
   * @param image The string to be parsed into a number.
   * @return Number The result of parsing the string.
   * @throws NumberFormatException If no number can be formed from that string.
   */
  public static Number valueOf(String image) throws NumberFormatException
    {
    // Integer needs to go first.
    try
      { return Integer.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Double needs to go second.
    try
      { return Double.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a couple of '\\'.
    try
      { return RadixNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a '/'.
    try
      { return RationalNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    // Unique - has a 'j' and possibly a '+', '*', etc...
    try
      { return ComplexNumber.valueOf(image); }
    catch(NumberFormatException e)
      { }

    return BooleanNumber.valueOf(image);
    }

  }

