package org.sldl.rosetta.parser.linker.symboltable;

import java.util.Vector;
import java.util.Enumeration;
import javax.swing.tree.TreeNode;

public class DefaultBasicSymbolTableEntry implements BasicSymbolTableEntry
  {
  private final BasicSymbolTableEntry myParent;
  private final Object myEntry;
  private Object myData;
  private Vector myChildren;


  public DefaultBasicSymbolTableEntry(Object root)
    { this(null, root, null); }

  public DefaultBasicSymbolTableEntry(DefaultBasicSymbolTableEntry parent, Object child)
    { this(parent, child, null); }

  public DefaultBasicSymbolTableEntry(DefaultBasicSymbolTableEntry parent, Object child, Object data)
    {
    myParent = parent;
    myEntry = child;
    myData = data;
    myChildren = new Vector();
    }





  public Enumeration children()
    { return myChildren.elements(); }

  public boolean getAllowsChildren()
    { return true; }

  public TreeNode getChildAt(int childIndex)
    { return (TreeNode)(myChildren.get(childIndex)); }

  public int getChildCount()
    { return myChildren.size(); }

  public int getIndex(TreeNode node)
    { return myChildren.indexOf(node); }

  public TreeNode getParent()
    { return myParent; }

  public boolean isLeaf()
    { return (getChildCount() == 0); }




  public Object getObject()
    { return myEntry; }

  public void setData(Object data)
    { myData = data; }

  public Object getData()
    { return myData; }




  public BasicSymbolTableEntry getParentEntry()
    { return myParent; }

  public void addChild(Object childObject)
    { addChildEntry(new DefaultBasicSymbolTableEntry(this, childObject)); }

  public void addChildEntry(BasicSymbolTableEntry child)
    { myChildren.add(child); }

  public BasicSymbolTableEntry getEntryAt(int childIndex)
    { return (BasicSymbolTableEntry)(myChildren.get(childIndex)); }

  public int getIndexOf(Object childObject)
    {
    int pos;
    BasicSymbolTableEntry nextChild;

    for (pos = 0; pos < myChildren.size(); pos++)
      {
      nextChild = getEntryAt(pos);
      if (nextChild.getObject() == childObject)
        { return pos; }
      }
    return -1;
    }

  public int getIndexOf(String childString)
    {
    int pos;
    BasicSymbolTableEntry nextChild;

    for (pos = 0; pos < myChildren.size(); pos++)
      {
      nextChild = getEntryAt(pos);
      if ((nextChild.getObject() != null) && (nextChild.getObject().toString().equals(childString)))
        { return pos; }
      }
    return -1;
    }

  public BasicSymbolTableEntry getChildEntry(Object childObject)
    { return (BasicSymbolTableEntry)(myChildren.get(getIndexOf(childObject))); }

  public BasicSymbolTableEntry getChildEntry(String childString)
    { return (BasicSymbolTableEntry)(myChildren.get(getIndexOf(childString))); }





  public BasicSymbolTableEntry resolveReference(String match)
    {
    int pos;
    BasicSymbolTableEntry nextEntry;

    for (pos = 0; pos < getChildCount(); pos++)
      {
      nextEntry = getEntryAt(pos);
      if (nextEntry.getObject().toString().equals(match))
        { return nextEntry; }
      }
    if (myEntry.toString().equals(match))
      { return this; }
    return resolveHigherReference(match);
    }

  public BasicSymbolTableEntry resolveHigherReference(String match)
    {
    int pos;
    BasicSymbolTableEntry nextEntry;

    if (myParent == null)
      { return null; }
    if (myParent.getObject().toString().equals(match))
      { return myParent; }

    for (pos = 0; pos < myParent.getChildCount(); pos++)
      {
      nextEntry = myParent.getEntryAt(pos);
      if ((nextEntry != this) && (nextEntry.getObject().toString().equals(match)))
        { return nextEntry; }
      }

    return myParent.resolveHigherReference(match);
    }

  public String toString()
    { return "[" + myEntry + "] #" + getChildCount() + " + {" + myData + "}"; }

  }

