//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: DataLiteralFactory.java,v 1.2 2002/11/15 16:58:47 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: DataLiteralFactory.java,v $
// Revision 1.2  2002/11/15 16:58:47  kimmell
// Changed DataLiteral to NumberLiteral. All instances of NumberLiteral
// are constructed using a Double.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.1  2002/03/12 22:35:10  cvs
// Added the LiteralFactory class that allows the scanner to generate literals. Two implementations are also added - one for generating DataLiterals (from the ROM), and another that generates Primitives.
// ROMScanner.jj was modified to take advantage of the Factory.
// -kimmell
//
package org.sldl.rosetta.parser.scanner;

/**
 * DataLiteralFactory.java
 *
 *
 * Created: Thu Mar  7 15:45:43 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.NumberLiteral;
import org.sldl.rosetta.ROM.CharacterLiteral;
import org.sldl.rosetta.ROM.StringLiteral;

public class DataLiteralFactory implements LiteralFactory {
  public DataLiteralFactory (){
    
  }
 
  public Expression createFloat(String value) {
    return new NumberLiteral(makeNumber(value));
  }

  public Expression createInteger(String value) {
    return new NumberLiteral(makeNumber(value));
  }
  
  public Expression createBased(String value) {
    return new NumberLiteral(makeNumber(value)) ;
  }
  
  public Expression createCharacter(String value) {
    return new CharacterLiteral(value);
  }

  public Expression createString(String value) {
    return new StringLiteral(value);
  }

  // TODO: Implement this method.
  public Expression createBoolean(String value) {
    return null;
  }

  private Number makeNumber(String s) {
    return new Double(s);
  }

}// DataLiteralFactory
