//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PrimitiveFactory.java,v 1.1.1.1 2002/07/01 15:47:36 cvs Exp $
//
// Revision History:
// ----------------
// $Log: PrimitiveFactory.java,v $
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.2  2002/03/18 20:21:05  cvs
// Fixed PrimitiveFactory to deal with the new layout of the lang package.
// -kimmell
//
// Revision 1.1.2.1  2002/03/12 22:35:11  cvs
// Added the LiteralFactory class that allows the scanner to generate literals. Two implementations are also added - one for generating DataLiterals (from the ROM), and another that generates Primitives.
// ROMScanner.jj was modified to take advantage of the Factory.
// -kimmell
//
package org.sldl.rosetta.parser.scanner;

/**
 * PrimitiveFactory.java
 *
 *
 * Created: Thu Mar  7 15:57:23 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.lang.values.*;

public class PrimitiveFactory implements LiteralFactory {

  public Expression createFloat(String value) {
    //System.out.println("Creating a float " + value);
    return new RealValue(value);
  }
  
  public Expression createInteger(String value) {
    //System.out.println("Creating an integer " + value);
    return new IntValue(value);
  }
  
  public Expression createBased(String value) {
    //System.out.println("Creating a based " + value);
    return new RealValue(value);
  }
  
  public Expression createCharacter(String value) {
    //System.out.println("Creating a character " + value);
    return new CharacterValue(value);
  }

  public Expression createString(String value) {
    //System.out.println("Creating a string " + value);
    return new StringValue(value);
  }

  public Expression createBoolean(String value) { 
    //System.out.println("Creating a boolean " + value);
    return new BooleanValue(value);
  }

}// PrimitiveFactory
