//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id:
//
//  Revision History:
//  ----------------
//  $Log:
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.antlr;

import antlr.CommonAST;
import antlr.Token;
import org.sldl.rosetta.parser.tarp.antlr.MyCommonToken;

/**
 *This Class over-rides the default Abstract Syntax Tree
 *used by antlr. It stores both line and column information.
 */
public class MyAST extends CommonAST {
    private int beginLine = 0;
    private int beginColumn = 0;
    private int endLine = 0;
    private int endColumn = 0;

    /**
     *This method initializes a MyAST, with line and column infor from the token
     *@param Token the token from which the Node is being made
     */
    public void initialize(Token tok) {
	super.initialize(tok);
	beginLine=((MyCommonToken)tok).getBeginLine();
	beginColumn=((MyCommonToken)tok).getBeginColumn();
	endLine=((MyCommonToken)tok).getEndLine();
	if ( ((MyCommonToken)tok).getText() != null )
	    {
		endColumn=((MyCommonToken)tok).getEndColumn()+((MyCommonToken)tok).getText().length();
	    }
	else endColumn = beginColumn;
    }

    /**
     *The method returns the beginning line number of the Token from which this AST node was derived
     *@return int The begin line number
     */
    public int getBeginLine(){
	return beginLine;
    }

    /**
     *The method returns the end line number of the Token from which this AST node was derived
     *@return int The end line number
     */
    public int getEndLine(){
	return endLine;
    }

    /**
     *The method returns the beginning column number of the Token from which this AST node was derived
     *@return int The begin Column number
     */
    public int getBeginColumn() {
	return beginColumn;
    }

    /**
     *The method returns the end column number of the Token from which this AST node was derived
     *@return int The end Column number
     */
    public int getEndColumn() {
	return endColumn;
    }
    /**
     *The method sets the beginning line number of the Token from which this AST node was derived
     *@param int The begin line number
     */
    public void setBeginLine(int line){
	beginLine = line;
    }

    /**
     *The method sets the end line number of the Token from which this AST node was derived
     *@param int The end line number
     */
    public void setEndLine(int line){
	endLine = line;
    }

    /**
     *The method sets the beginning column number of the Token from which this AST node was derived
     *@param int The begin Column number
     */
    public void setBeginColumn(int col) {
	beginColumn = col;
    }

    /**
     *The method sets the end column number of the Token from which this AST node was derived
     *@param int The end Column number
     */
    public void setEndColumn(int col) {
	endColumn = col;
    }

    /**
     *Re-define the toString method to include line and column information
     *Add in Line and Column info
     *@return String the updated string representation of this node
     */
    public String toString()
    {
	StringBuffer b = new StringBuffer();
	//use the default string conversion (requires access to private tokenNames table
	b.append(super.toString());
	//add info relating to source file position
	b.append(" [ Source: ("+getBeginLine()+","+getBeginColumn()+")-("+getEndLine()+","+getEndColumn()+") ]");
	return b.toString();
    }
    
}
