package org.sldl.rosetta.parser.tarp.antlr;

import antlr.CommonToken;

/** This implementation of <B>Token</B> stores column start/end and line start/end
 *  for Source-code referencing post-parse
 */
public class MyCommonToken extends CommonToken {
    private int beginLine=0;
    private int beginColumn=0;
    private int endLine=0;
    private int endColumn=0;
    
    public void setLine(int l)
    {
	line = l;
	beginLine=l;
	setEndLine(l);
    }
    public void setColumn(int c)
    {
	col = c;
	beginColumn = c;
	setEndColumn(c);
    }
    public int getLine()
    {
	return beginLine;
    }
    public int getColumn()
    {
	return beginColumn;
    }
    public int getBeginLine()
    {
	return beginLine;
    }
    public int getBeginColumn()
    {
	return beginColumn;
    }

    /** Method for setting the line at which this token ends
     *  @param int The line at which this Token ends
     *  @return void 
    */
    public void setEndLine(int l) {
	endLine=l;
    }

    /** Method for setting the column at which this token ends
     *  @param int The column at which this Token ends
     *  @return void 
    */
    public void setEndColumn(int c) {
	endColumn=c;
    }

    /** Method for getting the line at which this token ends
     *  @return int
    */
    public int getEndLine() {
	return endLine;
    }

    /** Method for getting the column at which this token ends
     *  @return int
    */
    public int getEndColumn() {
	return endColumn;
    }

    /** Simple Constructor, taking no parameters
     *  @return MyCommonToken
    */
    public MyCommonToken() {
	super();
	//System.out.print("() ");
	//System.out.print(this.getText()+this.getColumn()+this.getLine());
    }
    /**
     *Constructor taking string as parameter
     *@param String the string of this token
     */
    public MyCommonToken(String txt)
    {
	//super(txt);
	this.setEndLine(beginLine); //this token's line
	this.setEndColumn(beginColumn+txt.length());
	//System.out.print("(txt) ");
    }

    /** This constructor also sets column and line end-points of the token
     *  NOTE: default behaviour is to keep line same, but column=column+length
     *  @param int the type of this token
     *  @param String the text of this token
     *  @return MyCommonToken
     */
    public MyCommonToken(int t, String txt) {
	super(t,txt);
	//setType(t);
	//setText(txt);
	this.setEndLine(beginLine); //this token's line
	this.setEndColumn(beginColumn+txt.length());
	//System.out.print("(t,txt) ");
    }
}
