class RosettaLexer extends Lexer;

options {
	exportVocab=Rosetta;      // call the vocabulary "Rosetta"
	testLiterals=false;       // don't automatically test for literals
	k=4;                      // four characters of lookahead
//	charVocabulary='\u0000'..'\uFFFE';
	charVocabulary='\u0000'..'\u00FF';
	// without inlining some bitset tests, couldn't do unicode;
	// I need to make ANTLR generate smaller bitsets; see
	// bottom of JavaLexer.java
	codeGenBitsetTestThreshold=20;
}

// ----------------------------------------------------------------

{
    private static boolean done = false;

    public void uponEOF() throws TokenStreamException, CharStreamException {
	done=true;
    }

    public static void main(String[] args) throws Exception {
	RosettaLexer lexer = new RosettaLexer(System.in);
	while ( !done ) {
	    Token t = lexer.nextToken();
	    System.out.println("Token: "+t);
	}
    }
}


// ----------------------------------------------------------------
// Operators and delimiters
// ----------------------------------------------------------------

NUMBER_SIGN_TOKEN : '#';
PERCENT_SIGN_TOKEN : '%';
AMPERSAND_TOKEN : '&';
APOSTROPHE_TOKEN : '\'';
LEFT_PARENTHESIS_TOKEN : '(';
RIGHT_PARENTHESIS_TOKEN : ')';
ASTERISK_TOKEN : '*';
PLUS_SIGN_TOKEN : '+';
COMMA_TOKEN : ',';
HYPHEN_MINUS_TOKEN : '-';
FULL_STOP_TOKEN : '.';
SOLIDUS_TOKEN : '/';
COLON_TOKEN : ':';
SEMICOLON_TOKEN : ';';
LESS_THAN_SIGN_TOKEN : '<';
EQUALS_SIGN_TOKEN : '=';
GREATER_THAN_SIGN_TOKEN : '>';
COMMERCIAL_AT_TOKEN : '@';
LEFT_SQUARE_BRACKET_TOKEN : '[';
RIGHT_SQUARE_BRACKET_TOKEN : ']';
CIRCUMFLEX_ACCENT_TOKEN : '^';
LOW_LINE_TOKEN : '_';
LEFT_CURLY_BRACKET_TOKEN : '{';
VERTICAL_LINE_TOKEN : '|';
RIGHT_CURLY_BRACKET_TOKEN : '}';
GREATER_THAN_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN : ">>>";
EQUALS_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN : "=>>";
EQUALS_SIGN_EQUALS_SIGN_TOKEN : "==";
EQUALS_SIGN_GREATER_THAN_SIGN_TOKEN : "=>";
LESS_THAN_SIGN_EQUALS_SIGN_TOKEN : "<=";
SOLIDUS_EQUALS_SIGN_TOKEN : "/=";
EQUALS_SIGN_LESS_THAN_SIGN_TOKEN : "=<";
GREATER_THAN_SIGN_EQUALS_SIGN_TOKEN : ">=";
COLON_COLON_TOKEN : "::";
PLUS_SIGN_FULL_STOP_FULL_STOP_TOKEN : "+..";
AMPERSAND_FULL_STOP_FULL_STOP_TOKEN : "&..";
HYPHEN_MINUS_GREATER_THAN_SIGN_TOKEN : "->";
LESS_THAN_SIGN_ASTERISK_TOKEN : "<*";
ASTERISK_GREATER_THAN_SIGN_TOKEN : "*>";

// ----------------------------------------------------------------
// Line separators
// ----------------------------------------------------------------

LINE_SEPARATOR :
//  ( '\u000A'	// line feed
  ( '\n'	// line feed
  | '\u000B'	// vertical tabulation
  | '\u000C'	// form feed
//  | '\u000D'	// carriage return
  | '\r'	// carriage return
//  | '\u2028'	// line separator
//  | '\u2029'	// paragraph separator
  )
  { $setType(Token.SKIP); }
  ;

// ----------------------------------------------------------------
// Space separators
// ----------------------------------------------------------------

SPACE_SEPARATOR :
  ( '\u0020'	// space
  | '\u00A0'	// no-break space
//  | '\u2000'	// en quad
//  | '\u2001'	// em quad
//  | '\u2002'	// en space
//  | '\u2003'	// em space
//  | '\u2004'	// three-per-em space
//  | '\u2005'	// four-per-em space
//  | '\u2006'	// six-per-em space
//  | '\u2007'	// figure space
//  | '\u2008'	// punctuation space
//  | '\u2009'	// thin space
//  | '\u200A'	// hair space
//  | '\u200B'	// zero width space
//  | '\u200C'	// zero width non-joiner
//  | '\u200D'	// zero width joiner
//  | '\u200E'	// let-to-right mark
//  | '\u200F'	// right-to-left mark
//  | '\u3000'	// ideographic space
//  | '\uFEFF'	// zero width no-break space
  )
  { $setType(Token.SKIP); }
  ;

// ----------------------------------------------------------------
// Single-line comments
// ----------------------------------------------------------------

SINGLE_LINE_COMMENT :
  ( "//"
    ~ ( '\u000A'	// line feed
        | '\u000B'	// vertical tabulation
        | '\u000C'	// form feed
        | '\u000D'	// carriage return
//        | '\u2028'	// line separator
//        | '\u2029'	// paragraph separator
      ) 
    LINE_SEPARATOR )
  { $setType(Token.SKIP); }
  ;

// ----------------------------------------------------------------
// multiple-line comments
// ----------------------------------------------------------------

MULTI_LINE_COMMENT :
  ( "/*"
    ( { LA(2)!='/' }? '*'
      | ~'*'
    )*
    "*/" )
  {$setType(Token.SKIP);}
  ;

// ----------------------------------------------------------------
// Fix the following for Rosetta (they're still as in the Java grammar)


// ----------------------------------------------------------------
// character literals
// ----------------------------------------------------------------

CHAR_LITERAL
	:	'\'' ( ESC | ~'\'' ) '\''
	;

// ----------------------------------------------------------------
// string literals
// ----------------------------------------------------------------

STRING_LITERAL
	:	'"' (ESC|~('"'|'\\'))* '"'
	;

// ----------------------------------------------------------------
// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
// ----------------------------------------------------------------

protected
ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT 
		)
	;

protected
HEX_DIGIT
	:	('0'..'9'|'A'..'F'|'a'..'f')
	;

// ----------------------------------------------------------------
// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
// ----------------------------------------------------------------

// Fix the following: recast as labels, allow different forms of labels
// including __ forms.

IDENT
	options {testLiterals=true;}
	:	('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
	;

// ----------------------------------------------------------------
// a numeric literal
// ----------------------------------------------------------------

// Augment the following to include based numeric literals

NUMERIC_LITERAL :
	('0'..'9')+
	( '.' ('0'..'9')+ (EXPONENT)?
          |   EXPONENT
        )?
	;

protected
EXPONENT
	:	('e'|'E') ('+'|'-')? ('0'..'9')+
	;


// ================================================================


class RosettaParser extends Parser;

options {
//	k = 2;                           // two token lookahead
	exportVocab=Rosetta;             // Call its vocabulary "Java"
//	codeGenMakeSwitchThreshold = 2;  // Some optimizations
//	codeGenBitsetTestThreshold = 3;
//	defaultErrorHandler = false;     // Don't generate parser error handlers
//	buildAST = true;
}

// ----------------------------------------------------------------
// Design files and design units
// ----------------------------------------------------------------

design_file :
  ( design_unit )+
  EOF
  ;

// ----------------------------------------------------------------

design_unit :
  facet_declaration
  | package_declaration
  | component_declaration
  | domain_declaration
  | interaction_declaration
  ;

// ----------------------------------------------------------------
// Declarations
// ----------------------------------------------------------------

optional_declaration_list : 
  ( declaration_list_element )*
  ;

declaration_list_element :
  multiple_declaration
  | simple_declaration
  | enumeration_type_declaration
  | constructed_type_declaration
  | function_declaration
  | facet_declaration
  | package_declaration
  | component_declaration
  | domain_declaration
  | interaction_declaration
  | use_clause
  ;

multiple_declaration :
  labels:label_list COLON_COLON_TOKEN declaration_type:expression
    constant_expression:optional_constant_expression SEMICOLON_TOKEN
  ;


~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Following are the raw EBNF rules extracted from the draft standard
on 21-Dec-2001.
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


label ::=
simple_label | operator_interpretation_label

optional_label ::=
[ label ]

real_literal ::=
decimal_literal | based_literal

decimal_literal ::=
decimal_digits [ . decimal_digits ] [ exponent ]

exponent ::=
( e | E ) [ + | - ] decimal_digits

decimal_digits ::=
decimal_digit { decimal_digit }

decimal_digit ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9

based_literal ::=
base \ based_digits [ . based_digits ] \ [ exponent ]

base ::= decimal_digits

based_digits ::=
based_digit { based_digit }

based_digit ::=
0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | a | A | b | B | c | C | d | D | e | E | f | F

character_literal ::= graphic_character_literal | encoded_character_literal

graphic_character_literal ::= ' character '

encoded_character_literal ::= ' character_code_specifier '

character_code_specifier ::= unicode_specifier + based_digit based_digit based_digit based_digit

unicode_specifier ::= U | u

string_literal ::= " { character } "

expression ::=
precedence_0_expression

precedence_0_expression ::=
precedence_1_expression
| precedence_0_operation

precedence_0_operation ::=
left_operand:precedence_0_expression operator:precedence_0_operator
	right_operand:precedence_1_expression

precedence_0_operator ::=
>>> | =>>

precedence_1_expression ::=
precedence_2_expression
| precedence_1_operation

precedence_1_operation ::=
left_operand:precedence_1_expression operator:precedence_1_operator
	right_operand:precedence_2_expression

precedence_1_operator ::=
==

precedence_2_expression ::=
precedence_3_expression
| precedence_2_operation

precedence_2_operation ::=
left_operand:precedence_2_expression operator:precedence_2_operator
	right_operand:precedence_3_expression

precedence_2_operator ::=
=> | implies | <=

precedence_3_expression ::=
precedence_4_expression
| precedence_3_operation

precedence_3_operation ::=
left_operand:precedence_3_expression operator:precedence_3_operator
	right_operand:precedence_4_expression

precedence_3_operator ::=
or | nor | xor | xnor | max

precedence_4_expression ::=
precedence_5_expression
| precedence_4_operation

precedence_4_operation ::=
left_operand:precedence_4_expression operator:precedence_4_operator
	right_operand:precedence_5_expression

precedence_4_operator ::=
and | nand | min

precedence_5_expression ::=
precedence_6_expression
| precedence_5_operation

precedence_5_operation ::=
left_operand:precedence_5_expression operator:precedence_5_operator
	right_operand:precedence_6_expression

precedence_5_operator ::=
= | /= | < | > | =< | >= | in

precedence_6_expression ::=
precedence_7_expression
| precedence_6_infix_binary_operation

precedence_6_infix_binary_operation ::=
left_operand:precedence_6_expression operator:precedence_6_binary_operator
	right_operand:precedence_7_expression

precedence_6_binary_operator ::=
sub

precedence_7_expression ::=
precedence_8_expression
| precedence_7_operation

precedence_7_operation ::=
left_operand:precedence_7_expression operator:precedence_7_operator
	right_operand:precedence_8expression

precedence_7_operator ::=
+ | - | +.. | &.. | & | |

precedence_8_expression ::=
precedence_9_expression
| precedence_8_operation

precedence_8_operation ::=
left_operand:precedence_8_expression operator:precedence_8_operator
	right_operand:precedence_9_expression

precedence_8_operator ::=
* | / | mod | div

precedence_9_expression ::=
precedence_10_expression
| precedence_9_operation

precedence_9_operation ::=
left_operand:precedence_10_expression operator:precedence_9_operator
	right_operand:precedence_10_expression

precedence_9_operator ::=
^

precedence_10_expression ::=
precedence_11_expression
| precedence_10_operation
precedence_10_operation ::=
operator:precedence_10_operator
	operand:precedence_10_expression


precedence_10_operator ::=
not | + | - | * | # | %

precedence_11_expression ::=
precedence_12_expression
| at_operation
| tick_operation

at_operation ::=
left_operand:name @ right_operand:precedence_12_expression

tick_operation ::=
operand:name '

precedence_12_expression ::=
primary
| index_operation
| function_application
| facet_instantiation

index_operation ::=
left_operand:precedence_12_expression [ right_operand:expression ]

primary ::=
literal
| name
| parenthesized_expression
| anonymous_function
| anonymous_facet
| curly_bracket_formation
| square_bracket_formation
| type_formation
| let_expression
| if_expression
| case_expression
| quantified_expression

name ::=
label
| qualified_name

qualified_name
prefix:name . suffix:label

parenthesized_expression ::=
( contained_expression:expression )

curly_bracket_formation ::=
{ enclosed_expressions:optional_expression_list }

square_bracket_formation ::=
[ enclosed_expressions:optional_expression_list ]

optional_expression_list ::=
[ expression { , expression } ]

type_formation ::=
universal_or_subtype_formation
| set_type_formation
| array_type_formation
| function_type_formation
| facet_type_formation

universal_or_subtype_formation ::=
universal_type_formation
| subtype_formation

universal_type_formation ::=
type

subtype_formation ::=
subtype ( base_type:expression )

set_type_formation ::=
set ( base_type:expression )

array_type_formation ::=
array ( base_type:expression )

let_expression ::=
let ( binding_list:let_binding_list ) in encapsulated_expression:expression end let

let_binding_list ::=
let_binding { , let_binding_list }

let_binding ::=
let_function_binding | let_parameter_binding

let_function_binding ::=
function_label:label signature:function_signature be
	encapsulated_expression:expression

let_parameter_binding ::=
parameter:parameter_definition be bound_expression:expression

if_expression ::=
if condition:expression then true_alternative:expression
	false_alternative:if_expression_tail end if

if_expression_tail ::=
optional_elsif_expression
| optional_else_expression

optional_elsif_expression ::=
[ elsif_expression ]

elsif_expression ::=
elsif condition:expression then true_alternative:expression
	false_alternative:if_expression_tail

optional_else_expression ::=
[ else expression ]

case_expression ::=
case selector:expression is alternatives:case_alternative_list end case

case_alternative_list ::=
case_alternative { | case_alternative }

case_alternative ::=
tag:expression -> result:expression

quantified_expression ::=
quantifier_name:name ( parameters:optional_parameter_list |
				encapsulated_expression:expression )

optional_declaration_list ::=
{ declaration_list_element }

declaration_list_element ::=
multiple_declaration
| simple_declaration
| enumeration_type_declaration
| constructed_type_declaration
| function_declaration
| facet_declaration
| package_declaration
| component_declaration
| domain_declaration
| interaction_declaration
| use_clause

multiple_declaration ::=
labels:label_list :: declaration_type:expression
	constant_expression:optional_constant_expression ;

simple_declaration ::=
declaration_label:label :: declaration_type:expression
	constant_expression:optional_constant_expression ;

optional_constant_expression ::=
[ is expression_or_constant ]

expression_or_constant ::=
expression | constant

enumeration_type_declaration ::=
type_label:label :: subtype:expression is
	enumeration:enumeration_type_formation ;

enumeration_type_formation ::=
enumeration [ values:enumeration_list ]

enumeration_list ::=
enumeration_value { , enumeration_value }

enumeration_value ::=
label | real_literal | character_literal

constructed_type_declaration ::=
simple_constructed_type_declaration
| parameterized_constructed_type_declaration

simple_constructed_type_declaration ::=
type_label:label :: subtype:universal_or_type_formation is
	definition:constructed_type_definition ;

parameterized_constructed_type_declaration ::=
type_label:label signature:parameterize_constructed_type_signature is
	definition:constructed_type_definition ;

parameterize_constructed_type_signature ::=
( parameters:optional_parameter_list ) :: return_type:universal_or_subtype_formation

constructed_type_definition ::=
data alternatives:constructed_type_alternative_list

constructed_type_alternative_list ::=
constructed_type_alternative { | constructed_type_alternative }

constructed_type_alternative ::=
constructor_label:label ( observers:observer_definition_list ) :: recognizer_label:label

optional_observer_definition_list ::=
observer_definition { , observer_definition }

observer_definition ::=
observer_label:label :: observer_type:expression

function_declaration ::=
function_label:label signature:function_signature
	encapsulated_expression:optional_is_expression ;

facet_declaration ::=
facet facet_label:label signature:facet_signature
	body:optional_is_facet_body
end facet [ facet_label:label ] ;

optional_is_facet_body ::=
[ is facet_body ]

package_declaration ::=
package package_label:label signature:facet_signature is
	exports:optional_export_clause
begin
	declarations:optional_declaration_list
end package [ package_label:label ] ;

component_declaration ::=
component component_label:label signature:facet_signature is
	exports:optional_export_clause
	declarations:optional_declaration_list
begin
	component_assumptions:assumptions_clause
	component_definitions:definitions_clause
	component_implications:implications_clause
end component [ component_label:label ] ;

assumptions_clause ::=
assumptions
	terms:optional_term_list
end assumptions ;

definitions_clause ::=
definitions
	terms:optional_term_list
end definitions ;

implications_clause ::=
implications
	terms:optional_term_list
end implications ;

domain_declaration ::=
domain domain_label:label signature:facet_signature is
	body:facet_body
end domain [ domain_label:label ] ;

optional_term_list ::=
{ term_list_element }

term_list_element ::=
simple_term
| let_term

simple_term ::=
term_label:label : term_expression:expression ;

let_term ::=
let ( binding_list:let_binding_list ) in terms:optional_term_list end let ;

function_signature ::=
( parameters:optional_parameter_list ) :: return_type:expression

optional_parameter_list ::=
[ parameter_list_element { ; parameter_list_element } ]

parameter_list_element ::=
multiple_parameter_definition | parameter_definition

multiple_parameter_definition ::=
labels:label_list :: parameter_type:expression

label_list ::=
label { , label }

parameter_definition ::=
parameter_label:label :: parameter_type:expression

function_type_formation ::=
<* signature:function_signature *>

anonymous_function ::=
<* signature:function_signature is encapsulated_expression:expression *>

function_application ::=
applied_function:precedence_11_expression ( arguments:argument_list )

argument_list ::=
argument { , argument }

argument ::=
_ | optional_expression

optional_expression ::=
[ expression ]

facet_signature ::=
( parameters:optional_facet_parameter_list ) :: domain:expression

optional_facet_parameter_list ::=
[ facet_parameter_list_element { ; facet_parameter_list_element } ]

facet_parameter_list_element ::=
multiple_facet_parameter_definition | facet_parameter_definition

multiple_facet_parameter_definition ::=
labels:label_list ::
	parameter_kind:optional_label parameter_type:expression

facet_parameter_definition ::=
parameter_label:label ::
	parameter_kind:optional_label parameter_type:expression

facet_type_formation ::=
facet signature:facet_signature end facet

anonymous_facet ::=
facet signature:facet_signature is
	body:facet_body
end facet

facet_body ::=
exports:optional_export_clause
declarations:optional_declaration_list
begin
terms:optional_term_list

optional_export_clause ::=
[ export export_list ; ]

export_list ::=
label_list | all

facet_instantiation ::=
instantiated_facet:precedence_11_expression ( arguments:argument_list )

design_unit ::=
facet_declaration
| package_declaration
| component_declaration
| domain_declaration
| interaction_declaration

