//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: rosetta.g,v 1.9 2002/11/22 03:17:54 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: rosetta.g,v $
//  Revision 1.9  2002/11/22 03:17:54  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 2.10  2002/11/18 03:18:54  josephk
//  Renamed the tokens in the constructed type productions to reflect the correct
//  start/end tokens of the production.
//
//  Revision 2.9  2002/11/18 02:48:04  josephk
//  Fixed the enumeration and constructed_type declarations by increasing the
//  look-ahead of their parent production. Added the SEMI_TOKEN missing from
//  the 'simple constructed type' and 'parameterized constructed type' rules.
//
//  Revision 2.8  2002/11/15 04:35:53  josephk
//  Added the tilda '~' prefix operator to the precedence_10_expression.
//
//  Revision 2.7  2002/11/15 03:55:54  josephk
//  Fixed the file header so that the log is now appended to each checked-out file.
//  Altered the array indexing production and facet/function application production
//  so that arbitrary multiple applications can appear after any 'primary' without causing
//  ANTLR to die from a lhs-recursive grammar.
//
//
//=============================================================================
//----------------------------------------------------------------------------
//Grammar for Rosetta Lexer and Parser/Tree-builder)
//Using Antlr.Tool auto-code generator
//
//Numbering scheme reflects Accelera grammar definition
//  //T: comments begun this way represent sections with parts missing (To do)
//  //Q: Questions refer to grammar
//  //C: Changes I have made, but confirmed with Peter Ashendon
//  //E: Exception handler
//  //A: how to do Abstract Syntax Tree?
//  //D: not sure whether to trust document or .g file.
//  //JQ: Joseph's question
//  //N: change to NEW rom name required here
//----------------------------------------------------------------------------


header {
    package org.sldl.rosetta.parser.tarp.antlr;
    // classes for text,file i/o
    import java.io.*;
    import org.sldl.rosetta.parser.tarp.antlr.MyCommonToken;
    import java.util.Vector;
    //import org.sldl.rosetta.rom.*;
    //changed to KU version (Rosetta.tools.src.org.sldl.rosetta.ROM)
    //the following line shouldn't need to be mentioned here anyway
    //import org.sldl.rosetta.ROM.*;
    import org.sldl.rosetta.parser.tarp.misc.SourceReference;
}

//class to Parse the input (check for valid sentences)
class RosettaParser extends Parser;
options {
    exportVocab=Rosetta;             // Call its vocabulary "Rosetta"
    defaultErrorHandler = false;      // generate error handlers
    k=2;                             // Increased Lookahead (was 2)
    genHashLines=true;  //refer errors to the grammar (.g) file (not the generated .java file)
    buildAST=true;              //construct Abstract Syntax Tree
    //may need this?
    ASTLabelType = "MyAST"; // change default of "AST", but this is default anyway
}

tokens {
    //for ABSTRACT SYNTAX TREE (extra nodes)
    //items added as needed from bottom->up
    //T: prune this list to include only those used. Rearrange for top->down (easier to read)
 
    //5

    
    LABEL;
    LITERAL;
    REAL_LITERAL;
    CHARACTER_LITERAL;
    STRING_LITERAL;

    EXPRESSION;

    //unary operations requiring additional type
    UNARY_PLUS;
    UNARY_MINUS;
    UNARY_ASTERISK;

    AT_OPERATION;
    TICK_OPERATION;
    PRECEDENCE_12_EXPRESSION;
    FUNCTION_APPLICATION_OR_FACET_INSTANTIATION;
    INDEX_OPERATION; 
    PRIMARY;
    NAME;
    QUALIFIED_NAME;
    PARENTHESIZED_EXPRESSION;
    CURLY_BRACKET_FORMATION;
    SQUARE_BRACKET_FORMATION;
    EXPRESSION_LIST;
    TYPE_FORMATION;
    UNIVERSAL_OR_SUBTYPE_FORMATION; 
    UNIVERSAL_TYPE_FORMATION;
    SUBTYPE_FORMATION;
    SET_TYPE_FORMATION;
    ARRAY_TYPE_FORMATION;
    LET_EXPRESSION;
    LET_BINDING_LIST;
    LET_BINDING;
    LET_FUNCTION_BINDING;
    LET_PARAMETER_BINDING;
    PARAMETER;
    BOUND_EXPRESSION;
    IF_EXPRESSION;
    IF_EXPRESSION_TAIL;
    ELSIF_EXPRESSION;
    ELSE_EXPRESSION;
    CONDITION;
    TRUE_ALTERNATIVE;
    FALSE_ALTERNATIVE;
    CASE_EXPRESSION;
    SELECTOR;
    ALTERNATIVES;
    CASE_ALTERNATIVE_LIST;
    CASE_ALTERNATIVE;
    TAG;
    RESULT;
    QUANTIFIED_EXPRESSION;
    QUANTIFIER_NAME;
    QUANTIFIER;

    //6
    CONSTRUCTED_TYPE_DECLARATION;
    SUBTYPE;
    TYPE_LABEL;
    DEFINITION;
    PARAMETERIZED_CONSTRUCTED_TYPE_SIGNATURE;
    CONSTRUCTED_TYPE_DEFINITION;
    CONSTRUCTED_TYPE_ALTERNATIVE_LIST;
    CONSTRUCTED_TYPE_ALTERNATIVE;
    CONSTRUCTOR_LABEL;
    OBSERVERS;
    RECOGNIZER_LABEL;
    OBSERVER_DEFINITION;
    OBSERVER_LABEL;
    OBSERVER_TYPE;
    ENUMERATION_TYPE_DECLARATION;
    ENUMERATION_TYPE_FORMATION;
    ENUMERATION_LIST;
    ENUMERATION_VALUE;

    //7
    MULTIPLE_USE_CLAUSE;
    USE_CLAUSE;
    NAME_LIST;
    DECLARATION_LIST;
    DECLARATION_LIST_ELEMENT;
    MULTIPLE_DECLARATION;
    LABELS;
    SIMPLE_DECLARATION;
    DECLARATION_LABEL;
    DECLARATION_TYPE;
    CONSTANT_EXPRESSION;
    FUNCTION_DECLARATION;
    FUNCTION_LABEL;
    FACET_DECLARATION;
    FACET_LABEL;
    CHECK_LABEL;
    FACET_BODY;
    PACKAGE_DECLARATION;
    PACKAGE_LABEL;
    COMPONENT_DECLARATION;
    COMPONENT_LABEL;
    EXPORTS;
    DECLARATIONS;
    ASSUMPTIONS_CLAUSE;
    DEFINITIONS_CLAUSE;
    IMPLICATIONS_CLAUSE;
    DOMAIN_DECLARATION;
    DOMAIN_LABEL;
    BODY;

    //8
    TERM_LIST;
    SIMPLE_TERM;
    TERM_LABEL;
    TERM_EXPRESSION;
    LET_TERM;
    BINDING_LIST;
    TERMS;
    
    //9
    FUNCTION_SIGNATURE;
    RETURN_TYPE;
    PARAMETER_LIST;
    PARAMETER_LIST_ELEMENT;
    MULTIPLE_PARAMETER_DEFINITION;
    PARAMETER_DEFINITION;
    FUNCTION_TYPE_FORMATION;
    ANONYMOUS_FUNCTION;
    SIGNATURE;
    ENCAPSULATED_EXPRESSION;
    ARGUMENTS;
    FUNCTION_APPLICATION;
    APPLIED_FUNCTION;
    ARGUMENT_LIST;
    ARGUMENT;

    //10
    FACET_SIGNATURE;
    DOMAIN;
    PARAMETERS;
    FACET_PARAMETER_LIST;
    FACET_PARAMETER_LIST_ELEMENT;
    MULTIPLE_FACET_PARAMETER_DEFINITION;
    FACET_PARAMETER_DEFINITION;
    PARAMETER_LABELS;
    LABEL_LIST;
    PARAMETER_LABEL;
    PARAMETER_KIND;
    PARAMETER_TYPE;
    FACET_TYPE_FORMATION;
    ANONYMOUS_FACET;
    
    EXPORT_LIST;
    FACET_INSTANTIATION;

    DESIGN_FILE;
    //other 4

}


{

    //Methods to go in generated Parser here.

    //collect warnings during parse
    public java.util.Vector myWarnings = new java.util.Vector();
  
    public static class ParseWarning
    {
        public final String message;
        public final SourceReference position;
        public ParseWarning(String mesg, SourceReference pos)
        { message = mesg; position = pos; }
        public String toString()
        { return position + ": " + message; }
    ;
    }
    //T:no more source reference in KU ROM
    //need to modify this to be "decoration"
    protected void addWarning(java.util.Vector thisWarnings, String message, SourceReference position)
    {
        thisWarnings.add(new ParseWarning(message, position));
    }
    
    /**
    *take the positional info from one token and use it to annotate an AST node,
    *returning the annotated AST node
    *@param MyAST the node to be annotated
    *@param Token the token containing the end position for this node
    *@return MyAST the annotated node
    */
    private MyAST getEnd( MyAST thisNode, Token thisToken)

    {
        thisNode.setEndLine(((MyCommonToken)thisToken).getEndLine());
        thisNode.setEndColumn(((MyCommonToken)thisToken).getEndColumn());
        return thisNode;
    }

}


// The Initial Rule to be called
design_file
    :
        (
            //T: library section (does nothing yet, but will parse, if present)
            (LIBRARY_TOKEN name SEMICOLON_TOKEN)?
            design_unit
        )+ // at least one
        EOF!
        {#design_file = #([DESIGN_FILE, "DESIGN_FILE"], #design_file); }
    ;

//1.2 Terminals (not really required here)
//1.3 Tokens (defined in LEXER below)

//1.4 Labels
label
    :
        ( SIMPLE_LABEL
        | OPERATOR_INTERPRETATION_LABEL
        )
        { #label = #( (#[LABEL, "LABEL"]), #label );}
    ;

//1.4.1 Optional Labels
optional_label
    :
        ( //some optional labels are followed by label (literal)
            options {warnWhenFollowAmbig=false;}:
            label)?
    ;

//1.5 Literals
literal
    :
        ( real_literal 
        | character_literal 
        | string_literal
        )
    ;

// 1.5.1 Real Literals
real_literal
    :
        ( DECIMAL_LITERAL 
        | BASED_LITERAL
        )
        { #real_literal = #( (#[REAL_LITERAL, "REAL_LITERAL"]), #real_literal );}
    ;
// 1.5.2 Character literals
character_literal
    :
        ( GRAPHIC_CHARACTER_LITERAL 
        | ENCODED_CHARACTER_LITERAL
        )
        { #character_literal = #( (#[CHARACTER_LITERAL, "CHARACTER_LITERAL"]), #character_literal );}
    ;
// 1.5.3 String literals
string_literal
    :
        ( STRING_LITERAL
            //may concatenate to another string, or an encoded character
            //but don't need to worry about it as part of the string literal
            //will be picked up in the expression if it occurs.
            //             (
            //                 AMPERSAND_TOKEN^ //display concatenation in tree
            //                 ( character_literal
            //                 | STRING_LITERAL
            //                 )
            //             )*
        )

    ;

// 3 Expressions
//
// Operators are left-associative for interpretation of an unparenthesised expression
// 
//            a=>b=>c
//        is interpreted as
//           (a=>b)=>c     NOT   a=>(b=>c)
//
// And tree is built with operator as root, left/right operands  as first/second children
//
// ie
//
//         =>
//         |
//         +--=>
//         |  |
//         |  +--a
//         |  |
//         |  +--b
//         |
//         +--c
//
// 3.1 Operations //Q: do I have enough info to define these yet?
// 3.2 Expressions
expression
    :
        precedence_0_expression
        { #expression = #( (#[EXPRESSION, "EXPRESSION"]), #expression );}
    ;
// 3.3 Precedence 0 expression
precedence_0_expression
    :
        left_operand: precedence_1_expression
        ( precedence_0_operator:
            ( GREATER_THAN_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN^ 
            | EQUALS_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN^
            )
            right_operand: precedence_1_expression
        )*
    ;
// 3.4 Precedence 1 expression
precedence_1_expression
    :
        left_operand: precedence_2_expression
        ( precedence_1_operator: EQUALS_SIGN_EQUALS_SIGN_TOKEN^
            right_operand: precedence_2_expression
        )*
    ;
// 3.5 Precedence 2 expression
precedence_2_expression
    :
        left_operand: precedence_3_expression
        //Note: I couldn't make precedence_2_operator a separate rule, and just have it be the root node (as required)
        ( precedence_2_operator:
            ( EQUALS_SIGN_GREATER_THAN_SIGN_TOKEN^
            | IMPLIES_TOKEN^
            | LESS_THAN_SIGN_EQUALS_SIGN_TOKEN^
            )
            right_operand: precedence_3_expression
        )*
    ;
// 3.6 Precedence 3 expression
precedence_3_expression
    :
        left_operand: precedence_4_expression
        ( precedence_3_operator:
            ( OR_TOKEN^ 
            | NOR_TOKEN^ 
            | XOR_TOKEN^ 
            | XNOR_TOKEN^ 
            | MAX_TOKEN^
            )
            right_operand: precedence_4_expression
        )*
    ;
// 3.7 Precedence 4 expression
precedence_4_expression
    :
        left_operand: precedence_5_expression
        ( precedence_4_operator:
            ( AND_TOKEN^ 
            | NAND_TOKEN^ 
            | MIN_TOKEN^
            )
            right_operand: precedence_5_expression
        )*
    ;
// 3.8 Precedence 5 expression
precedence_5_expression
    :
        left_operand: precedence_6_expression
        ( precedence_5_operator:
            ( EQUALS_SIGN_TOKEN^
            | SOLIDUS_EQUALS_SIGN_TOKEN^ 
            | LESS_THAN_SIGN_TOKEN^ 
            | GREATER_THAN_SIGN_TOKEN^ 
            | EQUALS_SIGN_LESS_THAN_SIGN_TOKEN^
            | GREATER_THAN_SIGN_EQUALS_SIGN_TOKEN^
            | IN_TOKEN^
            )
            right_operand: precedence_6_expression
        )*
    ;
// 3.9 Precedence 6 expression
precedence_6_expression
    :
        left_operand: precedence_7_expression
        ( precedence_6_operator: SUB_TOKEN^
            right_operand: precedence_7_expression
        )*
    ;
// 3.10 Precedence 7 expression
precedence_7_expression
    :   
        left_operand: precedence_8_expression
        ( //match first VERTICAL_LINE_TOKEN
            options {warnWhenFollowAmbig=false;}:
            precedence_7_operator:
            ( PLUS_SIGN_TOKEN^
            | HYPHEN_MINUS_TOKEN^
            | PLUS_SIGN_FULL_STOP_FULL_STOP_TOKEN^
            | AMPERSAND_FULL_STOP_FULL_STOP_TOKEN^
            | AMPERSAND_TOKEN^
//            | VERTICAL_LINE_TOKEN^
            )
            right_operand: precedence_8_expression
        )*
    ;
// 3.11 Precedence 8 expression
precedence_8_expression
    :
        left_operand: precedence_9_expression
        ( options {warnWhenFollowAmbig=false;}:
            precedence_8_operator:
            ( ASTERISK_TOKEN^
            | SOLIDUS_TOKEN^
            | MOD_TOKEN^
            | DIV_TOKEN^
            )
            right_operand: precedence_9_expression
        )*
    ;
// 3.12 Precedence 9 expression
precedence_9_expression
//Q: 10 or 9? in operation
    :
        left_operand: precedence_10_expression
        ( precedence_9_operator:CIRCUMFLEX_ACCENT_TOKEN^ //ie '^'
            right_operand: precedence_10_expression
        )?
    ;
// 3.13 Precedence 10 expression
precedence_10_expression
//operators in this precedence are all unary, and must be distinguished from their binary predecessors (plus, minus, asterisk)
    :
        ( precedence_10_operator:
            ( NOT_TOKEN^
            | PLUS_SIGN_TOKEN^ {#PLUS_SIGN_TOKEN.setType(UNARY_PLUS);}
            | HYPHEN_MINUS_TOKEN^ {#HYPHEN_MINUS_TOKEN.setType(UNARY_MINUS);}
            | ASTERISK_TOKEN^ {#ASTERISK_TOKEN.setType(UNARY_ASTERISK);}
            | NUMBER_SIGN_TOKEN^
            | PERCENT_SIGN_TOKEN^
            | TILDA_TOKEN^
            )
        )*
        operand: precedence_11_expression
    ;
// 3.14 Precedence 11 expression
precedence_11_expression
    :
        (name COMMERCIAL_AT_TOKEN)=>at_operation //Q:precedence_12_expression not name?
    | (name APOSTROPHE_TOKEN)=>tick_operation //Q:precedence_12_expression not name?
    | precedence_12_expression 
    ;
// 3.14.1 At operations
at_operation
    :
        left_operand: name
        COMMERCIAL_AT_TOKEN^
        right_operand: precedence_12_expression
    ;
// 3.14.2 Tick operations
tick_operation
    :
        operand: name
        APOSTROPHE_TOKEN^
    ;
// 3.15 Precedence 12 expression
precedence_12_expression //Q:altered because of ambiguity with primary and index_operation, is definition same?
    :
        // Q: Original rule was:
        //    p12E = primary | p12E "[" expr "]" | p12E "(" optional_param_list ")"
        // But we can't have this (lhs recursive), so definition has been altered.

        primary
        ( index_operation
        | function_application_or_facet_instantiation
        )*
    ;
// 3.15.a
// A: AST structure?
function_application_or_facet_instantiation //T: requires semantic disambiguation
    :
        // Q: Can't have this (lhs recursive), so alter definition of precedence_12_expression
        //left_operand: precedence_12_expression
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(FUNCTION_APPLICATION_OR_FACET_INSTANTIATION);}
        arguments:argument_list
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;
// 3.15.1 Index operations
index_operation
    :
        // Q: Can't have this (lhs recursive), so alter definition of precedence_12_expression
        //left_operand: precedence_12_expression
        startToken:LEFT_SQUARE_BRACKET_TOKEN^ {#startToken.setType(INDEX_OPERATION);}
        right_operand: expression
        endToken:RIGHT_SQUARE_BRACKET_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;
// 3.16 Primaries
primary
    :
        ( literal 
        | name
        | parenthesized_expression
            //only difference between anonymous_function and anonymous_facet
        | ( LESS_THAN_SIGN_ASTERISK_TOKEN
                function_signature
                IS_TOKEN
            )=>anonymous_function
        | ( FACET_TOKEN
                facet_signature
                IS_TOKEN
            )=>anonymous_facet
        | curly_bracket_formation 
        | square_bracket_formation 
        | type_formation //function_type also has "<*" function_signature, but no IS_TOKEN
        | let_expression 
        | if_expression 
        | case_expression
        | quantified_expression
//        | (quantifier LEFT_PARENTHESIS_TOKEN optional_parameter_list VERTICAL_LINE_TOKEN )=>quantified_expression
        //JQ: | anonymous_(all facet analogues)
        )
        { #primary = #( (#[PRIMARY, "PRIMARY"]), #primary );}
    ;
// 3.17 Literals
// see 1.5 Literals
// 3.18 Names
name
//JQ:keywords set references
    :
        (
            (label FULL_STOP_TOKEN)=>qualified_name
        | label
        )
        { #name = #( (#[NAME, "NAME"]), #name );}
    ;
// 3.18.1 Qualified names
qualified_name
    :
        prefix: label 
        (
            FULL_STOP_TOKEN!
            suffix:label
        )*
    ;
// 3.19 Parenthesized expressions
parenthesized_expression
    :
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(PARENTHESIZED_EXPRESSION);}
        contained_expression: expression
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.20 Bracket formations
// 3.20.1 Curly bracket formation
curly_bracket_formation
    :
        startToken:LEFT_CURLY_BRACKET_TOKEN^ {#startToken.setType(CURLY_BRACKET_FORMATION);}
        enclosed_expressions: optional_expression_list
        endToken:RIGHT_CURLY_BRACKET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.20.2 Square bracket formation
square_bracket_formation
    :
        startToken:LEFT_SQUARE_BRACKET_TOKEN^ {#startToken.setType(SQUARE_BRACKET_FORMATION);}
        enclosed_expressions: optional_expression_list
        endToken:RIGHT_SQUARE_BRACKET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.20.3 Optional expression list
optional_expression_list
    :
        ( expression_list )?
    ;
//Q:added this, is this intention?
expression_list
    :
        ( expression ( COMMA_TOKEN! expression )* )
        { #expression_list = #( (#[EXPRESSION_LIST, "EXPRESSION_LIST"]), #expression_list );}        
    ;
// 3.21 Type formations
type_formation
    :
        ( universal_or_subtype_formation 
        | set_type_formation 
        | array_type_formation
        | function_type_formation
        | facet_type_formation
        )
        { #type_formation = #( (#[TYPE_FORMATION, "TYPE_FORMATION"]), #type_formation );}        
    ;
// 3.21.1 Universal or subtype formation
universal_or_subtype_formation
    :
        ( universal_type_formation
        | subtype_formation
        )
    ;
// 3.21.2 Universal type formation
universal_type_formation
    :
        token:TYPE_TOKEN {#token.setType(UNIVERSAL_TYPE_FORMATION);}
    ;
// 3.21.3 Subtype type formation
subtype_formation 
    :
        startToken:SUBTYPE_TOKEN^ {#startToken.setType(SUBTYPE_FORMATION);}
        LEFT_PARENTHESIS_TOKEN!
        base_type: expression
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.21.4 Set type formation
set_type_formation 
    :
        startToken:SET_TOKEN^ {#startToken.setType(SET_TYPE_FORMATION);}
        LEFT_PARENTHESIS_TOKEN!
        base_type: expression
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.21.5 Array type formation
array_type_formation
    :
        startToken:ARRAY_TOKEN^ {#startToken.setType(ARRAY_TYPE_FORMATION);}
        LEFT_PARENTHESIS_TOKEN!
        base_type: expression
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.22 Let expressions
let_expression
    :
        startToken:LET_TOKEN^ {#startToken.setType(LET_EXPRESSION);}
        LEFT_PARENTHESIS_TOKEN!
        binding_list: let_binding_list
        RIGHT_PARENTHESIS_TOKEN!
        IN_TOKEN!
        encapsulated_expression:expression
        END_TOKEN!
        endToken:LET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// 3.22.1 Let binding list
let_binding_list
    :
        ( let_binding 
            ( COMMA_TOKEN! let_binding )* //C:
        )
        { #let_binding_list = #( (#[LET_BINDING_LIST, "LET_BINDING_LIST"]), #let_binding_list );}        
    ;
// 3.22.2 Let binding
let_binding
    :
        ( let_function_binding
        | let_parameter_binding
        )
    ;
// 3.22.3 Let function binding
let_function_binding
    :
        function_label:label
        signature:function_signature
        token:BE_TOKEN^ {#token.setType(LET_FUNCTION_BINDING);}
        encapsulated_expression:expression
//         { #let_function_binding = #( (#[LET_FUNCTION_BINDING, "LET_FUNCTION_BINDING"]),
//                 #function_label,
//                 #signature,
//                 #encapsulated_expression );
//         }      
    ;
// 3.22.4 Let parameter binding
let_parameter_binding
    :
        parameter:parameter_definition
        token:BE_TOKEN^ {#token.setType(LET_PARAMETER_BINDING);}
        bound_expression:expression
//         { #let_parameter_binding = #( (#[LET_PARAMETER_BINDING, "LET_PARAMETER_BINDING"]),
//                 #parameter ,
//                 #bound_expression) ;
//         }     
    ;
// 3.23 If expressions
if_expression!
    :
        startToken:IF_TOKEN! {#startToken.setType(IF_EXPRESSION);}
        condition:expression
        THEN_TOKEN!
        true_alternative:expression
        false_alternative:if_expression_tail
        END_TOKEN!
        endToken:IF_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #if_expression = #( #startToken,
                (#[CONDITION, "CONDITION"], #condition),
                (#[TRUE_ALTERNATIVE, "TRUE_ALTERNATIVE"], #true_alternative),
                (#[FALSE_ALTERNATIVE, "FALSE_ALTERNATIVE"], #false_alternative) );
        }      
    ;
// 3.23.1 If expression tail
if_expression_tail
    :
        ( ( ELSIF_TOKEN )=>optional_elsif_expression 
        | optional_else_expression //(no syntactic predicate, as want to ensure the empty case is handled)
        )
        { #if_expression_tail = #( (#[IF_EXPRESSION_TAIL, "IF_EXPRESSION_TAIL"]), #if_expression_tail );}        
    ;
// 3.23.2 Optional elsif expression
optional_elsif_expression
    :
        ( // standard if-then-else ambiguity, but syntactic predicate over-rides ambiguity warning
            //options {warnWhenFollowAmbig=false;}:     //so this line not required.
            elsif_expression 
        )?
    ;
// 3.23.3 Elsif expression
elsif_expression!
    :
        token:ELSIF_TOKEN {#token.setType(ELSIF_EXPRESSION);}
        condition:expression 
        THEN_TOKEN!
        true_alternative:expression 
        false_alternative:if_expression_tail
        { #elsif_expression = #( #token,
                (#[CONDITION, "CONDITION"], #condition),
                (#[TRUE_ALTERNATIVE, "TRUE_ALTERNATIVE"], #true_alternative),
                (#[FALSE_ALTERNATIVE, "FALSE_ALTERNATIVE"], #false_alternative) );
        }      
    ;
// 3.23.4 Optional else expression 
optional_else_expression
    :
        ( //Q: why no standard if-then-else ambiguity?
            token:ELSE_TOKEN^ {#token.setType(ELSE_EXPRESSION);}
            expression 
        )?
    ;
// 3.24 Case expressions
case_expression
    :
        token:CASE_TOKEN^ {#token.setType(CASE_EXPRESSION);}
        selector:expression
        IS_TOKEN!
        alternatives:case_alternative_list
        END_TOKEN! CASE_TOKEN!
//         { #case_expression = #( #token,
//                 #selector,
//                 #alternatives );
//         }   
    ;
// 3.24.1 Case alternative list
case_alternative_list
    :
        case_alternative
        (
            VERTICAL_LINE_TOKEN!
            case_alternative
        )+
        { #case_alternative_list = #( (#[CASE_ALTERNATIVE_LIST, "CASE_ALTERNATIVE_LIST"]), #case_alternative_list );}
    ;
// 3.24.2 Case alternative
case_alternative!
    :
        //CE:
        start:LEFT_PARENTHESIS_TOKEN!
        tag:expression 
        token:HYPHEN_MINUS_GREATER_THAN_SIGN_TOKEN! {#token.setType(CASE_ALTERNATIVE);}
        result:expression
        end:RIGHT_PARENTHESIS_TOKEN!
        { #case_alternative = #( #token,
                (#[TAG, "TAG"], #tag),
                (#[RESULT, "RESULT"], #result) );
        }      
    ;
// 3.25 Quantified expressions
quantified_expression
    :
        quantifier_name:quantifier
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(QUANTIFIED_EXPRESSION);}
        parameters:optional_parameter_list
        VERTICAL_LINE_TOKEN!
        encapsulated_expression:expression
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
    ;
// not in peter's grammar rules, but in pdf.
quantifier
     :
     ( FORALL_TOKEN
     | EXISTS_TOKEN
     | SEL_TOKEN
     | MIN_TOKEN
     | MAX_TOKEN
     | RAN_TOKEN
     ) { #quantifier = #(#[QUANTIFIER, "QUANTIFIER"], #quantifier); }
     ;

// 6 Type Definitions
// 6.1 Constructed type defintions
constructed_type_declaration
//combining both of these into constructed_type_declaration in tree-grammar
    :
        ( simple_constructed_type_declaration
        | parameterized_constructed_type_declaration
        )
        {#constructed_type_declaration = #(#[CONSTRUCTED_TYPE_DECLARATION, "CONSTRUCTED_TYPE_DECLARATION"], #constructed_type_declaration);}
    ;
simple_constructed_type_declaration
    :
        type_label:label
        startToken:COLON_COLON_TOKEN!
        subtype:universal_or_subtype_formation 
        IS_TOKEN!
        definition:constructed_type_definition
        endToken:SEMICOLON_TOKEN!
    ;
parameterized_constructed_type_declaration
    :
        type_label:label
        signature:parameterized_constructed_type_signature 
        IS_TOKEN!
        definition:constructed_type_definition
        endToken:SEMICOLON_TOKEN!
    ;
parameterized_constructed_type_signature
    :
        LEFT_PARENTHESIS_TOKEN!
        parameters:optional_parameter_list
        RIGHT_PARENTHESIS_TOKEN!
        COLON_COLON_TOKEN!
        return_type:universal_or_subtype_formation
    ;
constructed_type_definition
    :
        token:DATA_TOKEN^ {#token.setType(CONSTRUCTED_TYPE_DEFINITION);}
        alternatives:constructed_type_alternative_list
        //{#constructed_type_definition = #(#[CONSTRUCTED_TYPE_DEFINITION, "CONSTRUCTED_TYPE_DEFINITION"], #constructed_type_definition);}
    ;
constructed_type_alternative_list
    :
        ( constructed_type_alternative 
            ( VERTICAL_LINE_TOKEN! 
                constructed_type_alternative )*
        )
    ;
constructed_type_alternative!
    :
        constructor_label:label 
        startToken:LEFT_PARENTHESIS_TOKEN! {#startToken.setType(CONSTRUCTED_TYPE_ALTERNATIVE);}
        observers:optional_observer_definition_list
        RIGHT_PARENTHESIS_TOKEN!
        endToken:COLON_COLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        recognizer_label:label
        { #constructed_type_alternative = #( #startToken,
                (#[CONSTRUCTOR_LABEL, "CONSTRUCTOR_LABEL"], #constructor_label),
                (#[OBSERVERS, "OBSERVERS"], #observers),
                (#[RECOGNIZER_LABEL, "RECOGNIZER_LABEL"], #recognizer_label) );
        }      
    ;
optional_observer_definition_list
    :
        ( observer_definition_list )?
    ;
observer_definition_list
    :
        (observer_definition 
            ( COMMA_TOKEN
                observer_definition )*
        )
    ;
observer_definition
    :
        observer_label:label
        token:COLON_COLON_TOKEN^ {#token.setType(OBSERVER_DEFINITION);}
        observer_type:expression
//         { #observer_definition = #( (#[OBSERVER_DEFINITION, "OBSERVER_DEFINITION"]),
//                 #observer_label,
//                 #observer_type);
//         }  
;
// 6.2 Enumeration type definitons
// 6.2.1.a Enumeration type declaration
enumeration_type_declaration
    :
        type_label:label
        startToken:COLON_COLON_TOKEN^ {#startToken.setType(ENUMERATION_TYPE_DECLARATION);}
        subtype:expression
        IS_TOKEN!
        enumeration:enumeration_type_formation
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
//         { #enumeration_type_declaration = #( (#[ENUMERATION_TYPE_DECLARATION, "ENUMERATION_TYPE_DECLARATION"]),
//                 (#type_label),
//                 (#subtype),
//                 (#enumeration) );
//         }     
    ;
// 6.2.1 Enumeration type formation
enumeration_type_formation
    :
        startToken:ENUMERATION_TOKEN^ {#startToken.setType(ENUMERATION_TYPE_FORMATION);}
        LEFT_SQUARE_BRACKET_TOKEN!
        labels: enumeration_list
        endToken:RIGHT_SQUARE_BRACKET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{#enumeration_type_formation = #(#[ENUMERATION_TYPE_FORMATION, "ENUMERATION_TYPE_FORMATION"], #enumeration_type_formation);}                
    ;
//6.2.1.1 Enumeration list
enumeration_list
    :
        ( enumeration_value
            ( COMMA_TOKEN! enumeration_value )*
        )
    ;
//6.2.1.2 Enumeration values
enumeration_value
    :
        ( SIMPLE_LABEL //ie NOT OPERATOR_INTERPRETATION_LABEL
        | real_literal
        | character_literal
        )
    ;


// 7 Declarations
// 7.1 Optional declaration list
optional_declaration_list
    :
        ( //if this is expected, then match it
            //options {warnWhenFollowAmbig = false;}:
            declaration_list )?
    ;
//Q: added this rule, is this what was intended?
declaration_list
    :
        ( declaration_list_element )+
        //since it's optional, should be at least one to be present at all
        {#declaration_list = #(#[DECLARATION_LIST, "DECLARATION_LIST"], #declaration_list);}                
    ;
// 7.1.1 Declaration list element
declaration_list_element
    :
        ( multiple_declaration
        | (label COLON_COLON_TOKEN expression optional_is_expression SEMICOLON_TOKEN
            )=>simple_declaration
        | (label COLON_COLON_TOKEN expression IS_TOKEN enumeration_type_formation
            )=>enumeration_type_declaration
        | (label ( COLON_COLON_TOKEN universal_or_subtype_formation IS_TOKEN constructed_type_definition
                 | parameterized_constructed_type_signature IS_TOKEN constructed_type_definition)
            )=>constructed_type_declaration
        | function_declaration
        | design_unit //JQ: could just put design_unit instead of these four declarations
            //         | facet_declaration
            //         | package_declaration
            //         | component_declaration
            //         | domain_declaration

            //| interaction_declaration //T:not yet defined
            //| use_clause //T:not yet defined, now in design_unit //JQ:
        )
    ;
// 7.1.2 Multiple declaration
multiple_declaration!
    :
        labels:label_list
        startToken:COLON_COLON_TOKEN! {#startToken.setType(MULTIPLE_DECLARATION);}
        declaration_type:expression
        constant_expression:optional_is_expression //T:Q:D: in latest grammar rules, optional_constant_expression defined, but no definition of constant
        //so using this definition, which is already defined, ( IS_TOKEN expression )?
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #multiple_declaration = #( #startToken,
                    #labels,
                    (#[DECLARATION_TYPE, "DECLARATION_TYPE"], #declaration_type),
                    #constant_expression );
        }      
    ;
// 7.1.3 Simple declaration
simple_declaration!
    :
        declaration_label:label
        startToken:COLON_COLON_TOKEN! {#startToken.setType(SIMPLE_DECLARATION);}
        declaration_type:expression
        constant_expression:optional_is_expression
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #simple_declaration = #( #startToken,
                    #declaration_label,
                    (#[DECLARATION_TYPE, "DECLARATION_TYPE"], #declaration_type),
                    #constant_expression );
        }      
    ;
// 7.1.4 Optional is expression
optional_is_expression
    :
        ( IS_TOKEN! expression 
        )?
    ;
// 7.2 Function declaration
function_declaration
    :
        function_label:label
        signature:function_signature
        encapsulated_expression:optional_is_expression
        token:SEMICOLON_TOKEN^ {#token.setType(FUNCTION_DECLARATION);}
//         { #function_declaration = #( (#[FUNCTION_DECLARATION, "FUNCTION_DECLARATION"]),
//                     #function_label,
//                     #signature,
//                     #encapsulated_expression );
//         }
    ;
// 7.3 Facet declaration
facet_declaration!
//JQ: why is this the only "stub" declaration
    :
        startToken: FACET_TOKEN! {#startToken.setType(FACET_DECLARATION);}
        facet_label: label
        signature:facet_signature
        body:optional_is_facet_body
        END_TOKEN!
        FACET_TOKEN!
        ( check_label: label )? 
        //JQ: should be manditory
        //Q: should this be the same label?
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #facet_declaration = #( #startToken, 
                    #facet_label,
                    #signature,
                    #body,
                    (#[CHECK_LABEL, "CHECK_LABEL"], #check_label) );
        }
    ;
//7.3.1 Optional is facet body
optional_is_facet_body
    :
        ( IS_TOKEN! facet_body 
        )?
    ;
// 7.4 Package declaration
package_declaration!
    :
        startToken:PACKAGE_TOKEN! {#startToken.setType(PACKAGE_DECLARATION);}
        package_label:label
        signature:facet_signature
        IS_TOKEN!
        exports:optional_export_clause
        BEGIN_TOKEN!
        declarations:optional_declaration_list
        END_TOKEN!
        PACKAGE_TOKEN!
        ( check_label:label )? 
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #package_declaration = #( #startToken,
                    #package_label,
                    #signature,
                    #exports,
                    #declarations,
                    (#[CHECK_LABEL, "CHECK_LABEL"], #check_label) );
        }
    ;
// 7.5 Component declaration
component_declaration!
    :
        startToken:COMPONENT_TOKEN! {#startToken.setType(COMPONENT_DECLARATION);}
        component_label:label
        signature:facet_signature
        IS_TOKEN!
        exports:optional_export_clause
        declarations:optional_declaration_list
        BEGIN_TOKEN!
        component_assumptions:assumptions_clause
        component_definitions:definitions_clause
        component_implications:implications_clause
        END_TOKEN!
        COMPONENT_TOKEN!
        ( check_label:label )?
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #component_declaration = #( #startToken,
                    #component_label,
                    #signature,
                    #exports,
                    #declarations,
                    #component_assumptions,
                    #component_definitions,
                    #component_implications,
                    (#[CHECK_LABEL, "CHECK_LABEL"], #check_label) );
        }
    ;
// 7.5.1 Assumptions clause
assumptions_clause
    :
        startToken:ASSUMPTION_TOKEN^ {#startToken.setType(ASSUMPTIONS_CLAUSE);}
        terms:optional_term_list
        END_TOKEN!
        ASSUMPTION_TOKEN!
        endToken:SEMICOLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;
// 7.5.2 Definitions clause
definitions_clause
    :
        startToken:DEFINITION_TOKEN^ {#startToken.setType(DEFINITIONS_CLAUSE);}
        terms:optional_term_list
        END_TOKEN!
        DEFINITION_TOKEN!
        endToken:SEMICOLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;
// 7.5.3 Implications clause
implications_clause
    :
        startToken:IMPLICATION_TOKEN^ {#startToken.setType(IMPLICATIONS_CLAUSE);}
        terms:optional_term_list
        END_TOKEN!
        IMPLICATION_TOKEN!
        endToken:SEMICOLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;
// 7.6 Domain declaration
domain_declaration!
    :
        startToken:DOMAIN_TOKEN!{#startToken.setType(DOMAIN_DECLARATION);}
        domain_label:label
        signature:facet_signature
        IS_TOKEN!
        body:facet_body
        END_TOKEN!
        DOMAIN_TOKEN!
        ( check_label:label )?
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #domain_declaration = #( #startToken,
                    #domain_label,
                    #signature,
                    #body,
                    (#[CHECK_LABEL, "CHECK_LABEL"], #check_label) );
        }
    ;
// 7.7 Interaction declaration (nothing)
// 7.8 Use clause (nothing)
//JQ: this should be possible at the start of every declaration
optional_use_clause
    :
        (multiple_use_clause)?
    ;
multiple_use_clause
    :
        (use_clause)+ //at least one
        { #multiple_use_clause = #( (#[MULTIPLE_USE_CLAUSE, "MULTIPLE_USE_CLAUSE"]), #multiple_use_clause );}
    ;
use_clause
    :
        startToken:USE_TOKEN^ {#startToken.setType(USE_CLAUSE);}
        name_list
        endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{ #optional_use_clause = #( (#[USE_CLAUSE, "USE_CLAUSE"]), #optional_use_clause );}
    ;
name_list
    :
        name (COMMA_TOKEN! name)*
        { #name_list = #( (#[NAME_LIST, "NAME_LIST"]), #name_list );}
    ;


// 8 Terms
// 8.1 Optional term list
optional_term_list
    :
        ( term_list )?
    ;
term_list
    :
        ( term_list_element )+
        {#term_list = #(#[TERM_LIST, "TERM_LIST"], #term_list);}        
    ;
// 8.1.1 Term list element
term_list_element
    :
        ( simple_term
        | let_term
        )
    ;
// 8.1.2 Simple term
simple_term
    :
        term_label:label //C:
        startToken:COLON_TOKEN^ {#startToken.setType(SIMPLE_TERM);}
        term_expression:expression
        endToken:SEMICOLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
//         { #simple_term = #( (#[SIMPLE_TERM, "SIMPLE_TERM"]),
//                     #term_label,
//                     #term_expression );
//         }
    ;
// 8.1.3 Let term
let_term
    :
        startToken:LET_TOKEN^ {#startToken.setType(LET_TERM);}
        LEFT_PARENTHESIS_TOKEN!
        binding_list:let_binding_list
        RIGHT_PARENTHESIS_TOKEN!
        IN_TOKEN!
        terms:optional_term_list
        END_TOKEN!
        LET_TOKEN!
        endToken:SEMICOLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
    ;

// 9 Functions
// 9.1 Function signatures
function_signature
    :
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(FUNCTION_SIGNATURE);}
        parameters:optional_parameter_list
        RIGHT_PARENTHESIS_TOKEN!
        endToken:COLON_COLON_TOKEN! {#startToken=getEnd(#startToken, endToken);}
        return_type:expression
//         { #function_signature = #( (#[FUNCTION_SIGNATURE, "FUNCTION_SIGNATURE"]),
//                     #parameters,
//                     #return_type );
//         }
    ;
// 9.1.1 Parameter lists
optional_parameter_list
    :
        ( parameter_list )?
    ;
//Q: Adding this rule, is this what was intended?
parameter_list
    :
        parameter_list_element ( SEMICOLON_TOKEN! parameter_list_element )* 
        {#parameter_list = #(#[PARAMETER_LIST, "PARAMETER_LIST"], #parameter_list);}
    ;
// 9.1.2 Parameter list elements
parameter_list_element
    :
        ( multiple_parameter_definition
        | parameter_definition
        )
    ;
// 9.1.3 Multiple parameter definition
multiple_parameter_definition
    :
        labels:label_list
        token:COLON_COLON_TOKEN^ {#token.setType(MULTIPLE_PARAMETER_DEFINITION);}
        parameter_type:expression
    ;      
// 9.1.4 Label list //T: export clause - should be able to do just one
label_list
    :
        label ( COMMA_TOKEN! label )+ //C:
        //Q: made this at least one, or else ambiguous with label from parameter_definition
        {#label_list = #(#[LABEL_LIST, "LABEL_LIST"], #label_list);}
    ;
// 9.1.5 Parameter definition
parameter_definition
    :
        parameter_label:label //C:
        token:COLON_COLON_TOKEN^ {#token.setType(PARAMETER_DEFINITION);}
        parameter_type:expression
    ;
// 9.2 Function type formations
function_type_formation
    :
        startToken:LESS_THAN_SIGN_ASTERISK_TOKEN^ {#startToken.setType(FUNCTION_TYPE_FORMATION);}
        signature:function_signature
        endToken:ASTERISK_GREATER_THAN_SIGN_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{#function_type_formation = #(#[FUNCTION_TYPE_FORMATION, "FUNCTION_TYPE_FORMATION"], #function_type_formation);}                        
    ;
// 9.3 Anonymous functions
anonymous_function
    :
        startToken:LESS_THAN_SIGN_ASTERISK_TOKEN^ {#startToken.setType(ANONYMOUS_FUNCTION);}
        signature:function_signature
        IS_TOKEN!
        encapsulated_expression:expression
        endToken:ASTERISK_GREATER_THAN_SIGN_TOKEN! {#startToken = getEnd(#startToken, endToken);}
//         { #anonymous_function = #( (#[ANONYMOUS_FUNCTION, "ANONYMOUS_FUNCTION"]),
//                     #signature,
//                     #encapsulated_expression );
//         }
    ;
// 9.4 Function application
function_application!
    :
        applied_function:precedence_12_expression //C:
        startToken:LEFT_PARENTHESIS_TOKEN! {#startToken.setType(FUNCTION_APPLICATION);}
        arguments:argument_list
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        { #function_application = #( #startToken,
                    (#[APPLIED_FUNCTION, "APPLIED_FUNCTION"], #applied_function),
                    (#[ARGUMENTS, "ARGUMENTS"], #arguments) );
        }
    ;
// 9.4.1 Argument lists
argument_list
    :
        argument ( COMMA_TOKEN! argument )*
        {#argument_list = #(#[ARGUMENT_LIST, "ARGUMENT_LIST"], #argument_list);}                        
    ;
// 9.4.2 Arguments
argument
    :
        ( LOW_LINE_TOKEN
        | optional_expression
        )
        //Q: so it could be nothing, nothing, nothing (ie , , , , ) !!!
    ;
// 9.4.3 Optional expressions
optional_expression
    :
        (
            expression 
        )?
    ;

// 10 Facets
// 10.1 Facet signatures
facet_signature
    :
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(FACET_SIGNATURE);}
        parameters:optional_facet_parameter_list
        RIGHT_PARENTHESIS_TOKEN!
        endToken:COLON_COLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        domain:expression
//         { #facet_signature = #( #startToken,
//                     #parameters,
//                     #domain );
//         }
    ;
// 10.1.1 Facet parameter lists
optional_facet_parameter_list
    :
        ( facet_parameter_list )?
    ;
//Q:added this one, is this intention?
facet_parameter_list
    :
        ( facet_parameter_list_element 
            ( SEMICOLON_TOKEN! facet_parameter_list_element )* 
        )
        {#facet_parameter_list = #(#[FACET_PARAMETER_LIST, "FACET_PARAMETER_LIST"], #facet_parameter_list);}                
    ;
// 10.1.2 Facet parameter list elements
facet_parameter_list_element
    :
        ( multiple_facet_parameter_definition 
        | facet_parameter_definition
        )
    ;
// 10.1.3 Multiple facet parameter definition
//Q: can't we just combine multiple_facet_param_def... into facet_par_def? (or vice versa)
multiple_facet_parameter_definition!
    :
        parameter_labels:label_list
        token:COLON_COLON_TOKEN! {#token.setType(MULTIPLE_FACET_PARAMETER_DEFINITION);}
        parameter_kind:optional_label
        parameter_type:expression
        { #multiple_facet_parameter_definition = #( #token,
                    #parameter_labels,
                    (#[PARAMETER_KIND, "PARAMETER_KIND"], #parameter_kind),
                    (#[PARAMETER_TYPE, "PARAMETER_TYPE"], #parameter_type) );
        }
    ;
// 10.1.4 Facet parameter definition
facet_parameter_definition!
    :
        parameter_label: label //C:
        token:COLON_COLON_TOKEN! {#token.setType(FACET_PARAMETER_DEFINITION);}
        parameter_kind:optional_label
        parameter_type:expression
        { #facet_parameter_definition = #( #token,
                    #parameter_label,
                    (#[PARAMETER_KIND, "PARAMETER_KIND"], #parameter_kind),
                    (#[PARAMETER_TYPE, "PARAMETER_TYPE"], #parameter_type) );
        }
    ;
// 10.1.5 Optional simple label
optional_simple_label
    :
        // since an expression may start with a literal (= SIMPLE_LABEL amongst others)
        // any (X)? which comes before an expression, which begins with A|B|..|X|.... will give nondeterminism.
        // However, if optional, we want to match it if its there first, and assume what's left is the valid expression we wanted.
        // Therefore, it's ok to turn off the warning.
        ( options {warnWhenFollowAmbig=false;}:  label )?
    ;
// 10.2 Facet type formations
// Q: using new syntax
facet_type_formation
    :
        startToken:FACET_TOKEN^ {#startToken.setType(FACET_TYPE_FORMATION);}
        signature:facet_signature 
        END_TOKEN!
        endToken:FACET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{#facet_type_formation = #(#[FACET_TYPE_FORMATION, "FACET_TYPE_FORMATION"], #facet_type_formation);}        
    ;
// 10.3 Anonymous facets
anonymous_facet
    :
        startToken:FACET_TOKEN^ {#startToken.setType(ANONYMOUS_FACET);}
        signature:facet_signature
        IS_TOKEN!
        body:facet_body
        END_TOKEN!
        endToken:FACET_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{#anonymous_facet = #(#[ANONYMOUS_FACET, "ANONYMOUS_FACET"], #anonymous_facet);}
    ;
// 10.3.1 Facet body
facet_body
    :
        exports:optional_export_clause
        declarations:optional_declaration_list
        token:BEGIN_TOKEN^ {#token.setType(FACET_BODY);}
        terms:optional_term_list
        //{#facet_body = #(#[FACET_BODY, "FACET_BODY"], #facet_body);}
    ;
// 10.3.2 Optional export clause
optional_export_clause
    :
        (
            startToken:EXPORT_TOKEN^ {#startToken.setType(EXPORT_LIST);}
            export_list
            endToken:SEMICOLON_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        )?
    ;
// 10.3.3 Export list
export_list
    :
        ( label_list
        | ALL_TOKEN
        )
        //{#export_list = #(#[EXPORT_LIST, "EXPORT_LIST"], #export_list);}
    ;
// 10.4 Term assertion
// 10.5 Facet Instantiation
facet_instantiation
    :
        instantiated_facet:precedence_12_expression //C:
        startToken:LEFT_PARENTHESIS_TOKEN^ {#startToken.setType(FACET_INSTANTIATION);}
        arguments:argument_list
        endToken:RIGHT_PARENTHESIS_TOKEN! {#startToken = getEnd(#startToken, endToken);}
        //{#facet_instantiation = #(#[FACET_INSTANTIATION, "FACET_INSTANTIATION"], #facet_instantiation);}
    ;
// 10.6 Facet inclusion (no syntax)

// 11 Facet composition (nothing)
// 12 Component definition (nothing)
// 13 Package definitions (nothing)
// 14 Domain definitions (nothing)
// 15 Interaction definition (nothing)
// 16 Scope and visibility (nothing)

// 17 Libraries
design_unit
    :
        //may appear above any design unit
        optional_use_clause
        //and this is what the accelera document specifies
        ( facet_declaration
        | package_declaration
        | component_declaration
        | domain_declaration
            //| interaction_declaration //Q:not defined yet.
        )
    ;
//Q:rule from chapter 17, not actually used anywhere
library_logical_name
    :
        SIMPLE_LABEL
    ;

// 18 The domain rosetta.lang (nothing)
// 19 Metafunctions (nothing)
// 20 The domain logic (nothing)
// 21 The domain discrete (nothing)
// 22 The domain continuous (nothing)

//============================================================================
//============================================================================
// The Rosetta Lexer (aka scanner, tokenizer)
//============================================================================
//============================================================================
class RosettaLexer extends Lexer;


options {
    charVocabulary = '\u0000'..'\u00FF'
    | '\u2000'..'\u200F'
    | '\u3000'
    //cant include this character due to java run-time error
    //|'\uFEFF'
    ; 
    exportVocab=Rosetta;  // call the vocabulary "Rosetta"
    testLiterals=false;    // don't automatically test for literals
    k=5;                   // only five characters of lookahead required to 
                        //distinguish any ambiguous tokens
    caseSensitiveLiterals=false;    //ignore case in keywords (literals)
    genHashLines=true; //intended to get line/position number in lexer exception
    defaultErrorHandler = false;      // don't generate error handlers
}


// 1.3 LEXICAL TOKENS

//the tokens{} section defines keywords which are ambiguous
//(eg fall under other lexical definitions, such as labels)
tokens {


    // 1.2.2 Operator interpretation tokens
    //only here because for some reason the label rule is nondeterministic with a double underscore
    UNARY_NOT_STRING             = "not__";
    
    // 1.3.2 Reserved Words
    // must be here or ambiguity occurs with label
    
    ALL_TOKEN                   = "all"; 
    AND_TOKEN                   = "and"; 
    ARRAY_TOKEN                 = "array"; 
    ASSUMPTION_TOKEN            = "assumption"; 
    BE_TOKEN                    = "be"; 
    BEGIN_TOKEN                 = "begin"; 
    CASE_TOKEN                  = "case"; 
    COMPONENT_TOKEN             = "component"; 
    CONSTANT_TOKEN              = "constant";
    DATA_TOKEN                  = "data";
    DEFINITION_TOKEN            = "definition"; 
    DIV_TOKEN                   = "div";
    //this was taken out, but it needs to be there for domain to work
    DOMAIN_TOKEN                = "domain";
    ELSE_TOKEN                  = "else"; 
    ELSIF_TOKEN                 = "elsif"; 
    END_TOKEN                   = "end"; 
    ENUMERATION_TOKEN           = "enumeration"; 
    EXISTS_TOKEN                = "exists"; 
    EXPORT_TOKEN                = "export"; 
    FACET_TOKEN                 = "facet"; 
    FORALL_TOKEN                = "forall"; 
    IF_TOKEN                    = "if"; 
    IMPLIES_TOKEN               = "implies";
    //implication not in new spec
    IMPLICATION_TOKEN           = "implication";
    IN_TOKEN                    = "in"; 
    IS_TOKEN                    = "is"; 
    LET_TOKEN                   = "let"; 
    MAX_TOKEN                   = "max"; 
    MIN_TOKEN                   = "min"; 
    MOD_TOKEN                   = "mod"; 
    NAND_TOKEN                  = "nand"; 
    NOR_TOKEN                   = "nor"; 
    NOT_TOKEN                   = "not"; 
    OR_TOKEN                    = "or"; 
    PACKAGE_TOKEN               = "package"; 
    RAN_TOKEN                   = "ran"; 
    SEL_TOKEN                   = "sel"; 
    SET_TOKEN                   = "set"; 
    SUB_TOKEN                   = "sub"; 
    SUBTYPE_TOKEN               = "subtype"; 
    THEN_TOKEN                  = "then"; 
    TYPE_TOKEN                  = "type"; 
    USE_TOKEN                   = "use"; 
    VERIFICATION_TOKEN          = "verification"; 
    XNOR_TOKEN                  = "xnor"; 
    XOR_TOKEN                   = "xor";



    
}

{

    //All methods to go into the Lexer go here.

}


// 1.3.1 Operator, punctuation and delimiter tokens

QUOTATION_MARK_TOKEN            : '\"';
NUMBER_SIGN_TOKEN               : '#';
PERCENT_SIGN_TOKEN              : '%';
TILDA_TOKEN                     : '~';
AMPERSAND_TOKEN                 : '&';
APOSTROPHE_TOKEN                : '\'';
LEFT_PARENTHESIS_TOKEN          : '('; 
RIGHT_PARENTHESIS_TOKEN         : ')';
ASTERISK_TOKEN                  : '*';
PLUS_SIGN_TOKEN                 : '+';
COMMA_TOKEN                     : ',';
HYPHEN_MINUS_TOKEN              : '-';
FULL_STOP_TOKEN                 : '.';
SOLIDUS_TOKEN                   : '/';
COLON_TOKEN                     : ':';
SEMICOLON_TOKEN                 : ';';
LESS_THAN_SIGN_TOKEN            : '<';
EQUALS_SIGN_TOKEN               : '=';
GREATER_THAN_SIGN_TOKEN         : '>';
COMMERCIAL_AT_TOKEN             : '@';
LEFT_SQUARE_BRACKET_TOKEN       : '[';
RIGHT_SQUARE_BRACKET_TOKEN      : ']';
CIRCUMFLEX_ACCENT_TOKEN         : '^';
LOW_LINE_TOKEN                  : '_';
LEFT_CURLY_BRACKET_TOKEN        : '{';
VERTICAL_LINE_TOKEN             : '|';
RIGHT_CURLY_BRACKET_TOKEN       : '}';

GREATER_THAN_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN : ">>>";
EQUALS_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN       : "=>>";
EQUALS_SIGN_EQUALS_SIGN_TOKEN           : "==";
EQUALS_SIGN_GREATER_THAN_SIGN_TOKEN     : "=>";
LESS_THAN_SIGN_EQUALS_SIGN_TOKEN        : "<=";
SOLIDUS_EQUALS_SIGN_TOKEN               : "/=";
EQUALS_SIGN_LESS_THAN_SIGN_TOKEN        : "=<";
GREATER_THAN_SIGN_EQUALS_SIGN_TOKEN     : ">=";
COLON_COLON_TOKEN                       : "::";
PLUS_SIGN_FULL_STOP_FULL_STOP_TOKEN     : "+..";
AMPERSAND_FULL_STOP_FULL_STOP_TOKEN     : "&..";
HYPHEN_MINUS_GREATER_THAN_SIGN_TOKEN    : "->";
LESS_THAN_SIGN_ASTERISK_TOKEN           : "<*";
ASTERISK_GREATER_THAN_SIGN_TOKEN        : "*>";


// 1.4 Labels
SIMPLE_LABEL
// check tokens{} section (and any other parser string) before applying this rule
options {testLiterals=true;}
    :
        l:(
        ('a'..'z'|'A'..'Z') //Q: If labels are not case sensitive,
        //should define with just 'a'..'z', and use rule
        //caseSensitive=false
        //no two LOW_LINE_TOKENS in a row
        //this "should" work, but for some reason results in a nondeterminism with "not__"
        //this warning is incorrect, but disabling warnWhenFollowAmbig is ineffective
        //however, testing of labels refuses to accept double underscore
        ( //options {warnWhenFollowAmbig=false;}:
            ('_')? ('a'..'z'|'A'..'Z'|'0'..'9') )*
        )
    ;

// // 1.4.2 Operator interpretation labels
// // assuming these have higher precedence than simple +, -, *, etc
OPERATOR_INTERPRETATION_LABEL
    :
        //"not__" //UNARY_NOT_STRING
        //in tokens above
        //nondeterminism with label
        //although rule should exclude it
        "+__" //UNARY_PLUS_SIGN_STRING
    | "-__" //UNARY_HYPHEN_MINUS_STRING
    | "*__" //UNARY_ASTERISK_STRING        
    | "#__" //UNARY_NUMBER_SIGN_STRING     
    | "%__" //UNARY_PERCENT_SIGN_STRING    
    | "__>>>__" //BINARY_GREATER_THAN_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_STRING      
    | "__=>>__" //BINARY_EQUALS_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_STRING            
    | "__==__" //BINARY_EQUALS_SIGN_EQUALS_SIGN_STRING                                    
    | "__=>__" //BINARY_EQUALS_SIGN_GREATER_THAN_SIGN_STRING                              
    | "__implies__" //BINARY_IMPLIES_STRING                                                    
    | "__<=__" //BINARY_LESS_THAN_SIGN_EQUALS_SIGN_STRING                                 
    | "__or__" //BINARY_OR_STRING             
    | "__nor__" //BINARY_NOR_STRING            
    | "__xor__" //BINARY_XOR_STRING            
    | "__xnor__" //BINARY_XNOR_STRING           
    | "__max__" //BINARY_MAX_STRING            
    | "__and__" //BINARY_AND_STRING            
    | "__nand__" //BINARY_NAND_STRING           
    | "__min__" //BINARY_MIN_STRING            
    | "__=__" //BINARY_EQUALS_SIGN_STRING    
    | "__/=__" //BINARY_SOLIDUS_EQUALS_SIGN_STRING            
    | "__<__" //BINARY_LESS_THAN_SIGN_STRING                 
    | "__>__" //BINARY_GREATER_THAN_SIGN_STRING              
    | "__=<__" //BINARY_EQUALS_SIGN_LESS_THAN_SIGN_STRING     
    | "__>=__" //BINARY_GREATER_THAN_SIGN_GREATER_THAN_SIGN_STRING    
    | "__in__" //BINARY_IN_STRING                                     
    | "__sub__" //BINARY_SUB_STRING                                    
    | "__+__" //BINARY_PLUS_SIGN_STRING                              
    | "__-__" //BINARY_HYPHEN_MINUS_STRING                           
    | "__+..__" //BINARY_PLUS_SIGN_FULL_STOP_FULL_STOP_STRING          
    | "__&..__" //BINARY_AMPERSAND_FULL_STOP_FULL_STOP_STRING          
    | "__&__" //BINARY_AMPERSAND_STRING                              
    | "__|__" //BINARY_VERTICAL_LINE_STRING                          
    | "__*__" //BINARY_ASTERISK_STRING                               
    | "__/__" //BINARY_SOLIDUS_STRING                                
    | "__mod__" //BINARY_MOD_STRING                                    
    | "__div__" //BINARY_DIV_STRING                                    
    | "__^__" //BINARY_CIRCUMFLEX_STRING                             
    | "__@__" //BINARY_COMMERCIAL_AT_STRING                          
    | "__'" //UNARY_APOSTROPHE_STRING                              
    | "__" //IMPLICIT_DEREFERENCE_STRING                          
    | "{__}" //UNARY_LEFT_CURLY_BRACKET_RIGHT_CURLY_BRACKET_STRING  
    | "[__]" //UNARY_LEFT_SQUARE_BRACKET_RIGHT_SQUARE_BRACKET_STRING    
    | "__[__]" //BINARY_LEFT_SQUARE_BRACKET_RIGHT_SQUARE_BRACKET_STRING
    ;

// 1.5.1 Real Literals

DECIMAL_OR_BASED_LITERAL
//not called from any parser rule, simply here to disambiguate between
//the two types during lexical analysis.
    :
        ( DECIMAL_DIGITS '\\' )=>BASED_LITERAL { $setType(BASED_LITERAL);}
    | ( DECIMAL_DIGITS )=>DECIMAL_LITERAL { $setType(DECIMAL_LITERAL);}
    ;

protected
DECIMAL_LITERAL
    :
        DECIMAL_DIGITS
        ( FULL_STOP_TOKEN DECIMAL_DIGITS )?
        ( EXPONENT )?
    ;

protected
DECIMAL_DIGITS
    :
        ( DECIMAL_DIGIT )+
    ;

protected
DECIMAL_DIGIT
    :
        //0|1|2|3|4|5|6|7|8|9
        '0'..'9'
    ;

protected
EXPONENT
	:
        ('e'|'E') ('+'|'-')? DECIMAL_DIGITS
    ;

protected
BASED_LITERAL
    :
        BASE
        '\\'
        BASED_DIGITS
        ( FULL_STOP_TOKEN
            BASED_DIGITS 
        )?
        '\\'
        ( EXPONENT )?
    ;

protected
BASE
    :
        //Q:assuming we only want a maximum of 2 digit base
        //unlimited is possible, with (DECIMAL_DIGIT)+, provided DECIMAL_OR_BASED_LITERAL rule remains
        ( DECIMAL_DIGIT 
            ( DECIMAL_DIGIT )?
        )
    ;

protected
BASED_DIGITS
    :
        ( BASED_DIGIT )+
    ;

protected
BASED_DIGIT 
    :
	    ('0'..'9'|'A'..'F'|'a'..'f')
    ;        

// 1.5.2 Character literals
GRAPHIC_CHARACTER_LITERAL
    :
        //' character '
        APOSTROPHE_TOKEN!
        CHARACTER
        APOSTROPHE_TOKEN!
    ;

protected
CHARACTER
    :
        //any ascii character
        ('\u0000'..'\u00FF')
    ;
ENCODED_CHARACTER_LITERAL 
    :
        //' character_code_specifier '
        APOSTROPHE_TOKEN!
        CHARACTER_CODE_SPECIFIER
        APOSTROPHE_TOKEN!
    ;

protected
CHARACTER_CODE_SPECIFIER 
    :
        UNICODE_SPECIFIER PLUS_SIGN_TOKEN
        //NOTE: A character code specifier shall contain exactly four hexadecimal digits
        BASED_DIGIT
        BASED_DIGIT
        BASED_DIGIT
        BASED_DIGIT
    ;

protected
UNICODE_SPECIFIER 
    :
        ( 'U' | 'u' )
    ;

// 1.5.3 String literals
// does not include concatenated strings (primitive type)
STRING_LITERAL 
    :
        (   QUOTATION_MARK_TOKEN!
            (   //keep one of the quotation marks as part of the text
                QUOTATION_MARK_TOKEN! QUOTATION_MARK_TOKEN | //encoding for quotation = ""
                //can't just be character or quotes would be allowed
                ~('\n'|'\r'|'\"')
            )*
            QUOTATION_MARK_TOKEN!
        )
    ;


// 1.6 Line separators

LINE_SEPARATOR 
    :
        ( '\n'	// line feed (\n)(000A)
        | '\u000B'	// vertical tabulation
        | '\u000C'	// form feed
        | '\r'	// carriage return (\r)(000D)
        | '\u2028'	// vertical tabulation
        | '\u2029'	// vertical tabulation
        )
        { $setType(Token.SKIP);
            newline(); 
        } //enable line counting
    ;

// ----------------------------------------------------------------
// Space separators
// (WhiteSpace - Ignored)
// ----------------------------------------------------------------

SPACE_SEPARATOR 
    :
        ( '\u0020'	// space
        | '\u00A0'	// no-break space
        | '\u0009'	// horizontal tab
        | '\u2000'	// EN QUAD
        | '\u2001'	// EM QUAD
        | '\u2002'	// EN SPACE
        | '\u2003'	// EM SPACE
        | '\u2004'	// THREE-PER-EM SPACE
        | '\u2005'	// FOUR-PER-EM SPACE
        | '\u2006'	// SIX-PER-EM SPACE
        | '\u2007'	// FIGURE SPACE
        | '\u2008'	// PUNCTUATION SPACE
        | '\u2009'	// THIN SPACE
        | '\u200A'	// HAIR SPACE
        | '\u200B'	// ZERO WIDTH SPACE
        | '\u200C'	// ZERO WIDTH NON-JOINER
        | '\u200D'	// ZERO WIDTH JOINER
        | '\u200E'	// LEFT-TO-RIGHT MARK
        | '\u200F'	// RIGHT-TO-LEFT MARK
        | '\u3000'	// IDEOGRAPHIC SPACE
//         | '\uFEFF'	// ZERO WIDTH NO-BREAK SPACE //T: can't put this in vocab as javac fails
        )
        { $setType(Token.SKIP); }
    ;

// 1.7 Comments

SINGLE_LINE_COMMENT 
    :
        ( "//"
            ( ~( '\u000A'	// line feed
                | '\u000B'	// vertical tabulation
                | '\u000C'	// form feed
                | '\u000D'	// carriage return
                ))* 
            LINE_SEPARATOR )
        {
            $setType(Token.SKIP);
        }
    ;

/* ----------------------------------------------------------------
Delimited Comment
   ----------------------------------------------------------------*/

DELIMITED_COMMENT
    :
        "/*"
        ( options {greedy=false;} :
            ~( '\n'	// line feed (\n)(000A)
            | '\u000B'	// vertical tabulation
            | '\u000C'	// form feed
            | '\r'	// carriage return (\r)(000D)
            | '\u2028'	// vertical tabulation
            | '\u2029'	// vertical tabulation
            )
        |   ( '\n'	// line feed (\n)(000A)
            | '\u000B'	// vertical tabulation
            | '\u000C'	// form feed
            | '\r'	// carriage return (\r)(000D)
            | '\u2028'	// vertical tabulation
            | '\u2029'	// vertical tabulation
            ){newline();}//still count lines
        )*
        "*/"
        {
            $setType(Token.SKIP);
        }
    ;
// ================================================================
// END OF GRAMMAR FILE
// ================================================================
