//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: rosetta.tree.g,v 1.9 2002/11/22 03:17:54 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: rosetta.tree.g,v $
//  Revision 1.9  2002/11/22 03:17:54  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 2.15  2002/11/22 03:17:26  josephk
//  Fixed a bug in the let_parameter_binding production which was building
//  the parameter item incorrectly.
//
//  Revision 2.14  2002/11/21 00:31:19  josephk
//  Fixed a bug in the LetExpression production which accidentily nested the
//  last item in a let parameter list twice.
//
//  Revision 2.13  2002/11/20 04:08:37  josephk
//  Fixed a bug where the ExportAll boolean wasn't being returned correctly.
//
//  Revision 2.12  2002/11/19 01:41:14  josephk
//  Added the parameter mode as a decoration to the parameter item.
//  Moved the ANTLR parser across to the ...misc.ParserConstants interface.
//
//  Revision 2.11  2002/11/19 00:14:52  josephk
//  Modified the 'function_signature' production to return the expression
//  for the function's range correctly.
//
//  Revision 2.10  2002/11/18 04:45:57  josephk
//  Fixed a typo that incorrectly handled the AST for constructed type observers.
//  The node was being treated as optional and the list as 1->? elements, when in
//  fact the node is always created with a list of 0->? elements.
//
//  Revision 2.9  2002/11/15 05:51:35  josephk
//  Added the code so that facet/domain parameter constraints are handled
//  correctly. There is still some question about where to put Package and
//  Component parameter constraints, since these classes have no terms per se.
//
//  Revision 2.8  2002/11/15 04:35:53  josephk
//  Added the tilda '~' prefix operator to the precedence_10_expression.
//
//  Revision 2.7  2002/11/15 03:55:54  josephk
//  Fixed the file header so that the log is now appended to each checked-out file.
//  Altered the array indexing production and facet/function application production
//  so that arbitrary multiple applications can appear after any 'primary' without causing
//  ANTLR to die from a lhs-recursive grammar.
//
//
//=============================================================================

/*
Rosetta Tree-Parser

Note: This file must be edited in conjunction with rosetta.g
as tree-constructor and tree-walker must agree on tree structure

The Adelaide Rosetta Project
Adelaide University Dept Computer Science
*/

header {
    package org.sldl.rosetta.parser.tarp.antlr;

    //ROM
    //import org.sldl.rosetta.rom.*;
    //import org.sldl.rosetta.rom.Package;
    //import org.sldl.rosetta.rom.tarp.DefaultMapFactory;
    //now using KU implementation
    import org.sldl.rosetta.ROM.*;
    import org.sldl.rosetta.ROM.Package;
    import org.sldl.rosetta.parser.tarp.map.MapParser;

    //ROMCreator
    //import org.sldl.rosetta.parser.tarp.ROMCreator;
    //using KU implementation
    import org.sldl.rosetta.parser.tarp.misc.ROMCreator;
    import org.sldl.rosetta.parser.tarp.misc.SourceReference;
    import org.sldl.rosetta.parser.tarp.misc.ParserConstants;

    //Duplicator
    //import org.sldl.rosetta.parser.duplicator.Duplicator;
    //import org.sldl.rosetta.parser.duplicator.DefaultDuplicator;
    //now using KU implementation
    import org.sldl.rosetta.parser.tarp.misc.Duplicator;

    import org.sldl.rosetta.lang.NumberParser;
    import org.sldl.rosetta.lang.logic.RadixNumber;

    //For error messages
    import java.util.Vector;
}
class RosettaTreeParser extends TreeParser;
options
{
    importVocab=Rosetta;
    ASTLabelType = "MyAST"; // change default of "AST", so can store line/column info
    defaultErrorHandler = false;
}

{
    //Extra Java Stuff goes here

    private MapParser myMapParser = new MapParser();
    private Duplicator myDuplicator = new Duplicator(myMapParser);
    //private ROMCreator myCreator = new ROMCreator(myMapParser);
    private ROMCreator myCreator = new ROMCreator();
    private LabelList myFilename = null;
    
    public void setMapParser(MapParser factory)
    {
        myMapParser = factory;
        //myCreator = new ROMCreator(myMapParser);
        myCreator = new ROMCreator();
        myDuplicator = new Duplicator(myMapParser);
    }

    public void setDefaultDuplicator()
    {
        myDuplicator = new Duplicator(myMapParser);
    }

    public void setDuplicator(Duplicator duplicator)
    {
        myDuplicator = duplicator;
    }

    public Duplicator getDuplicator()
    {
        return myDuplicator;
    }

//    public void setFactory(Factory useThisFactory)
//    {
//        myCreator = useThisFactory;
//    }
//    public Factory getFactory()
//    {
//        return myCreator;
//    }
    public void setFilename(String thisFilename)
    {
        
        myFilename = myCreator.makeLabelList();
        myFilename.add(myCreator.buildLabel(thisFilename));
    }
    public String getFilename()
    {
        return myFilename.toString();
    }
    protected ItemReference createReference(MyAST myNode)
    {
        ItemReference myReference = createReference(myNode.getText(), desource(myNode));
        return myReference;
    }
    protected ItemReference createUnaryReference(MyAST myNode)
    {
        ItemReference myReference = createReference(myNode.getText() + ParserConstants.OPERATOR_DELINIATOR_TAG, desource(myNode));
        return myReference;
    }
    protected ItemReference createPostfixUnaryReference(MyAST myNode)
    {
        ItemReference myReference = createReference(ParserConstants.OPERATOR_DELINIATOR_TAG + myNode.getText(), desource(myNode));
        return myReference;
    }
    protected ItemReference createUnaryReference(String myString)
    {
        ItemReference myReference = createReference(myString + ParserConstants.OPERATOR_DELINIATOR_TAG);
        return myReference;
    }
    protected ItemReference createBinaryReference(MyAST myNode)
    {
        ItemReference myReference = createReference(ParserConstants.OPERATOR_DELINIATOR_TAG + myNode.getText() + ParserConstants.OPERATOR_DELINIATOR_TAG, desource(myNode));
        return myReference;
    }
    protected org.sldl.rosetta.parser.tarp.misc.SourceReference getSRef(BaseObject thisObject)
    {
        return (org.sldl.rosetta.parser.tarp.misc.SourceReference)thisObject.getDecoration(SourceReference.annotationKey);
    }   
    protected ItemReference createReference(Item toReference)
    {
        //ItemReference myReference = createReference(toReference.getLabel().getData(), (org.sldl.rosetta.parser.tarp.misc.SourceReference)toReference.getDecoration(SourceReference.annotationKey));
        ItemReference myReference = createReference(toReference.getLabel().getData(), getSRef(toReference));
        return myReference;
    }
    protected ItemReference createReference(String toReference)
    {
        //make a label out of the string
        Label myLabel = myCreator.buildLabel(toReference);
        //pass it to another createReference method
        ItemReference myReference = createReference(myLabel);
        return myReference;
    }    
    protected ItemReference createReference(String toReference, SourceReference tag)
    {
        //make a label out of the string
        Label myLabel = myCreator.buildLabel(toReference);
        //pass it to another createReference method
        ItemReference myReference = createReference(myLabel);
        myReference.addDecoration(tag.annotationKey, tag);
        //System.out.print(tag);
        return myReference;
    }    
    protected ItemReference createReference(Label toReference)
    {
        LabelList name;
        ItemReference myReference;
        
        name = myCreator.makeLabelList();
        name.add(toReference);
        myReference = createReference(name, getSRef(toReference));
        return myReference;
    }
    protected ItemReference createReference(LabelList name)
    {
        ItemReference myReference = myCreator.buildItemReference(name);
        return myReference;
    }
    protected ItemReference createReference(LabelList name, org.sldl.rosetta.parser.tarp.misc.SourceReference tag)
    {
        ItemReference myReference = myCreator.buildItemReference(name);
        myReference.addDecoration(tag.annotationKey, tag);
        return myReference;
    }
    protected Expression duplicateExpression(Expression expr)
    {
        //return myDuplicator.duplicateExpression(myCreator, expr); 
        return myDuplicator.duplicateExpression(expr); 
    }
    protected org.sldl.rosetta.parser.tarp.misc.SourceReference desource(MyAST myNode)
    {
        if (myNode==null)
        {
            return null;
        }
        //use this to find out which nodes are being desourced
        //System.out.println("TEXT:"+myNode.getText()+" [L:"+myNode.getBeginLine()+ "-"+myNode.getEndLine()+" ;C:"+myNode.getBeginColumn()+"-"+myNode.getEndColumn()+"]");
        return myCreator.buildSourceReference(myFilename, myNode.getBeginColumn(), myNode.getBeginLine(), myNode.getEndColumn(), myNode.getEndLine());
    }

    //collect warnings during parse
    public java.util.Vector myWarnings = new java.util.Vector();
    
    public static class ParseWarning
    {
        public final String message;
        public final SourceReference position;
        public ParseWarning(String mesg, SourceReference pos)
        { message = mesg; position = pos; }
        public String toString()
        { return position + ": " + message; }
    }
    
    protected void addWarning(String message, SourceReference position)
    {
        addWarning(myWarnings, message, position);
    }
    protected void addWarning(java.util.Vector thisWarnings, String message, SourceReference position)
    {
        thisWarnings.add(new ParseWarning(message, position));
    }

    protected Expression ExpressionListAndBrackets(ExpressionList thisExpressionList, String theBrackets)
    {
        Expression returnExpression;

        returnExpression = null;
        if ("{}".equals(theBrackets))
          { returnExpression = myCreator.buildSetValue(thisExpressionList); }
        else if ("[]".equals(theBrackets))
          { returnExpression = myCreator.buildArrayValue(thisExpressionList); }
        //T:this is not really an array value, and should not be declared as such
//        Expression returnExpression = myCreator.buildArrayValue(thisExpressionList);
        //now just making the brackets a decoration
//        returnExpression.addDecoration(theBrackets, createUnaryReference(theBrackets));
        return returnExpression;
    }


  protected Item createParameterConstraintItem(Item param)
  {
    Label constraintLabel;
    Expression constraintValue;
    ExpressionList constraintApplicationParams;
    LabelList paramReference;

    param.addDecoration(ParserConstants.PARAMETER_CONSTRAINT_ITEM_TAG, param.getValue());
    constraintLabel = myCreator.buildLabel(ParserConstants.PARAMETER_CONSTRAINT_ITEM_TAG + param.getLabel().getData());
    paramReference = myCreator.makeLabelList();
    paramReference.add(myCreator.buildLabel(param.getLabel().getData()));
    constraintApplicationParams = myCreator.makeExpressionList();
    constraintApplicationParams.add(myCreator.buildItemReference(paramReference));
    constraintValue = myCreator.buildApplication(myDuplicator.duplicateExpression(param.getValue()), constraintApplicationParams);
    return myCreator.buildItem(constraintLabel, null, constraintValue);
  }

  protected void extractParameterConstraintItems(ItemList myParamList, ItemList myTermList)
  {
    int paramPos;

    for (paramPos = 0; paramPos < myParamList.size(); paramPos++)
    {
      if (myParamList.get(paramPos).getValue() != null)
      {
        myTermList.add(createParameterConstraintItem(myParamList.get(paramPos)));
        myParamList.get(paramPos).setValue(null);
      }
    }
  }



}

design_file returns [BaseObject thisDesignFile] //wraps the ItemList in a single Item (so it can be tree-displayed
{
    thisDesignFile = null;
    ItemList designFileItemList = myCreator.makeItemList();
}
    : #( DESIGN_FILE 
            (design_unit[designFileItemList])* 
        )
        {
            thisDesignFile = myCreator.buildFunction( designFileItemList, null, null );
        }
    ;
design_unit [ItemList designFileItemList]
{
    ExpressionList thisDesignUnitUses = myCreator.makeExpressionList();
}
    :
        (use_clause[thisDesignUnitUses])?
        ( facet_declaration[designFileItemList, thisDesignUnitUses]
        | package_declaration[designFileItemList, thisDesignUnitUses]
        | component_declaration[designFileItemList, thisDesignUnitUses]
        | domain_declaration[designFileItemList, thisDesignUnitUses]
            //| interaction_declaration[designFileItemList, thisDesignUnitUses] //Q:not defined yet.
        )
    ;
facet_declaration [ItemList thisItemList, ExpressionList thisFacetUses]
{
    Item thisFacetItem = null;
    //parameters required by buildItem
    Label thisFacetLabel, thisFacetCheckLabel = null;
    Facet thisFacetValue = null;
    Expression thisFacetType = null;
    //parameters required by buildFacet
    LabelList thisFacetLibrary = myCreator.makeLabelList();
    ItemList thisFacetParameters = myCreator.makeItemList();
    Expression thisFacetDomain = null;
    ItemList thisFacetDeclared = myCreator.makeItemList();
    Boolean thisFacetExportAll = new Boolean(false);
    ExpressionList thisFacetExports = myCreator.makeExpressionList();
    ItemList thisFacetTerms = myCreator.makeItemList();
}
    : #( node:FACET_DECLARATION 
            thisFacetLabel=label 
            thisFacetDomain=facet_signature[thisFacetParameters]
            (thisFacetExportAll=facet_body[thisFacetTerms, thisFacetExports, thisFacetDeclared])? 
            (#(CHECK_LABEL
                    (
                        thisFacetCheckLabel=label
                    )?
                )
                {
                    if (thisFacetCheckLabel==null)
                    {addWarning("end facet <label> missing.",desource(node));}
                    else if (!thisFacetCheckLabel.equals(thisFacetLabel))
                    {addWarning("end facet <label> does not match.",desource(node));}
                }
            )
        )
        {
            extractParameterConstraintItems(thisFacetParameters, thisFacetTerms);
            thisFacetValue = myCreator.buildFacet(thisFacetLibrary, thisFacetUses, thisFacetParameters, thisFacetDomain,
                thisFacetDeclared, thisFacetExportAll, thisFacetExports, thisFacetTerms);
            thisFacetItem = myCreator.buildItem(thisFacetLabel, thisFacetType, thisFacetValue);
            thisItemList.add(thisFacetItem);
        }
    ;
use_clause [ExpressionList thisUses]
    : #( MULTIPLE_USE_CLAUSE
            ( #( USE_CLAUSE
                    name_list[thisUses]
                )
            )*
        )
    ;
name_list [ExpressionList thisNameList]
{
    LabelList nextName = null;
}
    : #( NAME_LIST
            (
                nextName=name
                {
                    thisNameList.add(createReference(nextName) );
                }
            )*
        )
    ;
facet_body [ItemList thisFacetTerms, ExpressionList thisFacetExports, ItemList thisFacetDeclared] returns [Boolean thisFacetExportAll]
{
  thisFacetExportAll = new Boolean(false);
}
//Q:should also take in thisFacetUses?
    : #( FACET_BODY 
            (thisFacetExportAll=export_list[thisFacetExports])? 
            (declaration_list[thisFacetDeclared])?
            (term_list[thisFacetTerms])? 
        )
    ;   
export_list [ExpressionList thisExportList] returns [Boolean thisExportAll]
{
    LabelList thisLabelList = null;
    ItemReference thisItemReference = null;
    thisExportAll = new Boolean(false);
}
    : #( EXPORT_LIST 
            ( thisLabelList=label_list
                {
                    thisItemReference = myCreator.buildItemReference(thisLabelList); 
                    thisExportList.add(thisItemReference);
                }
            | all_token:ALL_TOKEN
                {
                    thisExportAll = new Boolean(true);
                }
            ) 
        )
    ;
facet_signature [ItemList facetParameters] returns [Expression facetDomain]
    : #( FACET_SIGNATURE 
            (facet_parameter_list[facetParameters])? 
            facetDomain=expression
        )
    ;
facet_parameter_list [ItemList facetParameters]
    : #( FACET_PARAMETER_LIST
            ( multiple_facet_parameter_definition[facetParameters]
            | facet_parameter_definition[facetParameters]
            )+
        )
    ;
multiple_facet_parameter_definition [ItemList facetParameters]
{
    int pos = 0;
    Item nextItem = null;
    LabelList myLabels = null;
    Label nextLabel = null;
    Item nextParameterItem = null;
    Expression thisType = null;
    ItemReference thisMode = null;
    Label myKind = null;
    Expression thisParameterValue = null;//Q:is this used?
}
    : #( MULTIPLE_FACET_PARAMETER_DEFINITION 
            myLabels=label_list 
            (#(PARAMETER_KIND myKind=label)
                {
                    thisMode = createReference(myKind);
                }
            )?
            (#(PARAMETER_TYPE thisType=expression))
        )
        {
            //Iterate through the list of labels, creating parameter Items for each one, with the same mode and type
            for (pos = 0; pos < myLabels.size(); pos++)
            {
                nextLabel = (Label)(myLabels.get(pos));
                nextItem = myCreator.buildItem(nextLabel, thisType, thisMode);
                facetParameters.add(nextItem);
                if (thisMode != null)
                { 
                    thisMode = myDuplicator.duplicateItemReference(thisMode);
                }
                
                if (thisType != null)
                { 
                    //T: returns null
                    thisType = duplicateExpression(thisType);
                }
            }
        }   
    ;
facet_parameter_definition [ItemList facetParameters]
{
    Label thisLabel = null;
    Label thisKind = null;
    Expression thisType = null;
    ItemReference thisMode = null;
    Item thisItem = null;
}
    : #( FACET_PARAMETER_DEFINITION 
            thisLabel=label 
            (#(PARAMETER_KIND thisKind=label)
                {
                    thisMode = createReference(thisKind);
                }
            )? 
            (#(PARAMETER_TYPE thisType=expression) )
            {
                thisItem = myCreator.buildItem(thisLabel, thisType, thisMode);
                facetParameters.add(thisItem);
            }
        )
    ;
declaration_list [ItemList thisDeclarationList]
    : #( DECLARATION_LIST
            // declarations add themselves to the declaration list
            ( multiple_declaration[thisDeclarationList]
            | simple_declaration[thisDeclarationList]
            | enumeration_type_declaration[thisDeclarationList]
            | constructed_type_declaration[thisDeclarationList]
            | function_declaration[thisDeclarationList]
            | design_unit[thisDeclarationList] //instead of the four declarations
                //             | facet_declaration[thisDeclarationList]
                //             | package_declaration[thisDeclarationList]
                //             | component_declaration[thisDeclarationList]
                //             | domain_declaration[thisDeclarationList]

                //| interaction_declaration[thisDeclarationList] //T:not yet defined
                //| use_clause[thisDeclarationList] //T:not yet defined
            )+
        )
    ;
multiple_declaration [ItemList thisDeclarationList]
{
    int pos = 0;
    LabelList thisMultipleDeclarationList = null;
    Item nextSimpleDeclarationItem = null;
    Label nextSimpleDeclarationLabel = null;
    Expression thisDeclarationType = null;
    Expression thisConstantExpression = null;
}
    : #( MULTIPLE_DECLARATION 
            thisMultipleDeclarationList=label_list 
            (#(DECLARATION_TYPE thisDeclarationType=expression)) 
            (thisConstantExpression=expression)? 
        )
        {
            for (pos=0; pos < thisMultipleDeclarationList.size(); pos++)
            {
                nextSimpleDeclarationLabel = (Label)(thisMultipleDeclarationList.get(pos));
                //T: returns null
                thisDeclarationType = duplicateExpression(thisDeclarationType);
                if (thisConstantExpression != null)
                {
                    //T: returns null
                    thisConstantExpression = duplicateExpression(thisConstantExpression);
                }
                nextSimpleDeclarationItem = myCreator.buildItem(nextSimpleDeclarationLabel, thisDeclarationType, thisConstantExpression);
                thisDeclarationList.add(nextSimpleDeclarationItem);
            }
        }
    ;
simple_declaration [ItemList thisDeclarationList]
{
    Item thisSimpleDeclarationItem = null;
    Label thisSimpleDeclarationLabel = null;
    Expression thisDeclarationType = null;
    Expression thisConstantExpression = null;
    
}
    : #( SIMPLE_DECLARATION 
            thisSimpleDeclarationLabel=label 
            (#(DECLARATION_TYPE thisDeclarationType=expression) )
            (thisConstantExpression=expression)? 
        )
        {
            thisSimpleDeclarationItem = myCreator.buildItem(thisSimpleDeclarationLabel, thisDeclarationType, thisConstantExpression);
            thisDeclarationList.add(thisSimpleDeclarationItem);
        }
    ;
enumeration_type_declaration [ItemList thisDeclarationList]
{
    Item thisEnumerationItem = null;
    Label thisEnumerationLabel = null;
    Expression thisEnumerationSubtype = null;
    ExpressionList thisExpressionList = myCreator.makeExpressionList();
    EnumerationType thisEnumerationValue = null;
} 
    : #( ENUMERATION_TYPE_DECLARATION
            thisEnumerationLabel = label
            thisEnumerationSubtype = expression
            enumeration_type_formation[thisExpressionList]
        )
        {
            thisEnumerationValue = myCreator.buildEnumerationType(thisExpressionList);
            thisEnumerationItem = myCreator.buildItem(thisEnumerationLabel, thisEnumerationSubtype, thisEnumerationValue);
            thisDeclarationList.add(thisEnumerationItem);
        }
    ;   
enumeration_type_formation [ExpressionList thisExpressionList]
{
    Expression nextExpression = null;
}
    : #( ENUMERATION_TYPE_FORMATION
            (
                ( sl:SIMPLE_LABEL
                    {   //usually, we don't specify JUST as simple label, so in this case, 
                        //the label object must be explicity created
                        nextExpression = myCreator.buildLabel(sl.getText());
                    }
                | nextExpression=real_literal
                | nextExpression=character_literal
                )
                {
                    thisExpressionList.add(nextExpression);
                }
            )+
        )
    ;
constructed_type_declaration [ItemList thisItemList]
{
    Item thisConstructedTypeItem = null;
    //parameters for buildItem
    Label thisConstructedTypeLabel = null;
    Expression thisConstructedType = null;
    ConstructedType thisConstructedTypeValue = null;;

    //no longer required as parameter, using Expression version instead
    //ItemList thisConstructedTypeList = myCreator.makeItemList();
    ExpressionList thisConstructedTypeExpressionList = myCreator.makeExpressionList();
    ItemList thisConstructedTypeParameters = myCreator.makeItemList();

    Function thisConstructedTypeFunction = null;
}
    : #(CONSTRUCTED_TYPE_DECLARATION
            thisConstructedTypeLabel=label
            //combined both parameterized and non-para' into one
            (parameter_list[thisConstructedTypeParameters])?
            ( thisConstructedType=universal_type_formation
            | thisConstructedType=subtype_formation
            )
            constructed_type_definition[thisConstructedTypeExpressionList]
        )
        {
            thisConstructedTypeValue = myCreator.buildConstructedType(thisConstructedTypeExpressionList);
            //for simple_constructed_type
            if (thisConstructedTypeParameters == null)
            {
                thisConstructedTypeItem = myCreator.buildItem(thisConstructedTypeLabel,
                    thisConstructedType, thisConstructedTypeValue);
                thisItemList.add(thisConstructedTypeItem);
            }
            else
            {
                thisConstructedTypeFunction = myCreator.buildFunction(thisConstructedTypeParameters, null, thisConstructedTypeValue);
                thisConstructedTypeItem = myCreator.buildItem(thisConstructedTypeLabel,
                    thisConstructedType, thisConstructedTypeFunction);
                thisItemList.add(thisConstructedTypeItem);
            }
        }
    ;
constructed_type_definition [ExpressionList thisConstructedTypeList]
{
    Label nextConstructorLabel, nextRecognizerLabel = null;
    Expression nextConstructorType = null;
    Expression nextRecognizerType = null;
    Expression nextConstructorValue = null;
    Expression nextRecognizerValue = null;
    Function nextContructorValueFunction = null;
    Item nextConstructorItem = null;
    Item nextRecognizerItem = null;
}
    : #( CONSTRUCTED_TYPE_DEFINITION
            (#( CONSTRUCTED_TYPE_ALTERNATIVE
                    {
                        ItemList nextParameterList = myCreator.makeItemList();
                        //Q:do I need new labels each time, or can I just write over the old ones having used them
                        //as parameters?
                    }
                    (#(CONSTRUCTOR_LABEL nextConstructorLabel=label))
                    (#(OBSERVERS (observer_definition[nextParameterList])* ))
                    (#(RECOGNIZER_LABEL nextRecognizerLabel=label))
                )
                {
                    thisConstructedTypeList.add(myCreator.buildConstructedTypeConstructor(nextConstructorLabel, nextParameterList, nextRecognizerLabel));
                }
            )+
        )
    ;
observer_definition [ItemList thisObserverList]
{
    Item thisObserverItem = null;
    Label thisObserverLabel = null;
    Expression thisObserverType = null;
    Expression thisObserverValue = null;
}
    : #( OBSERVER_DEFINITION
            thisObserverLabel=label
            thisObserverType=expression
        )
        {
            thisObserverItem = myCreator.buildItem(thisObserverLabel, thisObserverType, null);
            thisObserverList.add(thisObserverItem);
        }
    ;
function_declaration [ItemList thisDeclarationList]
{
    Item thisFunctionItem = null;
    //parameters for buildItem
    Label thisFunctionLabel = null;
    Function thisFunction = null;
    Expression thisFunctionType = null;
    //parameters for buildFunction
    ItemList thisFunctionParameters = myCreator.makeItemList();
    Expression thisFunctionRange = null; //return type
    Expression thisFunctionValue = null; //encapsulated expression
}
    : #( FUNCTION_DECLARATION
            thisFunctionLabel=label
            thisFunctionRange=function_signature[thisFunctionParameters]
            (thisFunctionValue=expression)?
        )
        {
            thisFunction = myCreator.buildFunction(thisFunctionParameters, thisFunctionRange, thisFunctionValue);
            thisFunctionItem = myCreator.buildItem(thisFunctionLabel, thisFunctionType, thisFunction);
            thisDeclarationList.add(thisFunctionItem);
        }
    ;
function_signature [ItemList thisFunctionParameters] returns [Expression thisFunctionRange]
//thisFunctionRange == return type
    : #( FUNCTION_SIGNATURE
            ( parameter_list[thisFunctionParameters] )?
            thisFunctionRange=expression
        )
    ;
parameter_list [ItemList thisParameterList]
    : #( PARAMETER_LIST
            ( multiple_parameter_definition[thisParameterList]
            | parameter_definition[thisParameterList]
            )+
        )
    ;
multiple_parameter_definition [ItemList thisParameterList]
{    
    int pos = 0;
    Item nextParameterItem = null;
    LabelList myLabels = null;
    Label nextParameterLabel = null;
    Expression nextParameterType = null;
}
    : #( MULTIPLE_PARAMETER_DEFINITION
            myLabels=label_list
            nextParameterType=expression
        )
        {
            for (pos=0; pos < myLabels.size(); pos++)
            {
                nextParameterLabel = (Label)(myLabels.get(pos));
                nextParameterItem = myCreator.buildItem(nextParameterLabel, nextParameterType, null);
                thisParameterList.add(nextParameterItem);
            }
        }
    ;
parameter_definition [ItemList thisParameterList]
{
    Item thisParameterItem = null;
    Label thisParameterLabel = null;
    Expression thisParameterType = null;
}
    : #( PARAMETER_DEFINITION
            thisParameterLabel=label
            thisParameterType=expression
        )
        {
            thisParameterItem = myCreator.buildItem(thisParameterLabel, thisParameterType, null);
            thisParameterList.add(thisParameterItem);
        }
    ;
package_declaration [ItemList thisItemList, ExpressionList thisPackageUses]
{
    Item thisPackageItem = null;

    //parameters for buildItem
    Label thisPackageLabel, thisPackageCheckLabel = null;
    Expression thisPackageType = null;
    org.sldl.rosetta.ROM.Package thisPackageValue = null;
    //org.sldl.rosetta.rom.Package thisPackageValue = null;
    //parameters for buildPackage
    LabelList thisPackageLibrary = myCreator.makeLabelList();
    ItemList thisPackageParameters = myCreator.makeItemList();
    Expression thisPackageDomain = null;
    ItemList thisPackageDeclared = myCreator.makeItemList();
    Boolean thisPackageExportAll = new Boolean(false);
    ExpressionList thisPackageExports = myCreator.makeExpressionList();
}
    : #( node:PACKAGE_DECLARATION 
            thisPackageLabel=label 
            thisPackageDomain=facet_signature[thisPackageParameters] 
            (thisPackageExportAll=export_list[thisPackageExports])? 
            (declaration_list[thisPackageDeclared])? 
            (#(CHECK_LABEL
                    (
                        thisPackageCheckLabel=label
                    )?
                )
                {
                    if (thisPackageCheckLabel==null)
                    {addWarning("end package <label> missing.",desource(node));}
                    else if (!thisPackageCheckLabel.equals(thisPackageLabel))
                    {addWarning("end package <label> does not match.",desource(node));}
                }
            )
        )
        {
            extractParameterConstraintItems(thisPackageParameters, thisPackageDeclared);
            thisPackageValue = myCreator.buildPackage(thisPackageLibrary, thisPackageUses, thisPackageParameters, thisPackageDomain,
                thisPackageDeclared, thisPackageExportAll, thisPackageExports);
            thisPackageItem = myCreator.buildItem(thisPackageLabel, thisPackageType, thisPackageValue);
            thisItemList.add(thisPackageItem);
        }
    ;
component_declaration [ItemList thisItemList, ExpressionList thisComponentUses]
{
    Item thisComponentItem= null;
    //parameters for buildItem
    Label thisComponentLabel;
    Expression thisComponentType = null;
    Component thisComponentValue = null;
    //parameters for buildComponent
    LabelList thisComponentLibrary = myCreator.makeLabelList();
    ItemList thisComponentParameters = myCreator.makeItemList();
    Expression thisComponentDomain = null;
    Boolean thisComponentExportAll = new Boolean(false); //not actually required for ROM, but an export_list could have it
    ExpressionList thisComponentExports = myCreator.makeExpressionList();
    ItemList thisComponentDeclared = myCreator.makeItemList();
    ItemList thisComponentAssumptionsTerms = myCreator.makeItemList();
    ItemList thisComponentDefinitionsTerms = myCreator.makeItemList();
    ItemList thisComponentImplicationsTerms = myCreator.makeItemList();
    //make assumptions, definitions, implications (verifiactions) into Facets, with only Term list specified
    Facet thisAssumptionsFacet = null;
    Facet thisDefinitionsFacet = null;
    Facet thisImplicationsFacet = null;
    Boolean theseFacetsExportAll = new Boolean(false);
    Label thisComponentCheckLabel = null;
}
    : #( node:COMPONENT_DECLARATION thisComponentLabel=label 
            thisComponentDomain=facet_signature[thisComponentParameters] 
            (thisComponentExportAll=export_list[thisComponentExports])? 
            (declaration_list[thisComponentDeclared])?
            (#(ASSUMPTIONS_CLAUSE (term_list[thisComponentAssumptionsTerms])?))
            (#(DEFINITIONS_CLAUSE (term_list[thisComponentDefinitionsTerms])?))
            (#(IMPLICATIONS_CLAUSE (term_list[thisComponentImplicationsTerms])?))
            (#(CHECK_LABEL
                    (
                        thisComponentCheckLabel=label
                    )?
                )
                {
                    if (thisComponentCheckLabel==null)
                    {addWarning("end component <label> missing.",desource(node));}
                    else if (!thisComponentCheckLabel.equals(thisComponentLabel))
                    {addWarning("end component <label> does not match.",desource(node));}
                }
            )
        )
        {
            thisAssumptionsFacet = myCreator.buildFacet(null, null, null, null, null, theseFacetsExportAll, null, thisComponentAssumptionsTerms);
            thisDefinitionsFacet = myCreator.buildFacet(null, null, null, null, null, theseFacetsExportAll, null, thisComponentDefinitionsTerms);
            thisImplicationsFacet = myCreator.buildFacet(null, null, null, null, null, theseFacetsExportAll, null, thisComponentImplicationsTerms);
            extractParameterConstraintItems(thisComponentParameters, thisComponentDeclared);
            thisComponentValue = myCreator.buildComponent(thisComponentLibrary, thisComponentUses, thisComponentParameters, thisAssumptionsFacet, thisDefinitionsFacet, thisImplicationsFacet);
            thisComponentValue.setDeclaredItems(thisComponentDeclared);
            thisComponentItem = myCreator.buildItem(thisComponentLabel, thisComponentType, thisComponentValue);
            thisItemList.add(thisComponentItem);
        }
    ;
term_list [ItemList thisTermList]
    : #( TERM_LIST 
            ( simple_term[thisTermList]
            | let_term[thisTermList]
            )+
        )
    ;
simple_term [ItemList thisTermList]
{
    Label thisSimpleTermLabel = null;
    Expression thisSimpleTermExpression = null;
    Item thisTermItem = null;
}    
    : #( SIMPLE_TERM
            thisSimpleTermLabel=label
            thisSimpleTermExpression=expression
        )
        {
            thisTermItem = myCreator.buildItem(thisSimpleTermLabel, null, thisSimpleTermExpression); 
            thisTermList.add(thisTermItem);
        }
    ;
let_term [ItemList thisTermList]
//interpret let with multiple terms as multiple let terms
//let_binding_list interpreted same as let_expression
//so some code duplicated here, but unavoidable.
{
    //the bindings associated with the let_term
    ItemList thisLetBindingList = myCreator.makeItemList();
    //the terms contained within the let_term
    ItemList thisLetTermList = myCreator.makeItemList();
    //for iteration through the ItemLists
    int currentPosition = 0;
    Item currentBindingItem = null;
    ItemList thisLetParameters = myCreator.makeItemList();
    Expression thisLetExpression = null;
    int pos = 0;
    Item nextLetTermItem = null;
    Expression nextLetFunctionValue = null;
}
    : #( LET_TERM
            let_binding_list[thisLetBindingList]
            (term_list[thisLetTermList])?
        )
        {
            //iterate through thisLetTermList making each value contained within a let expression
            //the leaf node in a tree representing the let_binding_list
            for (pos = 0; pos < thisLetTermList.size(); pos++)
            {
                nextLetTermItem = (thisLetTermList.get(pos));
                //make the nextLetFunctionValue similarly to let_expression, but with nextLetTermItem.getValue() as leaf node instead
                //of thisLetValue (as in let_expression)
                {
                    //initialize letFunction with last one having thisLetValue as it's expression, and last letBinding
                    //must be at least one let_binding in thisLetBindingList, but not necessary to be more
                    //set currentPosition at start of the ExpressionList
                    currentPosition = thisLetBindingList.size() - 1;
                    //get the bindingExpression at that position
                    currentBindingItem = thisLetBindingList.get(currentPosition);
                    //T:soon LetFunction will only take(Item, Expression), NOT (ItemList, Expression)
                    //for now, add the binding to an itemlist
                    //thisLetParameters.add(currentBindingItem);
                    //nextLetFunctionValue = myCreator.buildLetFunction(thisLetParameters, nextLetTermItem.getValue());
                    nextLetFunctionValue = myCreator.buildLetFunction(currentBindingItem, nextLetTermItem.getValue());
                    //iterate backwards through thisLetBindingList adding each to the ROM
                    //***changing this to stop over use of the first binding - already added first one ->take it off to start
                    //for(;currentPosition>=0;currentPosition--)
                    for(currentPosition--;currentPosition>=0;currentPosition--)
                    {
                        currentBindingItem = thisLetBindingList.get(currentPosition);
                        //T: clear the ItemList (not required when ROM interface changes to (Item, Expression) )
                        //thisLetParameters.remove(0);
                        //add the new one
                        //thisLetParameters.add(currentBindingItem);
                        //update the let_expression, with the old one and the new letBinding
                        //nextLetFunctionValue = myCreator.buildLetFunction(thisLetParameters, nextLetFunctionValue);
                        nextLetFunctionValue = myCreator.buildLetFunction(currentBindingItem, nextLetFunctionValue);
                    }
                }
                nextLetTermItem = myCreator.buildItem(nextLetTermItem.getLabel(), null, nextLetFunctionValue);
                thisTermList.add(nextLetTermItem);
            }
        }
    ;
let_binding_list [ItemList thisLetBindingList]
    : #( LET_BINDING_LIST 
            (let_binding[thisLetBindingList])+ 
        )
    ;
let_binding [ItemList thisLetBindingList]
{
    Item thisLetItem = null;
}
    :
        ( thisLetItem=let_function_binding
        | thisLetItem=let_parameter_binding
        )
        {
            thisLetBindingList.add(thisLetItem);
        }
    ;
let_function_binding returns [Item thisLetItem]
{
    //initialize the return object
    thisLetItem = null;
    //parameters for buildFunction
    Function thisFunction = null;
    LetFunction thisLetFunction = null;
    Label thisLetFunctionBindingLabel = null;
    Expression thisLetFunctionBindingExpression = null;
    ItemList thisLetFunctionParameters = myCreator.makeItemList();
    Expression thisLetFunctionRange = null;
}
    : #( LET_FUNCTION_BINDING 
            thisLetFunctionBindingLabel=label 
            thisLetFunctionRange=function_signature[thisLetFunctionParameters]
            thisLetFunctionBindingExpression=expression 
        )
        {
            //make it a function
            thisFunction = myCreator.buildFunction(thisLetFunctionParameters, thisLetFunctionRange, thisLetFunctionBindingExpression);
            //Q:which parameter should be returned?
            thisLetFunction = myCreator.buildLetFunction(thisLetFunctionParameters.get(0), thisFunction);
            //NOTE: using thisLetFunctionBindingExpression twice here
            thisLetItem = myCreator.buildItem(thisLetFunctionBindingLabel, thisLetFunctionBindingExpression, thisLetFunction);
        }
    ;
let_parameter_binding returns [Item thisLetParameterBinding]
{
    thisLetParameterBinding = null;
    ItemList thisLetParameters = myCreator.makeItemList();
    Expression thisLetParameterBindingExpression = null;
    //temporary params
    //MESSY, I REALISE
    Item thisItem = null;
}
    : #( LET_PARAMETER_BINDING
            parameter_definition[thisLetParameters]
            thisLetParameterBindingExpression=expression
        )
        {
            //get the Item from the ItemList
            thisItem = thisLetParameters.get(0);
            thisLetParameterBinding = myCreator.buildItem(thisItem.getLabel(), thisItem.getType(), thisLetParameterBindingExpression );
        }
    ;
domain_declaration [ItemList thisItemList, ExpressionList thisDomainUses]
{
    Item thisDomainItem = null;
    //parameters required by buildItem
    Label thisDomainLabel, thisDomainCheckLabel = null;
    Domain thisDomainValue = null;
    Expression thisDomainType = null;

    //parameters required by buildDomain
    LabelList thisDomainLibrary = myCreator.makeLabelList();
    ItemList thisDomainParameters = myCreator.makeItemList();
    Expression thisDomainDomain = null;
    ItemList thisDomainDeclared = myCreator.makeItemList();
    Boolean thisDomainExportAll = new Boolean(false);
    ExpressionList thisDomainExports = myCreator.makeExpressionList();
    ItemList thisDomainTerms = myCreator.makeItemList();
}
    : #( node:DOMAIN_DECLARATION
            thisDomainLabel=label 
            thisDomainDomain=facet_signature[thisDomainParameters]
            thisDomainExportAll=facet_body[thisDomainTerms, thisDomainExports, thisDomainDeclared]
            (#(CHECK_LABEL
                    (
                        thisDomainCheckLabel=label
                    )?
                )
                {
                    if (thisDomainCheckLabel==null)
                    {addWarning("end domain <label> missing.",desource(node));}
                    else if (!thisDomainCheckLabel.equals(thisDomainLabel))
                    {addWarning("end domain <label> does not match.",desource(node));}
                }
            )
        )
        {
            extractParameterConstraintItems(thisDomainParameters, thisDomainTerms);
            thisDomainValue = myCreator.buildDomain(thisDomainLibrary, thisDomainUses, thisDomainParameters, thisDomainDomain,
                thisDomainDeclared, thisDomainExportAll, thisDomainExports, thisDomainTerms);
            thisDomainItem = myCreator.buildItem(thisDomainLabel, thisDomainType, thisDomainValue);
            thisItemList.add(thisDomainItem);
        }
    ;
label_list returns [LabelList thisLabelList]
{
    thisLabelList = myCreator.makeLabelList();
    Label nextLabel = null;
}
    : #( LABEL_LIST 
            (
                nextLabel=label 
                {thisLabelList.add(nextLabel);}
            )+ 
        )
    ;
label returns [Label thisLabel]
{
    thisLabel = null;
}
    : #( LABEL
            ( sl:SIMPLE_LABEL
                { thisLabel = myCreator.buildLabel(sl.getText()); }
            | oil:OPERATOR_INTERPRETATION_LABEL
                { thisLabel = myCreator.buildLabel(oil.getText()); }
            )
        )
        
    ;
real_literal returns [NumberLiteral thisRealLiteral]
{
    thisRealLiteral = null;
}
    : #( REAL_LITERAL
            ( dl:DECIMAL_LITERAL
                { thisRealLiteral = myCreator.buildNumberLiteral(NumberParser.valueOf(dl.getText())); }
            | bl:BASED_LITERAL
                {
                    thisRealLiteral = myCreator.buildNumberLiteral(NumberParser.valueOf(bl.getText()));
                    //NOTE: to use the actual method (for debugging) could call:
                    //thisRealLiteral = myCreator.buildNumberLiteral(org.sldl.rosetta.lang.logic.RadixNumber.valueOf(bl.getText()));
                }
            )
        )
    ;
character_literal returns [CharacterLiteral thisCharacterLiteral]
{
    thisCharacterLiteral = null;
}
    : #( CHARACTER_LITERAL
            ( gcl:GRAPHIC_CHARACTER_LITERAL
                { thisCharacterLiteral = myCreator.buildCharacterLiteral( new Character(gcl.getText().charAt(0)) ); }
            | ecl:ENCODED_CHARACTER_LITERAL
                //convertRosettaToCharacter requrires format "`U+XXXX`" (ie reverse apostrophe at both ends)
                //CL: {thisCharacterLiteral = myCreator.buildCharacterLiteral( CharacterLiteral.CharConverter.convertRosettaToCharacter("`"+ecl.getText()+"`") ); }
            )
        )
    ;
string_literal returns [Expression thisStringLiteral]
//handling concatenation done in expression (AMPERSAND operator)
{
    thisStringLiteral = null;
}
    :
        ( //simple string
            sl:STRING_LITERAL //primitive type
            { thisStringLiteral = myCreator.buildStringLiteral(sl.getText()); }
        )
    ;
expression returns [Expression thisExpression]
{
    thisExpression = null;
}
    : #( EXPRESSION thisExpression=expr )
        //the EXPRESSION node appears only once at the top level of an expression
    ;
expr returns [Expression thisExpression]
{
    thisExpression = null;
    LabelList thisName = null;
    //operator
    ItemReference thisOperator = null;
    //operands
    Expression left_operand = null;
    Expression right_operand = null;
    //for case of index_operation/function_appl_or_facet_inst.
    Expression primaryExpression = null;
    //container for operands
    ExpressionList thisExpressionList = myCreator.makeExpressionList();
    ExpressionList followingParameters = null;
}

    :       //precedence_0
        ( #(p0_1:GREATER_THAN_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p0_1); }
        | #(p0_2:EQUALS_SIGN_GREATER_THAN_SIGN_GREATER_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p0_2); }

            //precedence_1
        | #(p1_1:EQUALS_SIGN_EQUALS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p1_1); }

            //precedence_2
        | #(p2_1:EQUALS_SIGN_GREATER_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p2_1); }
        | #(p2_2:IMPLIES_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p2_2); }
        | #(p2_3:LESS_THAN_SIGN_EQUALS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p2_3); }

            //precedence_3
        | #(p3_1:OR_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p3_1); }
        | #(p3_2:NOR_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p3_2); }
        | #(p3_3:XOR_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p3_3); }
        | #(p3_4:XNOR_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p3_4); }
        | #(p3_5:MAX_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p3_5); }

            //precedence_4
        | #(p4_1:AND_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p4_1); }
        | #(p4_2:NAND_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p4_2); }
        | #(p4_3:MIN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p4_3); }

            //precedence_5
        | #(p5_1:EQUALS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_1); }
        | #(p5_2:SOLIDUS_EQUALS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_2); }
        | #(p5_3:LESS_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_3); }
        | #(p5_4:GREATER_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_4); }
        | #(p5_5:EQUALS_SIGN_LESS_THAN_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_5); }
        | #(p5_6:GREATER_THAN_SIGN_EQUALS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_6); }
        | #(p5_7:IN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p5_7); }

            //precedence_6
        | #(p6_1:SUB_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p6_1); }

            //precedence_7
        | #(p7_1:PLUS_SIGN_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_1); }
        | #(p7_2:HYPHEN_MINUS_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_2); }
        | #(p7_3:PLUS_SIGN_FULL_STOP_FULL_STOP_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_3); }
        | #(p7_4:AMPERSAND_FULL_STOP_FULL_STOP_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_4); }
        | #(p7_5:AMPERSAND_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_5); }
        | #(p7_6:VERTICAL_LINE_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p7_6); }

            //precedence_8
        | #(p8_1:ASTERISK_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p8_1); }
        | #(p8_2:SOLIDUS_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p8_2); }
        | #(p8_3:MOD_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p8_3); }
        | #(p8_4:DIV_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p8_4); }

            //precedence_9
        | #(p9_1:CIRCUMFLEX_ACCENT_TOKEN left_operand=expr right_operand=expr)
            { thisOperator = createBinaryReference(p9_1); }

            //precedence_10 (unary) - all assign right operand _only_ (as in the tarpjj parser)
        | #(p10_1:NOT_TOKEN right_operand=expr)
            { thisOperator = createUnaryReference(p10_1); }
        | #(p10_2:UNARY_PLUS right_operand=expr)
            { thisOperator = createUnaryReference(p10_2); }
        | #(p10_3:UNARY_MINUS right_operand=expr)
            { thisOperator = createUnaryReference(p10_3); }
        | #(p10_4:UNARY_ASTERISK right_operand=expr)
            { thisOperator = createUnaryReference(p10_4); }
        | #(p10_5:NUMBER_SIGN_TOKEN right_operand=expr)
            { thisOperator = createUnaryReference(p10_5); }
        | #(p10_6:PERCENT_SIGN_TOKEN right_operand=expr)
            { thisOperator = createUnaryReference(p10_6); }
        | #(p10_7:TILDA_TOKEN right_operand=expr)
            { thisOperator = createUnaryReference(p10_7); }

            //precedence_11
        | #(p11_1:COMMERCIAL_AT_TOKEN thisName=name right_operand=expr)
            {
                thisOperator = createBinaryReference(p11_1);
                //NOTE: changed name->expr as left_operand
                left_operand = createReference(thisName);
            }
        | #(p11_2:APOSTROPHE_TOKEN thisName=name) //right_operand remains null
            {
                //must be POSTFIX
                thisOperator = createPostfixUnaryReference(p11_2);
                //NOTE: changed name->expr as left_operand
                left_operand = createReference(thisName);
            }

            //precedence_12//T:Q:not sure about how to handle this yet
        | thisExpression=primary
           ( followingParameters=function_application_or_facet_instantiation
               { thisExpression = myCreator.buildApplication(thisExpression, followingParameters); }
           | followingParameters=index_operation
               {
               followingParameters.add(0, thisExpression);
               thisExpression = myCreator.buildApplication(createReference(ParserConstants.DEREF_OPERATOR_TAG), followingParameters);
               }
           )*
        )
        {
            if (thisExpression==null) //ie not a primary
            {
                Application markedExpression;
                if (left_operand!=null)
                  {thisExpressionList.add(left_operand);}
                if (right_operand!=null)
                  {thisExpressionList.add(right_operand);}
                markedExpression = myCreator.buildApplication(thisOperator, thisExpressionList );
                if ((left_operand != null) && (right_operand != null))
                  { markedExpression.setApplicationType(Application.BINARY); }
                else if ((left_operand == null) && (right_operand != null))
                  { markedExpression.setApplicationType(Application.PREFIX); }
                else if ((left_operand != null) && (right_operand == null))
                  { markedExpression.setApplicationType(Application.POSTFIX); }
                thisExpression = markedExpression;
            }
        }
    ;
function_application_or_facet_instantiation returns [ExpressionList thisFAFI]
{
    thisFAFI = myCreator.makeExpressionList();
}
    : #( FUNCTION_APPLICATION_OR_FACET_INSTANTIATION
            argument_list[thisFAFI]
        )
    ;
index_operation returns [ExpressionList thisIndexExpression]
{
    thisIndexExpression = myCreator.makeExpressionList();
    Expression indexExpression = null;
}
    : #( INDEX_OPERATION
            indexExpression = expression
            { thisIndexExpression.add(indexExpression); }
        )
    ;
argument_list [ExpressionList thisExpressionList]
{
    Expression thisExpression = null;
}
    : #( ARGUMENT_LIST
            ( llt:LOW_LINE_TOKEN
                {
                    thisExpressionList.add(createReference(llt));
                }
            | thisExpression=expression
                {
                    thisExpressionList.add(thisExpression);
                }
            )*
        )
    ;
//only called from expr, where it expects an expression to be initialized
primary returns [Expression thisPrimaryExpression]
{
    thisPrimaryExpression = null;
    LabelList thisName = null;
}
    : #(PRIMARY
            ( thisPrimaryExpression=literal
            | thisPrimaryExpression=quantified_expression
            | thisName=name
                {thisPrimaryExpression=createReference(thisName);}
            | thisPrimaryExpression=parenthesized_expression
            | thisPrimaryExpression=anonymous_function
            | thisPrimaryExpression=anonymous_facet
            | thisPrimaryExpression=curly_bracket_formation
            | thisPrimaryExpression=square_bracket_formation
            | thisPrimaryExpression=type_formation
            | thisPrimaryExpression=let_expression
            | thisPrimaryExpression=if_expression
            | thisPrimaryExpression=case_expression
            )
        )
    ;
literal returns [Expression thisLiteralExpression]
{
    thisLiteralExpression = null;
}
    : 
        ( thisLiteralExpression=real_literal
        | thisLiteralExpression=character_literal
        | thisLiteralExpression=string_literal
        )
    ;   
quantified_expression returns [QuantifierReference thisQuantifiedExpression]
{
    thisQuantifiedExpression = null;
    //parameters for buildQuantifierReference
    LabelList thisQuantifierName = null;
    ItemList thisQuantifiedExpressionParameters = myCreator.makeItemList();
    Expression thisExpression = null;
}
   : #( QUANTIFIED_EXPRESSION 
            thisQuantifierName=quantifier
            (parameter_list[thisQuantifiedExpressionParameters])? 
            thisExpression=expression
        )
        {
            thisQuantifiedExpression = myCreator.buildQuantifierReference(createReference(thisQuantifierName), thisQuantifiedExpressionParameters, thisExpression);
        }
    ;
quantifier returns [LabelList thisName]
{
    thisName = myCreator.makeLabelList();
}
    : #(QUANTIFIER
          ( f:FORALL_TOKEN { thisName.add(myCreator.buildLabel(f.getText())); }
          | e:EXISTS_TOKEN { thisName.add(myCreator.buildLabel(e.getText())); }
          | s:SEL_TOKEN { thisName.add(myCreator.buildLabel(s.getText())); }
          | i:MIN_TOKEN { thisName.add(myCreator.buildLabel(i.getText())); }
          | a:MAX_TOKEN { thisName.add(myCreator.buildLabel(a.getText())); }
          | r:RAN_TOKEN { thisName.add(myCreator.buildLabel(r.getText())); }
           )
        )
    ;

name returns [LabelList thisName]
{
    thisName = myCreator.makeLabelList();
    Label nextLabel = null;
}
    : #(NAME
            (
                (nextLabel=label
                    { //construct similarly to label_list
                        thisName.add(nextLabel);
                    }
                )+
            )
        )
    ;
parenthesized_expression returns [Expression thisParenthesizedExpression]
//just removes parenthesis used to control precedence operations
{
    thisParenthesizedExpression = null;
}
    : #(PARENTHESIZED_EXPRESSION
            thisParenthesizedExpression=expression
            { thisParenthesizedExpression.setParens(true); }
        )
    ;
anonymous_function returns [Function thisAnonymousFunction]
{
    thisAnonymousFunction = null;
    //parameters for buildFunction
    ItemList thisFunctionParameters = myCreator.makeItemList();
    Expression thisFunctionRange = null;
    Expression thisFunctionValue = null;
}
    : #(ANONYMOUS_FUNCTION
            thisFunctionRange=function_signature[thisFunctionParameters]
            thisFunctionValue=expression
        )
        {
            thisAnonymousFunction = myCreator.buildFunction(thisFunctionParameters, thisFunctionRange, thisFunctionValue);
        }
    ;
anonymous_facet returns [Facet thisAnonymousFacet]
{
    thisAnonymousFacet = null;
    //parameters for buildFacet
    LabelList thisAnonymousFacetLibrary = myCreator.makeLabelList();
    ExpressionList thisAnonymousFacetUses = myCreator.makeExpressionList(); //Not defined so far
    ItemList thisAnonymousFacetParameters = myCreator.makeItemList();
    Expression thisAnonymousFacetDomain = null;
    ItemList thisAnonymousFacetDeclared = myCreator.makeItemList();
    Boolean thisAnonymousFacetExportAll = new Boolean(false);
    ExpressionList thisAnonymousFacetExports = myCreator.makeExpressionList();
    ItemList thisAnonymousFacetTerms = myCreator.makeItemList();
}
    : #(ANONYMOUS_FACET 
            thisAnonymousFacetDomain=facet_signature[thisAnonymousFacetParameters]
            thisAnonymousFacetExportAll=facet_body[thisAnonymousFacetTerms, thisAnonymousFacetExports, thisAnonymousFacetDeclared]
        )
        {
            extractParameterConstraintItems(thisAnonymousFacetParameters, thisAnonymousFacetTerms);
            thisAnonymousFacet = myCreator.buildFacet(thisAnonymousFacetLibrary, thisAnonymousFacetUses, thisAnonymousFacetParameters, thisAnonymousFacetDomain,
                thisAnonymousFacetDeclared, thisAnonymousFacetExportAll, thisAnonymousFacetExports, thisAnonymousFacetTerms);
        }
    ;
curly_bracket_formation returns [Expression thisCurlyBracketFormationExpression]
{
    thisCurlyBracketFormationExpression = null;
    ExpressionList thisExpressionList = myCreator.makeExpressionList();
}
    : #(CURLY_BRACKET_FORMATION
            (expression_list[thisExpressionList])?
        )
        {
            //thisCurlyBracketFormationExpression = myCreator.buildApplication(createUnaryReference("{}"), new CompositeValue(thisExpressionList));
            thisCurlyBracketFormationExpression = ExpressionListAndBrackets(thisExpressionList, "{}");
        }
    ;
square_bracket_formation returns [Expression thisSquareBracketFormationExpression]
{
    thisSquareBracketFormationExpression = null;
    ExpressionList thisExpressionList = myCreator.makeExpressionList();
}
    : #(SQUARE_BRACKET_FORMATION
            (expression_list[thisExpressionList])?
        )
        {
            //thisSquareBracketFormationExpression = myCreator.buildApplication(createUnaryReference("[]"), thisExpressionList);
            thisSquareBracketFormationExpression = ExpressionListAndBrackets(thisExpressionList, "[]");
        }
    ;
type_formation returns [Expression thisTypeFormationExpression]
{
    thisTypeFormationExpression = null;
}
    : #(TYPE_FORMATION 
            ( thisTypeFormationExpression=universal_type_formation 
            | thisTypeFormationExpression=subtype_formation
            | thisTypeFormationExpression=set_type_formation
            | thisTypeFormationExpression=array_type_formation
            | thisTypeFormationExpression=function_type_formation
            | thisTypeFormationExpression=facet_type_formation
            )
        )
    ;
universal_type_formation returns [Expression thisUniversalTypeFormation]
{
    thisUniversalTypeFormation = null;
}
    :
        //was TYPE_TOKEN, now just UNIVERSAL_TYPE_FORMATION
        node:UNIVERSAL_TYPE_FORMATION
        {
            //using the string "TYPE", as this is what the ROM is expecting
            //
            thisUniversalTypeFormation = createReference("TYPE", desource(node));
        }
    ;
subtype_formation returns [Expression thisSubtypeFormation]
{
    thisSubtypeFormation = null;
    Expression thisValue = null;
    ItemReference thisRelationship = null;
}
    : #(node:SUBTYPE_FORMATION
            thisValue=expression
        )
        {
            //thisRelationship = createReference("SUBTYPE", desource(node));
            //thisSubtypeFormation = myCreator.buildTypeExpression(thisRelationship, thisValue);
            thisSubtypeFormation = myCreator.buildTypeExpression("SUBTYPE", thisValue);
        }
    ;
set_type_formation returns [Expression thisSetTypeFormation]
{
    thisSetTypeFormation = null;
    Expression thisValue = null;
    ItemReference thisRelationship = null;
}
    : #(node:SET_TYPE_FORMATION
            thisValue=expression
        )
        {
            //thisRelationship = createReference("SET", desource(node));
            //thisSetTypeFormation = myCreator.buildTypeExpression(thisRelationship, thisValue);
            thisSetTypeFormation = myCreator.buildTypeExpression("SET", thisValue);
        }
    ;
array_type_formation returns [Expression thisArrayTypeFormation]
{
    thisArrayTypeFormation = null;
    Expression thisValue = null;
    ItemReference thisRelationship = null;
}
    : #(node:ARRAY_TYPE_FORMATION 
            thisValue=expression
        )
        {
            //thisRelationship = createReference("ARRAY", desource(node));
            //thisArrayTypeFormation = myCreator.buildTypeExpression(thisRelationship, thisValue);
            thisArrayTypeFormation = myCreator.buildTypeExpression("ARRAY", thisValue);
        }
    ;
function_type_formation returns [Expression thisFunctionTypeFormation]
{
    thisFunctionTypeFormation = null;
    ItemList thisFunctionTypeFormationParameters = myCreator.makeItemList();
    Expression thisFunctionTypeFormationDomain = null;
    //parameters for buildTypeExpression
    Expression thisValue = null;
    ItemReference thisRelationship = createReference("FUNCTION");
}
    : #(FUNCTION_TYPE_FORMATION
            thisFunctionTypeFormationDomain=function_signature[thisFunctionTypeFormationParameters]
        )
        {
            thisValue = myCreator.buildFunction(thisFunctionTypeFormationParameters, null, thisFunctionTypeFormationDomain);
            //thisFunctionTypeFormation = myCreator.buildTypeExpression(thisRelationship, thisValue);
            thisFunctionTypeFormation = myCreator.buildTypeExpression("FUNCTION", thisValue);
        }
    ;
facet_type_formation returns [Expression thisFacetTypeFormation]
{
    thisFacetTypeFormation = null;
    ItemList thisFacetTypeFormationParameters = myCreator.makeItemList();
    Expression thisFacetTypeFormationDomain = null;
    //parameters for buildTypeExpression
    Expression thisValue = null;
    ItemReference thisRelationship = createReference("FACET");
}
    : #(FACET_TYPE_FORMATION
            thisFacetTypeFormationDomain=facet_signature[thisFacetTypeFormationParameters]
        )
        {
            thisValue = myCreator.buildFunction(thisFacetTypeFormationParameters, null, thisFacetTypeFormationDomain);
            //thisFacetTypeFormation = myCreator.buildTypeExpression(thisRelationship, thisValue);
            thisFacetTypeFormation = myCreator.buildTypeExpression("FACET", thisValue);
        }
    ;
let_expression returns [Expression thisLetExpression]
{
    ItemList thisLetBindingList = myCreator.makeItemList();
    thisLetExpression = null;
    LetFunction thisLetFunction = null;
    //parameters for buildLetFunction
    Expression thisLetValue = null;
    ItemList thisLetParameters = myCreator.makeItemList();
    //variables for iteration
    Item currentBindingItem = null;
    int currentPosition = 0;
}
    : #(LET_EXPRESSION
            let_binding_list[thisLetBindingList]
            thisLetValue=expression
        )
        {
            //NOTE: could also do this by removing letBindings from the list, but not done this way here

            //initialize letFunction with last one having thisLetValue as it's expression, and last letBinding
            //must be at least one let_binding in thisLetBindingList, but not necessary to be more
            //set currentPosition at start of the ExpressionList
            currentPosition = thisLetBindingList.size() - 1;
            //get the bindingExpression at that position
            currentBindingItem = thisLetBindingList.get(currentPosition);
            //T:soon LetFunction will only take(Item, Expression), NOT (ItemList, Expression)
            //for now, add the binding to an itemlist
            //thisLetParameters.add(currentBindingItem);
            //thisLetExpression = myCreator.buildLetFunction(thisLetParameters, thisLetValue);
            thisLetExpression = myCreator.buildLetFunction(currentBindingItem, thisLetValue);
            //iterate backwards through thisLetBindingList adding each to the ROM
            for(currentPosition--;currentPosition>=0;currentPosition--)
            {
                currentBindingItem = thisLetBindingList.get(currentPosition);
                //T: clear the ItemList (not required when ROM interface changes to (Item, Expression) )
                //thisLetParameters.remove(0);
                //add the new one
                //thisLetParameters.add(currentBindingItem);
                //update the let_expression, with the old one and the new letBinding
                //thisLetExpression = myCreator.buildLetFunction(thisLetParameters, thisLetExpression);
                thisLetExpression = myCreator.buildLetFunction(currentBindingItem, thisLetExpression);
            }
        }

    ;
if_expression returns [Expression thisIfExpression]
//can be used for elsif_expression also, but leave separate until ROM creation implemented
{
    thisIfExpression = null;
    Expression firstConditionExpression = null;
    Expression firstTrueAlternativeExpression = null;    
    Expression thisIfCondition = null;
    Expression thisIfTrueAlternative = null;
    Expression thisElseExpression = null;
}
    : #(IF_EXPRESSION 
            (#(CONDITION firstConditionExpression=expression))
            { thisIfCondition = firstConditionExpression; }
            (#(TRUE_ALTERNATIVE firstTrueAlternativeExpression=expression))
            { thisIfTrueAlternative = firstTrueAlternativeExpression; }
            (#(FALSE_ALTERNATIVE thisElseExpression=if_expression_tail))
        )
        {
            thisIfExpression = myCreator.buildIfExpression(thisIfCondition, thisIfTrueAlternative, thisElseExpression);
        }
    ;
if_expression_tail returns [Expression thisElseExpression]
{
    //pre-declare it.
    thisElseExpression = null;
}
    : #(IF_EXPRESSION_TAIL
            ( thisElseExpression=elsif_expression
            | thisElseExpression=else_expression
            )?
        )
    ;
elsif_expression returns [Expression thisElseExpression]
{
    thisElseExpression = null;
    Expression nextConditionExpression = null;
    Expression nextTrueAlternativeExpression = null;
    Expression nextFalseAlternative = null;
}
    : #( ELSIF_EXPRESSION
            (#(CONDITION nextConditionExpression=expression))
            (#(TRUE_ALTERNATIVE nextTrueAlternativeExpression=expression))
            (#(FALSE_ALTERNATIVE nextFalseAlternative=if_expression_tail))
        )
        {
            thisElseExpression = myCreator.buildIfExpression(nextConditionExpression, nextTrueAlternativeExpression, nextFalseAlternative);
        }
    ;
else_expression returns [Expression thisElseExpression]
{
    thisElseExpression = null;
}
    : #( ELSE_EXPRESSION
            thisElseExpression=expression
        )
    ;
case_expression returns [Expression thisCaseExpression]
{
    thisCaseExpression = null;
    //parameters for buildCaseExpression
    Expression thisTestExpression = null;
    ExpressionList thisCaseTests = myCreator.makeExpressionList();
    ExpressionList thisCaseResults = myCreator.makeExpressionList();
}
    : #(CASE_EXPRESSION
            thisTestExpression=expression
            case_alternative_list[thisCaseTests, thisCaseResults]
        )
        {
            thisCaseExpression = myCreator.buildCaseExpression(thisTestExpression, thisCaseTests, thisCaseResults);
        }
    ;
case_alternative_list [ExpressionList thisCaseTests, ExpressionList thisCaseResults]
{
    Expression nextTagExpression = null;
    Expression nextResultExpression = null;
}
    : #(CASE_ALTERNATIVE_LIST
            ( #(CASE_ALTERNATIVE 
                    (#(TAG nextTagExpression=expression))
                    {
                        thisCaseTests.add(nextTagExpression);
                    }
                    (#(RESULT nextResultExpression=expression))
                    {
                        thisCaseResults.add(nextResultExpression);
                    }
                )
            )+
        )
    ;
expression_list [ExpressionList thisExpressionList]
{
    Expression nextExpression = null;
}
    : #(EXPRESSION_LIST 
            (
                nextExpression=expression
                {
                    thisExpressionList.add(nextExpression);
                }
            )+
        )
        
    ;
