//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: MapObjectDeconstructor.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: MapObjectDeconstructor.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.1  2002/08/27 07:15:25  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.Serializable;
import java.util.Map;


/** Basic interface to provide the deconstruction transformation
 *  from an object to a map of (key,value) pairs.
 */
public interface MapObjectDeconstructor extends Serializable
  {
  /** Method to check if this deconstructor can handle a given object.
   * @param target The object to deconstruct.
   * @return boolean Can this deconstructor handle that object?
   */
  public boolean isValidDeconstructor(Object target);


  /** Method to perform the deconstruction.
   * @param object The object to be deconstructed.
   * @return Map The map containing the contents of the object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Map deconstruct(Object object) throws Exception;

  }
