//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: MapObjectReconstructor.java,v 1.3 2002/10/30 17:33:11 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: MapObjectReconstructor.java,v $
//  Revision 1.3  2002/10/30 17:33:11  cvs
//  Added the files relevant to ANTLR to the cvs.
//  --Murthy.
//
//  Revision 1.1  2002/08/27 07:15:26  josephk
//  Added the new implimentation of the Mapping tools.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.map;

import java.io.Serializable;
import java.util.Map;


/** Class to provide the reconstruction transformation from
 *  a map of (key,value) pairs to an object.
 */
public interface MapObjectReconstructor extends Serializable
  {
  /** Method to check if this reconstructor can handle a given map.
   * @param target The map to reconstruct.
   * @return boolean Can this reconstructor handle that map?
   */
  public boolean isValidReconstructor(Map target);


  /** Method to perform the reconstruction.
   * @param map The map containing the contents of the object.
   * @return Object The reconstructed object.
   * @throws Exception If an error occurs it should be propogated.
   */
  public Object reconstruct(Map map) throws Exception;

  }
