//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: ConstructorSignatures.java,v 1.5 2002/11/21 05:34:13 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: ConstructorSignatures.java,v $
//  Revision 1.5  2002/11/21 05:34:13  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 1.6  2002/11/18 04:31:26  josephk
//  Added the key to the list of constructors to the ConstructedTypeSignature.
//
//  Revision 1.5  2002/11/18 03:09:22  josephk
//  Extended the signature for ItemReference to take the original string used
//  to create the reference as well as the LabelList it turns into.
//
//  Revision 1.4  2002/09/09 08:59:30  josephk
//  Fixed a bug in the PackageSignature. Removed ConstructedValue methods.
//
//  Revision 1.3  2002/09/06 06:00:22  josephk
//  Updated some problems in the Creator routines.
//
//  Revision 1.2  2002/09/05 01:36:50  josephk
//  Fixed several array out-of-bounds errors for "Facet.keySet" and "Item.keySet".
//
//  Revision 1.1  2002/08/27 07:16:07  josephk
//  Moved the various 'one-off' files to a common misc directory.
//
//
//=============================================================================
package org.sldl.rosetta.parser.tarp.misc;

import java.io.Serializable;
import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;


/** This is a wrapper class for making it easier to call the large
 *  number of constructors a parser will call when building a ROM,
 *  such that changes in the ROM need only be updated here.
 */
public interface ConstructorSignatures extends Serializable
  {
  public static final String[] ItemListSignature = { BaseObjectList.keySet[0] };
  public static final String[] ExpressionListSignature = { BaseObjectList.keySet[0] };
  public static final String[] LabelListSignature = { BaseObjectList.keySet[0] };

  public static final String[] LabelSignature = { Label.keySet[0] };
  public static final String[] CharacterLiteralSignature = { CharacterLiteral.keySet[0] };
  public static final String[] NumberLiteralSignature = { NumberLiteral.keySet[0] };
  public static final String[] StringLiteralSignature = { StringLiteral.keySet[0] };

  public static final String[] ArrayValueSignature = { CompositeValue.keySet[0] };
  public static final String[] EnumerationTypeSignature = { CompositeValue.keySet[0] };
  public static final String[] SetValueSignature = { CompositeValue.keySet[0] };

  public static final String[] ApplicationSignature = { Application.keySet[0], Application.keySet[1] };
  public static final String[] CaseExpressionSignature = { CaseExpression.keySet[0], CaseExpression.keySet[1], CaseExpression.keySet[2] };
  public static final String[] ComponentSignature = { Facet.keySet[0], Facet.keySet[1], Facet.keySet[2], Facet.keySet[3], Component.keySet[0], Component.keySet[1],  Component.keySet[2] };
  public static final String[] ConstructedTypeSignature = { ConstructedType.keySet[0], ConstructedType.keySet[1] };
  public static final String[] ConstructedTypeConstructorSignature = { ConstructedTypeConstructor.keySet[0], ConstructedTypeConstructor.keySet[1], ConstructedTypeConstructor.keySet[2] };
  public static final String[] DomainSignature = { Facet.keySet[0], Facet.keySet[1], Facet.keySet[2], Facet.keySet[3], Facet.keySet[4], Facet.keySet[5], Facet.keySet[6] };
  public static final String[] FacetSignature = { Facet.keySet[0], Facet.keySet[1], Facet.keySet[2], Facet.keySet[3], Facet.keySet[4], Facet.keySet[5], Facet.keySet[6] };
  public static final String[] FunctionSignature = { Function.keySet[0], Function.keySet[1], Function.keySet[2] };
  public static final String[] IfExpressionSignature = { IfExpression.keySet[0], IfExpression.keySet[1], IfExpression.keySet[2] };
  public static final String[] InteractionSignature = { Facet.keySet[0], Facet.keySet[1], Facet.keySet[2], Facet.keySet[3], Facet.keySet[4], Facet.keySet[5], Facet.keySet[6] };
  public static final String[] ItemSignature = { Item.keySet[0], Item.keySet[1], Item.keySet[2] };
  public static final String[] ItemReferenceSignature = { ItemReference.keySet[1], ItemReference.keySet[3] };
  public static final String[] LetFunctionSignature = { LetFunction.keySet[0], LetFunction.keySet[1] };
  public static final String[] PackageSignature = { Facet.keySet[0], Facet.keySet[1], Facet.keySet[2], Facet.keySet[3], Facet.keySet[5], Facet.keySet[6] };
  public static final String[] QuantifierReferenceSignature = { QuantifierReference.keySet[0], QuantifierReference.keySet[1], QuantifierReference.keySet[2] };
  public static final String[] TypeExpressionSignature = { TypeExpression.keySet[0], TypeExpression.keySet[1] };

  public static final String[] SourceReferenceSignature = { SourceReference.keySet[0], SourceReference.keySet[1], SourceReference.keySet[2], SourceReference.keySet[3], SourceReference.keySet[4] };
  }
