//============================================================================
//  Copyright (c) 2002 SLDL
//  All rights reserved.
//
//  SLDL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SLDL SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//  $Id: ParseWarning.java,v 1.1 2002/11/21 05:34:13 josephk Exp $
//
//  Revision History:
//  ----------------
//  $Log: ParseWarning.java,v $
//  Revision 1.1  2002/11/21 05:34:13  josephk
//  Updating KU ANTLR mirror.
//
//  Revision 1.2  2002/11/21 05:26:30  josephk
//  Cleaned up a few minor bugs in the ParseWarning class.
//
//  Revision 1.1  2002/11/21 05:25:17  josephk
//  Moved the object which describes parsing errors into the 'misc' directory.
//
//
//============================================================================
package org.sldl.rosetta.parser.tarp.misc;

import java.io.Serializable;
import java.util.Map;


/** This is the <B>ParseWarning</B> class which represents that an
 *  error has been generated during the parsing of a file which does
 *  not prevent the parser from continuing through the rest of the
 *  file. In this way a single pass over a document might turn up
 *  several errors to be corrected, rather than having to re-run the
 *  parser to find each error iteratively.
 */
public class ParseWarning implements Serializable
  {
  public static final String[] keySet = { "Message", "Position" };
  private final String myMessage;
  private final SourceReference myPosition;



  /** Method to access the message which describes this error.
   * @return String The message describing the error.
   */
  public String getMessage()
    { return myMessage; }

  /** Method to access the marker which shows where this error occurred.
   * @return SourceReference Where did this error occur?
   */
  public SourceReference getPosition()
    { return myPosition; }




  /** Map Constructor.
   * @param settings The settings used to construct this object.
   */
  public ParseWarning(Map settings)
    {
    myMessage = (String)settings.get(ParseWarning.keySet[0]);
    myPosition = (SourceReference)settings.get(ParseWarning.keySet[1]);
    }

  /** Direct Constructor.
   * @param mesg The message which describes the error that occurred.
   * @param pos The marker of where that error occurred.
   */
  public ParseWarning(String mesg, SourceReference pos)
    {
    myMessage = mesg;
    myPosition = pos;
    }



  /** Method to get the string representation of the object
   * @return String representing the string of this object
   */
  public String toString()
    { return myPosition + ": " + myMessage; }

  }
