//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//      authors: Garrin kimmell@ittc.ku.edu
//               Joseph josephk@cs.adelaide.edu.au
//
//  $Id: BasicTypeResolver.java,v 1.1.1.1 2002/07/01 15:47:36 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BasicTypeResolver.java,v $
//  Revision 1.1.1.1  2002/07/01 15:47:36  cvs
//  Rosetta tools and publications repository
//
//  Revision 1.1.2.1  2002/01/17 00:22:26  cvs
//  Continued adding files.
//  -kimmell
//
//  Revision 1.1.2.2  2002/01/10 04:05:39  cvs
//  Updated the BasicTypeResolver.java class.
//
//  Revision 1.1.2.1  2002/01/04 13:05:07  cvs
//  Updated the old 'AggregateReferences' to 'QuantifiedReferences'.
//
//
//=============================================================================
package org.sldl.rosetta.parser.typechecker;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.ROM.Package;

import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.TCC;
import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.TypeCheckerMonitor;
import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.UnableToTypeCheckException;
import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.TypeResolver;

import org.sldl.rosetta.lang.Rational;
import org.sldl.rosetta.lang.RadixNumber;
import org.sldl.rosetta.lang.ComplexNumber;

/** This represents the <B>BasicTypeResolverr</B> object. An instance of
 *  this class represents a primitive literal/rosetta.lang based type checker.
 * @author <I> Garrin Kimmell, Joseph Kuehn </I>
 * (kimmell@ittc.ku.edu, josephk@cs.adelaide.edu.au)
 */
public class BasicTypeResolver implements TypeResolver
  {
  private static final Label logicLabel = new Label("logic");
  private static final Label stateBasedLabel = new Label("state_based");

  private static final Label enumerationLabel = new Label("enumeration");
  private static final Label booleanLabel = new Label("boolean");
  private static final Label complexLabel = new Label("complex");
  private static final Label realLabel = new Label("real");
  private static final Label rationalLabel = new Label("rational");
  private static final Label integerLabel = new Label("integer");
  private static final Label naturalLabel = new Label("natural");
  private static final Label characterLabel = new Label("character");
  private static final Label stringLabel = new Label("string");
  private static final Label constructedLabel = new Label("constructed");
  private static final Label functionLabel = new Label("function");
  private static final Label setLabel = new Label("set");
  private static final Label arrayLabel = new Label("array");
  private static final Label facetLabel = new Label("facet");
  private static final Label domainLabel = new Label("domain");
  private static final Label packageLabel = new Label("package");
  private static final Label componentLabel = new Label("component");
  private static final Label interactionLabel = new Label("interaction");



  private ItemList langItems, logicItems, stateBasedItems;

  public BasicTypeResolver(DeferredItem langItem)
    {
    int pos;

    langItems = langItem.getImportList();
    for (pos = 0; pos < langItems.size(); pos++)
      {
      if (langItems.get(pos).getLabel().matches(logicLabel))
        { logicItems = ((DeferredItem)(langItems.get(pos))).getImportList(); }
      if (langItems.get(pos).getLabel().matches(stateBasedLabel))
        { stateBasedItems = ((DeferredItem)(langItems.get(pos))).getImportList(); }
      }
    }



  public static boolean ShowGood = false;

  private void print(boolean isGood, String message, TypeCheckerMonitor monitor)
    {
    if (ShowGood || !isGood)
      { monitor.typeCheckingError(message); }
    }


  private ItemReference fetch(Label key)
    {
    int pos;
    for (pos = 0; pos < logicItems.size(); pos++)
      {
      if (logicItems.get(pos).getLabel().matches(key))
        { return new ItemReference(logicItems.get(pos)); }
      }
    return null;
    }

//  foo = new TCC(String msg, BaseObject loc, boolean failed);
//  public static interface TypeCheckerMonitor
//    {
//    public void addTCC(TCC requirement);
//    public void typeCheckingError(String errorMessage);
//    }



  private void setType(Expression root, Expression theType)
    { setType(root, theType, true); }

  private void setType(Expression root, Expression theType, boolean createSet)
    {
//    FiniteSet newSet;

//    if (!createSet)
      { root.setType(theType); return; }
//    newSet = new FiniteSet();
//    newSet.addElement(theType);
//    root.setType(newSet);
    }


  private boolean match(Expression inner, Expression outer)
    {
    if ((inner == null) || (outer == null))
      { return false; }

    if (inner.matches(outer))
      { return true; }

    if ((outer instanceof ItemReference) && (((ItemReference)outer).getItemReferred().getLabel().matches(new Label("univ"))))
      { return true; }

    if (outer instanceof TypeExpression)
      {
      Expression outer2 = ((TypeExpression)outer).getExpression();
      if ((outer2 instanceof ItemReference) && (((ItemReference)outer2).getItemReferred().getLabel().matches(new Label("univ"))))
        { return true; }
      }

    try
      {
      ItemReference seeker;
      seeker = (ItemReference)inner;
      while (seeker != null)
        {
        if (seeker.matches(outer))
          { return true; }
        if (seeker.getItemReferred().getType() instanceof ItemReference)
          { seeker = (ItemReference)(seeker.getItemReferred().getType()); }
        else if (seeker.getItemReferred().getType() instanceof TypeExpression)
          { seeker = (ItemReference)(((TypeExpression)(seeker.getItemReferred().getType())).getExpression())  ; }
        else
          { seeker = null; }
        }
      }
    catch(Exception e)
      { }

    return false;
    }




  public void typeCheckBaseObject(BaseObject root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
    {
    if ((root == null) || (monitor == null) || (root instanceof DeferredItem))
      { return; }

    if (root instanceof Literal)
      { visitLiteral((Literal)root, monitor); }

    if (root instanceof ItemReference)
      { visitItemReference((ItemReference)root, monitor); }

    if (root instanceof LetFunction)
      { visitLetFunction((LetFunction)root, monitor); }

    if (root instanceof Formula)
      { visitFormula((Formula)root, monitor); }




    if ((root instanceof Expression) && (!(root instanceof ItemReference)) && (((Expression)root).getType() != null))
      { print(true, "Type [" + ((Expression)root).getType() + "] assigned to [" + root + "]", monitor); }
    }


  public void visitItemReference(ItemReference root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
    { setType(root, root.getItemReferred().getType(), false); }

  public void visitLetFunction(LetFunction root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
    { setType(root, root.getExpression().getType(), false); }


  public void visitFormula(Formula root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
    {
    try
      {
      Item opItem;
      Function fnVal;
      Expression lhsT, rhsT;

      opItem = root.getOperator().getItemReferred();
      if (opItem.getType() != null)
        { fnVal = (Function)(opItem.getType()); }
      else
        { fnVal = (Function)(opItem.getValue()); }
      setType(root, fnVal.getRange(), false);

      lhsT = root.getLeftOp().getType();
      rhsT = root.getLeftOp().getType();
      if (match(lhsT, fnVal.getInterfaceItems().get(0).getType()) && match(rhsT, fnVal.getInterfaceItems().get(1).getType()))
        { print(true, "Type Operator MATCH! [" + root + "]@" + root.getSourceReference() + " => " + lhsT + " in " + fnVal.getInterfaceItems().get(0).getType() + " && " + rhsT + " in " + fnVal.getInterfaceItems().get(1).getType(), monitor); }
      else
        { print(false, "Type Operator FAILURE! [" + root + "]@" + root.getSourceReference() + " => " + lhsT + " ?in? " + fnVal.getInterfaceItems().get(0).getType() + " && " + rhsT + " ?in? " + fnVal.getInterfaceItems().get(1).getType(), monitor); }
      }
    catch(Exception e)
      { }
    }


//  public void visitX(X root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
//    {
//    }

  public void visitLiteral(Literal root, TypeCheckerMonitor monitor) throws UnableToTypeCheckException
    {
    if (root instanceof DataLiteral)
      {
      Number data;
      data = ((DataLiteral)root).getNumericData();
      if (data instanceof Integer)
        {
        if (data.intValue() > 0)
          { setType(root, fetch(naturalLabel), false); }
        else
          { setType(root, fetch(integerLabel), false); }
        }
      if (data instanceof RadixNumber)
        { 
        if (((RadixNumber)data).isInteger())
          { setType(root, fetch(integerLabel), false); }
        else
          { setType(root, fetch(realLabel), false); }
        }
      if (data instanceof Rational)
        { setType(root, fetch(rationalLabel), false); }
      if (data instanceof Double)
        { setType(root, fetch(realLabel), false); }
      if (data instanceof ComplexNumber)
        { setType(root, fetch(complexLabel), false); }
      }

    if (root instanceof CharacterLiteral)
      { setType(root, fetch(characterLabel), false); }

    if (root instanceof StringLiteral)
      { setType(root, fetch(stringLabel), false); }

    if (root instanceof AtomicLiteral)
      { setType(root, fetch(enumerationLabel), false); }
    }



  }
