//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: Demo.java,v 1.2 2002/11/15 17:51:32 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: Demo.java,v $
// Revision 1.2  2002/11/15 17:51:32  kimmell
// Fixed compilation errors.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.3  2002/06/06 19:17:34  kimmell
// The demo now sets usage profile information for software.
// -kimmell
//
// Revision 1.1.2.2  2002/06/06 13:45:18  kimmell
// Cleaned up the demo output.
// -kimmell
//
// Revision 1.1.2.1  2002/06/05 16:42:20  kimmell
// Added 2002 DAC demo tool.
// -kimmell
//
//
package org.sldl.rosetta.tools.demo;

/**
 * Demo.java
 *
 *
 * Created: Wed May 29 15:43:30 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.io.FileInputStream;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.tools.glinker.Linker;
import org.sldl.rosetta.tools.evaluator.Evaluator;

import org.sldl.rosetta.lang.functions.meta.MetaGetOperators;
import org.sldl.rosetta.lang.functions.PrimRealEquals;
import org.sldl.rosetta.lang.values.RealValue;

/**
 * Describe class <code>Demo</code> here.
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version 1.0
 */
public class Demo {

  private Evaluator eval;
  
  // Execution profile properties
  Properties profile;

  // Individual parts of the spec
  Item power;
  Item cmos_power;
  Item sw_power;
  Item interaction;
  Item bitvector;
  Item getMaxResampleRate;
  Item increment_count;
  Item resampler;

  /**
   * Creates a new <code>Demo</code> instance.
   *
   */
  public Demo (){
    eval = new Evaluator();
  }


  /**
   * Describe <code>rundemo</code> method here.
   *
   * @param root an <code>Item</code> value
   */
  public void rundemo(Item root) {
    // System.out.println("Root Label: " + root.getLabel().toString());

    Facet rootFacet = (Facet) root.getValue();

    // System.out.println("sw_power facet:");
    power = (Item) getItem("power", rootFacet);
    cmos_power = (Item) getItem("cmos_power", rootFacet);
    sw_power = (Item) getItem("sw_power", rootFacet);
    interaction = (Item) getItem("DandP", rootFacet);
    bitvector = (Item) getItem("bitvector", rootFacet);
    getMaxResampleRate = (Item) getItem("getMaxResampleRate", rootFacet);
    increment_count = (Item) getItem("increment_count", rootFacet);

    resampler = (Item) getItem("resampler", rootFacet);

    // Calculate the interactions
    System.out.println("-------------------------------------");
    calculateSoftwarePower();
    System.out.println("\n\n-------------------------------------");
    calculateCMOSPower();
  }

  private void calculateSoftwarePower() {

    // Creating the projected facet sw_resampler
    setExecutionProfile((Facet)sw_power.getValue());

    System.out.println("sw_resampler:");

    Item vcc = getItem("Vcc", (Facet) sw_power.getValue());
    vcc.setValue(new RealValue(3.3));

    ((Facet)sw_power.getValue()).addTermItem(makeInteractionTerm((Facet)sw_power.getValue(), 
								 makeOperators(resampler)));
    
    ((Facet)sw_power.getValue()).addTermItem(getItem("p1", (Facet)
						     power.getValue()));
    System.out.println(sw_power.publishROM());




    
    
    // Evaluating the interacted facet
    System.out.println("====================================");
    System.out.println("SW activity: " + getItem("activity", (Facet)
						 power.getValue()).getValue());
    
    calculatePower(new RealValue(1.0));
  }


  private void calculateCMOSPower() {

    System.out.println("CMOS_resampler:");

    setExecutionProfile((Facet)cmos_power.getValue());
    // Item vcc = getItem("Vcc", (Facet) sw_power.getValue());
    // vcc.setValue(new RealValue(5.0));

    ((Facet)cmos_power.getValue()).addTermItem(makeInteractionTerm((Facet)cmos_power.getValue(), 
								 makeOperators(resampler)));
    
    ((Facet)cmos_power.getValue()).addTermItem(getItem("p1", (Facet)
						     power.getValue()));
    System.out.println(cmos_power.publishROM());
    System.out.println("====================================");
    System.out.println("CMOS activity: " + getItem("activity", (Facet)
						 power.getValue()).getValue());

    calculatePower(new RealValue(1.0));
  }

  
  private void calculatePower(RealValue leakage) {
    Facet powerFacet = (Facet) power.getValue();
    getItem("leakage", powerFacet).setValue(leakage);
    Item powerCalc = getItem("p1", powerFacet);
    Application powerApp = (Application) powerCalc.getValue();
    Application powerFunction = (Application)
      powerApp.getParameters().get(1);
    System.out.println("Power consumed: " + eval.traverse(powerFunction));
  }
    
  /**
   * The <code>makeOperators</code> invokes the "MetaGetOperators"
   * primitive on the facet parameter, and returns the result.
   *
   * @param f a <code>Item</code> value
   * @return an <code>ExpressionList</code> value

  */
  private ExpressionList makeOperators(Item f) {
    /* TODO: This requires that Item be a subclass of Expression,
  which is no longer a valid assumption. See Bug 259
   
    Function function = new MetaGetOperators();
    ExpressionList params = new ExpressionList();
    params.add(f);
    Application app = new Application(function, params);
    return (ExpressionList) eval.traverse(app);
    */
    return new ExpressionList();
  }

  
  private void setExecutionProfile(Facet f) {
    getItem("usage_plus", f).setValue(new
      RealValue(profile.getProperty("usage_plus")));
    getItem("usage_minus", f).setValue(new
      RealValue(profile.getProperty("usage_minus")));
    getItem("usage_not", f).setValue(new
      RealValue(profile.getProperty("usage_not")));
    getItem("usage_equal", f).setValue(new
      RealValue(profile.getProperty("usage_equal")));
    getItem("usage_and", f).setValue(new
      RealValue(profile.getProperty("usage_and")));
  }

  private Item makeInteractionTerm(Facet f, ExpressionList operators) {
    ExpressionList params = new ExpressionList();

    // Make the "Get operators" term.
    Item calcActivityDefinition = getItem("calculateOpActivity", f);
    params.add(operators);
    ItemReference functionRef = new
      ItemReference("calculateOpActivity");
    functionRef.setItemReferred(calcActivityDefinition);

    Application calculateOpActivity = new Application(functionRef, params);
    functionRef.setParent(calculateOpActivity);


    getItem("activity", (Facet)power.getValue()).setValue(evalCalculateOpActivity(calculateOpActivity));

    params = new ExpressionList();
    ItemReference activity = new ItemReference("activity");
    activity.setItemReferred(getItem("activity", (Facet)
				     power.getValue()));

    params.add(activity);
    params.add(calculateOpActivity);
    Application term = new Application(new PrimRealEquals(), params,
				   Application.BINARY);
    return new Item(new Label("discrete_projection"), null, term);
  }


  private RealValue evalCalculateOpActivity(Application app) {
    ItemReference functionRef = (ItemReference) app.getFunction();
    Item functionItem =  functionRef.getItemReferred();
    Function function = (Function) functionItem.getValue();
    IfExpression ifExpr = (IfExpression) function.getExpression();
    // System.out.println(ifExpr);
    Application elseExpr = (Application) ifExpr.getElseValue();
    CaseExpression cond = (CaseExpression) elseExpr.getParameters().get(0);
    // System.out.println("-----------------");
    // System.out.println(cond);
    
    ExpressionList parameters = app.getParameters();
    //ExpressionList values = (ExpressionList) parameters.get(0);
    RealValue result = new RealValue(0.0);
    for (Iterator valueIter = parameters.iterator(); valueIter.hasNext();){
      result.add(evalCase(cond, (Expression) valueIter.next()));
    } // end of for (Iterator valueIter = values.iterator(); valueIter.hasNext();)
    
    //    System.out.println("case result: " + result);
      
    
    return result;
  }
    

  private RealValue evalCase(CaseExpression cond, Expression e) {
    // System.out.println("expression class: " + e.getClass().getName());
    // System.out.println("expression value: " + e);
    
    RealValue result = new RealValue(0.0);

    ExpressionList options = cond.getOptions();
    ExpressionList values = cond.getResults();
    Iterator optionIter = options.iterator();
    Iterator valueIter = values.iterator();
    
    while (optionIter.hasNext()) {
      Expression expr = (Expression) optionIter.next();
      Expression value = (Expression) valueIter.next();
      // System.out.println("option class: " + expr.getClass().getName());
      // System.out.println("option value: " + expr);
      if (("\"" + e.toString() + "\"").equals(expr.toString())) {
	// System.out.println("Matches");
	// System.out.println("Value is " + ((RealValue)value).getPrimitiveValue());
	// System.out.println("Value type: " +
	// value.getClass().getName());
	result.add((RealValue)eval.traverse(value));
      } 
    } 
    
    
    // System.out.println("Result: " + result);
    return result;
  }

  
  /**
   * The <code>getItem</code> method is a utility method for finding
   * in a scope.
   *
   * @param name a <code>String</code> value
   * @param f a <code>Facet</code> value
   * @return an <code>Item</code> value
   */
  private Item getItem(String name, Facet f) {
    Enumeration children = f.children();
    while (children.hasMoreElements()) {
      Item next = (Item) children.nextElement();
      if (next.getLabel().toString().equals(name)) {
	return next;
      } 
    } // end of while (children.hasNextElement())

    return null;
  }    


  public void setupProperties(String filename) {
    profile = new Properties();
    try {
      profile.load(new FileInputStream(filename));
    } catch (Exception e) {
      System.out.println(e);
    } // end of try-catch
    
  }
  
  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args) {
    
    if (args.length != 2) {
      System.out.println("args: " + args.length);
      System.out.println("usage: Demo [sldlfile] [execution profile]");
      
      return;
    } // end of if (args.length != 1)
    


    Demo demo = new Demo();
    demo.setupProperties(args[1]);

    TestParser parser = new TestParser();
    
    System.out.println("Parsing...");
    Object root = parser.compileFile(args[0]);
    //    System.out.println(((BaseObject)root).publishROM());
    
    //    System.out.println("Linking...");
    Linker linker = new Linker(5);
    linker.link(root);
    
  
    demo.rundemo((Item)root);

  } // end of main ()

  
}// Demo
