//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: Evaluator.java,v 1.3 2002/11/15 17:50:41 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: Evaluator.java,v $
// Revision 1.3  2002/11/15 17:50:41  kimmell
// Removed the evaluation of Quantifiers.
// -kimmell
//
// Revision 1.2  2002/07/08 20:52:36  kimmell
// 1. Removed the dependency on the instantiate method of facet when
// evaluating a facet application. This breaks the evaluation of facets,
// and introduces Bug #177.
//
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.5  2002/06/05 16:40:12  kimmell
// Changed the facet instantiation to use the Facet::instantiate() method, rather than a method within Evaluator.
// -kimmell
//
// Revision 1.1.2.4  2002/05/24 21:04:35  kimmell
// 1. Added some debug logging information.
// 2. Re-activated the dynamic lookup of symbols. They will be resolved dynamically if the static analysis did not resolve them.
// -kimmell
//
// Revision 1.1.2.3  2002/05/23 20:32:16  kimmell
// Added some runtime linking code, currently commented out.
// -kimmell
//
// Revision 1.1.2.2  2002/04/17 19:56:03  cvs
// Added the handling of interactions.
// -kimmell
//
// Revision 1.1.2.1  2002/03/21 20:37:13  cvs
// Added an initial implementation of an expression evaluator which uses the Multivisitor class.
// -kimmell
//
package org.sldl.rosetta.tools.evaluator;

import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import org.sldl.rosetta.tools.multivisitor.MultiVisitor;
import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.tools.glinker.Linker;

import org.sldl.rosetta.parser.primitive.PrimitiveValue;
import org.sldl.rosetta.parser.primitive.PrimitiveFunction;

import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.BaseObjectList;
import org.sldl.rosetta.ROM.CaseExpression;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.Interaction;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.QuantifierReference;


import org.sldl.rosetta.lang.types.BooleanType;
import org.sldl.rosetta.lang.values.BooleanValue;
/**
 * Evaluator.java
 *
 *
 * Created: Wed Mar 20 11:38:31 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

public class Evaluator extends MultiVisitor {
  /**
   * Creates a new <code>Evaluator</code> instance.
   *
   */
  public Evaluator (){
    toolName = "EVAL ";
    logLevel = ERROR;
  }
  
  /**
   * Describe <code>process</code> method here.
   *
   * @param val a <code>PrimitiveValue</code> value
   * @return an <code>Object</code> value
   */
  public Object process(PrimitiveValue val) {
	return val;
  }
  
  /**
   * Describe <code>process</code> method here.
   *
   * @param item an <code>Item</code> value
   * @return an <code>Object</code> value
   */
  public Object process(Item item) {
    if (item.getValue() != null) {
      return traverse(item.getValue());
    } else {
      return null;
    } // end of else
    
    
  }    

  /**
   * Describe <code>process</code> method here.
   *
   * @param fun a <code>Function</code> value
   * @return an <code>Object</code> value
   */
  public Object process(Function fun) {
    return fun;
  }

  /**
   * Describe <code>process</code> method here.
   *
   * @param ir an <code>ItemReference</code> value
   * @return an <code>Object</code> value
   */
  public Object process(ItemReference ir) {
    if (ir.getItemReferred() != null) {
      log(DEBUG, "Name linked: " + ir.getItemReferred());
      return traverse(ir.getItemReferred());
    } // end of if (ir.getItemReferred() != null)
    
    try {
      return lookup(ir.getReferenceName(), ir);
    } catch (Exception e) {
      log(ERROR, e.toString());
      log(ERROR, "Error looking up " + ir.getReferenceName());
      return null;
    } // end of try-catch
  }


  /**
   * Describe <code>process</code> method here.
   *
   * @param interaction an <code>Interaction</code> value
   * @return an <code>Object</code> value
   */
  public Object process (Interaction interaction) {
    return interaction;
  }
  
  /**
   * Describe <code>process</code> method here.
   *
   * @param facet a <code>Facet</code> value
   * @return an <code>Object</code> value
   */
  public Object process(Facet facet) {

    for (Iterator termIter = facet.getTermItems().iterator();
	 termIter.hasNext();)
      {
	traverse(termIter.next());
      } 
    return facet;
  }

  /**
   * Describe <code>process</code> method here.
   *
   * @param obj a <code>Package</code> value
   * @return an <code>Object</code> value
   */
  public Object process(Package obj) {
    //System.out.println("Visiting a " + obj.getClass().getName());
    //System.out.println("Value:\n" + obj + "\n\n");
    Object[] children = obj.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i]);
    } // end of for (int i = 0; i < children.length; i++)
    return obj;
  }


  
  /**
   * Describe <code>process</code> method here.
   *
   * @param app an <code>Application</code> value
   * @return an <code>Object</code> value
   */
  public Object process(Application app) {
    log(MESSAGE, "Evaluating " + app.toString());
    Object function =  traverse(app.getFunction());
    // log(MESSAGE, "Type of function is " + function.getClass().getName());
    ExpressionList params = app.getParameters();
    if (function instanceof PrimitiveFunction) {
      log(DEBUG, "Evaluating a primitive function");
      Expression result = (Expression)
	 ((PrimitiveFunction)function).invoke(makePrimitiveParameters(params), this);
      log(MESSAGE, "Result is " + result);
      //log(DEBUG, "Result type is " + result.getClass().getName());
      return result;
    } else if (function instanceof Function) {
      log(DEBUG, "Evaluating a non-primitive function");
      return evalFunction((Function) function, app.getParameters());
    } else if (function instanceof Facet) {
      Facet f = (Facet) function;
      // FIXME: need to instantiate the facet
      return f;
      // return f.instantiate(f.getInterfaceItems(), params);
    } else if (function instanceof Interaction) {
      return evalInteraction((Interaction) function, app.getParameters());
    } // end of else if (function instanceof Interaction)

    else {
	log(MESSAGE, "Evaluating something not a primitive or a function");
	return null;
    } // end of els
  }


  public Object process(CaseExpression caseExpr) {
    log(MESSAGE, "Evaluating a CaseExpression");
    Expression test = caseExpr.getTestExpression();
    ExpressionList values = caseExpr.getOptions();
    ExpressionList results = caseExpr.getResults();
    
    Expression testResult = (Expression) traverse(test);
    log(DEBUG, "testResult is " + testResult.toString());
    Iterator valuesIter = values.iterator();
    Iterator resultsIter = results.iterator();
    Expression value = null;
    Expression result = null;

    while (valuesIter.hasNext()) {
      value = (Expression) traverse(valuesIter.next());
      result = (Expression) resultsIter.next();
      if (value.equals(testResult)) {
	log(DEBUG, "case expression result " + result.toString());
	return traverse(result);
      } 
    } // end of while (valuesIter.hasNext())

    log(DEBUG, "case expression result " + result.toString());
    return traverse(result);
  }


  /**
   * Describe <code>process</code> method here.
   *
   * @param quant a <code>QuantifierReference</code> value
   * @return an <code>Object</code> value
   */
  /*

  // TODO: The following code does not work, as it still relies on
  // BaseObjectList to be an instance of Expression. With the ROM
  // restructuring, this is not the case anymore. See Bug 259.

  public Object process(QuantifierReference quant) {
    if (quant.getReference().equals("sel")) {
      log(MESSAGE, "Evaluating a select statement");
    } // end of if (quant.getReferenceName().equals("sel"))

    // Evaluate each variable to the get range of values. 
    // Pass a list of lists to makeQuantParams to get all of the
    // combinations of input variables. Then, evaluate the expression
    // of the quantifier with each of the possible input variable combinations.
    ItemList params = quant.getVariables();
    ExpressionList paramValues = new ExpressionList();
    for (Iterator paramIter = params.iterator(); paramIter.hasNext();){
       Item theItem = (Item) paramIter.next();
       // log(MESSAGE, "Processing " + theItem);
       // log(MESSAGE, "with type " + theItem.getType());
       Expression paramValue = (Expression) traverse(theItem.getType());
       // log(MESSAGE, theItem + "'s value is " + paramValue);
       if (! (paramValue instanceof BaseObjectList)) {
	 // It is important that each member of the paramValues is a
	 // list for the makeQuantParams function to work correctly.
	 ExpressionList tmp = new ExpressionList();
	 tmp.add(paramValue);
	 paramValues.add(tmp);
       } else {
	 paramValues.add(paramValue);
       } // end of else
       
    } //  of for (Iterator paramIter = params.iterator();
    // paramIter.hasNext();)
    
    ExpressionList paramCombinations = makeQuantParams(paramValues);
    Application app;
    ExpressionList results = new ExpressionList();
    for (Iterator cmbIter = paramCombinations.iterator(); cmbIter.hasNext();){
      ExpressionList current = (ExpressionList)  cmbIter.next();
      Function fun = ((Function) quant.getExpression());
      log(MESSAGE, "Combination: " + current);
      app = new Application(fun,  current);
      BooleanValue appResult = (BooleanValue) traverse(app);
      if (appResult.booleanValue()) {
	  log(MESSAGE, "Selected " + current);
	  results.add(current);
      } else {
	  log(MESSAGE, "Did not select " + current);
      } // end of else
      
    } // end of for (Iterator cmbIter = paramCombinations.iterator(); cmbIter.hasNext();)
    
    for (Iterator resultsIter = results.iterator(); resultsIter.hasNext();){
	System.out.println("Selected: " + resultsIter.next());
    } // end of for (Iterator resultsIter = results.iterator(); resultsIter.hasNext();)
    
    return results;

  }
  
  // TODO: Commented due to bug 259.
  /**
   * Describe <code>makeQuantParams</code> method here.
   *
   * @param params an <code>ExpressionList</code> value
   * @return an <code>ExpressionList</code> value
   */

  /*
  
  public ExpressionList makeQuantParams(ExpressionList params) {
    log(MESSAGE, "Creating quantifier combinations");
    if (params.size() == 0) {
	//log(MESSAGE, "No Params");
      return new ExpressionList();
    } else {
      BaseObjectList head = (BaseObjectList) params.get(0);
      ExpressionList tail = new ExpressionList();

      // cdr would be nice.
      for (int pos = 1; pos < params.size(); pos++) {
	tail.add((Expression) params.getObjectAt(pos));
      } // end of for (int pos = 0; pos < params.size(); pos++)
      
     
      
      // Get a list of the rest params
      ExpressionList restTuples = makeQuantParams(tail);
      ExpressionList myReturn = new ExpressionList();
      ExpressionList tmpList;
      for (Iterator headIter = head.iterator(); headIter.hasNext();){
	Expression currentHead = (Expression) headIter.next();
	if (restTuples.size() == 0) {
	  tmpList = new ExpressionList();
	  tmpList.add(0, currentHead);
	  myReturn.add(tmpList);
	} else {
	
	  for (Iterator restIter = restTuples.iterator();
	       restIter.hasNext();){
	    tmpList = new ExpressionList();
	    tmpList.add((ExpressionList)restIter.next());
	    tmpList.add(0, currentHead);
	    myReturn.add(tmpList);
	  } // end of for (Iterator restIter = restTuples.iterator(); restIter.hasNext();)
	}
      } // end of for (Iterator headIter = head.iterator(); headIter.hasNext();)
	
      return myReturn;
    } // end of else
    }
  */
 




  
  private List makePrimitiveParameters(ExpressionList params) {
    List primParams = new LinkedList();
    for (Iterator paramIter = params.iterator();
	 paramIter.hasNext();){
      Expression item = (Expression) paramIter.next();
      // log(MESSAGE, "Type of expression is " + item.getClass().getName());
      if (item instanceof ItemReference) {
	  primParams.add(((ItemReference)item).getItemReferred());
      } else {
	primParams.add(item);
      }
    } // end of for (Iterator paramIter = params.iterator(); paramIter.hasNext();)
    return primParams;
  }
  
  private Object evalFunction(Function f, ExpressionList actuals) {
    ItemList formals = f.getInterfaceItems();
    ExpressionList curValues = new ExpressionList();
    // Set the parameter values
    Iterator formalIter = formals.iterator();
    Iterator actualIter = actuals.iterator();
    while (formalIter.hasNext() && actualIter.hasNext()) {
      Item formal = (Item) formalIter.next();
      Expression actual = (Expression) actualIter.next();
      curValues.add(formal.getValue());
      formal.setValue(actual);
    }

    Expression result =
      (Expression) traverse(f.getExpression());
    // Clean up the parameters
    formalIter = formals.iterator();
    Iterator savedIter = curValues.iterator();
    while (formalIter.hasNext() && actualIter.hasNext()) {
      ((Item) formalIter.next()).setValue((Expression) savedIter.next());
    } // end of while (formalIter.hasNext() && actualIter.hasNext())
    
    return result;
    
  }

  private Object evalInteraction(Interaction inter, ExpressionList
				 actuals) {
    ItemList formals = inter.getInterfaceItems();
    ExpressionList curValues = new ExpressionList();
    // Set the parameter values
    Iterator formalIter = formals.iterator();
    Iterator actualIter = actuals.iterator();
    while (formalIter.hasNext() && actualIter.hasNext()) {
      Item formal = (Item) formalIter.next();
      Expression actual = (Expression) actualIter.next();
      curValues.add(formal.getValue());
      formal.setValue(actual);
    }

    ItemList interactionTerms = inter.getTermItems();
    Item firstAndSecond = interactionTerms.get(0);
    
    try { 
      // A hack. Fix this.
      Facet first = (Facet) traverse(actuals.get(0));
      //	((ItemReference)
      //	     actuals.get(0)).getItemReferred().getValue();
      Expression newExpression = (Expression) traverse(firstAndSecond);
      Item newTerm = new Item(new Label("first_and_second"), 
					new ItemReference("boolean"), 
			      newExpression);
      first.addTermItem(newTerm);
      log(MESSAGE, "Evaluated interaction: new first:\n" +
	  first.publishROM("    "));
      
      
    } catch (Exception e) {
      log(ERROR, "Error evaluating interaction " + e);
    } // end of try-catch

    Item secondAndFirst = interactionTerms.get(1);
    
    try { 
      // A hack. Fix this.
      Facet second = (Facet) ((ItemReference)
		     actuals.get(1)).getItemReferred().getValue();
      Expression newExpression = (Expression) traverse(secondAndFirst);
      Item newTerm = new Item(new Label("second_and_first"), 
					new ItemReference("boolean"), 
			      newExpression);
      second.addTermItem(newTerm);
      
      log(MESSAGE, "Evaluated interaction: new second:\n " +
	  second.publishROM("    "));
      
    } catch (Exception e) {
      log(ERROR, "Error evaluating interaction " + e);
    } // end of try-catch

    return null;
  }


  private BaseObject lookup(String name, BaseObject target) {
    log(MESSAGE, "looking up " + name);
    if (target == null) {
      log(ERROR, "Target null: Couldn't find reference to " + name);
      return null;
    } else if (target.isLeaf()) {
      log(DEBUG, "Am a leaf" + target.toString());
      target = (BaseObject) target.getParent();
      return lookup(name, target);	 
    } else {
      log(DEBUG, "I have " + target.getChildCount() + " children");
      // This target has children - look through them

      // Break the name into two parts - one for the immediately
      // needed symbol, one for the remaining part (if it exists)
      int delimiter = name.indexOf(".");
      String immediate;
      String rest;
      if (delimiter != -1) {
	immediate = name.substring(0,delimiter);
	rest = name.substring(delimiter + 1, name.length());
      } else {
	immediate = name;
	rest = null;
      } // end of else

      log(DEBUG, "Immediate: " + immediate);
      log(DEBUG, "Rest: " + rest);

      // Iterate through the children
      java.util.Enumeration children = target.children();
      Item parentItem = null;
      log(DEBUG, "target " + target.toString());
      while (children.hasMoreElements()) {
	log(DEBUG, "in while");
	try {
	  parentItem = (Item) children.nextElement();
	  log(DEBUG, "parentItem " + parentItem);
	  if (parentItem.getLabel().toString().equals(immediate)) {
	    log(MESSAGE, "Found " + immediate);
	    break;
	  } else {
	    parentItem = null;
	  } // end of else
	  
	} catch (Exception e) {
	  log(DEBUG, e.toString());
	  parentItem = null;
	} // end of try-catch

      } // end of while (e.hasMoreElements())
      
      log(DEBUG, "After while");
      if ((parentItem != null) && (rest == null)) {
	log(DEBUG, "Return case 1");
	return parentItem.getValue();
      } else if ((parentItem != null) && (rest != null)) {
	log(DEBUG, "Return case 2");
	return lookup(rest, parentItem.getValue());
      } else { // No matching item was found
	// Check the domain
	try {
	  Facet f = (Facet) target;
	  String domainName = ((ItemReference)f.getDomain()).getReferenceName();
	  log(DEBUG, "Looking in domain: " + domainName);  
	  Facet domain = (Facet) lookup(domainName, (BaseObject)target.getParent());

	  BaseObject result = lookup(name, domain);
	  if (result != null) {
	    return result;
	  } // end of if (result == null)
	} catch (ClassCastException e) {
	  log(DEBUG, e.toString());
	} // end of try-catch
	
	log(DEBUG, "Return case 3");
	return lookup(name, (BaseObject) target.getParent());
      } // end of else
      
    }
      
  }	
      
        
  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args) {
    
    if (args.length != 1) {
      System.out.println("usage: Evaluator [sldlfile]");
      return;
    } // end of if (args.length != 1)
    
    Evaluator eval = new Evaluator();
    TestParser parser = new TestParser();
    
    System.out.println("Parsing...");
    Object root = parser.compileFile(args[0]);
    
    //    System.out.println("Linking...");
    Linker linker = new Linker();
    linker.link(root);
    
    System.out.println("Evaluating");
    eval.traverse(root);
    
  } // end of main ()
  
}// Evaluator
