//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: BooleanVariable.java,v 1.2 2002/09/09 15:21:59 cvs Exp $
//
// Revision History:
// ----------------
// $Log: BooleanVariable.java,v $
// Revision 1.2  2002/09/09 15:21:59  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//
package org.sldl.rosetta.tools.generatedata;

import java.lang.*;
import org.sldl.rosetta.ROM.Expression;

/**
 * BooleanVariable.java
 *
 *
 * Created: Fri Feb  8 18:11:51 2002
 *
 * @author <a href="mailto:zwang@kirk.ittc.ku.edu">Zhongjun Wang</a>
 * @version
 */

public class BooleanVariable  {
  
  /**
   * Describe variable <code>name</code> here.
   *
   */
  protected String  name;
  private int value;
  private int negateValue;
  private Expression expression;
  private BooleanVariableComplement complement = null;
  
  /**
   * Creates a new <code>BooleanVariable</code> instance.
   * This is a default constructor.
   */
  public BooleanVariable (){
    super();
    name = "";
    value= 1;   //the default valur of variable.
    negateValue = 0;  // the default value of variable.
    expression = null;
    //complement = new BooleanVariableComplement(this);
  }

  public BooleanVariable(String nameInput){
    name = nameInput;
    value = 1;
    negateValue =0;
    expression = null;
    //complement = new BooleanVariableComplement(this);
  }

  /**
   * Creates a new <code>BooleanVariable</code> instance.
   * This is a primary constructor.
   * @param nameInput a <code>String</code> value
   * @param valueInput an <code>int</code> value
   */
  public BooleanVariable(String nameInput, int valueInput){
    name = nameInput;
    value= valueInput;
    negateValue = 1 - valueInput; 
    //complement = new BooleanVariableComplement(this);
    expression = null;
  }

  /**
   * Creates a new <code>BooleanVariable</code> instance.
   * This is a primary constructor.
   * @param nameInput a <code>String</code> value
   * @param exp an <code>Expression</code> value
   */
  public BooleanVariable(String nameInput, Expression exp){
    super();
    name = nameInput;
    value = 1;
    negateValue = 0;
    expression = exp;
    //complement = new BooleanVariableComplement(this);
  }
    
  /**
   * Creates a new <code>BooleanVariable</code> instance.
   *
   * @param nameInput a <code>String</code> value
   * @param valueInput an <code>int</code> value
   * @param exp an <code>Expression</code> value
   */
  public BooleanVariable(String nameInput, int valueInput, Expression exp){
    name = nameInput;
    value= valueInput;
    negateValue = 1 - valueInput; 
    expression = exp;
    //complement = new BooleanVariableComplement(this);

  }


  /**
   * The <code>getComplement</code> method returns the complement of
   * BooleanVariable. Although the return type is BooleanVariable, it
   * returns BooleanVariableComplement instance,
   * BooleanVariableComplement is a subset of BooleanVariable.
   *
   * @return a <code>BooleanVariable</code> value
   */
  public BooleanVariable getComplement(){
    if (complement == null) {
      complement = new BooleanVariableComplement(this);
    }
    return complement;

  }
  /**
   * The <code>getValue</code> method returns the value of the BooleanVariable.
   * This is a accessor to value of Boolean Variable.
   * @return a <code>Boolean</code> value
   */
  public int getValue(){
    return value;
  }
  /**
   * Describe <code>getName</code> method here.
   * This is accessor to name of Boolean variable.
   * @return a <code>String</code> value
   */
  public String getName(){
    return name;
  }
  /**
   * Describe <code>GetNegateValue</code> method here.
   * This is a accessor to the negate value of Boolean variable.
   * @return a <code>Boolean</code> value
   */
  public int GetNegateValue(){
    return negateValue;
  }
  /**
   * The <code>SetValue</code> method set the value of BooleanVariable
   * to the input value (valueInput).
   * This is a mutator to change the value of Boolean variable.
   * @param valueInput a <code>Boolean</code> value
   */
  public void setValue(int valueInput){
    value = valueInput;
    negateValue = 1 - valueInput;
  }
  /**
   * Describe <code>setName</code> method here.
   * This is a mutator to change the name of Boolean Variable.
   * @param nameInput a <code>String</code> value
   */
  public void setName(String nameInput){
    name = nameInput;
  }
  /**
   * Describe <code>toString</code> method here.
   * This is to return the name to a string.
   * @return a <code>String</code> value
   */
  public String toString(){
    return name;
  }

  /**
   * Describe <code>printExpr</code> method here.
   *
   * @return a <code>String</code> value
   */
  public String printExpr() {
    return expression.toString();
  }

  /**
   * Describe <code>getExpression</code> method here.
   *
   * @return an <code>Expression</code> value
   */
  public Expression getExpression() {
    return expression;
  }
 
  /**
   * Describe <code>setExpression</code> method here.
   *
   * @param exp an <code>Expression</code> value
   */
  public void setExpression(Expression exp) {
    expression = exp;
  }

  /**
   * Describe <code>main</code> method here.
   *
   * @param args[] a <code>String</code> value
   */
  public static void main (String args[])
  {
    BooleanVariable a,b,c;

    a = new BooleanVariable("a");
    b = new BooleanVariable("b");

    System.out.println(" Boolean Variable "+ a.getName()); 
    System.out.println(" Boolean Variable "+ a.getValue());
   // System.out.println(" Boolean Variable "+ b.getName());
   // System.out.println(" Boolean Variable "+ b.getValue());
    
    System.out.println(" Boolean Variable "+ a.getComplement().getValue()); 
	
    a.setValue(0);

     c = a.getComplement();
     
     if ( c instanceof BooleanVariableComplement)
       System.out.println(" success");
     
     a.getComplement().setValue(1);
     if ( a instanceof BooleanVariableComplement)
       System.out.println(" unsuccess");
     
     System.out.println(" Boolean Variable "+ a.getValue());
     System.out.println(" Boolean Variable "+ a.getComplement().getValue()); 
     a.setName("a2");
     a.setValue(1);
     b.setName("b2");
     b.setValue(1);

  }

    
}// BooleanVariable
