//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: BooleanVariableComplement.java,v 1.5 2002/10/22 20:17:55 cvs Exp $
//
// Revision History:
// ----------------
// $Log: BooleanVariableComplement.java,v $
// Revision 1.5  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.4  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//
package org.sldl.rosetta.tools.generatedata;

import java.util.*;
import java.lang.*;

import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Application;

// The following implementation for BooleanVariableComplement class
// has been discussed with Garrin

/**
 * BooleanVariableComplement.java
 *
 *
 * Created: Wed Feb 13 18:08:19 2002
 *
 * @author <a href="mailto:zwang@kirk.ittc.ku.edu">Zhongjun Wang</a>
 * @version
 */

public class BooleanVariableComplement extends BooleanVariable {

  private BooleanVariable variable;

  /**
   * Creates a new <code>BooleanVariableComplement</code> instance.
   *
   */
  public BooleanVariableComplement (){
    super();
  }
  
  /**
   * Creates a new <code>BooleanVariableComplement</code> instance.
   *
   * @param variableInput a <code>BooleanVariable</code> value
   */
  public BooleanVariableComplement(BooleanVariable variableInput){
    super();
    variable = variableInput;
  }
  /**
   * The <code>getValue</code> method returns the negate value of
   * original BooleanVariable.
   *
   * @return an <code>int</code> value
   */
  public int getValue(){
    return (1 - variable.getValue());
  }
  /**
   * The <code>setValue</code> method set the value of original
   * BooleanVariable as the negate of input.
   *
   * @param valueInput an <code>int</code> value
   */
  public void setValue(int valueInput){
    variable.setValue(1-valueInput);
  }
  /**
   * The <code>getComplement</code> method return the original BooleanVariable.
   *
   * @return a <code>BooleanVariable</code> value
   */
  public BooleanVariable getComplement()
  {
    return variable;
  }

  public Expression getExpression() {

    Expression expr = variable.getExpression();
    
    Expression newNegApp = null;
    Application negApp = (Application)expr;
    String Operator = negApp.getFunction().toString();
    //System.out.println("\nThe Operator is: "+Operator);
    ExpressionList parameters = negApp.getParameters();
    if (Operator.equals("<")) {
      newNegApp = new Application(new ItemReference(">="),parameters,Application.BINARY);
    } // end of if (Operator.equals("<"))
    else if (Operator.equals("=<")) {
      newNegApp = new Application(new ItemReference(">"),parameters,Application.BINARY); 
    } // end of if (Operator.equals("=<"))
    else if (Operator.equals(">")) {
      newNegApp = new Application(new ItemReference("=<"),parameters,Application.BINARY); 
    } 
    else if (Operator.equals(">=")) {
      newNegApp = new Application(new ItemReference("<"),parameters,Application.BINARY); 
    }
    else if (Operator.equals("=")) {
      newNegApp = new Application(new ItemReference("/="),parameters,Application.BINARY); 
      
    }
    else if (Operator.equals("/=")) {
      newNegApp = new Application(new ItemReference("="),parameters,Application.BINARY); 
    } 
    else if (Operator.equals("==")) {
      newNegApp = new Application(new ItemReference("/="),parameters,Application.BINARY); 
    } 
    else if (Operator.equals("not")) {
      newNegApp = parameters.get(0);
    } else {
      ExpressionList newParams = new ExpressionList();
      newParams.add(expr);
      newNegApp = new Application(new ItemReference("not"), newParams,
				  Application.PREFIX);
    }
    return newNegApp;

    //    return variable.getExpression();  // need to negate it
    // return new Application(new ItemReference("not"), variable.getExpression(), Application.PREFIX);
  }

  public String toString() {
    return "/" + variable.getName();
  }

  public String printExpr() {
    return getExpression().toString();
  }
  
}// BooleanVariableComplement

