//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
// Revision History:
// ----------------
// $Log: CostFunction.java,v $
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.LinkedList;
import java.util.Iterator;
import java.lang.reflect.Array;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;

/**
 * Describe class <code>CostFunction</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Satyanarayana Kakarlamudi</a>
 * @version 1.0
 */
public class CostFunction implements SimplexFunc {
  
  private LinkedList independentVarList;
  private LinkedList costFncElmList;
  private LinkedList boolVarList;
  private double arg[];
  private double functionValue;

  /**
   * Creates a new <code>CostFunction</code> instance.
   * The argument stores all the independent Boolean 
   * Variables.
   * @param boolVarListIn a <code>LinkedList</code> value
   */
  public CostFunction(LinkedList boolVarListIn) {
    boolVarList = boolVarListIn;
    independentVarList = new LinkedList();
    costFncElmList = new LinkedList();
  }

  /**
   * Describe <code>getArguments</code> method here.
   *
   * @return a <code>double[]</code> value
   */
  public double[] getArguments() {
    return arg;
  }

  /**
   * Describe <code>generateIndptVarList</code> method here.
   * This method is used to calculate the independent variables 
   * in the application.All the independent variables are stored
   * in the LinkedList independentVarList.
   */
  public void generateIndptVarList() {
    for (Iterator boolVarListIter = boolVarList.iterator();boolVarListIter.hasNext();) {
      Application app = (Application)boolVarListIter.next();
      ExpressionList paramList = app.getParameters();
      Expression leftExp = paramList.get(0);
      Expression rightExp = paramList.get(1);
      String leftString = leftExp.toString();
      String rightString = rightExp.toString();

      if (!(independentVarList.contains(leftString))) {
	independentVarList.add(leftString);
      } // end of if (!(independentVarList.contains(leftExp))
	  
      if (!(rightExp instanceof PrimitiveValue)) {
	// Here in b' = b both b' and b are treated 
	//as independent variables.

	independentVarList.add(rightString);
      } // end of if (!(rightExp instanceof DataLiteral))
    } // end of for (Iterator boolVarListIter = boolVarList.iterator();boolVarListIter.hasNext();)
  }



  /**
   * Describe <code>costFunc</code> method here.
   * This is the method in which the CostFunction gets
   * generated.This method is called repeatedly by the 
   * Simplex Algorithm.This method checks the operator in the 
   * Application and transfers the control to the corresponding
   * method.
   * @param x a <code>double[]</code> value
   * @return a <code>double</code> value
   */
  public double costFunc(double[] x) {
    arg =x;
    double f = 0;
    for (Iterator boolIter = boolVarList.iterator();boolIter.hasNext();){
      Application boolVar = (Application)boolIter.next();
      if (boolVar.getFunction().toString().equals("<") || boolVar.getFunction().toString().equals("=<")) {
	double lessExp = this.lessFunction(boolVar);
	f=f+lessExp;
      } // end of if (boolVar.getFunction().toString().equals("<") || boolVar.getFunction().toString().equals("<="))
      if (boolVar.getFunction().toString().equals(">") || boolVar.getFunction().toString().equals(">=")) {
	double greatExp = this.greatFunction(boolVar);
	f=f+greatExp;
      }
      if (boolVar.getFunction().toString().equals("=")){
	double equalExp = this.equalFunction(boolVar);
	f=f+equalExp;
      }
      if (boolVar.getFunction().toString().equals("/=")){
	double notEqualExp = this.notEqualFunction(boolVar);
	f=f+notEqualExp;
      }
    }
    return f;
  }
  
  /**
   * Describe <code>lessFunction</code> method here.
   * Example: if we have an Application of form 
   * x < 1.0 then the cost function should be
   * exp(1.0 - x).Based on this, the implementation is 
   * done.
   * @param lessApp an <code>Application</code> value
    * @return a <code>double</code> value
   */
  public  double lessFunction(Application lessApp) {
    double d = 0;
    ExpressionList paramsList = lessApp.getParameters();
    String leftArgStr = paramsList.get(0).toString();
    String rightArgStr = paramsList.get(1).toString();
    if (independentVarList.contains(rightArgStr)) {
      d = d + arg[independentVarList.indexOf(rightArgStr)];
    } // end of if (independentVarList.contains(rightArgStr))
    else {
      Double rightDouble = new Double(rightArgStr);
      d = d + rightDouble.doubleValue();
    } // end of else
    d = d - arg[independentVarList.indexOf(leftArgStr)];
    return Math.exp(-d);
  }


  /**
   * Describe <code>greatFunction</code> method here.
   * Example: if the application is: x > 10 then the 
   * cost function is exp(x - 10).
   * @param greatApp an <code>Application</code> value
   * @return a <code>double</code> value
   */
  public double greatFunction(Application greatApp) {
    double d = 0;
    ExpressionList paramsList = greatApp.getParameters();
    String leftArgStr = paramsList.get(0).toString();
    String rightArgStr = paramsList.get(1).toString();
    d = d + arg[independentVarList.indexOf(leftArgStr)];
    if (independentVarList.contains(rightArgStr)) {
      d = d - arg[independentVarList.indexOf(rightArgStr)];
    } // end of if (independentVarList.contains(rightArgStr))
    else {
      Double rightDouble = new Double(rightArgStr);
      d = d - rightDouble.doubleValue();
    } // end of else
    return Math.exp(-d);
  }

  /**
   * Describe <code>equalFunction</code> method here.
   * Example: if the application is: x = 1.0 
   * The cost function is exp(abs(x - 1.0)).
   * @param equalApp an <code>Application</code> value
   * @return a <code>double</code> value
   */
  public double equalFunction(Application equalApp) {
    double d = 0;
    ExpressionList paramsList = equalApp.getParameters();
    String leftArgStr = paramsList.get(0).toString();
    String rightArgStr = paramsList.get(1).toString();
    d = d + arg[independentVarList.indexOf(leftArgStr)];
    if (independentVarList.contains(rightArgStr)) {
      d = d - arg[independentVarList.indexOf(rightArgStr)];
    } // end of if (independentVarList.contains(rightArgStr))
    else {
      Double rightDouble = new Double(rightArgStr);
      d = d - rightDouble.doubleValue();
    } // end of else
    return Math.exp(Math.abs(d));
  }

  /**
   * Describe <code>notEqualFunction</code> method here.
   * if the application is: x /= 1.0 
   * then the cost function is: exp(-abs(x - 1.0)).
   * @param notEqualApp an <code>Application</code> value
   * @return a <code>double</code> value
   */
  public double notEqualFunction(Application notEqualApp) {
    double d = 0;
    ExpressionList paramsList = notEqualApp.getParameters();
    String leftArgStr = paramsList.get(0).toString();
    String rightArgStr = paramsList.get(1).toString();
    d = d + arg[independentVarList.indexOf(leftArgStr)];
    if (independentVarList.contains(rightArgStr)) {
      d = d - arg[independentVarList.indexOf(rightArgStr)];
    } // end of if (independentVarList.contains(rightArgStr))
    else {
      Double rightDouble = new Double(rightArgStr);
      d = d - rightDouble.doubleValue();
    } // end of else
    return Math.exp(-Math.abs(d));
  }

  /**
   * Describe <code>getIndptVarListSize</code> method here.
   *
   * @return an <code>int</code> value
   */
  public int getIndptVarListSize() {
    return independentVarList.size();
  }

  /**
   * Describe <code>getIndptVarList</code> method here.
   * This function is used to get the independent variables 
   * in  a term.
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getIndptVarList() {
    return independentVarList;
  }


//    public void accessCostFnc() {
//      for (Iterator costIter = costFncElmList.iterator();costIter.hasNext();) {
//        System.out.print(costIter.next());
//        if (costIter.hasNext()) {
//  	System.out.print(" + ");
	
//        } // end of if (costIter.hasNext())
      
//      } // end of for (Iterator costIter = costFncElmList.iterator();costIter.hasNext();)
//    }
}    
