//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
// Revision History:
// ----------------
// $Log: CostFunctionGenerator.java,v $
// Revision 1.3  2002/11/06 18:33:20  ksatya
// Implemented the functionality of generating XML format for Optimized True Points
// --Murthy.
//
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.lang.reflect.Array;

/**
 * Describe class <code>CostFunctionGenerator</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Satyanarayana Kakarlamudi</a>
 * @version 1.0
 */
public class CostFunctionGenerator {

  private FileInputStream fileInStream;
  private LinkedList falsePointsList;
  private BooleanTerm boolTerm;
  private LinkedList falseVarList;
  private byte b[];
  private int index = 1;

  /**
   * Creates a new <code>CostFunctionGenerator</code> instance.
   *
   * @param fileNameIn a <code>String</code> value
   * @exception FileNotFoundException if an error occurs
   * @exception IOException if an error occurs
   */
  public CostFunctionGenerator(String fileNameIn) throws FileNotFoundException,IOException {
    fileInStream = new FileInputStream(fileNameIn);
  }

  /**
   * Creates a new <code>CostFunctionGenerator</code> instance.
   *
   * @param falsePointsListIn a <code>LinkedList</code> value
   */
  public CostFunctionGenerator(LinkedList falsePointsListIn){
    falsePointsList = falsePointsListIn;
  }

  /**
   * Creates a new <code>CostFunctionGenerator</code> instance.
   *
   * @param boolTermIn a <code>BooleanTerm</code> value
   */
  public CostFunctionGenerator(BooleanTerm boolTermIn) {
    boolTerm = boolTermIn;
  }


  /**
   * Describe <code>createCostFnc</code> method here.
   * This is the method from where you will be calling Class 
   * CostFunction to build the cost function.This method will
   * iter through all the false points and generate the optimised 
   * value for each false point.
   */
  public int createCostFnc(String fileNameIn,int countIn) throws FileNotFoundException {
    //int i=1;
    DnfExpression scenDnf = new DnfExpression("falsePoint",boolTerm.getBooleanVarList());
    scenDnf.add(boolTerm);
    scenDnf.nearFalsePoints();
    LinkedList falsePointsList = scenDnf.getFalsePointExpr(); 
    for (Iterator falseTermIter = falsePointsList.iterator();falseTermIter.hasNext();) {
      CostFunction cstFnc = new CostFunction((LinkedList)falseTermIter.next());
      cstFnc.generateIndptVarList();
      double function  = 0;
      double[] args = new double[cstFnc.getIndptVarListSize()];
      cstFnc.costFunc(args);
      double[] params = cstFnc.getArguments();
      double[] dx = new double[cstFnc.getIndptVarListSize()];
      for (int count = 0; count < Array.getLength(params);count++) {
	params[count] = 1.0;
	dx[count] = 0.1;
      } // end of for (int count = 0; count < Array.getLength(params);count++)
      double sigma = -1;
      int mm = 1000;
      int n = cstFnc.getIndptVarListSize();
      FileOutputStream optOut = new FileOutputStream(fileNameIn+countIn+".xml");
      PrintWriter pwOpt = new PrintWriter(optOut,true);
      pwOpt.println("<optvalues>");
      double fmin = SimplexMethod.SmplxOne(n,params,cstFnc,dx,sigma,mm);
      pwOpt.println("\t<function>"+fmin+"</function>");
      //pwOpt.println("The minimum value of the fnc is: "+ fmin);
      LinkedList indptVarList = cstFnc.getIndptVarList();
      Iterator indptIter = indptVarList.iterator();
      for (int count = 0; count < Array.getLength(params);count++) {
	if (indptIter.hasNext()) {
	  String variable = indptIter.next().toString();
	  //pwOpt.println("The independent value "+ variable+" is: "+params[count]);
	  pwOpt.println("\t<parameter name = \""+variable+"\">");
	  pwOpt.println("\t\t<value>"+params[count]+"</value>");
	  pwOpt.println("\t</parameter>");
	}
      }
      countIn++;
      pwOpt.println("</optvalues>");
    }

    return countIn;
  }


}// Ending brace of the class.

  
