//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
// Revision History:
// ----------------
// $Log: DnfExpression.java,v $
// Revision 1.5  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.4  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.*;
import java.lang.Number;

/**
 * DnfExpression.java
 *
 *
 * Created: Mon Feb 11 12:12:47 2002
 *
 * @author <a href="mailto:zwang@kirk.ittc.ku.edu">Zhongjun Wang</a>
 * @version
 */

public class DnfExpression extends LinkedList {

  private String dnfName;

  //the boolean variables used in Dnf expression 
  private LinkedList symbolsList; 

  // the linked list to store the unique true points and near false
  // points of the Dnf expression.

  private LinkedList truePointExpr = new LinkedList();
  private LinkedList falsePointExpr = new LinkedList();

  private LinkedList resultPoint = new LinkedList();
  /**
   * Creates a new <code>DnfExpression</code> instance.
   *
   */
  public DnfExpression (){
    super();
    dnfName = "";
    symbolsList = null;
  }
  /**
   * Creates a new <code>DnfExpression</code> instance. Two input
   * parameters are needed: the name of the DNF expression and a
   * LinkedList of Booleanvariable(symbols) used in the DNF expression.
   * 
   * @param nameInput a <code>String</code> value
   * @param symbolsInput a <code>LinkedList</code> value
   */
  public DnfExpression (String nameInput, LinkedList symbolsInput){
    super();
    dnfName = nameInput;
    symbolsList = symbolsInput;
    
  }

  /**
   * Describe <code>getTruePointExpr</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getTruePointExpr(){
    return truePointExpr;
  }

  /**
   * Describe <code>getFalsePointExpr</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getFalsePointExpr(){
    return falsePointExpr;
  }

  /**
   * The <code>addTerm</code> method adds a BooleanTerm instance to
   * the DNF expression. It overwites the add method in LinkedList class.
   *
   * @param termInput a <code>BooleanTerm</code> value
   */
  public void addTerm (BooleanTerm termInput)
  {
    add(termInput);
  }
  /**
   * The <code>getValue</code> method returns the logic vaule of the
   * DNF expression. If it is true, 1 is returned. Otherwise, 0 is returned.
   *
   * @return an <code>int</code> value
   */
  public int getValue(){
    int value = 0;
    if ( size() != 0)
      {
	for (int i = 0; i < size(); i++)
	  {
	    if ( ( (BooleanTerm)get(i) ).getValue() == 1)
	      {
		value = 1;
	      }
	  }
	return value;
      }
    else
      return value = 88;
  }
  /**
   * The <code>isTrue</code> method check if the DNF expression is
   * true or false. If it is true, true is returned, otherwise, false
   * is returned.
   *
   * @return a <code>boolean</code> value
   */
  public boolean isTrue(){
    int value = 0;

    if ( size() !=0 )
      value = getValue();

    if( (value == 1) && (size() !=0))
      return true;
    else
      return false;
  }
  /**
   * The <code>getDnfName</code> method returns the name of the DNF expression.
   *
   * @return a <code>String</code> value
   */
  public String getDnfName(){
    return dnfName;
  }

  /**
   * The <code>setDnfName</code> method sets the name of the DNF expression.
   *
   * @param nameInput a <code>String</code> value
   */
  public void setDnfName(String nameInput){
    dnfName = nameInput;
  }

  /**
   * Describe <code>getSymbolsList</code> method here.
   *
   * @return a <code>List</code> value
   */
  public List getSymbolsList() {
    return symbolsList;
  }

  /**
   * Describe <code>toString</code> method here.
   *
   * @return a <code>String</code> value
   */
  public String toString(){
    String str = "";
    boolean first = true;
    for(Iterator iter = iterator(); iter.hasNext();) {
      str = str + ( first ? "" : " + ");
      first = false;
      str = str + iter.next().toString();
    }
    return str;
  }

  /**
   * Describe <code>printExpr</code> method here.
   *
   * @return a <code>String</code> value
   */
  public String printExpr() {
    String str = "";
    boolean first = true;
    for(Iterator iter = iterator(); iter.hasNext();) {
      str = str + (first ? "" : " or ");
      first = false;
      str = str + "(" + ((BooleanTerm)iter.next()).printExpr() + ")";
    }
    return str;
  }

  /**
   * Describe <code>propogate</code> method here.
   *
   * @param var a <code>BooleanVariable</code> value
   */
  public void propogate(BooleanVariable var) {
    for (Iterator iter = iterator(); iter.hasNext();){
      BooleanTerm term = (BooleanTerm) iter.next();
      term.add(0, var);
    } // end of for (Iterator iter = iterator.iterator(); iter.hasNext();)
  }

  /**
   * Describe <code>propogate</code> method here.
   *
   * @param paramTerm a <code>BooleanTerm</code> value
   */
  public void propogate(BooleanTerm paramTerm) {
    for (Iterator iter = iterator(); iter.hasNext();){
      BooleanTerm term = (BooleanTerm) iter.next();
      for (Iterator termIter = paramTerm.iterator(); termIter.hasNext();){
	BooleanVariable var = (BooleanVariable) termIter.next();
	term.add(0, var);
      } 
    } // end of for (Iterator iter = iterator.iterator(); iter.hasNext();)
  }

  /**
   * THe <code>uniqueTruePoints</code> method calcultes the unique
   * true points of the DNF expression.
   *
   */
  public void uniqueTruePoints(){
    int numSymbols = symbolsList.size();
    int numTerms   = size();

    int choice = 1;

    // processing the true term
    for ( int i = 0; i < numTerms; i++)
      {
	//System.out.println(" processing term "+ i);

	// this is for the true term
	BooleanTerm termi =(BooleanTerm)get(i);
	
	desiredTruePoints(termi,i,choice);
      }
  }
  /**
   * The <code>desiredTruePoints</code> method calculates the Unique
   * True Points of i-th term in the Dnf expression. This i-th term
   * has to be true and other terms have to be false.
   *
   * @param termi a <code>BooleanTerm</code> value
   * @param indexOfTermi an <code>int</code> value
   * @param choice an <code>int</code> value, if choice =1, the unique
   * true points are calculated, otherwise, near false points are calculated
   */
  public void desiredTruePoints(BooleanTerm termi, int indexOfTermi,
				int choice){


    int numSymbols = symbolsList.size(); //number of boolean variables
    int numTerms   = size(); // number of terms in Dnf expresion.
    int sizeOfTermi = termi.size();// number of variables in i-th term

    // dealing with each variable in the i-th term.
    for (int j = 0; j < sizeOfTermi; j++)
      {
	BooleanVariable variable = (BooleanVariable)termi.get(j); 
	
	// Every variable in i-th term has to be true, such that the
	// whole term will be true.
	// If the variable is in its complement form in i-th term, the following
	// setValue method will assign 1(true) to its complement form, and
	// automatically, change the variable value to 0 in its
	// original form. That is to say, the setValue method in
	// class BooleanVariableComplement is called.
	variable.setValue(1);
      }

    // The number of boolean variables that are not in i-th term(true term).
    int sizeOutTermi = numSymbols - sizeOfTermi;

    // There are about 2**n group assignemnts to variables outside
    // the true term, where n the number of variables not in the
    // true term. Actually, this is a 3-sat problem.
    
    int n = (int)Math.pow(2, sizeOutTermi);
    for ( int k = 0;  k < n ; k++)
      {
	int kuse = k;
	for ( int m = 0; m < numSymbols; m++) 
	  {
	    BooleanVariable variableOut = (BooleanVariable)symbolsList.get(m);
	    
	    //symbols(variables) in true term will not be processed
	    if ( (!termi.contains(variableOut)) &&
		 (!termi.contains(variableOut.getComplement()) ) )
	      {
		// the binary expression of n is used to assign 0 or 1
		// to thoses variables out side i-th term(true term). 
		//the last digit of k is assigned to bit 
		int bit = (int)(kuse & 1); 
		variableOut.setValue(bit);
		kuse = kuse >> 1;  // shift k to rigth by 1 bit.
	      }
	  }
	
	// test if the current assignment will give other terms
	// false values, if it is, output this set of values. it
	// is one unique true point(near false point) of the DNF expression.
	// Otherwise, it is not one of the test points.
	int value = 0;
	for ( int m = 0; m < numTerms; m++)
	  {  
	    if ( m != indexOfTermi )   //the true term will not be evaluated.
	      {
		BooleanTerm termm = (BooleanTerm)get(m);
		value = value + termm.getValue();
	      }
	  }
	
	//
	// the following linked list will store one test scenario or
	// test point
	//
	LinkedList testScenario = new LinkedList();
	LinkedList testPoint    = new LinkedList();

	if (value == 0 ) //output the test point of term i.
	  {
	    for (int m = 0; m < numSymbols; m++)
	      {
		BooleanVariable tempVariable = (BooleanVariable)symbolsList.get(m);
		int valueFound = tempVariable.getValue();

		//
		// if the unique true points or near false points have
		// already existed, they will not be added into the
		// corresponding lists
		//
		//		if ( (choice == 1) and truePointExpr.contains(
		if (valueFound == 1)
		  {   
		    testScenario.add(tempVariable.getExpression());
		 		    // else //(choice ==2) near false point
		    //testScenario.add(tempVariable.getExpression());
		  }
		else //valueFoud =0
		  {
		    //  		    if (choice ==1)
		    testScenario.add( (tempVariable.getComplement()).getExpression());
		    //testScenario.add(tempVariable.getExpression());
		      // else // choice ==2  near false point
  		      //falsePointExpr.add(
		      //		 (tempVariable.getComplement()).getExpression());
  		  }
		
		testPoint.add(new Integer(valueFound));
		//	System.out.print("   "+ valueFound);
	      }	

	    System.out.println("");
	    //
	    // store the test point into the corresponding true or
	    // false linked list
	    //
	    if ( (choice == 1) &&
	      (!(resultPoint.contains(testPoint))))
	      //        !(truePointExpr.contains(testScenario)) )
	      {
		truePointExpr.add(testScenario);
		
	      }
	    else if ( (choice == 2) &&
		      (!(resultPoint.contains(testPoint))))
	      //     (!(falsePointExpr.contains(testScenario)) ))
	      {
		//System.out.print(" choice= "+ choice+"  ");
		//System.out.println(" inside = "+
		//	   falsePointExpr.contains(testScenario)+"  "+
		//	   resultPoint.contains(testPoint));

		resultPoint.add(testPoint);
		falsePointExpr.add(testScenario);
		for (int ii=0; ii<testPoint.size();ii++)
		  {
		    //System.out.print(testPoint.get(ii).toString()+"  ");
		  }
		//System.out.println();
		for (int ii=0; ii<testPoint.size();ii++)
		  {
		    //System.out.print(testScenario.get(ii).toString()+"  ");
		  }

	      }
	    else
	      System.out.print("");

	  }
      }
  }
  
  /**
   * The <code>nearFalsePoints</code> method calculates the all near
   * false points of a disjunctive normal form boolean expression.
   *
   */
  public void nearFalsePoints(){

    int choice = 2;
    // number of boolean variables in the Dnf expression.
    int numSymbols = symbolsList.size();
    // number of terms in the Dnf expression.
    int numTerms   = size();    
    //System.out.println("The number of boolean terms are: "+numTerms);

    for (int i = 0; i < numTerms; i++)
      {
	// processing the i-th term in the Dnf expression.
	BooleanTerm termi = (BooleanTerm)get(i);
	// Number of boolean variables in the i-th term.
	int sizeOfTermi = termi.size();
	//System.out.println("The number of boolean variables are: "+sizeOfTermi);

	// dealing with each symbol in the i-th term of Dnf expression.
	for (int j = 0; j < sizeOfTermi; j++)
	  {
	    // the j-th boolean variable in the i-th term.
               
	    BooleanVariable variable = ((BooleanVariable)termi.get(j));

	    // set the j-th variable to its complement from
	    termi.set(j, variable.getComplement());

	    // calculate the unique true points of the modified Dnf
	    // expression for this particular j-th term.
	    desiredTruePoints(termi,i,choice);

	    // resume the original Dnf form by set the j-th variable
	    // to its original form.
	    termi.set(j, variable);
	  }
      }
  }
  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args){
    BooleanVariable a,b,c,d,e;
    a = new BooleanVariable("a",0);
    b = new BooleanVariable("b",0);
    c = new BooleanVariable("c",0);
    d = new BooleanVariable("d",0);
    //e = new BooleanVariable("e",1);

    LinkedList symbols;

    symbols = new LinkedList();

    symbols.add(a);
    symbols.add(b);
    symbols.add(c);
    symbols.add(d);
    //symbols.add(e);

    BooleanTerm abc,cd,ee;
   
    abc = new BooleanTerm ("abc");
    cd  = new BooleanTerm ("cd");
    ee  = new BooleanTerm ("ee");

    abc.add(a);
    abc.add(b.getComplement());
    abc.add(d);
   
    //abc.add(a.getComplement());
    

    cd.add(c.getComplement());
    cd.add(d);

    //cd.add(a);
    //cd.add(b.getComplement());

    //    ee.add(e);
   
    ee.add(a);
    ee.add(b);
    ee.add(c.getComplement());

    DnfExpression f;
    
    f = new DnfExpression("f1",symbols);

    f.add(abc);
    f.add(cd);
    // f.add(ee);

    //System.out.println(" c's value befor "+ c.getValue());

    //System.out.println(" Unique True Points ");
    f.uniqueTruePoints();
    
    //System.out.println(" Near False Points ");
    f.nearFalsePoints();

    //System.out.println(" c is instance of BooleanVariableComplement "+
    //(c instanceof BooleanVariableComplement) );

  }
  
}// DnfExpression
