//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: FalsePointsGenerator.java,v 1.13 2002/11/11 13:15:59 ksatya Exp $
// Revision History:
// ----------------
// $Log: FalsePointsGenerator.java,v $
// Revision 1.13  2002/11/11 13:15:59  ksatya
// Bugs:
// -- Standard Declarations within a Package are having problems.
// -- Nested Facets are not supported till now.
// Those bugs were fixed.
// --Murthy.
//
// Revision 1.10  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.9  2002/10/17 04:27:47  cvs
// Made minor changes to support all the examples.
// --Murthy.
//
// Revision 1.8  2002/10/10 17:10:19  cvs
// Implemented the functionality of evaluating expressions using Jython.
// --Murthy.
//
// Revision 1.7  2002/10/09 06:05:59  cvs
// Designed a new DTD for storing the False points in XML format and generated the False points in that format.
// --Murthy.
//
// Revision 1.5  2002/10/08 14:33:55  cvs
// Made the tools compatable to generate the false points in XML format.Added a new file ValueGenerator.java to generate the vectors.This file implements a new logic as compared to the logic in GenerateVectors class.
// --Murthy.
//
// Revision 1.4  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.LinkedList;
import java.util.HashMap;
import java.util.Set;
import java.util.Map;
import java.io.FileOutputStream;
import java.io.File;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.util.Iterator;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.lang.values.*;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;

/**
 * Describe class <code>FalsePointsGenerator</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Satyanarayana Kakarlamudi</a>
 * @version 1.0
 */
public class FalsePointsGenerator {

  private LinkedList originalScenarios;
  private LinkedList finalScenarios;

  private LinkedList falsePoints;
  
  //This variable is used to store the requirements 
  //which will be used to generate the xml format for 
  //False Points.
  private ReadRequirements requirements;
  private ReadScenarios scenarios;
  /**
   * Describe variable <code>destDirectory</code> here.
   * This String stores the directory name where the
   * FalsePoints and the OptimumValues get stored.
   */
  private String destDirectory;

  private LinkedList outputCondition;

  /**
   * Creates a new <code>FalsePointsGenerator</code> instance.
   * The primary constructor.
   * @param origScenIn a <code>LinkedList</code> value
   */
  public FalsePointsGenerator(LinkedList origScenIn) {
    originalScenarios = origScenIn;
    falsePoints = new LinkedList();
    finalScenarios = new LinkedList();
  }

  /**
   * Describe <code>setDirectoryName</code> method here.
   * This Method is used to set the destDirectory name.
   * @param directoryNameIn a <code>String</code> value
   */
  public void setDirectoryName(String directoryNameIn) {
    destDirectory = directoryNameIn;
  }

  /**
   * Describe <code>setRequirements</code> method here.
   *
   * @param reqIn a <code>ReadRequirements</code> value
   * @param scenIn a <code>ReadScenarios</code> value
   */
  public void setRequirements(ReadRequirements reqIn,ReadScenarios scenIn) {
    requirements = reqIn;
    scenarios = scenIn;
  }

  /**
   * Describe <code>getRequirements</code> method here.
   *
   * @return a <code>ReadRequirements</code> value
   */
  public ReadRequirements getRequirements() {
    return requirements;
  }

  /**
   * Describe <code>setFinalScenarios</code> method here.
   *
   * @param scenIn a <code>LinkedList</code> value
   */
  public void setFinalScenarios(LinkedList scenIn) {
    finalScenarios = scenIn;
    //  System.out.println("############################################");
//      System.out.println("in false point generator");
//      for (Iterator finalIter = finalScenarios.iterator(); finalIter.hasNext();) {
//        System.out.println(finalIter.next());
//      } // end of for (Iterator finalIter = finalScenarios.iterator(); finalIter.hasNext();)
//      System.out.println("############################################");
  }

  
 public void generateOutputCondition(){
    outputCondition = new LinkedList();
    LinkedList outputVariables = scenarios.getOutputVar();
    
    LinkedList finalScenarios = scenarios.getFinalScenariosList();
    for (Iterator finalScenIter = finalScenarios.iterator(); finalScenIter.hasNext();) {
      LinkedList innerList = (LinkedList)finalScenIter.next();
      
      for (Iterator innerScenIter = innerList.iterator(); innerScenIter.hasNext();) {
	Application app = (Application)innerScenIter.next();
	String leftOp = ((ExpressionList)app.getParameters()).get(0).toString();
	
	for (Iterator optVarIter = outputVariables.iterator(); optVarIter.hasNext();) {
	  String opt_var = optVarIter.next().toString();
	  if (leftOp.equalsIgnoreCase(opt_var)) {
	    outputCondition.add(app);
	  } // end of if (leftOp.equalsIgnoreCase("opt_var"))
	  
	} // end of for (Iterator optVarIter = outputVariables.iterator(); optVarIter.hasNext();)
	
      } // end of for (Iterator innerScenIter = innerList.iterator(); innerScenIter.hasNext())
      
    } // end of for (Iterator finalScenIter = finalScenarios.iterator(); finalScenIter.next();)
    //  System.out.println("#########################################################");
//      System.out.println("Testing the output conditions");
//      for (Iterator condIter = outputCondition.iterator(); condIter.hasNext();) {
//        System.out.println(condIter.next());
//      } // end of for (Iterator condIter = outputCondition.iterator(); condIter.hasNext();)
//      System.out.println("#########################################################");
    
  }


  /**
   * Describe <code>buildFalsePoints</code> method here.
   * This method builds the false points after removing the 
   * redundancy from the original scenarios.
   * --Murthy.
   */
  public void buildFalsePoints() {
    
    LinkedList finalList = new LinkedList();

    for (Iterator finalListIter = finalScenarios.iterator(); finalListIter.hasNext();) {
      BooleanTerm outer_term = new BooleanTerm();
      LinkedList innerList = (LinkedList)finalListIter.next();
      for (Iterator innerIter = innerList.iterator();innerIter.hasNext();) {
	BooleanVariable inner_var = new BooleanVariable("bool",(Application)innerIter.next());
	outer_term.add(inner_var);
      } // end of for (Iterator innerIter = innerList.iterator();innerIter.hasNext();)
      finalList.add(outer_term);
    } // end of for (Iterator finalListIter = finalList.iterator(); finalListIter.hasNext();)
    
    
    for (Iterator iter = finalList.iterator();iter.hasNext();) {
      RedundancyRemoving redRem = new RedundancyRemoving((BooleanTerm)iter.next());
      //This linkedList contains all the nodes in the Term with 
      //Redundancy removed.
      LinkedList falsePointTerm = redRem.redundancyRemoval();
      BooleanTerm boolTerm = new BooleanTerm();
      LinkedList innerList = new LinkedList();
      for (Iterator falseIter = falsePointTerm.iterator();falseIter.hasNext();) 
	{
	  //Each node in the term is treated as a boolean variable 
	  //and is added to the boolean term.
	  Expression expr = (Expression)falseIter.next();
	  BooleanVariable boolVar = new BooleanVariable("bool",expr);
	  boolTerm.add(boolVar);
	  innerList.add(expr);
	}
      
      //This falsePoints linked list contains all the boolean terms.
      falsePoints.add(boolTerm);
    } // end of for (Iterator iter = originalScenarios.iterator();iter.hasNext();)
  }


  /**
   * Describe <code>generateFalsePoints</code> method here.
   * This method is used to generate the false points 
   * for every scenario.
   * --Murthy.
   */
  public void generateFalsePoints() {
      int i=1;
      //Used to create the Directories.
      File f_main = new File(destDirectory);
      f_main.mkdir();
      File f_false = new File(destDirectory+"/False");
      f_false.mkdir();
      File f_optValues = new File(destDirectory+"/OptValues");
      f_optValues.mkdir();
      

      try {
	for (Iterator scenIter = falsePoints.iterator();scenIter.hasNext();) {
	
	  BooleanTerm scenBoolTerm = (BooleanTerm)scenIter.next();
	  DnfExpression scenDnf = new DnfExpression("falsePoint",scenBoolTerm.getBooleanVarList());
	  
	  scenDnf.add(scenBoolTerm);
	  scenDnf.nearFalsePoints();
	  
	  LinkedList falsePointsList = scenDnf.getFalsePointExpr();
	  //  System.out.println("####################################");
//  	  System.out.println("The FalsePoint LinkedList: "+falsePointsList);
//  	  System.out.println("####################################");

	  for (Iterator falsePointIter = falsePointsList.iterator();falsePointIter.hasNext();) {
	    //Used to write the false points into the corresponding files.
	    FileOutputStream fout = new FileOutputStream(destDirectory+"/False/FalsePoint_"+i+".txt");
	    PrintWriter pfalsePoint = new PrintWriter(fout,true);
	    FileOutputStream fxml = new FileOutputStream(destDirectory+"/False/FalsePoint_"+i+".xml");
	    PrintWriter xmlPw = new PrintWriter(fxml,true);
	    LinkedList falseList = (LinkedList)falsePointIter.next();
	    xmlPw.println("<vectorslist>");
	    evaluateExpression2(falseList,pfalsePoint,xmlPw);
	    xmlPw.println("</vectorslist>");
	    i++;
	  }
	  //System.out.println();
	} // end of for (Iterator falsePointIter = falsePointsList.iterator();falsePointIter.hasNext();)
      } catch (FileNotFoundException e) {
	System.out.println("Error in printing the FalsePoint files:"+e); 
      }
  }

  public void evaluateExpression2(LinkedList falsePointList,PrintWriter textPw,PrintWriter xmlPw) throws FileNotFoundException {
    LinkedList iptVars = new LinkedList();
    LinkedList declVars = new LinkedList();
    LinkedList optVars = new LinkedList();

    LinkedList input_vars = scenarios.getInputVar();
    LinkedList output_vars = scenarios.getOutputVar();
    HashMap input_values = requirements.getInputs();

    for (Iterator falseIter = falsePointList.iterator(); falseIter.hasNext();) {
      Application app = (Application)falseIter.next();
      String leftOp = ((ExpressionList)app.getParameters()).get(0).toString();
      
      if (input_vars.contains(leftOp)) {
	iptVars.add(app);
      } // end of if (input_vars.contains(leftOp))
      else if (output_vars.contains(leftOp)) {
	optVars.add(app);
      } // end of if (output_vars.contains(leftOp))
      else {
	declVars.add(app);
      } // end of else
      
    }

    int size = iptVars.size();

    switch (size) {
    case 1:
      //System.out.println("I am into the correct case");
      HashMap retObj = new HashMap();
      Application app_1 = (Application)iptVars.get(0);
      String leftOp_1 = ((ExpressionList)app_1.getParameters()).get(0).toString();
      LinkedList values_1 = (LinkedList)input_values.get(leftOp_1);
      //System.out.println("The right class name is: "+((ExpressionList)app_1.getParameters()).get(1).getClass().getName());
      String rightOp_1 = ((ExpressionList)app_1.getParameters()).get(1).toString();
      String oper_1 = app_1.getFunction().toString();
      
      if (oper_1.equalsIgnoreCase("<") || oper_1.equalsIgnoreCase("=<")) {
	//System.out.println("I am in the less operator in switch ");
	for (double cnt = getDouble(values_1.get(0).toString()); cnt < getDouble(rightOp_1); cnt += getDouble(values_1.get(2).toString())) {
	  textPw.print(leftOp_1+" = "+cnt+" \t");
	  retObj.put(leftOp_1,(new Double(cnt)));
	  printXml2(xmlPw,iptVars,optVars,declVars,retObj);
	} // end of for (double cnt = getDouble(values_1.get(0)))
      } // end of if (oper_1.equalsIgnoreCae("<"))
      if (oper_1.equalsIgnoreCase(">") || oper_1.equalsIgnoreCase(">=")) {
	//System.out.println("I am in the greater operator in switch");
	
	if (values_1.get(1).toString().equalsIgnoreCase("bit")) {
	  textPw.print(leftOp_1+" = "+getDouble(rightOp_1)+"\t");
	  retObj.put(leftOp_1,rightOp_1);
	  printXml2(xmlPw,iptVars,optVars,declVars,retObj);
	} // end of if (values_1.get(1).toString().equalsIgnoreCase("bit"))
	else {
	  for (double cnt =  getDouble(rightOp_1);cnt < getDouble(values_1.get(1).toString()); cnt += getDouble(values_1.get(2).toString())) {
	    textPw.print(leftOp_1+" = "+cnt+" \t");
	    retObj.put(leftOp_1,(new Double(cnt)));
	    printXml2(xmlPw,iptVars,optVars,declVars,retObj);
	  } // end of if (oper_1.equalsIgnoreCase(">"))   
	} // end of else
      }
            
      break;
      
    default:
      //System.out.println("There are no input variables");
      break;
    } // end of switch (size)
  }

  public double getDouble(Object valueIn) {
    String value = valueIn.toString();
    //System.out.println("The value in getDouble() is: "+value);
    return (new Double(value)).doubleValue();
  }    
  

 /**
   * Describe <code>evaluate</code> method here.
   *
   * @param falsePointsIn a <code>LinkedList</code> value
   * @return a <code>String</code> value
   */
  public Object evaluate2(LinkedList declVarListIn) {
    HashMap conditions = new HashMap();

    for (Iterator declIter = declVarListIn.iterator(); declIter.hasNext();) {
      Application appIn = (Application)declIter.next();
      String left_op =((ExpressionList)appIn.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)appIn.getParameters()).get(1).toString();
      String operator = appIn.getFunction().toString();
      
      conditions.put(left_op,appIn);
    } // end of for (Iterator falseIter = falsePointsIn.iterator(); falseIter.hasNext();)
    return conditions;
  }    
  

  public void printXml2(PrintWriter xmlPwIn, LinkedList iptVarList,LinkedList optVarList,LinkedList declVarList, HashMap retObjIn) {
    for (Iterator iptIter = iptVarList.iterator(); iptIter.hasNext();) {
      Application app = (Application)iptIter.next();
      String left_op = ((ExpressionList)app.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)app.getParameters()).get(1).toString();
      String oper = app.getFunction().toString();
      xmlPwIn.println("<vector>");
      xmlPwIn.println("<condition>");
      xmlPwIn.println("\t<parameter mode = \"in\">"+left_op+"</parameter>");
      xmlPwIn.println("\t<value> "+retObjIn.get(left_op)+" </value>");
      //  xmlPwIn.println("\t<Application>");
      //  	xmlPwIn.println("\t\t<operator> "+oper+" </operator>");
      //  	xmlPwIn.println("\t\t<left_operand> "+left_op+" </left_operand>");
      //  	xmlPwIn.println("\t\t<right_operand> "+right_op+" </right_operand>");
      //xmlPwIn.println("\t</Application>");
      xmlPwIn.println("</condition>");
    }
    
    for (Iterator outputIter = outputCondition.iterator(); outputIter.hasNext();) {
      Application out_app = (Application)outputIter.next();
      String left_op = ((ExpressionList)out_app.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)out_app.getParameters()).get(1).toString();
      
      HashMap condIn = (HashMap)evaluate2(declVarList);
      Application condApp = (Application)condIn.get(right_op);
      
      xmlPwIn.println("<condition>");
      xmlPwIn.println("\t<parameter mode = \"out\">"+left_op+"</parameter>");
      xmlPwIn.println("\t<Application>");
      xmlPwIn.println("\t\t<operator> "+condApp.getFunction().toString()+" </operator>");
      xmlPwIn.println("\t\t<left_operand> "+left_op+" </left_operand>");
      String rightOperand = ((ExpressionList)condApp.getParameters()).get(1).toString();
      xmlPwIn.println("\t\t<right_operand> "+rightOperand+" </right_operand>");
      xmlPwIn.println("\t</Application>");
      xmlPwIn.println("</condition>");
      
    } // end of for (Iterator outputIter = outputCondition.iterator(); outputIter.hasNext();)
    
    xmlPwIn.println("</vector>");
  }
  

  /**
   * Describe <code>evaluateExpression</code> method here.
   *
   * @param falsePoint a <code>LinkedList</code> value
   * @param textIn a <code>PrintWriter</code> value
   * @param xmlPwIn a <code>PrintWriter</code> value
   * @exception FileNotFoundException if an error occurs
   */
  public void evaluateExpression(LinkedList falsePoint,PrintWriter textIn,PrintWriter xmlPwIn) throws FileNotFoundException {
    //System.out.println("in evaluate expression: "+falsePoint);
    HashMap inputs = requirements.getInputs();
    Set input_set = inputs.entrySet();
    Iterator input_set_iter = input_set.iterator();
    
    LinkedList optVars = scenarios.getOutputVar();
    
    //This is dumb.Here I am considering that I am having 
    //only one input variable.
    double lower_bound = 0.0;
    double upper_bound = 0.0;
    double increment = 0.0;
    String left_key = null;
    while (input_set_iter.hasNext()) {
      Map.Entry inputs_entry = (Map.Entry)input_set_iter.next();
      left_key = (String)inputs_entry.getKey();
      LinkedList values = (LinkedList)inputs_entry.getValue();
      System.out.println("The values are: "+values.get(0).getClass().getName());
      lower_bound = (new Double(values.get(0).toString())).floatValue();
      upper_bound = (new Double(values.get(1).toString())).floatValue();
      increment = (new Double(values.get(2).toString())).floatValue();
    } // end of while (input_set_iter.hasNext())
    
    
    
    //This loop iterates through all the the nodes in the Application
    //to grab the node that contains the input variable.
    for (Iterator falsePointIter = falsePoint.iterator(); falsePointIter.hasNext();) {
      
      //This grabs each node in the false point term and checks 
      //for the one containing the input_value.
      Application app = (Application)falsePointIter.next();
      ExpressionList paramsList = app.getParameters();
      String leftOp = paramsList.get(0).toString();
      double rightOp = 0.0;
      if (paramsList.get(1) instanceof RealValue) {
	rightOp =(new Double(paramsList.get(1).toString())).floatValue();	   
	System.out.println("The right operand is: "+rightOp);
      } // end of if (paramsList.get(1) instanceof RealValue)
      String operator = app.getFunction().toString();
      
      //##############################################################
      //This is the actual logic.
      if (left_key.equalsIgnoreCase(leftOp)) {
	System.out.println("I am into the outer if loop");
	
	for (double count = lower_bound; count < upper_bound; count = count+increment) {	  
	  
	  if (count == rightOp) {
	    
	    if (operator.equalsIgnoreCase("<") || operator.equalsIgnoreCase("=<")) {
	      xmlPwIn.println("<vectorslist>");
	      for (double tmp = lower_bound; tmp < rightOp; tmp = tmp+increment) {
		System.out.println("I am into the inner for loop");
		printXml(xmlPwIn,leftOp,tmp,optVars,falsePoint);
		printText(textIn,leftOp,tmp,optVars,falsePoint);
	      } // end of for (int tmp = lower_bound; tmp < rightOp; tmp = tmp+increment)
	      xmlPwIn.println("</vectorslist>");
	      break;
	    } // end of if (operator.equalsIgnoreCase("<"))
	    if (operator.equalsIgnoreCase(">") || operator.equalsIgnoreCase(">=")) {
	      for (double tmp = count; tmp < upper_bound; tmp = tmp+increment) {
		printXml(xmlPwIn,leftOp,tmp,optVars,falsePoint);
		printText(textIn,leftOp,tmp,optVars,falsePoint);
	      }
	      break;
	    } // end of if (operator.equalsIgnoreCase(">") || operator.equalsIgnoreCase("="))
	  }
	}
	
      }
    }
  }    
  
  
  /**
   * Describe <code>evaluate</code> method here.
   *
   * @param falsePointsIn a <code>LinkedList</code> value
   * @return a <code>String</code> value
   */
  public Object evaluate(LinkedList falsePointsIn) {
    LinkedList ipts = new LinkedList();
    LinkedList opts = new LinkedList();
    HashMap conditions = new HashMap();
    String return_value = null;

    LinkedList inptList = scenarios.getInputVar();
    String inputValue = inptList.get(0).toString();
    LinkedList optList = scenarios.getOutputVar();
    String outputValue = optList.get(0).toString();

    for (Iterator falseIter = falsePointsIn.iterator(); falseIter.hasNext();) {
      Application appIn = (Application)falseIter.next();
      String left_op =((ExpressionList)appIn.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)appIn.getParameters()).get(1).toString();
      String operator = appIn.getFunction().toString();
      
      if ((!(left_op.equalsIgnoreCase(inputValue))) && (!(left_op.equalsIgnoreCase(outputValue)))){
	conditions.put(left_op,appIn);

	//return_value = operator+right_op;	   
      } // end of if (left_op.equalsIgnoreCase(inputValue) || (left_op.equalsIgnoreCase(outputValue))

      //Here I am assuming that the node containing the output variable
      //will always be at the end.This needs to be addressed.
      //  if (left_op.equalsIgnoreCase(outputValue)) {
//  	return_value = (String)conditions.get(right_op);
//  	//return_value = (new Double(retValue)).floatValue();
//        } // end of if (left_op.equalsIgnoreCase(outputValue))
      
    } // end of for (Iterator falseIter = falsePointsIn.iterator(); falseIter.hasNext();)
    return conditions;
  }    


  public void printXml(PrintWriter xmlPwIn, String leftOp, double valueIn, LinkedList optVarList,LinkedList falsePoint) {
    for (Iterator falseIter = falsePoint.iterator(); falseIter.hasNext();) {
      Application app = (Application)falseIter.next();
      String left_op = ((ExpressionList)app.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)app.getParameters()).get(1).toString();
      String oper = app.getFunction().toString();
      xmlPwIn.println("<vector>");
      if (left_op.equalsIgnoreCase(leftOp)) {
	
	xmlPwIn.println("<condition>");
	xmlPwIn.println("\t<parameter mode = \"in\">"+leftOp+"</parameter>");
	xmlPwIn.println("\t<value> "+valueIn+" </value>");
	//  xmlPwIn.println("\t<Application>");
//  	xmlPwIn.println("\t\t<operator> "+oper+" </operator>");
//  	xmlPwIn.println("\t\t<left_operand> "+left_op+" </left_operand>");
//  	xmlPwIn.println("\t\t<right_operand> "+right_op+" </right_operand>");
	//xmlPwIn.println("\t</Application>");
	xmlPwIn.println("</condition>");
      } // end of if (left_op.equalsIgnoreCase(leftOp))
      

    } // end of for (Iterator falseIter = falsePoint.iterator(); falseIter.hasNext();)
    
    for (Iterator outputIter = outputCondition.iterator(); outputIter.hasNext();) {
      Application out_app = (Application)outputIter.next();
      String left_op = ((ExpressionList)out_app.getParameters()).get(0).toString();
      String right_op = ((ExpressionList)out_app.getParameters()).get(1).toString();
      HashMap condIn = (HashMap)evaluate(falsePoint);
      Application condApp = (Application)condIn.get(right_op);
      
      xmlPwIn.println("<condition>");
      xmlPwIn.println("\t<parameter mode = \"out\">"+optVarList.get(0).toString()+"</parameter>");
      xmlPwIn.println("\t<Application>");
      xmlPwIn.println("\t\t<operator> "+condApp.getFunction().toString()+" </operator>");
      xmlPwIn.println("\t\t<left_operand> "+left_op+" </left_operand>");
      String rightOperand = ((ExpressionList)condApp.getParameters()).get(1).toString();
      xmlPwIn.println("\t\t<right_operand> "+rightOperand+" </right_operand>");
      xmlPwIn.println("\t</Application>");
      xmlPwIn.println("</condition>");

    } // end of for (Iterator outputIter = outputCondition.iterator(); outputIter.hasNext();)
    
    xmlPwIn.println("</vector>");
  }
  
  
  public void printText(PrintWriter textIn, String leftOp, double valueIn, LinkedList optVarList,LinkedList falsePoint) {
    textIn.print(leftOp+" = "+valueIn+"\t");
    textIn.println(optVarList.get(0).toString()+" =  "+valueIn);
    //textIn.println(evaluate(falsePoint));
  }


  /**
   * Describe <code>instantiateCostFunction</code> method here.
   *
   */
  public void instantiateCostFunction() {
    try {
      
      int i=1;
      int count = 1;
      int retCount;
      for (Iterator falsePtIter = falsePoints.iterator();falsePtIter.hasNext();) {
	CostFunctionGenerator costFncGen = new CostFunctionGenerator((BooleanTerm)falsePtIter.next()); 
	String pntWtrName = destDirectory+"/OptValues/optvalue_";
	retCount = costFncGen.createCostFnc(pntWtrName,count);
	count = retCount;
      }
      
    } catch (Exception e) {
      System.out.println("Error in instantiateCostFunction");
      
    } // end of try-catch
  }
}  
