//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: GenerateVectors.java,v 1.18 2002/11/18 11:57:46 ksatya Exp $
//
// Revision History:
// ----------------
// $Log: GenerateVectors.java,v $
// Revision 1.18  2002/11/18 11:57:46  ksatya
// Removed the debugging statements from all the files.
// --Murthy.
//
// Revision 1.17  2002/11/13 01:53:02  ksatya
// Added changes to generate scenarios for inner facets in a facet and multiple for multiple facets in a Package.
// --Murthy.
//
// Revision 1.16  2002/11/11 13:15:59  ksatya
// Bugs:
// -- Standard Declarations within a Package are having problems.
// -- Nested Facets are not supported till now.
// Those bugs were fixed.
// --Murthy.
//
// Revision 1.14  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.13  2002/10/17 04:27:48  cvs
// Made minor changes to support all the examples.
// --Murthy.
//
// Revision 1.11  2002/10/14 13:10:04  cvs
// Bits need to be handled seperately for generating the vectprs.Changes are made to support them.
// --Murthy.
//
// Revision 1.10  2002/10/10 17:10:19  cvs
// Implemented the functionality of evaluating expressions using Jython.
// --Murthy.
//
// Revision 1.9  2002/10/09 09:33:51  cvs
// Added support to handle multiple inputs.
// --Murthy.
//
// Revision 1.7  2002/10/08 22:21:09  cvs
// Made changes to support XML format for false points.
// --Murthy.
//
// Revision 1.6  2002/10/08 14:33:57  cvs
// Made the tools compatable to generate the false points in XML format.Added a new file ValueGenerator.java to generate the vectors.This file implements a new logic as compared to the logic in GenerateVectors class.
// --Murthy.
//
// Revision 1.5  2002/10/01 06:31:45  cvs
// Made changes in GenerateVectors appropriate to the changes in the types directory.
// --Murthy
//
// Revision 1.4  2002/09/29 02:43:18  cvs
//  Made changes to print the vectors in XML format.
// --Murthy.
//
// Revision 1.3  2002/09/27 05:22:04  cvs
// Implemented the complete functionality of BitVectors.
// --Murthy.
//
// Revision 1.2  2002/09/08 21:50:41  cvs
// Added a new class to take care of Literals.Split the functionality of ReadTestRequirements class to make the tool more flexible.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.lang.values.RealValue;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;
import org.sldl.rosetta.tools.generatedata.types.*;
import org.sldl.rosetta.tools.generatedata.scanner.*;

import java.util.*;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;

//Jython stuff.
import org.python.core.*;


/**
 * Describe class <code>GenerateVectors</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class GenerateVectors {

  private ReadRequirements requirements;

  private static ReadScenarios scenarios;

  private LinkedList finalScenList;

  public LinkedList finalScenarios;

  //This variable is used to store the conditions in the 
  //terms that has the relationship between output variable 
  //and any declared variables.
  private LinkedList outputCondition;


  private String fileName;

  //Needs to take care of this variable.
  private static String previous_state;

  //This field takes care of whether or not to 
  //generate the false points.
  protected static boolean generateFalsePoints = false;

  /**
   * Creates a new <code>GenerateVectors</code> instance.
   * Primary Constructor.
   */
  public GenerateVectors() {
    finalScenList = new LinkedList();
  }

  /**
   * Creates a new <code>GenerateVectors</code> instance.
   * Secondary Constructor.
   * @param reqsIn a <code>ReadRequirements</code> value
   * @param scenIn a <code>ReadScenarios</code> value
   */
  public GenerateVectors(ReadRequirements reqsIn, ReadScenarios scenIn) {
    requirements = reqsIn;
    scenarios = scenIn;
    finalScenList = new LinkedList();
    finalScenarios = new LinkedList();
  }
  
  /**
   * Describe <code>setRequirements</code> method here.
   * This sets the requirements object.
   * @param reqsIn an <code>Object</code> value
   */
  public void setRequirements(Object reqsIn) {
    requirements = (ReadRequirements)reqsIn;
  }

  /**
   * Describe <code>getRequirements</code> method here.
   * This returns the requirements object.
   * @return an <code>Object</code> value
   */
  public Object getRequirements() {
    return requirements;
  }

  public void setFileName(String fileNameIn) {
    fileName = fileNameIn.substring(0,fileNameIn.indexOf("_SCENARIOS"));
  }



  /**
   * Describe <code>setScenarios</code> method here.
   * This sets the scenarios object.
   * @param scenIn an <code>Object</code> value
   */
  public static void setScenarios(Object scenIn) {
    scenarios = (ReadScenarios)scenIn;
  }

  /**
   * Describe <code>getScenarios</code> method here.
   * This returns the scenarios.
   * @return an <code>Object</code> value
   */
  public static Object getScenarios() {
    return scenarios;
  }

  /**
   * Describe <code>optimizeScenList</code> method here.
   * This method is used to optimize the scenarios.Firstly the scenarios 
   * that doesn't contain any output or control variables are
   * removed.
   */
  public LinkedList optimizeScenList() {
    LinkedList declVars = (LinkedList)scenarios.getFctDeclaredItems();
    LinkedList origScen = (LinkedList)scenarios.scenList;
    finalScenList = new LinkedList();
    
    //This code needs to be deleted.
    //**************************************************************//
   //   System.out.println("###########################################");
//      System.out.println("Testing the original scenarios");
//      for (Iterator origOtr = origScen.iterator();origOtr.hasNext();) {
//        LinkedList OrgInr = (LinkedList)origOtr.next();
//        for (Iterator OrgIter = OrgInr.iterator();OrgIter.hasNext();) {
//  	System.out.println(OrgIter.next());
//        } // end of for (Iterator OrgIter = OrgInr.iterator();OrgIter.hasNext();)
//      } // end of for (Iterator origOtr = origScen.iterator();origOtr.hasNext();)
//      System.out.println("End of testing the original scenarios");
//      System.out.println("###########################################");
    //**************************************************************//
    
    //The following code is used to remove the scenarios that
    //does not contain either any control variables or 
    //the output variables.
    
    for (Iterator scenIter = origScen.iterator(); scenIter.hasNext();) {
      boolean flag = false;
      LinkedList innerScenarios = (LinkedList)scenIter.next();	 
      for (Iterator innerScenIter = innerScenarios.iterator();innerScenIter.hasNext();) {
	Application app = (Application)innerScenIter.next();
	
	flag  = checkControlVar(app,declVars) || checkOutputVar(app,scenarios.getOutputVar());
	if (flag == true) {
	  finalScenList.add(innerScenarios);
	  break;
	} // end of if (flag == true)
      }
    } // end of while (scenIter.hasNext())
    
    //This code needs to be deleted.
    //**************************************************************//
   //   System.out.println("###########################################");
//      System.out.println("Testing the removal of control vars");
//      for (Iterator finScen = finalScenList.iterator();finScen.hasNext();) {
//        LinkedList finInr = (LinkedList)finScen.next();
//        for (Iterator finInnerIter = finInr.iterator();finInnerIter.hasNext();) {
//  	System.out.println(finInnerIter.next());
//        } // end of for (Iterator OrgIter = OrgInr.iterator();OrgIter.hasNext();)
//      } // end of for (Iterator origOtr = origScen.iterator();origOtr.hasNext();)
//      System.out.println("End of testing the removal of control vars");
//      System.out.println("###########################################");
    //**************************************************************//
    
    
    //This piece of code is used to split the scenarios.if we have a scenario 
    //of the form a>=b then it is split into a>b and a=b for implementation
    //purposes.
    //   LinkedList finalOuterList = new LinkedList();
//      for (Iterator outerScenIter = finalScenList.iterator();outerScenIter.hasNext();) {
//        LinkedList innerScenList = (LinkedList)outerScenIter.next();
//        LinkedList finalInnerList = new LinkedList();
//  	for (Iterator innerScenIter = innerScenList.iterator();innerScenIter.hasNext();) {
//  	  Application app = (Application)innerScenIter.next();
//  	  ExpressionList params = app.getParameters();
//  	  String oper = app.getFunction().toString();
//  	  if (oper.equals(">=")) {
//  	    //innerScenList.remove(app);
//  	    finalInnerList.add(new Application(new ItemReference(">"),params,Application.BINARY));
//  	    finalInnerList.add(new Application(new ItemReference("="),params,Application.BINARY));
//  	  } // end of if (oper.equals(">="))
//  	  if (oper.equals("=<")) {
//  	    //innerScenList.remove(app);
//  	    finalInnerList.add(new Application(new ItemReference("<"),params,Application.BINARY));
//  	    finalInnerList.add(new Application(new ItemReference("="),params,Application.BINARY)); 
//  	  } // end of if (oper.equals("=<"))
//  	  if ((oper.equals("<")) || (oper.equals("=")) || (oper.equals(">"))) {
//  	    finalInnerList.add(app);
//  	  } // end of if ((oper.equals("<") || (oper.equals("=") || oper.equals(">"))
//  	} // end of for (Iterator innerScenIter = innerScenList.iterator();innerScenIter.hasNext();)
//  	finalOuterList.add(finalInnerList);
//      } // end of for (Iterator outerScenIter = scenList.iterator();outerScenIter.hasNext();)

//      //Needs to be deleted.
//      //*******************************************************************//
//      System.out.println("###########################################");
//      System.out.println("Testing in Optimize scenario list");
//      Iterator finIter = finalOuterList.iterator();
//      while (finIter.hasNext()) {
//        LinkedList finInnerList = (LinkedList)finIter.next();
//        Iterator finInnerIter = finInnerList.iterator();
//        while (finInnerIter.hasNext()) {
//  	System.out.println(finInnerIter.next());
//        } // end of while (finInnerIter.hasNext())
//      } // end of while (finIter.hasNext())
//      System.out.println("###########################################");
//      //*******************************************************************//
    

    //    return finalOuterList;

    return finalScenList;

  }


  /**
   * Describe <code>checkControlVar</code> method here.
   * This piece of code is used to check if the scenarios
   * have any control variables, which are declared in the facet.
   * @param appIn an <code>Application</code> value
   * @param declVarsIn a <code>LinkedList</code> value
   * @return a <code>boolean</code> value
   */
  public boolean checkControlVar(Application appIn,LinkedList declVarsIn) {
    Iterator declIter = declVarsIn.iterator();
    while (declIter.hasNext()) {
      ExpressionList paramsList = (ExpressionList)appIn.getParameters();
      String leftParam = initialState(paramsList.get(0).toString());
      String rightParam = initialState(paramsList.get(1).toString());
      String declVar = (String)declIter.next().toString();
      if (declVar.equals(leftParam) || declVar.equals(rightParam)) {
	return true;
      } // end of if (declVar.equals(leftParam) || declVar.equals(rightParam))
      else {
	continue;
      } // end of else
    } // end of while (declIter.hasNext())
    return false;
  }
  
  /**
   * Describe <code>checkOutputVar</code> method here.
   * This code is used to check if the scenarios
   * contain any output variables.if there ane any scenarios
   * that contains the output variables they are not modified.
   * @param appIn an <code>Application</code> value
   * @param outputVarsIn a <code>LinkedList</code> value
   * @return a <code>boolean</code> value
   */
  public boolean checkOutputVar(Application appIn,LinkedList outputVarsIn) {
    Iterator optIter = outputVarsIn.iterator();
    while (optIter.hasNext()) {
      ExpressionList paramsList = (ExpressionList)appIn.getParameters();
      //System.out.println("The right op class name is: "+paramsList.get(1).getClass().getName());
      String leftParam = initialState(paramsList.get(0).toString());
      //System.out.println("The left op in checkOutputVar: "+leftParam);
      String rightParam = initialState(paramsList.get(1).toString());
      //System.out.println("The right op in checkOutputVar: "+rightParam);
      String optVar = (String)optIter.next().toString();
      if (optVar.equals(leftParam) || optVar.equals(rightParam)) {
	return true;
      } // end of if (declVar.equals(leftParam) || declVar.equals(rightParam))
      else {
	continue;
      } // end of else
    } // end of while (declIter.hasNext())
    return false;
    //   ExpressionList paramsList = (ExpressionList)appIn.getParameters();
    //      String leftParam = initialState(paramsList.get(0).toString());
    //      System.out.println("The left op in checkOutputVar: "+leftParam);
    //      if (paramsList.get(1) instanceof Application) {
    //        appIn = (Application)paramsList.get(1);
    //        System.out.println("The next app is: "+appIn);
    //        String rightOp = ((ExpressionList)appIn.getParameters()).get(0).toString();
    //        System.out.println("The rightOp is: "+rightOp);
    
    //      } // end of if (paramsList.get(1) instanceof Application)
    //      return true;    
    
  }
  
  /**
   * Describe <code>initialState</code> method here.
   * This method returns the initial state of any variable.
   * @param varIn a <code>String</code> value
   * @return a <code>String</code> value
   */
  public static String initialState(String varIn){
    if (varIn.endsWith("'")) {
      return (new String(varIn.substring(0,varIn.indexOf("'"))));
    } // end of if (varIn.endsWith("'"))
    else {
      return varIn;
    }
  }    

  /**
   * Describe <code>getDouble</code> method here.
   *
   * @param valueIn an <code>Object</code> value
   * @return a <code>double</code> value
   */
  public double getDouble(Object valueIn) {
    String value = valueIn.toString();
    return (new Double(value)).doubleValue();
  }

  /**
   * Describe <code>displayFinalScenarios</code> method here.
   * This is basically used for testing purposes.
   */
  public void displayFinalScenarios() {
    for (Iterator scenIter = finalScenList.iterator(); scenIter.hasNext();) {
      LinkedList innerList = (LinkedList)scenIter.next();
      for (Iterator innerIter = innerList.iterator(); innerIter.hasNext();) {
	System.out.print(innerIter.next().toString()+"\t");
      } // end of for (Iterator innerIter = innerList.iterator(); innerIter.hasNext();)
      System.out.println();
    } // end of for (Iterator scenIter = origScen.iterator(); scenIter.hasNext();)
  }
  
  /**
   * Describe <code>checkBitVector</code> method here.
   * This method checks to see if the input specified is 
   * a BitVector.The logic is that it iterates thru all the 
   * bits and checks to see if they are 1 or 0.
   * @param valueIn an <code>Object</code> value
   * @return a <code>boolean</code> value
   */
  public boolean checkBitVector(Object valueIn) {
    boolean returnValue = false;
    //System.out.println("in checkBitVector: "+valueIn);
    ArrayValue value_tmp = (ArrayValue)valueIn;
    for (int index = 0; index < value_tmp.size(); index++) {
      if (value_tmp.getElementAt(index).toString().equalsIgnoreCase("1") || value_tmp.getElementAt(index).toString().equalsIgnoreCase("0")) {
	returnValue = true;
      }
      else {
	returnValue = false;
      } // end of else
    }
    return returnValue;
  }


  public boolean containsBits(HashMap iptObjectIn) {
    Set mapSet = iptObjectIn.entrySet();
    Iterator mapSetIter = mapSet.iterator();
    
    while (mapSetIter.hasNext()) {
      Map.Entry mapEntry = (Map.Entry)mapSetIter.next();
      LinkedList values = (LinkedList)mapEntry.getValue();
      if (values.contains("bit")) {
	return true;
      } // end of if (values.contains("bit"))
    } // end of while (mapSetIter.hasNext())
    return false;
  }
    
  public void getBitList(HashMap iptObjIn,LinkedList iptBitListIn, LinkedList iptNonBitListIn) {
    Set iptSet = iptObjIn.entrySet();
    Iterator iptSetIter = iptSet.iterator();
    
    while (iptSetIter.hasNext()) {
      Map.Entry iptEntry = (Map.Entry)iptSetIter.next();
      LinkedList values = (LinkedList)iptEntry.getValue();
      if (values.contains("bit")) {
	iptBitListIn.add(iptEntry.getKey()); 
      } // end of if (values.contains("bit"))
      else {
	iptNonBitListIn.add(iptEntry.getKey());
      } // end of else
      
    } // end of while (iptSetIter.hasNext())
    //System.out.println("The ipBitList: "+iptBitListIn.size());
    //System.out.println("The ipNonBitList: "+iptNonBitListIn.size());
  }
    

  /**
   * Describe <code>getNext</code> method here.
   * This is used for returning complement not value of 
   * the bit.
   * @param valueIn an <code>int</code> value
   * @return an <code>int</code> value
   */
  public double getNext(double valueIn) {
    if (valueIn == 1) {
      return 0;
    } // end of if (valueIn == 1)
    else {
      return 1;
    } // end of else
  }


  /**
   * Describe <code>Combo</code> method here.
   * This function is called recursively to cover all
   * the inputs.A bit scary though ........:-)
   * @param low[] a <code>double</code> value
   * @param high[] a <code>double</code> value
   * @param inc[] a <code>double</code> value
   * @param val[] a <code>double</code> value
   * @param count an <code>int</code> value
   * @param index an <code>int</code> value
   * @param mapIn a <code>HashMap</code> value
   * @param listIn a <code>LinkedList</code> value
   * @param testIn a <code>ValueGenerator</code> value
   * @param pw a <code>PrintWriter</code> value
   * @param xmlPw a <code>PrintWriter</code> value
   * @exception FileNotFoundException if an error occurs
   */
  public void Combo(double low[],double high[], double inc[], double val[], int count, int index,HashMap mapIn,LinkedList listIn,ValueGenerator testIn,PrintWriter pw,PrintWriter xmlPw) throws FileNotFoundException{
    try {
          
      if (index<count) {
	val[index] = low[index];
	while (val[index] <= high[index]) {
	  Combo(low,high,inc,val,count,index+1,mapIn,listIn,testIn,pw,xmlPw);
	  val[index] += inc[index];
	  //System.out.println(val[index]);
	} // end of while (val[index] < high[index])
	
      } // end of if (index<count)
      else {
	// Print(val,count);
	for (int cnt = 0; cnt < count; cnt++) {
	  mapIn.put(listIn.get(cnt),new Double(val[cnt]));
      } // end of for (int cnt = 0; cnt < count; cnt++)
	testIn.generateOpValue(mapIn,pw,xmlPw);
      } // end of else
    } catch (FileNotFoundException e) {
      System.out.println("Error in the recursive function Combo");    
    } // end of try-catch
    
  }
  
  /**
   * Describe <code>buildVectors</code> method here.
   * This is the actual method from where the ValueGenerator 
   * gets called to generate the output value.
   * @exception FileNotFoundException if an error occurs
   */
  public void buildVectors() throws FileNotFoundException {
    
    int index = 1;
    //File to print the vectors in a text format.
    FileOutputStream fileIn = new FileOutputStream(fileName+"_VECTORS.txt");
    PrintWriter pw = new PrintWriter(fileIn,true);
    
    //File to print the vectors in XML format.
    FileOutputStream xmlFile = new FileOutputStream(fileName+"_VECTORS.xml");
    PrintWriter xmlPw = new PrintWriter(xmlFile,true);
    xmlPw.println("<vectorslist>");
    
    LinkedList optList = optimizeScenList();
    LinkedList finalList = optList;//addOutVar(optList);
    //finalScenarios = finalList;
    //Type type;

    scenarios.setFinalScenarios(finalList);
    ValueGenerator test = new ValueGenerator(scenarios,requirements);
    test.generateOutputCondition();
    
    HashMap iptObject = requirements.getInputs();

    //The below condition is used to deal with the
    //bits in the inputs. Here I am seperating all the bits
    //and the non-bit inputs and I am iterating thru the 
    //bits first.
    if (containsBits(iptObject)) {
      
      //Stuff to be filled.
      LinkedList iptBitList = new LinkedList();
      LinkedList iptNonBitList = new LinkedList();
      
      getBitList(iptObject,iptBitList,iptNonBitList);
      
      
      int size = iptBitList.size();
      HashMap retObj = new HashMap();

      //This needs to be redesigned. Here I am using a switch statement.
      //This can also be designed using recursion.
      switch (size) {
      case 1:
	
	LinkedList values_list = (LinkedList)iptObject.get(iptBitList.get(0));
	double value = getDouble(values_list.get(0));
	for (double cnt = 0; cnt < 2; cnt++,value = getNext(value)) {
	  retObj.put(iptBitList.get(0),(new Double(value)));
	  
	  Iterator nonBitListIter = iptNonBitList.iterator();
	  while (nonBitListIter.hasNext()) {
	    String key = (String)nonBitListIter.next();
	    LinkedList key_list = (LinkedList)iptObject.get(key);
	    retObj.put(key,key_list.get(0));
	  } // end of while (nonBitListIter.hasNext())
	  
	  //for testing purpose.
	  //  System.out.println("##############################################");
//  	  System.out.println("Testing in containsBits() the retValue object");
//  	  Set retObjSet = retObj.entrySet();
//  	  Iterator retObjSetIter = retObjSet.iterator();
//  	  while (retObjSetIter.hasNext()) {
//  	    Map.Entry retEntry = (Map.Entry)retObjSetIter.next();
//  	    System.out.println(retEntry.getKey()+":"+retEntry.getValue());
//  	  } // end of while (retObjSetIter.hasNext())
//  	  System.out.println("##############################################");
	  
	  test.generateOpValue(retObj,pw,xmlPw);
	} // end of for (int cnt = 0; cnt < 2; cnt++,value = getNext(value))
	
	break;

      case 2:
	LinkedList values_list2_1 = (LinkedList)iptObject.get(iptBitList.get(0));
	double value1 = getDouble(values_list2_1.get(0));
	LinkedList values_list2_2 = (LinkedList)iptObject.get(iptBitList.get(1));
	
	for (double cnt1=0;cnt1<2;cnt1++,value1 = getNext(value1)) {
	  retObj.put(iptBitList.get(0),new Double(value1));
	  
	  double value2 = getDouble(values_list2_2.get(0));
	  for (double cnt2 = 0;cnt2 < 2; cnt2++,value2 = getNext(value2)) {
	    retObj.put(iptBitList.get(1),new Double(value2));

	    Iterator nonBitListIter2 = iptNonBitList.iterator();
	    while (nonBitListIter2.hasNext()) {
	      String key_2 = (String)nonBitListIter2.next();
	      LinkedList key_2_list = (LinkedList)iptObject.get(key_2);
	      retObj.put(key_2,key_2_list.get(0));
	    } // end of while (nonBitListIter.hasNext())
	    
	    //for testing purpose.
	    //  System.out.println("##############################################");
//  	    System.out.println("Testing in containsBits() the retValue object");
//  	    Set retObjSet_2 = retObj.entrySet();
//  	    Iterator retObjSetIter_2 = retObjSet_2.iterator();
//  	    while (retObjSetIter_2.hasNext()) {
//  	      Map.Entry retEntry_2 = (Map.Entry)retObjSetIter_2.next();
//  	      System.out.println(retEntry_2.getKey()+":"+retEntry_2.getValue());
//  	    } // end of while (retObjSetIter.hasNext())
//  	    System.out.println("##############################################");

	    test.generateOpValue(retObj,pw,xmlPw);	    	

	  } // end of for (int cnt2 = 0;cnt < 2; cnt++,value2 = getNext(value2))
	} // end of for (int cnt1=0;cnt<2;cnt++,value1 = getNext(value1))
		
	break;

	
      case 3:
	LinkedList values_list3_1 = (LinkedList)iptObject.get(iptBitList.get(0));
	double value3_1 = getDouble(values_list3_1.get(0));
	LinkedList values_list3_2 = (LinkedList)iptObject.get(iptBitList.get(1));
	LinkedList values_list3_3 = (LinkedList)iptObject.get(iptBitList.get(2));
	
	for (double cnt1=0;cnt1<2;cnt1++,value3_1 = getNext(value3_1)) {
	  retObj.put(iptBitList.get(0),new Double(value3_1));
	  
	  double value3_2 = getDouble(values_list3_2.get(0));
	  for (double cnt2 = 0;cnt2 < 2; cnt2++,value3_2 = getNext(value3_2)) {
	    retObj.put(iptBitList.get(1),new Double(value3_2));

	    double value3_3 = getDouble(values_list3_3.get(0));
	    for (double cnt3 = 0; cnt3<2;cnt3++,value3_3 = getNext(value3_3)) {
	      retObj.put(iptBitList.get(2),new Double(value3_3));	    
	    
	    Iterator nonBitListIter2 = iptNonBitList.iterator();
	    while (nonBitListIter2.hasNext()) {
	      String key_2 = (String)nonBitListIter2.next();
	      LinkedList key_2_list = (LinkedList)iptObject.get(key_2);
	      retObj.put(key_2,key_2_list.get(0));
	    } // end of while (nonBitListIter.hasNext())

	    //for testing purpose.
	    //  System.out.println("##############################################");
//  	    System.out.println("Testing in containsBits() the retValue object");
//  	    Set retObjSet_2 = retObj.entrySet();
//  	    Iterator retObjSetIter_2 = retObjSet_2.iterator();
//  	    while (retObjSetIter_2.hasNext()) {
//  	      Map.Entry retEntry_2 = (Map.Entry)retObjSetIter_2.next();
//  	      System.out.println(retEntry_2.getKey()+":"+retEntry_2.getValue());
//  	    } // end of while (retObjSetIter.hasNext())
//  	    System.out.println("##############################################");
 

	    test.generateOpValue(retObj,pw,xmlPw);

	    }
	  }
	}
	break;

      default:
	break;
      } // end of switch (size)
    } // end of if (containsBits(iptObject))
    
    else {
       
      Set set = iptObject.entrySet();
      Iterator iter = set.iterator();
      
      if(iter.hasNext()) {
	Map.Entry entry = (Map.Entry)iter.next();
	System.out.print(entry.getKey()+"\t");
	LinkedList values = (LinkedList)entry.getValue();
	
	if (values.getFirst() instanceof ArrayValue) {
	  int size = ((ArrayValue)values.getFirst()).size();
	  
	  //The following code is used to distinguish between a 
	  //bitvector and a sequence.
	  boolean isBitVector = checkBitVector(values.getFirst());
	  
	  if (isBitVector) {
	    BitVectorType bitType = new BitVectorType(iptObject);
	    bitType.setInputs(iptObject);
	    bitType.setSize(size);
	    bitType.buildVectors(finalList,pw,xmlPw);
	  }
	  else {
	    Sets setType = new Sets(iptObject);
	    setType.buildVectors(finalList,pw,xmlPw);
	  } // end of else
	} // end of if (initial_value instanceof SetValue)
	
	if (values.getFirst() instanceof PrimitiveValue) {
	  
	  //First get all the input variables from the scenarios file.
	  LinkedList iptVariablesList = scenarios.getInputVar();
	  int size = iptVariablesList.size();
	  	  
	  HashMap inputValues = requirements.getInputs();
	  int iptsSize = inputValues.size();

	  //These data structures stores all the lower bound,
	  //higher bound and the incremental values of all
	  //the input variables.
	  double low[] = new double[iptsSize];
	  double high[] = new double[iptsSize];
	  double inc[] = new double[iptsSize];
	  double val[] = new double[iptsSize];
	  int index_rec = 0;
	  
	  LinkedList ipt_values_hashmap = new LinkedList();
	  Set iptSet = inputValues.entrySet();
	  Iterator ipIter = iptSet.iterator();
	  while (ipIter.hasNext()) {
	    Map.Entry ipEntry = (Map.Entry)ipIter.next();
	    ipt_values_hashmap.add(ipEntry.getKey());
	    LinkedList map_values = (LinkedList)ipEntry.getValue();

	    //The below statements are used to extract the values from the input hashmap
	    //and initialize the above declared data structures.
	    double low_val = ((Double)((RealValue)map_values.get(0)).getPrimitiveValue()).doubleValue();
	    double high_val = ((Double)((RealValue)map_values.get(1)).getPrimitiveValue()).doubleValue();
	    double inc_val = ((Double)((RealValue)map_values.get(2)).getPrimitiveValue()).doubleValue();
	    low[index_rec] = low_val;
	    high[index_rec] = high_val;
	    inc[index_rec] = inc_val;

	    index_rec++;
	  } // end of while (ipIter.hasNext())
	  
	  //This val is the intermediate data structure that contains
	  //all the 
	  for (int i = 0; i < iptsSize; i++) {
	    val[i] = 0.0;
	  } // end of for (int i = 0; i < iptsSize; i++)

	  HashMap value_obj = new HashMap();
	  Combo(low,high,inc,val,iptsSize,0,value_obj,ipt_values_hashmap,test,pw,xmlPw);
	  
	} // end of if (values.getFirst() instanceof PrimitiveValue)
      }
    }
    xmlPw.println("</vectorslist>");
  }
  
  

  
  /**
   * Describe <code>printVectors</code> method here.
   * This method is used to print the abstract test vectors
   * in a text file.
   * @param count a <code>float</code> value
   * @param listIn a <code>LinkedList</code> value
   * @param pw a <code>PrintWriter</code> value
   * @exception FileNotFoundException if an error occurs
   */
  public static void printVectors(float count,LinkedList listIn,PrintWriter pw) throws FileNotFoundException {
    //This is only temporary.Here I am coding taking the assumption that I have 
    //only one input and one output variable.I need to modify this as this is
    //going to be a LinkedList.
    LinkedList iptList = scenarios.getInputVar();
    LinkedList optList = scenarios.getOutputVar();
    
    String input_var = (String)iptList.get(0).toString();
    String output_var = (String)optList.get(0).toString();
    //String previous_state = null;

    pw.print("("+input_var + " = " + count+")");
    for (Iterator vectorIter = listIn.iterator();vectorIter.hasNext();) {
      Application appIn = (Application)vectorIter.next();
      ExpressionList paramsList = appIn.getParameters();
      String leftOp = paramsList.get(0).toString();
      String rightOp = paramsList.get(1).toString();
      
      if ((!(leftOp.equals(input_var))) && (!(leftOp.equals(output_var)))) {
	if (paramsList.get(1) instanceof PrimitiveValue) {
	  previous_state  = (String)((PrimitiveValue)paramsList.get(1)).getPrimitiveValue().toString();
	} // end of if (leftOp.equals(input_var) && leftOp.equals(output_var))
      }
      if (!(input_var.equals(leftOp))) {
	if (paramsList.get(1) instanceof RealValue) {
	  pw.print(" "+appIn.toString());
	} // end of if (paramsList.get(1) instanceof RealValue)
	else {
	  pw.print(" ("+leftOp+" = "+previous_state+")"); 
	} // end of else
      } // end of if (input_variable.equals(leftOp))
    } // end of for (Iterator vectorIter = listIn.iterator();vectorIter.hasNext();)
    pw.println();
  }
  
  
  /**
   * Describe <code>printXml</code> method here.
   * This method is used to print the vectors 
   * in XML file.
   * @param count a <code>float</code> value
   * @param listIn a <code>LinkedList</code> value
   * @param xml a <code>PrintWriter</code> value
   */
  public static void printXml(float count,LinkedList listIn, PrintWriter xml,int indexIn) {
    //This is only temporary.Here I am coding taking the assumption that I have 
    //only one input and one output variable.I need to modify this as this is
    //going to be a LinkedList.
    LinkedList iptList = scenarios.getInputVar();
    LinkedList optList = scenarios.getOutputVar();
    
    String input_var = (String)iptList.get(0).toString();
    String output_var = (String)optList.get(0).toString();
    
    xml.println("<vector count = \""+indexIn+"\">");
    xml.println("<condition>");
    xml.println("\t<parameter mode=\"in\"> "+input_var+" </parameter>");
    xml.println("\t<value> "+count+" </value>");
    xml.println("</condition>");
    for (Iterator vectorIter = listIn.iterator();vectorIter.hasNext();) {
      Application appIn = (Application)vectorIter.next();
      ExpressionList paramsList = appIn.getParameters();
      String leftOp = paramsList.get(0).toString();
      String rightOp = paramsList.get(1).toString();
      if ((!(leftOp.equals(input_var))) && (!(leftOp.equals(output_var)))) {
	if (paramsList.get(1) instanceof PrimitiveValue) {
	  previous_state  = (String)((PrimitiveValue)paramsList.get(1)).getPrimitiveValue().toString();
	} // end of if (leftOp.equals(input_var) && leftOp.equals(output_var))
      }
      //if (!(input_var.equals(leftOp))) {
      if (output_var.equals(initialState(leftOp))) {
	if (paramsList.get(1) instanceof RealValue) {
	  xml.println("<condition>");
	  xml.println("\t<parameter mode=\"out\"> "+leftOp+" </parameter>");
	  xml.println("\t<value> "+rightOp+" </value>");
	  xml.println("</condition>");
	  //pw.print(" "+appIn.toString());
	} // end of if (paramsList.get(1) instanceof RealValue)
	else {
	  xml.println("<condition>");
	  xml.println("\t<parameter mode=\"out\"> "+leftOp+" </parameter>");
	  xml.println("\t<value> "+previous_state+" </value>");
	  xml.println("</condition>");
	  //pw.print(" ("+leftOp+" = "+previous_state+")"); 
	} // end of else
      } // end of if (input_variable.equals(leftOp))
    } // end of for (Iterator vectorIter = listIn.iterator();vectorIter.hasNext();)
    xml.println("</vector>");
  }
  
  
  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args) {
    
    TestParser test = new TestParser();
    
    Item reqPackItem = test.compileFile(args[0]);
    Item scenPackItem = test.compileFile(args[1]);
    
    ReadRequirements req = new ReadRequirements(reqPackItem);
    req.populateInputs();
    
    ReadScenarios scen = new ReadScenarios(scenPackItem);
    scen.readScenarios();
    
    GenerateVectors vectors = new GenerateVectors(req,scen);
    vectors.setFileName(args[1]);
    //vectors.optimizeScenList();
    try {
      vectors.buildVectors();       
      System.out.println("Done with the buildVectors()");
    } catch (FileNotFoundException e) {
      
    } // end of try-catch
    
    //********************************************************//
    //Used to generate the false points in xml format.
    //   GenerateScenarios genScen = new GenerateScenarios();
    //      String fileName = args[1].substring(0,args[1].indexOf("_SCENARIOS.sld"))+".sld";
    //      Item packItem = test.compileFile(fileName);
    //      org.sldl.rosetta.ROM.Package pack = (org.sldl.rosetta.ROM.Package)packItem.getValue();
    //      //******************************************************************//
    
//      //Walking through the tree for the package.
    //      ItemList declItemList = pack.getDeclaredItems();
    //      Iterator declIter = declItemList.iterator();
    //      while (declIter.hasNext()) {
    //        Item fctItem = (Item)declIter.next();
    //        if (fctItem.getValue() instanceof Facet) {
    //  	//genScen.process((Facet)(fctItem.getValue()));  	 
    //  	genScen.process(fctItem);  	 
    //        } // end of if (Item instanceof Facet)
      
    //        if (generateFalsePoints == true) {
    //  	//System.out.println("Starting the False Point generation stuff");
    //  	FalsePointsGenerator falsePointGen = new FalsePointsGenerator(genScen.getScenarios());
    //  	falsePointGen.setRequirements(req,scen);
    //  	falsePointGen.generateOutputCondition();
    //  	falsePointGen.setFinalScenarios(vectors.finalScenList);
    //  	falsePointGen.buildFalsePoints();
    //  	falsePointGen.setDirectoryName(args[1].substring(0,args[1].indexOf("_SCENARIOS.sld")));
    //  	falsePointGen.generateFalsePoints();
    //  	falsePointGen.instantiateCostFunction();
    //  	//********************************************************//
    //        }
    //for testing purpose.
    //vectors.displayFinalScenarios();
    
  } // end of main ()
}

 



