//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: IfWrapper.java,v 1.2 2002/09/09 15:22:00 cvs Exp $
// Revision History:
// ----------------
// $Log: IfWrapper.java,v $
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import org.sldl.rosetta.ROM.IfExpression;
import org.sldl.rosetta.ROM.Application;

import java.util.LinkedList;

/**
 * Describe class <code>IfWrapper</code> here.
 * This class is a wrapper class for the IfExpression.
 * This was created because of deprecated methods in ROM.
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class IfWrapper {

  private IfExpression ifExpr;
  private LinkedList ifsList;
  private LinkedList thensList;
  private Application elseExpr;
  private int ifThenCount = 0;

  /**
   * Creates a new <code>IfWrapper</code> instance.
   * Primary Constructor.
   * @param ifExprIn an <code>IfExpression</code> value
   */
  public IfWrapper(IfExpression ifExprIn) {
    ifExpr = ifExprIn;
    ifsList = new LinkedList();
    thensList = new LinkedList();
  }

  /**
   * Describe <code>processIfsThens</code> method here.
   * 
   */
  public void processIfsThens() {
    this.process(ifExpr);
  }

  /**
   * Describe <code>process</code> method here.
   * This method calculates the number of if expressions 
   * and populates the ifsList, thensList.
   * @param ifExpression an <code>IfExpression</code> value
   */
  public void process(IfExpression ifExpression) {
    if (ifExpression.getIfValue() != null) {
      ifThenCount++;
      ifsList.add(ifExpression.getIfValue());
      thensList.add(ifExpression.getThenValue());
      Object TmpElseExpr = ifExpression.getElseValue();
      if (TmpElseExpr instanceof IfExpression) {
	IfExpression ElseExpr = (IfExpression)TmpElseExpr;
	process(ElseExpr);
      } // end of if (ElseExpr instanceof IfExpression)
      else {
	elseExpr = (Application)TmpElseExpr;
      } // end of else
    } // end of if (ifExpression.getIfValue() != null)
  }

  /**
   * Describe <code>displayIfs</code> method here.
   * This is used for testing purpose.It just displays the 
   * if statements.
   */
  public void displayIfs() {
    for (int count = 0; count < ifsList.size(); count++) {
      System.out.println("The if class is: "+ifsList.get(count).getClass().getName());
      System.out.println("The if expr is: "+ifsList.get(count).toString());
    } // end of for (int count = 0; count < ifsList.size(); count++)
  }
  /**
   * Describe <code>displayThens</code> method here.
   *
   */
  public void displayThens() {
    for (int count = 0; count < thensList.size(); count++) {
      System.out.println("The then class is: "+thensList.get(count).getClass().getName());
      System.out.println("The then expr is: "+thensList.get(count).toString());
    } // end of for (int count = 0; count < thensList.size(); count++)
  }

  /**
   * Describe <code>displayElse</code> method here.
   *
   */
  public void displayElse() {
    System.out.println("The else class name is: "+elseExpr.getClass().getName());
    System.out.println("The else expr is: "+elseExpr.toString());
  }

  /**
   * Describe <code>ifThenCount</code> method here.
   * This returns the number of ifs the the expression.
   * @return an <code>int</code> value
   */
  public int ifThenCount() {
    return ifThenCount;
  }

  /**
   * Describe <code>getIfs</code> method here.
   * This returns the list of if expressions.
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getIfs() {
    return ifsList;
  }

  /**
   * Describe <code>getThens</code> method here.
   * This returns the list of then expressions.
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getThens() {
    return thensList;
  }

  /**
   * Describe <code>getElse</code> method here.
   * This returns the else expression.
   * @return an <code>Object</code> value
   */
  public Object getElse() {
    return elseExpr;
  }

}
    



    



