//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: ReadRequirements.java,v 1.9 2002/10/22 20:17:55 cvs Exp $
// Revision History:
// ----------------
// $Log: ReadRequirements.java,v $
// Revision 1.9  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.8  2002/10/14 13:10:05  cvs
// Bits need to be handled seperately for generating the vectprs.Changes are made to support them.
// --Murthy.
//
// Revision 1.7  2002/10/08 14:33:59  cvs
// Made the tools compatable to generate the false points in XML format.Added a new file ValueGenerator.java to generate the vectors.This file implements a new logic as compared to the logic in GenerateVectors class.
// --Murthy.
//
// Revision 1.6  2002/10/01 06:31:46  cvs
// Made changes in GenerateVectors appropriate to the changes in the types directory.
// --Murthy
//
// Revision 1.5  2002/09/29 02:43:19  cvs
//  Made changes to print the vectors in XML format.
// --Murthy.
//
// Revision 1.3  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//
package org.sldl.rosetta.tools.generatedata;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.tools.testparser.TestParser;

import java.util.LinkedList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Map;

/**
 * Describe class <code>ReadRequirements</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class ReadRequirements {
  
  private Item packItem;
  private HashMap inputs;
  private LinkedList reqDeclItems;

  /**
   * Creates a new <code>ReadRequirements</code> instance.
   * The Primary Constructor.
   * @param packItemIn an <code>Item</code> value
   */
  public ReadRequirements(Item packItemIn) {
    packItem = packItemIn;
    inputs = new HashMap();
    reqDeclItems = new LinkedList();
  }

  /**
   * Describe <code>getInputs</code> method here.
   * This returns the inputs.
   * @return a <code>HashMap</code> value
   */
  public HashMap getInputs() {
    return inputs;
  }

  /**
   * Describe <code>getReqDeclItems</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getReqDeclItems() {
    return reqDeclItems;
  }

  /**
   * Describe <code>populateInputs</code> method here.
   *
   */
  public void populateInputs() {

    HashMap paramsMap = new HashMap();
    org.sldl.rosetta.ROM.Package pack = (org.sldl.rosetta.ROM.Package)packItem.getValue();
    Iterator packDeclIter = pack.getDeclaredItems().iterator();
    
    while (packDeclIter.hasNext()) {
      Item declItm = (Item)packDeclIter.next();
      
      if (declItm.getValue() instanceof Facet) {
	Facet fct = (Facet)declItm.getValue();

	//To deal with the parameters of the facet.
	ItemList fctParams = fct.getInterfaceItems();
	Iterator fctParamsIter = fctParams.iterator();

	//This is only for testing.This needs to be deleted.
	//System.out.println("####Filling the parameters mode########");
	while (fctParamsIter.hasNext()) {
	  Item paramItem = (Item)fctParamsIter.next();
	  //System.out.println(paramItem.getLabel()+":"+paramItem.getType());
	  paramsMap.put(paramItem.getLabel().toString(),paramItem.getType().toString());
	} // end of while (fctParamsIter.hasNext())
	//System.out.println("####Filling the parameters mode########");

	//To deal with the declared items in the facet.
	ItemList declItmsLst = fct.getDeclaredItems();
	Iterator declItmsIter = declItmsLst.iterator();
	while (declItmsIter.hasNext()) {
	  reqDeclItems.add(((Item)declItmsIter.next()));
	} // end of while (declItmsIter.hasNext())

	//Iterating through the terms of the facet.
	Iterator fctTermsIter = fct.getTermItems().iterator();
	while (fctTermsIter.hasNext()) {
	  LinkedList values = new LinkedList();
	  Item fctTerm = (Item)fctTermsIter.next();
	  Application fctTermApp = (Application)fctTerm.getValue();
	  ExpressionList param = fctTermApp.getParameters();
	  Iterator paramIter = param.iterator();
	  
	  //The first paramter always becomes the key to the 
	  //hashmap.
	  Object keyObject = paramIter.next();
	  //As the class for the keyObject is ItemReference it is 
	  //changed to String.
	  String key = keyObject.toString();

	  //To add the initial,final,increment values of the input
	  //variable to the linkedlist and store it in the hashmap.
	  while (paramIter.hasNext()) {
	    Object next = paramIter.next();
	    values.add(next);
	  } // end of while (paramIter.hasNext())
	  values.addLast(paramsMap.get(key));

	  inputs.put(key,values);
	} // end of while (fctTermsIter.hasNext())
      
      } // end of if (declItm.getValue() instanceof Facet)
    
    } // end of while (packDeclIter.hasNext())
  
  }

  /**
   * Describe <code>displayInputs</code> method here.
   *
   */
  public void displayInputs() {
    Set set = inputs.entrySet();
    Iterator iter = set.iterator();
    while (iter.hasNext()) {
      Map.Entry entry = (Map.Entry)iter.next();
      System.out.print(entry.getKey()+":");

      LinkedList valueList = (LinkedList)entry.getValue();
      Iterator valLstIter = valueList.iterator();
      while (valLstIter.hasNext()) {
	System.out.print(valLstIter.next()+"\t");
      } // end of while (valLstIter.hasNext())
      System.out.println();
    } // end of while (iter.hasNext())
  }
    

  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args) {
    TestParser test = new TestParser();

    Item packItem = test.compileFile(args[0]);
    ReadRequirements req = new ReadRequirements(packItem);
    req.populateInputs();
    req.displayInputs();
  } // end of main ()
}




