//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: ReadScenarios.java,v 1.5 2002/11/11 13:15:59 ksatya Exp $
// Revision History:
// ----------------
// $Log: ReadScenarios.java,v $
// Revision 1.5  2002/11/11 13:15:59  ksatya
// Bugs:
// -- Standard Declarations within a Package are having problems.
// -- Nested Facets are not supported till now.
// Those bugs were fixed.
// --Murthy.
//
// Revision 1.4  2002/10/08 14:33:59  cvs
// Made the tools compatable to generate the false points in XML format.Added a new file ValueGenerator.java to generate the vectors.This file implements a new logic as compared to the logic in GenerateVectors class.
// --Murthy.
//
// Revision 1.3  2002/09/27 05:22:05  cvs
// Implemented the complete functionality of BitVectors.
// --Murthy.
//
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//
package org.sldl.rosetta.tools.generatedata;

import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.ROM.*;
import java.util.*;

/**
 * Describe class <code>ReadScenarios</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class ReadScenarios {

  private Item scenPackItm;
  
  //This variable to used to store the scenarios
  //without any processing.The scenarios are read 
  //from the SCENARIOS file and stored in this variable.
  protected LinkedList scenList;

  //The scenarios are checked for all the conditions 
  //and after getting the final scenarios they are stored here.
  private LinkedList finalScenarios;

  //This variable is used to store the declared items 
  //in the facet.The use of this variable is explained 
  //in GenerateVectors class.
  private LinkedList fctDeclItems;

  /**
   * Describe variable <code>inputVarList</code> here.
   * This variable stores the input variables.
   */
  protected static LinkedList inputVarList;


  /**
   * Describe variable <code>outputVarList</code> here.
   * This variable is used to store the output variables.
   */
  protected static LinkedList outputVarList;
  

  /**
   * Creates a new <code>ReadScenarios</code> instance.
   * Primary Constructor.
   */
  public ReadScenarios() {
    scenList = new LinkedList();
    inputVarList = new LinkedList();
    outputVarList = new LinkedList();
    fctDeclItems = new LinkedList();
  }

  /**
   * Creates a new <code>ReadScenarios</code> instance.
   * Secondary Constructor.
   * @param scenPackIn an <code>Item</code> value
   */
  public ReadScenarios(Item scenPackIn) {
    scenPackItm = scenPackIn;
    scenList = new LinkedList();
    inputVarList = new LinkedList();
    outputVarList = new LinkedList();
    fctDeclItems = new LinkedList();
  }

  
  /**
   * Describe <code>getInputVar</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getInputVar() {
    return inputVarList;
  }
  
  /**
   * Describe <code>getOutputVar</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
  public LinkedList getOutputVar() {
    return outputVarList;
  }

  public LinkedList getScenariosList() {
    return scenList;
  }

  public LinkedList getFctDeclaredItems() {
    return fctDeclItems;
  }

  public void setFinalScenarios(LinkedList finalListIn) {
    finalScenarios = finalListIn;
  }

  public LinkedList getFinalScenariosList() {
    return finalScenarios;
  }
  
  
  /**
   * Describe <code>readScenarios</code> method here.
   * Reads the scenarios and stores them in a linked list.
   */
  public void readScenarios() {
    org.sldl.rosetta.ROM.Package scenPack =(org.sldl.rosetta.ROM.Package)scenPackItm.getValue();
    Iterator scenFctItem = scenPack.getDeclaredItems().iterator();
    while (scenFctItem.hasNext()) {
      Item nxtItem = (Item)scenFctItem.next();
      if (nxtItem.getValue() instanceof Facet) {
	Facet fct = (Facet)nxtItem.getValue();
	
	this.populateIpOpVar(fct);
	
	//To deal with the declared items in the facet.
	ItemList declItmsLst = fct.getDeclaredItems();
	Iterator declItmsIter = declItmsLst.iterator();
	while (declItmsIter.hasNext()) {
	  Item fctDeclItem = (Item)declItmsIter.next();
	  if (fctDeclItem.getValue() instanceof Facet) {
	    //this.populateIpOpVar(fctDeclItem.getValue());
	    processInnerFacet((Facet)fctDeclItem.getValue());
	  } // end of if (fctDeclItem.getValue() instanceof Facet)
	  
	  fctDeclItems.add(fctDeclItem.getLabel().toString());
	} // end of while (declItmsIter.hasNext())
	

	//To deal with the terms in the facet.
	ItemList itmList = fct.getTermItems();
	Iterator itmIter = itmList.iterator();
	while (itmIter.hasNext()) {
	  Item trmItm = (Item)itmIter.next();
	  //if (!(trmItm.getLabel().toString().equals("precondition"))) {
	  LinkedList list = new LinkedList();
	  buildList((Application)trmItm.getValue(),list);
	  scenList.add(list);
	  //}
	} // end of while (paramsIter.hasNext())
      } // end of while (itmIter.hasNext())
    } 
  }
  

  public void processInnerFacet(Facet fct) {
    
    this.populateIpOpVar(fct);
    //To deal with the declared items in the facet.
    ItemList declItmsLst = fct.getDeclaredItems();
    Iterator declItmsIter = declItmsLst.iterator();
    while (declItmsIter.hasNext()) {
      Item fctDeclItem = (Item)declItmsIter.next();
      if (fctDeclItem.getValue() instanceof Facet) {
	//this.populateIpOpVar(fctDeclItem.getValue());
	processInnerFacet((Facet)fctDeclItem.getValue());
      } // end of if (fctDeclItem.getValue() instanceof Facet)
      
      fctDeclItems.add(fctDeclItem.getLabel().toString());
    } // end of while (declItmsIter.hasNext())
    
    
    //To deal with the terms in the facet.
    ItemList itmList = fct.getTermItems();
    Iterator itmIter = itmList.iterator();
    while (itmIter.hasNext()) {
      Item trmItm = (Item)itmIter.next();
      //if (!(trmItm.getLabel().toString().equals("precondition"))) {
      LinkedList list = new LinkedList();
      buildList((Application)trmItm.getValue(),list);
      scenList.add(list);
    }
  }


  /**
   * Describe <code>buildList</code> method here.
   *
   * @param appIn an <code>Application</code> value
   * @param listIn a <code>LinkedList</code> value
   */
  public  void buildList(Application appIn,LinkedList listIn) {
    ExpressionList paramsIn = appIn.getParameters();
    Expression leftParam = paramsIn.get(0);
    Expression rightParam = paramsIn.get(1);
    if (appIn.getFunction().toString().equals("and")) {
      listIn.add(rightParam);
      if ((leftParam instanceof Application) && ((((Application)leftParam).getFunction().toString()).equals("and"))) {
	buildList((Application)leftParam,listIn);
      } // end of if ((rightParam instanceof Application) && ((Application)rightParam.getFunction().toString().equals("and")))
      else {
	listIn.add(leftParam);
      } // end of else
      
    } // end of if (appIn.getFuncion().toString().equals("and"))
    else {
      listIn.add(appIn);
    } // end of else
    
  }
  
  
  
  /**
   * Describe <code>populateIpOpVar</code> method here.
   *
   * @param fct a <code>Facet</code> value
   */
  public void populateIpOpVar(Facet fct) {
    ItemList interList = fct.getInterfaceItems();
    for (Iterator interIter = interList.iterator();interIter.hasNext();) {
      Item interItem = (Item)interIter.next();
      String intrItmMode = (String)interItem.getDecoration("kind").toString();
      if (intrItmMode.equals("in")) {
	inputVarList.add(interItem.getLabel().toString());
      } // end of if (interItem.getMode().toString().equals("in"))
      if (intrItmMode.equals("out")) {
	//output_var = interItem.getLabel().toString();
	outputVarList.add(interItem.getLabel().toString());
      } // end of if (interItem.getMode().toString().equals("out"))
    } // end of for (Iterator interIter = interList.iterator();interIter.hasNext();)
  }
  
  
  /**
   * Describe <code>displayScenarios</code> method here.
   *
   */
  public void displayScenarios() {
    for (Iterator finalIter = scenList.iterator();finalIter.hasNext();) {
      LinkedList innerList = (LinkedList)finalIter.next();
      for (Iterator innerIter = innerList.iterator();innerIter.hasNext();) {
	System.out.println("the value is: "+innerIter.next().toString());
      }
    }
  }
  
  
  /**
   * Describe <code>displayIpOpVar</code> method here.
   *
   */
  public void displayIpOpVar() {
    for (Iterator ipIter = inputVarList.iterator();ipIter.hasNext();) {
      System.out.print("ipVar: "+ipIter.next().toString()+"    ");
    } // end of for (Iterator ipIter = inputVarList.iterator();ipIter.hasNext();;)
    System.out.println();
    for (Iterator opIter = outputVarList.iterator();opIter.hasNext();) {
      System.out.print("opvar: "+opIter.next().toString()+"   ");
    } // end of for (Iterator opIter = outputVarList.iterator();opIter.hasNext();;)
    System.out.println();
  }


  /**
   * Describe <code>displayDeclaredItems</code> method here.
   *
   */
  public void displayDeclaredItems() {
    Iterator declIter = fctDeclItems.iterator();
    while (declIter.hasNext()) {
      System.out.println("Decl: "+declIter.next()+" ");
    } // end of while (declIter.hasNext())
  }
  
  /**
   * Describe <code>main</code> method here.
   *
   * @param args a <code>String[]</code> value
   */
  public static void main (String[] args) {
    
    TestParser test = new TestParser();
    Item scenPackItem = test.compileFile(args[0]);

    ReadScenarios scen = new ReadScenarios(scenPackItem);
    scen.readScenarios();
    
    //for Testing purpose.
    scen.displayIpOpVar();
    scen.displayScenarios();
    scen.displayDeclaredItems();

    //****************************************************//
    //Used to generate the false points in xml format.
    


  } // end of main ()
}  
    
