//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: ReadXmlRequirements.java,v 1.1 2002/11/06 05:55:55 ksatya Exp $
//
// Revision History:
// $Log: ReadXmlRequirements.java,v $
// Revision 1.1  2002/11/06 05:55:55  ksatya
// Implemented the initial functionality of reading the requirements from an XML file.Needs to be thoroughly tested...
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;

import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

/**
 * Describe class <code>ReadXmlRequirements</code> here.
 * This class is reads the requirements specified in XML.
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class ReadXmlRequirements {

  static Document doc;
  private static HashMap paramsMap;
  private static HashMap valuesMap;
  private String filename;


  public ReadXmlRequirements(String filenameIn) {
    filename = filenameIn;
    paramsMap = new HashMap();
    valuesMap = new HashMap();
  }


  public HashMap getInputs() {
    return valuesMap;
  }
    
  public void populateInputs() {

    try {
      DocumentBuilderFactory dFactory =  DocumentBuilderFactory.newInstance(); 
      DocumentBuilder dBuilder = dFactory.newDocumentBuilder(); 
      File file = new File(filename); 
      doc = dBuilder.parse(file);

    } catch (ParserConfigurationException pce) {
      // Parser with specified options can't be built
      pce.printStackTrace();

    } catch (SAXException sxe) {
      // Error generated during parsing
      Exception  x = sxe;
      if (sxe.getException() != null)
	x = sxe.getException();
      x.printStackTrace();

    } catch (IOException ioe) {
      // I/O error
      ioe.printStackTrace();
    } // end of catch

    Element root = doc.getDocumentElement();
    System.out.println("The root is: "+root);

    //This gets the first set of children
    NodeList children = root.getChildNodes();
    
    //This iterates thru the interfaces,initialisations parts of the children.
    for (Node child = root.getFirstChild(); child != null; child = child.getNextSibling()) {

      //This check should be performed always to look only for an ELEMENT_NODE.
      if (child.getNodeType() != Node.TEXT_NODE) {
	System.out.println("The child name is: "+child.getNodeName()+" Value: "+child.getNodeValue());	 
	
	//To access all the subchildren of a node.
	if (child.hasChildNodes()) {
	   
	  for (Node grandChild = child.getFirstChild(); grandChild != null;
	       grandChild = grandChild.getNextSibling()) {
	    
	    if (grandChild.getNodeType() != Node.TEXT_NODE) {
	      System.out.println("The Grand child is: "+grandChild.getNodeName());
	    
	      if (grandChild.getNodeName().equalsIgnoreCase("parameter")) {

		NamedNodeMap paramAttr = grandChild.getAttributes();
		String parameter = null;
		String type = null;
		for (int index = 0; index < paramAttr.getLength(); index++) {
		  Node param_attr = paramAttr.item(index);
		  System.out.println("The grand child Attr: "+param_attr.getNodeName()+" Value: "+param_attr.getNodeValue());
		  if (param_attr.getNodeName().equalsIgnoreCase("name")) {
		    parameter = param_attr.getNodeValue();
		  } // end of if (param_attr.getNodeName().equalsignoreCase("name"))
		  if (param_attr.getNodeName().equalsIgnoreCase("type")) {
		    type = param_attr.getNodeValue();
		  } // end of if (param_attr.getNodeName.equalsIgnoreCase("type"))
		} // end of for (int index = 0; index < paramAttr.getLength(); index++)
		paramsMap.put(parameter,type);

	      } // end of if (grandChild.getNodename().equalsignoreCase("parameter"))
	      
	      //Walk thru all the children and populate the values in valuesMap.
	      if (grandChild.getNodeName().equalsIgnoreCase("function")) {

		LinkedList valuesList = new LinkedList();
		String key = null;

		for (Node grand_grand_child = grandChild.getFirstChild(); grand_grand_child != null; grand_grand_child = grand_grand_child.getNextSibling()) {
		  
		  if (grand_grand_child.getNodeType() != Node.TEXT_NODE) {

		    //This part is used grab the key for the valuesMap.
		    if (grand_grand_child.getNodeName().equalsIgnoreCase("parameter")) {
		      key = grand_grand_child.getFirstChild().getNodeValue();
		      System.out.println("The key value is: "+key);
		    } // end of if (grand_grand_key.getNodeName().equalsIgnoreCase("parameter"))

		    else {
		      System.out.println("The value added is: "+grand_grand_child.getFirstChild().getNodeValue());
		      valuesList.add(grand_grand_child.getFirstChild().getNodeValue());
		    } // end of else
		  } // end of if (grand_grand_child.getNodeType() != Node.TEXT_NODE)
		} // end of for (Node grand_grand_child = grandChild.getFirstChild(); grand_grand_child

		valuesList.addLast(paramsMap.get(key));
		valuesMap.put(key,valuesList);
									   
		
	      } // end of if (grandChild.getNodeName().equalsIgnoreCase("function"))
	      
	    } // end of if (grandChild.getNodeType != Node.TEXT_NODE)
	    
	  } // end of for (Node grandChild = child.getFirstChild(); grandChild != null;
	  
	} // end of if (child.hasChildNodes())
	
      } // end of if (child.getNodeType() != TEXT_NODE)
    } // end of for (Node child = root.getFirstChild(); child != null; child = child.getNextSibling())
    
  }

  public static void printMap() {
    System.out.println("##########################################################");
    System.out.println("Displaying the values in valuesMap");
    Set map_set = valuesMap.entrySet();
    Iterator setIter = map_set.iterator();
    while (setIter.hasNext()) {
      Map.Entry setEntry = (Map.Entry)setIter.next();
      System.out.println("Name: "+setEntry.getKey()+" Value: "+setEntry.getValue());
    } // end of while (setIter.hasNext())
    System.out.println("##########################################################");
  }

}
