//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
//  $Id: RedundancyRemoving.java,v 1.6 2002/11/11 17:29:42 ksatya Exp $
//
// Revision History:
// $Log: RedundancyRemoving.java,v $
// Revision 1.6  2002/11/11 17:29:42  ksatya
// Removed DataLiteral from RedundancyRemoving and replaced it with NumericLiterl.
// --Murthy
//
// Revision 1.5  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//
//
//=============================================================================

package org.sldl.rosetta.tools.generatedata;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.tools.multivisitor.MultiVisitor;

import java.lang.*;
import java.util.*;
import java.util.Iterator;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.FileNotFoundException;


/**
 * Describe class <code>RedundancyRemoving</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Satyanarayana Kakarlamudi</a>
 * @version 1.0
 */
public class RedundancyRemoving {


  private LinkedList exprList = null;
  private Expression expr;
  private BooleanTerm boolTerm;
  
  /**
   * The primary constructor for DVTG_Facet
   */
  public RedundancyRemoving(){
		super();
		exprList = new LinkedList();
	}

  /**
   * Creates a new <code>RedundancyRemoving</code> instance.
   *
   * @param exprListIn an <code>ExpressionList</code> value
   */
  public RedundancyRemoving(LinkedList exprListIn){
    super();
    exprList = exprListIn;
  }

  public RedundancyRemoving(Expression exprIn)
  {
    expr = exprIn;
  }

  public RedundancyRemoving(BooleanTerm booltermIn){
    super();
    boolTerm = booltermIn;
  }


  /**
   * Describe <code>setExpressionList</code> method here.
   *
   * @param exprListIn an <code>ExpressionList</code> value
   */
  public void setExpressionList(LinkedList exprListIn){
    exprList = exprListIn;
  }

  /**
   * Describe <code>getExpressionList</code> method here.
   *
   * @return an <code>ExpressionList</code> value
   */
  public LinkedList getExpressionList(){
    return exprList;
  }


  // the following redundancy removing should remove all the
  // redundant. 
  // Zhongjun Wang, April 23, 2002
  //
  public LinkedList redundancyRemove2() {
    
    LinkedList finalList = new LinkedList();
    // initialize the final linked list
    //
    for (int i = 0; i< (boolTerm.size()); i++)
      finalList.add( (Application)((BooleanVariable)boolTerm.get(i)).getExpression() );


    int isRedundant = 0;
  
    for (int i=0; i<(boolTerm.size());i++) {
      Application leftApp = (Application)((BooleanVariable)boolTerm.get(i)).getExpression();
      ExpressionList leftParamsList = leftApp.getParameters();
      Iterator leftParamIter = leftParamsList.iterator();
      String leftLeftOp = leftParamIter.next().toString();
      String leftOperator = leftApp.getFunction().toString();
      String leftRightOp = leftParamIter.next().toString();
      System.out.println("leftOp: "+leftLeftOp+" Oper: "+leftOperator+" rightOp: "+leftRightOp);

      //
      // testing if the i-th expression is redundant, if so, it is
      // removed from the finalList.
      //
      for (int j=1; j<(boolTerm.size()); j++) {

	if ( i == j) 
	  continue;

	Application rightApp = (Application)((BooleanVariable)boolTerm.get(j)).getExpression();
	ExpressionList rightParamsList = rightApp.getParameters();
	Iterator rightParamIter = rightParamsList.iterator();
	String rightLeftOp = rightParamIter.next().toString();
	String rightOperator = rightApp.getFunction().toString();
	String rightRightOp = rightParamIter.next().toString();
	System.out.println("leftOp: "+rightLeftOp+"Oper:"+rightOperator+" rightOp: "+rightRightOp);

	if (leftLeftOp.equals(rightLeftOp)) {
	  if ( ((leftOperator.equals("<")) || (leftOperator.equals("=<")) )
	       && ( (rightOperator.equals("<")) || ( rightOperator.equals("=<")))) {
	    Float leftFloat = new Float(leftRightOp);
	    System.out.println("The left float is: "+leftFloat.floatValue());
	    Float rightFloat = new Float(rightRightOp);
	    System.out.println("The right float is: "+rightFloat.floatValue());
	    
	    if (leftFloat.floatValue() > rightFloat.floatValue()) {
	      finalList.remove(leftApp);
	    }
	    
	    if ((leftFloat.floatValue() ==  rightFloat.floatValue()) 
		&& (leftOperator.equals("=<")) ) {
	      finalList.remove(leftApp);
	    } // end of else
	  }

	  if (((leftOperator.equals(">"))||leftOperator.equals(">="))
	      && ((rightOperator.equals(">"))|| (rightOperator.equals(">=")))) {
	    System.out.println("I am in this condition");
	    Float leftFloat = new Float(leftRightOp);
	    Float rightFloat = new Float(rightRightOp);
	    
	    if (leftFloat.floatValue() < rightFloat.floatValue()) {
	      finalList.remove(leftApp);
	    }	 

	    if ((leftFloat.floatValue() ==  rightFloat.floatValue()) 
		&& (rightOperator.equals(">=")) ) {
	      finalList.remove(leftApp);
	    } // 
	    
	  }
	}
      }
    }
    return finalList;  //redundancy has been removed 
    
  }

  //Written by Murthy for a simple case.
  public LinkedList redundancyRemoval() {
    LinkedList finalList = new LinkedList();
    if (boolTerm.size() ==1) {
      Application app  = (Application)((BooleanVariable)boolTerm.get(0)).getExpression();
      finalList.add(app);
    }
    else {
      for (int i=0; i<=(boolTerm.size()-1);i++) {
	for (int j=i+1;j<(boolTerm.size());j++) {
	  Application leftApp = (Application)((BooleanVariable)boolTerm.get(i)).getExpression();
	  ExpressionList leftParamsList = leftApp.getParameters();
	  Iterator leftParamIter = leftParamsList.iterator();
	  String leftLeftOp = leftParamIter.next().toString();
	  String leftOperator = leftApp.getFunction().toString();
	  String leftRightOp = null;
	  if (!(leftOperator.equals("%"))) {
	     leftRightOp = leftParamIter.next().toString();	     
	  } // end of if (!(leftOperator.equals("%"))
	  Application rightApp = (Application)((BooleanVariable)boolTerm.get(j)).getExpression();
	  ExpressionList rightParamsList = rightApp.getParameters();
	  Iterator rightParamIter = rightParamsList.iterator();
	  String rightLeftOp = rightParamIter.next().toString();
	  String rightOperator = rightApp.getFunction().toString();
	  String rightRightOp = rightParamIter.next().toString();
	  
	  if (leftLeftOp.equals(rightLeftOp)) {
	    if (leftOperator.equals("<") && rightOperator.equals("<")) {
	      Float leftFloat = new Float(leftRightOp);
	      Float rightFloat = new Float(rightRightOp);
	      if (leftFloat.floatValue() < rightFloat.floatValue()) {
		finalList.add(leftApp);
	      }
	    }
	    if (((leftOperator.equals(">"))||leftOperator.equals(">=")) && ((rightOperator.equals(">"))||rightOperator.equals(">="))){
	      Float leftFloat = new Float(leftRightOp);
	      Float rightFloat = new Float(rightRightOp);
	      
	      if (leftFloat.floatValue() < rightFloat.floatValue()) {
		finalList.add(rightApp);
		break;
	      }
	    }
	    if (((leftOperator.equals(">"))||(leftOperator.equals(">="))) &&
		((rightOperator.equals("<")) || (rightOperator.equals("=<")))) 

	      {
		finalList.add(leftApp);
	      }
	  }
	  else {
	    if (!(finalList.contains(leftApp))) {
	      finalList.add(leftApp);
	    } 
	  }
	}
      }
      finalList.add((Application)((BooleanVariable)boolTerm.getLast()).getExpression());
    }
    return finalList;
  }    
    



  /**
   * The scenarios that are generated for each term expression are
   *added to a expression list. These are printed out to the
   *scenarios file. Each scenario has subExpressions combined by the
   "and" operator. The redundancy is removed and then printed
   * @return void
   */
  public LinkedList managerRedundancyRemove(){

    //Application app = (Application)expr;
    //ExpressionList paramList = app.getParameters();
    
    if (exprList != null){
      System.out.println("I am into manager Redundancy");
      //Iterator exprIter = exprList.iterator();
      LinkedList newList = null;
      //if (exprIter != null){
      //int count;
      //count = 0;
      //while (exprIter.hasNext()){
	  	  
      //  Expression exprElement =
      //    (Expression)((BooleanVariable)exprIter.next()).getExpression();
      //  System.out.println("The class name is: "+((Object)exprElement).getClass().getName());
      //  LinkedList subList = new LinkedList();
	  //
	  //get the simplest expression,
	  //i.e. simplify the expression of every test scenario
	  //
	  
	  //this.getSubExpressions((Expression)exprElement,subList);
	  //there could be an error in the remove redundancy
	  //check it with perry removing redundancy from
	  // every test scenario
      System.out.println("The original size is: "+exprList.size());
	  newList = this.removeRedundancy(exprList);
	  if (newList != null) {
	    System.out.println("The final list size is: "+newList.size());
	  } // end of if (finalList != null)
	  return newList;
    }
    else {
      return null;
    } // end of else
    
  }
 

  //    pw.print("END ");
  //pw.println(";");


    
  
  
  /** To remove the redundancy in the subexpressions of a scenario
   * @param List the sub expressions list
   * @return List the list of subexpressions after removing the
   redundancy
  */
  public static  LinkedList removeRedundancy(LinkedList exprList){
    ListIterator exprIter = exprList.listIterator();
    LinkedList ret = new LinkedList();
    if (exprIter != null){
      while(exprIter.hasNext()){
	Expression expr = (Expression)exprIter.next();
	String oper = null;
	Expression left = null;
	Expression right = null;
	int yes = 3;
	if (expr instanceof Application){
	  oper = ((Application)expr).getFunction().toString();
	  ExpressionList params = ((Application)expr).getParameters();
	  Iterator paramsIter = params.iterator();
	  left =(Expression)paramsIter.next();
	  System.out.println("The left Op is: "+left.toString());
	  System.out.println("The op is: "+oper);
	  right =(Expression) paramsIter.next();
	  System.out.println("The right Op is: "+right.toString());
	  // the foolowing will not be
	  // considerated temporally
	  if ((left == null) || (right == null)){

	  }else
	    {
	      yes =
		checkExpression(exprList,oper,left,right,exprIter.nextIndex());
	    }
	}
	else
	  {
	    System.out.println("");
	  }
	if (yes==1)
	  ret.add(expr);
	else if (yes == 2)
	  return null;
      }
   
    }
    return ret;
  }

  
  
  /** The function returns 2 if the expression is redundant when
   *	compared to the other expressions in the List else it returns
   *1.
   * @param List list of expressions
   * @param String operator of the expression to test
   * @param Expression left the left operand of expression tested
   * @param Expression right the right operand of expression tested
   * @param int index of the expression tested in the list
   * @return int result
   */
  //
  // revised on April 20, 2002
  //
  public static int  checkExpression(LinkedList exprList,String oper,Expression leftexpr,
			       Expression rightexpr,int index){
    
    ListIterator exprIter = exprList.listIterator();
    String left = null;
    String right = null;
    if (rightexpr != null)
      right = rightexpr.toString();
    if (leftexpr != null)
      left = leftexpr.toString();
    int result = 1;
    
    Object rightVal = getValue(rightexpr);
    
    if (exprIter != null){
      while(exprIter.hasNext()){
	Expression expr = (Expression)exprIter.next();
	String checkoper = null;
	String checkleft = null;
	String checkright = null;
	Object checkexpr = null;
	if (expr instanceof Application){
	  ExpressionList params = ((Application)expr).getParameters();
	  Iterator paramsIter = params.iterator();
	  checkleft = ((Expression)paramsIter.next()).toString();
	  checkright = ((Expression)paramsIter.next()).toString();
	  //checkexpr = getValue(((Application)expr).getRightOp());

	  checkoper =
	    ((Application)expr).getFunction().toString();
	  //if ((checkleft != null) &&
	  //  (checkright != null)){
	  //checkleft = ( ((Application)expr).getLeftOp()).toString();
	  //checkright = ( ((Application)expr).getRightOp()).toString();
	  //}
	}
	int thisIndex = exprIter.nextIndex();
	if ((left.equals(checkleft)) && (index != thisIndex)){
	  int subresult =1;
	  if (oper.equals("=="))
	    subresult =
	      checkNotEqual(checkoper,rightVal,checkexpr);
	  else if (oper.equals("/="))
	    subresult= checkNotEqual(checkoper,rightVal,checkexpr);
	  else if ((oper.equals(">")) || (oper.equals(">=")))
	    subresult= checkGreat(checkoper,rightVal,checkexpr);
	  else if ((oper.equals("<")) || (oper.equals("=<")))
	    subresult= checkLess(checkoper,rightVal,checkexpr);
	  if ((subresult ==0)||(subresult == 2))
	    return subresult;
	}
      }
    }
    return result;
  }
  
  /**Function to return the value of Expression if the expression is
   *a DataLiteral
   * @param Expression
   * @return Object
   */
  public static Object getValue(Expression expr){
    //if (expr instanceof DataLiteral){
    if (expr instanceof NumberLiteral){
      //return ((DataLiteral)expr).getNumericData();
      return ((NumberLiteral)expr).getData();
    }else 
      return null;
  }
	
	
  /**To remove the recundancy when a "/=" operator is encountered
   * @param String
   * @param Object this expression value
   * @param Object to check
   * @return int
   */
  public static int checkNotEqual(String oper,Object exprVal,Object checkVal){
    int result = 1;
    if ((exprVal instanceof Integer)&&(checkVal instanceof Integer)){
      int expr = ((Integer)exprVal).intValue();
      int check = ((Integer)checkVal).intValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if (expr < check) result =0;
      if ((oper.equals("<")) || (oper.equals("=<")))
	if (expr > check) result =0;
    }else if ((exprVal instanceof Float)&&(checkVal instanceof Float)){
      float expr = ((Float)exprVal).floatValue();
      float check = ((Float)checkVal).floatValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if (expr < check) result =0;
      if ((oper.equals("<")) || (oper.equals("=<")))
	if (expr > check) result =0;
    }
    return result;
  }
  
  
  /**To remove the recundancy when a ">" operator is encountered
   * @param String
   * @param Object this expression value
   * @param Object to check
   * @return int
   */
  public static int checkGreat(String oper,Object exprVal,Object checkVal){
    int result =1 ;
    
    System.out.println(" in checkGreat operator: "+oper+" exprVal: "+exprVal);
    
    if ((exprVal instanceof Integer)&&(checkVal instanceof Integer)){
      int expr = ((Integer)exprVal).intValue();
      int check = ((Integer)checkVal).intValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if (expr < check) result =0;
      if ((oper.equals("<"))  || (oper.equals("=<")))
	if (check < expr) result =2;
    }
    else if ((exprVal instanceof Float)&&(checkVal instanceof Float)){
      float expr = ((Float)exprVal).floatValue();
      float check = ((Float)checkVal).floatValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if (expr < check) result =0;
      if ((oper.equals("<"))  || (oper.equals("=<")))
	if  (check < expr) result =2;
    }
    return result;
  }
  
  
  /**To remove the recundancy when a "<" operator is encountered
   * @param String
   * @param Object this expression value
   * @param Object to check
   * @return int
   */
  public static int checkLess(String oper,Object exprVal,Object checkVal){
    int result = 1;
    
    if ((exprVal instanceof Integer)&&(checkVal instanceof
				       Integer)){
      int expr = ((Integer)exprVal).intValue();
      int check = ((Integer)checkVal).intValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if (expr < check) result=2;
      if ((oper.equals("<"))  || (oper.equals("=<")))
	if (check < expr) result =0;
    }
    else if ((exprVal instanceof Float)&&(checkVal instanceof
					  Float)){
      float expr = ((Float)exprVal).floatValue();
      float check = ((Float)checkVal).floatValue();
      if ((oper.equals(">")) || (oper.equals(">=")))
	if  (expr < check) result=2;
      if ((oper.equals("<"))  || (oper.equals("=<")))
	if(check > expr) result=0;
    }
    return result;
  }
  
 
  
  /** Each scenario generated has subExpressions that are combined
   *by the "and" operator. Each subExpression is added to the List
   * @param Expression the scenario
   * @param List the list in which all the subexpressions are added
   * @return void
   */
  
  //
  //this code has been modified to accept Expression and Application
  //Zhongjun Wang, April 18, 2002
  //
  //  public static void getSubExpressions(Expression expr,LinkedList ret){
    
//      if (expr instanceof Application){
//        String oper = ((Application)expr).getFunction().toString();
//        ExpressionList parameters = ((Application)expr).getParameters();
//        if (oper.equals("and")){
//  	Expression left = ((Application)expr).getLeftOp();
//  	Expression right = ((Application)expr).getRightOp();
	
//  	//
//  	// make sure that the following are correct.!!!
//  	// 
//  	Iterator paramsIter = parameters.iterator();
//  	String leftStr =paramsIter.next().toString();
//  	String rightStr =paramsIter.next().toString();
//  	if ((leftStr.indexOf(" and "))>0){
//  	  getSubExpressions(left,ret);
//  	}else{
//  	  ret.add(left);
//  	}
//  	if ((rightStr.indexOf(" and "))>0){
//  	  getSubExpressions(right,ret);
//  	}else{
//  	  ret.add(right);
//  	}
//        }else
//  	ret.add(expr);
//      }else
//        ret.add(expr);
//      return;
//    }


}// end of class "DVTG_Facet"



