//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
// Revision History:
// $Log: SimplexOpti.java,v $
// Revision 1.4  2002/11/11 17:58:33  ksatya
// Removed DataLiteral and added NumberLiteral
// --Murthy.
//
// Revision 1.3  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.lang.*;
import java.util.*;

import org.sldl.rosetta.ROM.*;

/**
 * Describe class <code>SimplexOpti</code> here.
 *
 * Created: Mon Mar  4 09:22:35 2002
 *
 * @author <a href="mailto:zwang@kirk.ittc.ku.edu">Zhongjun Wang</a>
 * @version 1.0
 */
public class SimplexOpti extends Object implements SimplexFunc {

  private String name;
  private FormulaList relationExp;
  private LinkedList dnfSolution;
  
  //a list of variables and their corresponding ranges
  private LinkedList variableList;  


  public SimplexOpti(){
  
  }  
  
  public SimplexOpti(String nameInput, FormulaList expInput,
		     LinkedList solutionInput, LinkedList variableInput) {
  
    name = nameInput;
    relationExp = expInput;
    dnfSolution = solutionInput;
    variableList = variableInput;

  }

  
  /**
   * Describe <code>main</code> method here.
   *
   * @param args[] a <code>String</code> value
   */
  public static void main (String args[]) {
    
    double x[] = new double[5];
    double dx[] = new double[5];

    LinkedList variables = new LinkedList();

    NumericalVariable a = new NumericalVariable("x", 0.0, 0.0, 10.0);
    NumericalVariable b = new NumericalVariable("y", 0.0, 0.0, 10.0);
  
    variables.add(a);
    variables.add(b);

    Label labge = new Label(">=");
    Item tstItem = new Item(labge,null,null,null);
    ItemReference tstItmRefGe = new ItemReference(tstItem);
    
    Label labeq = new Label("=");
    Item tstItemeq = new Item(labeq,null,null,null);
    ItemReference tstItmRefEq = new ItemReference(tstItemeq);
    
    ExpressionList params = new ExpressionList();
    params.add(new ItemReference("x"));
    //params.add(new DataLiteral(new Double(5.0)));
    params.add(new NumberLiteral(new Double(5.0)));
    Application app2 = new Application(tstItmRefEq, params);

    params = new ExpressionList();
    params.add(new ItemReference("x"));
    //params.add(new DataLiteral(new Double(3.0)));
    params.add(new NumberLiteral(new Double(3.0)));
    Application app3 = new Application(tstItmRefEq, params);

       
    FormulaList formula = new FormulaList();

    formula.add(app2);
    formula.add(app3);

    //    SimplexOpti simplexTest = new SimplexOpti("",formula,null);
  
    
    System.out.print("\n\n*********Test of Simplex*********\n\n");
    
    
    x[0] = x[1] = x[2] = x[3] = x[4] = 1.0;
    dx[0] = dx[1] = dx[2] = dx[3] = dx[4] = 0.1;
    
    double sigma = -1;
    int mm = 1000;
    //int n = 2;

    int n =variables.size();


    // double fmin = SimplexMethod.SmplxOne(n,x,simplexTest,dx,sigma,mm);
    
    //    System.out.print("\nThe f value is " + fmin + "\n");
    
    System.out.print("\nThe x vector is \n\n");
    
    System.out.print(x[0] + "  " + x[1] + "  " + x[2] + "  " +
		     x[3] + "\n");
    
    
    System.out.println("\n");
    
  }
  
  /**
   * Describe <code>costFunc</code> method here.
   *
   * @param x[] a <code>double</code> value
   * @return a <code>double</code> value
   */
  public double costFunc(double x[]) {
    
    double f;
    double fuser;
    f = 0.0;
    fuser = 0.0;

     for ( int i = 0; i < relationExp.size(); i++)
       {
	 Application expres =(Application) relationExp.get(i);
	 ExpressionList parameters = expres.getParameters();

	 if ( expres.getFunction().toString() == ">=")
	   {
	     // f = f + Math.exp (- (x[i] -
	     //		((DataLiteral)parameters.get(0)).getDoubleValue() ) );

	     f = f + Math.exp (- (x[i] -((NumberLiteral)parameters.get(0)).getData().doubleValue() ));
	   }
	 else if ( expres.getFunction().toString() == "=")
	   {
	     //f = f + Math.exp ( Math.abs( x[i] -
	     //		((DataLiteral)parameters.get(1)).getDoubleValue() ) );
	     f = f + Math.exp ( Math.abs( x[i] -
					  ((NumberLiteral)parameters.get(1)).getData().doubleValue() ));
	   }
       }

     //
     // constraints on the ranges of variables [lowerBound, upperBound]
     //
     for (int i = 0; i < variableList.size(); i++)
       {
	 NumericalVariable xyz = (NumericalVariable)
	   variableList.get(i);

	 fuser = fuser + Math.exp ( - (x[i] - xyz.getLowerBound() ) )
	   + Math.exp ( - (xyz.getUpperBound() - x[i] ) );
	   
       }

     return f+fuser;

     /*
      *   double f,f1,f2,f3,f4,f5,f6;
      *   
      
      *  f1 = Math.exp ( - (x[0] - 5.0) );
      
      *  f2 = Math.exp ( Math.abs(x[0] - x[1]) ) -1.0;
      
      *  f3 = Math.exp( -(x[0] -0.0) );
      *  f4 = Math.exp( -(7.0 - x[0]) );
      
      *  f = f1 + f2 + f3 + f4;
      
      *  return f;
      */   
    
  }
  
}
