//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: ValueGenerator.java,v 1.13 2002/11/18 11:57:46 ksatya Exp $
//
// Revision History:
// ----------------
// $Log: ValueGenerator.java,v $
// Revision 1.13  2002/11/18 11:57:46  ksatya
// Removed the debugging statements from all the files.
// --Murthy.
//
// Revision 1.12  2002/11/13 01:53:02  ksatya
// Added changes to generate scenarios for inner facets in a facet and multiple for multiple facets in a Package.
// --Murthy.
//
// Revision 1.11  2002/11/11 13:15:59  ksatya
// Bugs:
// -- Standard Declarations within a Package are having problems.
// -- Nested Facets are not supported till now.
// Those bugs were fixed.
// --Murthy.
//
// Revision 1.9  2002/10/22 20:17:55  cvs
// Made some minor changes while printing the scenarios file.Included all the conditions in the terms in parenthesis.
// --Murthy.
//
// Revision 1.7  2002/10/17 04:27:48  cvs
// Made minor changes to support all the examples.
// --Murthy.
//
// Revision 1.6  2002/10/14 20:23:45  cvs
// Made changs to iterate through bits.
// --Murthy.
//
// Revision 1.5  2002/10/14 13:10:06  cvs
// Bits need to be handled seperately for generating the vectprs.Changes are made to support them.
// --Murthy.
//
// Revision 1.3  2002/10/09 09:33:52  cvs
// Added support to handle multiple inputs.
// --Murthy.
//
// Revision 1.2  2002/10/08 22:21:10  cvs
// Made changes to support XML format for false points.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Set;
import java.util.Map;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.FileNotFoundException;

//Stuff for jython(JPython).
import org.python.util.PythonInterpreter; 
import org.python.core.*;

import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;

/**
 * Describe class <code>ValueGenerator</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class ValueGenerator {

  //private LinkedList scenarios;
  private ReadScenarios scenarios;
  private ReadRequirements reqs;
  private LinkedList outputCondition;

  //This variable is used to store the values of the
  //variables in the previous state.
  HashMap previous_state;

  /**
   * Creates a new <code>ExampleOp</code> instance.
   *
   * @param scenIn a <code>LinkedList</code> value
   */
  public ValueGenerator(ReadScenarios scenIn,ReadRequirements reqIn) {
    scenarios = scenIn;
    reqs = reqIn;
    previous_state = new HashMap();
  }

  /**
   * Describe <code>generateOutputCondition</code> method here.
   * This method is used to iterate through all the the terms
   * and grab all the conditions that contain the output variables.
   */
  public void generateOutputCondition(){
    outputCondition = new LinkedList();
    LinkedList outputVariables = scenarios.getOutputVar();
    //System.out.println("The output variables are: "+outputVariables);

    LinkedList finalScenarios = scenarios.getFinalScenariosList();
    for (Iterator finalScenIter = finalScenarios.iterator(); finalScenIter.hasNext();) {
      LinkedList innerList = (LinkedList)finalScenIter.next();
      
      for (Iterator innerScenIter = innerList.iterator(); innerScenIter.hasNext();) {
	Application app = (Application)innerScenIter.next();
	//System.out.println("The app is: "+app);
	String leftOp_tmp = ((ExpressionList)app.getParameters()).get(0).toString();
	
	//This is used to check if there is any state associated with the 
	//output variable.In some cases the condition involves the output variable 
	//in the next state and hence it may not be stored.so "initialState(blah) 
	//returns the variable in initialState.
	String leftOp = initialState(leftOp_tmp);
	//System.out.println("The left Op is: "+leftOp);
	for (Iterator optVarIter = outputVariables.iterator(); optVarIter.hasNext();) {
	  String opt_var = optVarIter.next().toString();
	  if (leftOp.equalsIgnoreCase(opt_var)) {
	    //System.out.println("The boolean value is: "+outputCondition);
	    //System.out.println("The app is: "+app);
	    if (!(containsCondition(outputCondition,app))) {
	      outputCondition.add(app);	       
	    } // end of if (outputCondition.contains(app))
	  } // end of if (leftOp.equalsIgnoreCase("opt_var"))
	  
	} // end of for (Iterator optVarIter = outputVariables.iterator(); optVarIter.hasNext();)
	
      } // end of for (Iterator innerScenIter = innerList.iterator(); innerScenIter.hasNext())
      
    } // end of for (Iterator finalScenIter = finalScenarios.iterator(); finalScenIter.next();)
    //  System.out.println("#########################################################");
//      System.out.println("Testing the output conditions");
//      for (Iterator condIter = outputCondition.iterator(); condIter.hasNext();) {
//        System.out.println(condIter.next());
//      } // end of for (Iterator condIter = outputCondition.iterator(); condIter.hasNext();)
//      System.out.println("#########################################################");
    
  }

  /**
   * Describe <code>containsCondition</code> method here.
   * This method checks to see if the condition being added 
   * to the datastructure already contains the condition.
   * @param listIn a <code>LinkedList</code> value
   * @param appIn an <code>Application</code> value
   * @return a <code>boolean</code> value
   */
  public boolean containsCondition(LinkedList listIn, Application appIn){
    for (Iterator listIter = listIn.iterator();listIter.hasNext();) {
      Application optApp = (Application)listIter.next();
      if ((optApp.toString()).equalsIgnoreCase(appIn.toString())) {
	return true;
      } // end of if ((optApp.toString()).equalsIgnoreCase(appIn.toString()))
    } // end of for (Iterator listIter = listIn.iterator();listIter.hasNext();)
    return false;
  }
  
  /**
   * Describe <code>generateOpValue</code> method here.
   * This method is called from outer classes.This basically
   * evaluates the terms.
   * @param valueObjIn a <code>HashMap</code> value
   * This contains the values of all the input variables 
   * in all the ietartions.
   * @param textPw a <code>PrintWriter</code> value
   * PrintWriter used to vectors in a text file.
   * @param xmlPwIn a <code>PrintWriter</code> value
   * PrintWriter used to print the vectors in an XML file.
   * @exception FileNotFoundException if an error occurs
   */
  public void generateOpValue(HashMap valueObjIn,PrintWriter textPw,PrintWriter xmlPwIn)  throws FileNotFoundException {
    xmlPwIn.println("<vector>");
    Set value_set = valueObjIn.entrySet();
    Iterator value_set_iter = value_set.iterator();
    while (value_set_iter.hasNext()) {

      //System.out.println("I am into the generateOpValue while loop");
      Map.Entry value_entry = (Map.Entry)value_set_iter.next();
      textPw.print(value_entry.getKey()+" = "+value_entry.getValue()+"\t");
      //textPw.flush();
      xmlPwIn.println("<condition>");
      xmlPwIn.println("\t<parameter mode = \"in\">"+value_entry.getKey()+" </parameter>");
      xmlPwIn.println("\t<value> "+value_entry.getValue()+" </value>");
      xmlPwIn.println("</condition>");
    } // end of while (value_set_iter.hasNext())
    


    LinkedList finalScenarios = scenarios.getFinalScenariosList();
    //LinkedList finalScenarios = scenarios.getScenariosList();

    for (Iterator outerIter = finalScenarios.iterator(); outerIter.hasNext();) {
      
      LinkedList innerList = (LinkedList)outerIter.next();
      //System.out.println("I am calling the evaluateTerm"+innerList);
      if (evaluateTerm(innerList,valueObjIn,textPw,xmlPwIn)){
	break;
      }
      
    } // end of for (Iterator outerIter = scenarios.iterator(); outerIter.hasNext();)
    xmlPwIn.println("</vector>");
  }

  
    
    
  /**
   * Describe <code>evaluateTerm</code> method here.
   * This method basically iterates thru all the terms.
   * @param innerListIn a <code>LinkedList</code> value
   * @param valueObjIn a <code>HashMap</code> value
   * @param textPw a <code>PrintWriter</code> value
   * @param xmlPwIn a <code>PrintWriter</code> value
   * @return a <code>boolean</code> value
   * @exception FileNotFoundException if an error occurs
   */
  public boolean evaluateTerm(LinkedList innerListIn,HashMap valueObjIn,PrintWriter textPw,PrintWriter xmlPwIn)  throws FileNotFoundException {
    
    //System.out.println("I am into the evaluateTerm: "+innerListIn);
    LinkedList declaredCondition = new LinkedList();
    for (Iterator innerIter = innerListIn.iterator(); innerIter.hasNext();) {
      Application app = (Application)innerIter.next();
      //System.out.println("I am calling the evaluateNode");
      if (!(evaluateNode(app,valueObjIn))){
	//System.out.println("I am here atlease once");
	return false;
      }
    } // end of for (Iterator innerIter = innerListIn.iterator(); innerIter.hasNext();)
    
    //Populate the declared conditions if there are any.
    //System.out.println("I am calling the populateCondition");
    populateCondition(innerListIn,declaredCondition);
    //System.out.println("I am calling the evaluateFunction method");
    HashMap value = evaluateFunction(declaredCondition,innerListIn,valueObjIn);
    printText(textPw,value,declaredCondition);
    printXml(xmlPwIn,value,declaredCondition);
    return true;
  }
  
  
  /**
   * Describe <code>evaluateNode</code> method here.
   *
   * @param appIn an <code>Application</code> value
   * @param valueObjIn a <code>HashMap</code> value
   * @return a <code>boolean</code> value
   */
  public boolean evaluateNode(Application appIn,HashMap valueObjIn) {
    
    //Get the operands and operator from the application
    //and compare it with the valueIn.
    String leftOp = ((ExpressionList)(appIn.getParameters())).get(0).toString();
    
    //This if loop is here because there will be some expressions in Rosetta
    //that does not contain rightOp. for example %setTime(from alaram example)
    //does not have a roght operand.
    String rightOp = null;
    if (appIn.getApplicationType() == Application.BINARY) {
      rightOp = ((ExpressionList)(appIn.getParameters())).get(1).toString();       
    } // end of if (((ExpressionList)(appIn.getParameters())).get(1) != null)
    String operator = appIn.getFunction().toString();
    
    LinkedList iptVarList = scenarios.getInputVar();
    LinkedList optVarList = scenarios.getOutputVar();
    
    //Doing like this is not recommended.Change this.
    String optVar = optVarList.get(0).toString();
    
    for (Iterator iptVarIter = iptVarList.iterator(); iptVarIter.hasNext();) {
      String iptVar = iptVarIter.next().toString();
      
      if (leftOp.equalsIgnoreCase(iptVar)) {
	double right_op = 0.0;
	if (rightOp != null) {
	  right_op = (new Double(rightOp)).doubleValue();	   
	} // end of if (rightOp != null)
	
	double valueIn = ((Double)valueObjIn.get(leftOp)).doubleValue();
	//The following code grabs the operator and performs the 
	//actual calculations.

	if (operator.equalsIgnoreCase("<") || operator.equalsIgnoreCase("=<")) {
	  if (valueIn <= right_op) {
	    return true;
	  } // end of if (valueIn < right_op)
	  else {
	    return false;
	  } // end of else
	  
	} // end of if (leftOp.equalsIgnoreCase(iptVar))
	
	//I am doing like this only for the demo purpose for Edaptive.Better check this.
	//if (operator.equalsIgnoreCase(">") || operator.equalsIgnoreCase(">=")) {
	if (operator.equalsIgnoreCase(">")) {
	  //if (valueIn >= right_op) {
	  if (valueIn > right_op) {
	    return true;
	  } // end of if (valueIn > right_op)
	  else {
	    return false;
	  } // end of else
	  
	} // end of if (operator.equalsIgnoreCase(">") || operator.equalsIgnoreCase(">="))
	
	
	if (operator.equalsIgnoreCase(" == ")) {
	  if (valueIn == right_op) {
	    return true;
	  } // end of if (valueIn == right_op)
	  else {
	    return false;
	  } // end of else
	  
	} // end of if (operator.equalsIgnoreCase(" == "))
	
	//This code deals with the % operator.This needs to be handled 
	//seperately.if the input value is "0" then this returns false 
	//and if the input value is "1" this reutrns true.
	if (operator.equalsIgnoreCase("%")) {
	  if (valueIn == 1.0) {
	    return true;
	  } // end of if (valueIn == 1)
	  else {
	    return false;
	  } // end of else
	} // end of if (operator.equalsIgnoreCase("%"))
	
      } // end of if (leftOp.equalsIgnoreCase(optVar))
    }
    return true;
  }
  
 //   public void getLeftOp_backup(Application appIn,String leftOp) {

//      if (((ExpressionList)appIn.getParameters()).get(0) instanceof Application) {
//        System.out.println(((ExpressionList)appIn.getParameters()).get(0));
//        getLeftOp((Application)((ExpressionList)appIn.getParameters()).get(0),leftOp);
//      } // end of if (((ExpressionList)appIn.getParameters()).get(0) instanceof Application)
//      else {
//        System.out.println("the leftOp in else part is: "+leftOp);
//        if (leftOp == null) {
//  	leftOp = ((ExpressionList)appIn.getParameters()).get(0).toString();	 
//  	System.out.println("the leftOp in rec fnc is: "+leftOp);
//        } // end of if (leftOp == null)
      
//      } // end of else
//      System.out.println("the leftOp in rec fnc is: "+leftOp);

//    }

  public String getLeftOp(Application appIn) {
    while (((ExpressionList)appIn.getParameters()).get(0) instanceof Application) {
      appIn = (Application)((ExpressionList)appIn.getParameters()).get(0);
    } // end of while (((ExpressionList)appIn.getParameters()).get(0) instanceof Application)
      return ((ExpressionList)appIn.getParameters()).get(0).toString();
  }
 
  
  /**
   * Describe <code>populateCondition</code> method here.
   *
   * @param innerListIn a <code>LinkedList</code> value
   * @param conditionIn a <code>LinkedList</code> value
   */
  public void populateCondition(LinkedList innerListIn,LinkedList conditionIn) {
    
    LinkedList iptVarList = scenarios.getInputVar();
    LinkedList optVarList = scenarios.getOutputVar();
    
    for (Iterator innerIter = innerListIn.iterator(); innerIter.hasNext();) {
      Application app = (Application)innerIter.next();
      String leftOp = null;
      leftOp = getLeftOp(app);
      if (!(inInputVarList(iptVarList,leftOp)) && !(inOutputVarList(optVarList,leftOp))) {
	conditionIn.add(app);
      } // end of if (!(leftOp.equalsIgnoreCase(iptVar) && (!(leftOp.equalsIgnoreCase(optVar))))
      
    } // end of for (Iterator innerIter = innerListIn.iterator(); innerIter.hasNext();)
    
    //  System.out.println("#########################################################");
//      System.out.println("Testing the populate condition method");
//      for (Iterator condIter = conditionIn.iterator();condIter.hasNext();) {
//        System.out.println(condIter.next());
//      } // end of for (Iterator condIter = conditionIn.iterator();condIter.hasNext();)
//      System.out.println("#########################################################");
  }
  

  /**
   * Describe <code>inInputVarList</code> method here.
   * This method is called from populateCondition() method.
   * This method basically iterates thru all the input variables 
   * list and determines if the node has any input variable.
   * @param ipt_var_list a <code>LinkedList</code> value
   * @param left_op a <code>String</code> value
   * @return a <code>boolean</code> value
   */
  public boolean inInputVarList(LinkedList ipt_var_list, String left_op) {
    for (Iterator ipt_var_list_iter = ipt_var_list.iterator();ipt_var_list_iter.hasNext();) {
      if (left_op.equalsIgnoreCase((String)ipt_var_list_iter.next())) {
	return true;
      } // end of if (left_op.equalsIgnoreCase(ipt_var_list_iter.next())
    } // end of for (Iterator ipt_var_list_iter = ipt_var_list.iterator();ipt_var_list_iter.hasNext();)
    return false;
  }
  
  /**
   * Describe <code>inOutputVarList</code> method here.
   * The documentation is same as the inInputVarList() but it
   * checks for all the output variables.
   * @param opt_var_list a <code>LinkedList</code> value
   * @param left_op a <code>String</code> value
   * @return a <code>boolean</code> value
   */
  public boolean inOutputVarList(LinkedList opt_var_list, String left_op) {
    for (Iterator opt_var_list_iter = opt_var_list.iterator();opt_var_list_iter.hasNext();) {
      if (left_op.equalsIgnoreCase((String)opt_var_list_iter.next())) {
	return true;
      } // end of if (left_op.equalsIgnoreCase(ipt_var_list_iter.next())
    } // end of for (Iterator ipt_var_list_iter = ipt_var_list.iterator();ipt_var_list_iter.hasNext();)
    return false;
  }

  //This method needs to be deleted.
  public String appendTick(String strIn) {
    if (strIn.endsWith("'")) {
      return (strIn.substring(0,strIn.indexOf("'"))+"tick");
    } // end of if (strIn.endsWith("'"))
    else {
      return strIn;
    } // end of else
  }

  /**
   * Describe <code>evaluateFunction</code> method here.
   *
   * @param declCond a <code>LinkedList</code> value
   * @param term a <code>LinkedList</code> value
   * @param iptValues a <code>HashMap</code> value
   * @return a <code>HashMap</code> value
   */
  public HashMap evaluateFunction(LinkedList declCond,LinkedList term,HashMap iptValues) {

    //System.out.println("I am into evaluateFunction");

    //This HashMap is used to contain the output variables and 
    //their corresponding values.
    HashMap ret_value = new HashMap();    
    
    PythonInterpreter interp = new PythonInterpreter();
    
    //There may be cases where the declared conditions are null.for 
    //example in the alarm examples I don't have any declared conditions.
    if (declCond.size() != 0) {
       
      for (Iterator declCondIter = declCond.iterator(); declCondIter.hasNext();) {
	Application app = (Application)declCondIter.next();
	String left_op_tmp = ((ExpressionList)app.getParameters()).get(0).toString();
	String left_op = hasTick(left_op_tmp);
	String right_op = ((ExpressionList)app.getParameters()).get(1).toString();
	
	//The below if loop is used to check for the previous state.The logic is that 
	//the value gets stored in a datastructure for the previous state and that value 
	//is retrieved in this state.
	if (((ExpressionList)app.getParameters()).get(1) instanceof PrimitiveValue ) {
	  //System.out.println("The values being set are: "+left_op+":"+right_op);
	  
	  previous_state.put(initialState(left_op_tmp),(new Double(right_op)));
	  interp.set(left_op,new Double(right_op));
	} // end of if ((ExpressionList)(app.getParameters()).get(1) instanceof RealValue )
	else {
	  interp.set(left_op,previous_state.get(initialState(left_op_tmp)));
	} // end of else
	
      } // end of for (Iterator declCondIter = declCond.iterator(); declCondIter.hasNext();)
      
      //There are also cases where the output conditions needs to have input paramters apart
      //from the declared variables.
      Set valueSet = iptValues.entrySet();
      Iterator value_set_iter = valueSet.iterator();
      while (value_set_iter.hasNext()) {
	Map.Entry value_entry = (Map.Entry)value_set_iter.next();
	Double value = new Double(value_entry.getValue().toString());
	interp.set((String)value_entry.getKey(),value);
      } // end of while (value_set_iter.hasNext())
      
      
      try {
	for (Iterator optIter = outputCondition.iterator(); optIter.hasNext();) {
	  Application optApp = (Application)optIter.next();
	  
	  //The condition will be of the form (c = a).The parenthesis should 
	  //be removed to feed the condition to the Jython interpretor.
	  //So the method removeParens is used here.
	  String opCond = replaceState(optApp);
	  //System.out.println("The output condition is: "+opCond);
	  String left_opcond = ((ExpressionList)optApp.getParameters()).get(0).toString();
	  //System.out.println("The left_opcond is: "+left_opcond);
	  String left_opcond_tmp = appendTick(left_opcond);
	  interp.exec(opCond);
	  //System.out.println("upto here");
	  PyObject obj = interp.get(left_opcond_tmp);
	  //System.out.println("The value from Jython: "+obj);
	  ret_value.put(initialState(left_opcond),obj.toString());
	} // end of for (Iterator optIter = outputCondition.iterator(); optIer.hasNext();)
	
      } catch (PyException e) {
	System.out.println("Error in generating the value"+e);
      } // end of try-catch
    } // end of if (declCond != null)    

    //This else part deals with the % operator(for the time being) and 
    //if there are no declared conditions.
    else {
      LinkedList optConditions = new LinkedList();
      for (Iterator nodeIter = term.iterator();nodeIter.hasNext();) {
	Application nodeApp = (Application)nodeIter.next();
	String left_oper = initialState(((ExpressionList)nodeApp.getParameters()).get(0).toString());
	
	//I should iterate thru all the output variables and grab the node
	//in the term that contains the output variable.
	LinkedList output_variables = scenarios.getOutputVar();
	for (Iterator output_iter = output_variables.iterator(); output_iter.hasNext();) {
	  String right_var = (String)output_iter.next();
	  if (left_oper.equalsIgnoreCase(right_var)) {
	    optConditions.add(nodeApp);
	    break;
	  } // end of if (left_oper.equalsIgnoreCase(right_oper))
	} // end of for (Iterator output_iter = output_variables.iterator(); output_iter.hasNext())
      } // end of for (Iterator nodeIter = term.iterator();nodeIter.hasNext();)

      //  System.out.println("##############Testing the optConditions for no declared conditions######");
//        for (Iterator optCond_tmp_iter = optConditions.iterator();optCond_tmp_iter.hasNext();) {
//  	System.out.println(optCond_tmp_iter.next());
//        } // end of for (Iterator optCond_tmp_iter = optConditions.iterator();optCond_tmp_iter.hasNext();)
//        System.out.println("##############Testing the optConditions for no declared conditions######");
      
      for (Iterator output_conditions_iter = optConditions.iterator();output_conditions_iter.hasNext();) {
	
	PythonInterpreter interp_tmp = new PythonInterpreter();
	Application output_app = (Application)output_conditions_iter.next();
	String condition = replaceState(output_app);
	String left_tmp = ((ExpressionList)output_app.getParameters()).get(0).toString();

	//This piece of code is choking when there are any negative parameters on the rhs 
	//in an application. Needs to be taken care of.
	
	while(((ExpressionList)output_app.getParameters()).get(1) instanceof Application) {
	  Application temp_app = (Application)((ExpressionList)output_app.getParameters()).get(1);
	  if (temp_app.getApplicationType() == Application.BINARY) {
	    System.out.println("I am into the loop once");
	    output_app = (Application)((ExpressionList)output_app.getParameters()).get(1);
	    String leftTmp = ((ExpressionList)output_app.getParameters()).get(0).toString();
	    System.out.println("the leftTmp value in while loop: "+leftTmp);
	    //System.out.println("The leftTmp value: "+iptValues.get(initialState(leftTmp)));
	    if (iptValues.get(initialState(leftTmp)) != null) {
	      interp_tmp.set(leftTmp,iptValues.get(initialState(leftTmp)));
	    }
	  }
	  else {
	    break;
	  } // end of else
	  
	} // end of while (((ExpressionList)output_app.getParameters()).get(0) instanceof Application)
	
	String right_tmp = ((ExpressionList)output_app.getParameters()).get(1).toString();
	//System.out.println("the right_tmp is: "+right_tmp);
	String right_str = null;
	if ( iptValues.get(initialState(right_tmp)) instanceof Object) {
	  right_str =  iptValues.get(initialState(right_tmp)).toString();
	  interp_tmp.set(right_tmp,new Double(right_str));
	} // end of if ( iptValues.get(initialState(right_tmp) != null)
	
	//interp_tmp.set(right_tmp,new Double(right_str));
	//System.out.println("the output_app string: "+condition);
	interp_tmp.exec(condition);

	//PyObject objValue = interp_tmp.get(initialState(left_tmp));
	PyObject objValue = interp_tmp.get(hasTick(left_tmp));
	//System.out.println("The value evaluated is: "+objValue);

	ret_value.put(initialState(left_tmp),objValue);
	//ret_value.put(initialState(left_tmp),iptValues.get(initialState(right_tmp)));
      } // end of for (Iterator output_conditions_iter = optConditions.iterator();output_conditions_iter.hasNext();)
      
      //  System.out.println("#####Testing the final return value#######");
//        Set ret_value_set = ret_value.entrySet();
//        Iterator ret_value_iter = ret_value_set.iterator();
//        while (ret_value_iter.hasNext()) {
//  	Map.Entry ret_entry = (Map.Entry)ret_value_iter.next();
//  	System.out.println(ret_entry.getKey()+":"+ret_entry.getValue());
//        } // end of while (ret_value_iter.hasNext())
//        System.out.println("###########################################");

    } // end of else
    
    return ret_value;
  }    

  /**
   * Describe <code>hasTick</code> method here.
   * This method is used as Jython cannot support "'".
   * for example if we have b'=0 then Jython fails in recognizing
   * the "'" operator.So I am converting b' to btick in this method.
   * @param strIn a <code>String</code> value
   * @return a <code>String</code> value
   */
  public String hasTick(String strIn) {
    String newStr;
    if (strIn.endsWith("'")) {
      newStr =  (strIn.substring(0,strIn.indexOf("'"))+"tick");
    } // end of if (strIn.endsWith("'"))
    else {
      newStr = strIn;
    } // end of else
    return newStr;
  }

  /**
   * Describe <code>removeParens</code> method here.
   * This method is used to replace any variables having "'".
   * for example if i have a = b' then this method converts it to 
   * a = btick.This is because b' is stored as btick in the Jython 
   * Interpretor environment.
   * @param appIn an <code>Application</code> value
   * @return a <code>String</code> value
   */
  public String replaceState(Application appIn) {

    String left_op = hasTick(((ExpressionList)appIn.getParameters()).get(0).toString());
    String right_op = hasTick(((ExpressionList)appIn.getParameters()).get(1).toString());
    String oper = appIn.getFunction().toString();
    
    return (left_op+oper+right_op);
  }


  /**
   * Describe <code>initialState</code> method here.
   *
   * @param strIn a <code>String</code> value
   * @return a <code>String</code> value
   */
  public String initialState(String strIn) {
    if (strIn.endsWith("'")) {
      return strIn.substring(0,strIn.indexOf("'"));
    } // end of if (strIn.endsWith("'"))
    else {
      return strIn;
    } // end of else
  }
    

  /**
   * Describe <code>printText</code> method here.
   *
   * @param textPw a <code>PrintWriter</code> value
   * @param valueIn a <code>HashMap</code> value
   */
  public void printText(PrintWriter textPw,HashMap valueIn,LinkedList declCondIn) {
   
    //This loop is used to print the values from the declared conditions
    //if the spec has any declared conditions.
    if (declCondIn.size() != 0) {
      for (Iterator optCondIter = outputCondition.iterator(); optCondIter.hasNext();) {
	Application optApp = (Application)optCondIter.next();
	String left_op = ((ExpressionList)optApp.getParameters()).get(0).toString();
	textPw.println(left_op+" = "+valueIn.get(initialState(left_op)));
	textPw.println();
      } // end of for (Iterator optCondIter = outputCondition.iterator(); optCondIter.hasNext();)
    } // end of if (declCondIn.size() != 0)
    else {
      Set value_set = valueIn.entrySet();
      Iterator value_set_iter = value_set.iterator();
      while (value_set_iter.hasNext()) {
	Map.Entry value_entry = (Map.Entry)value_set_iter.next();
	textPw.println(value_entry.getKey()+" = "+value_entry.getValue());
	textPw.println();
      } // end of while (value_set_iter.hasNext())       
      
    } // end of else
  }

  /**
   * Describe <code>printXml</code> method here.
   *
   * @param xmlPwIn a <code>PrintWriter</code> value
   * @param valueIn a <code>HashMap</code> value
   */
  public void printXml(PrintWriter xmlPwIn,HashMap valueIn,LinkedList declCondIn) {

    if (declCondIn.size() != 0) {
      for (Iterator optCondIter = outputCondition.iterator(); optCondIter.hasNext();) {
	
	Application optApp = (Application)optCondIter.next();
	String left_op = ((ExpressionList)optApp.getParameters()).get(0).toString();
	xmlPwIn.println("<condition>");
	xmlPwIn.println("\t<parameter mode = \"out\">"+left_op+" </parameter>");
	xmlPwIn.println("\t<value> "+valueIn.get(initialState(left_op))+" </value>");
	xmlPwIn.println("</condition>");
      }       
    } // end of if (declCondIn.size() != 0)
    else {
      Set value_set = valueIn.entrySet();
      Iterator value_set_iter = value_set.iterator();
      while (value_set_iter.hasNext()) {
	Map.Entry value_entry = (Map.Entry)value_set_iter.next();
	xmlPwIn.println("<condition>");
	xmlPwIn.println("\t<parameter mode = \"out\">"+value_entry.getKey()+" </parameter>");
	xmlPwIn.println("\t<value> "+value_entry.getValue()+" </value>");
	xmlPwIn.println("</condition>");
      } // end of while
            
    }
  }
}
  
  
