//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: XmlToVec.java,v 1.2 2002/09/09 15:22:00 cvs Exp $
//
// Revision History:
// $Log: XmlToVec.java,v $
// Revision 1.2  2002/09/09 15:22:00  cvs
// Added KU License to all the files and made some minor changes.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata;

import java.util.LinkedList;
import java.util.Iterator;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;

import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;



/**
 * Describe class <code>XmlToVec</code> here.
 * This class is used to generate the ".vec" format 
 * from the XML files.
 * @author <a href="mailto:ksatya@ittc.ku.edu">Satyanarayana Kakarlamudi</a>
 * @version 1.0
 */
public class XmlToVec {

  //This field stores the parsed XML file.
  static Document doc;

  //This is the starting number for the iterations.
  static String start = null;

  //This is the ending number for the iterations.
  static String stop = null;

  //This represents the interval between the iterations.
  static int interval = 0;

  //This stores the list of all the input variables.
  static LinkedList inputVariables;

  //This stores the list of all the output variables.
  static LinkedList outputVariables;

  /**
   * Describe <code>main</code> method here.
   *
   * @param argv a <code>String[]</code> value
   */
  public static void main (String[] argv) {
    try {
      DocumentBuilderFactory dFactory =  DocumentBuilderFactory.newInstance(); 
      DocumentBuilder dBuilder = dFactory.newDocumentBuilder(); 
      File file = new File(argv[0]); 
      doc = dBuilder.parse(file);
    } catch (ParserConfigurationException e) {
    } catch (SAXException e) {
    } catch (IOException e) {
    } // end of catch
    
    inputVariables = new LinkedList();
    outputVariables = new LinkedList();

    Element root = doc.getDocumentElement();    
    //System.out.println("The root name is: "+root.getNodeName());
    
    NodeList children = root.getChildNodes();
    //System.out.println("The number of children are: "+children.getLength());

    for (Node child = root.getFirstChild();child!=null;child=child.getNextSibling()) {
      if (!(child.getNodeType() == child.TEXT_NODE)) {
	//System.out.println("the child is: "+child.getNodeName()+"="+child.getNodeValue());
	
	//Into accessing the attributes.
	if (child.getNodeType() == child.ELEMENT_NODE) {
	  NamedNodeMap attr = child.getAttributes();
	  for (int i=0;i<attr.getLength();i++) {
	    Node childAttr = attr.item(i);
	    if (start == null) {
	      start = childAttr.getNodeValue();
	    }
	    stop = childAttr.getNodeValue();	
	    if (interval == 0) {
	      interval = ((new Integer(stop)).intValue()) - ((new Integer(start)).intValue());
	    } // end of if (interval == null)
	  }
	}
	
	//Accessing the sub-children in the main child element.
	for (Node subChild= child.getFirstChild();subChild != null;subChild=subChild.getNextSibling()) {
	  stepThrough(subChild);
	} // end of for (Node subChild= child.getFirstChild();child != null;subChild=subChild.getNextSibling())
      }
    }
    printVecFormat(argv[0]);
    System.out.println("Exiting Successfully :-)");
  }


  /**
   * Describe <code>printVecFormat</code> method here.
   * This method is used to print in the ".vec" file.
   * @param fileName a <code>String</code> value
   */
  public static void printVecFormat(String fileName) {
    try {
      FileOutputStream XmlOut = new FileOutputStream(fileName.substring(0,fileName.indexOf(".xml"))+".vec");
      PrintWriter pOut = new PrintWriter(XmlOut,true);
      pOut.println("START: "+start);
      pOut.println("STOP: "+stop);
      pOut.println("INTERVAL: "+interval);
      for (Iterator iptIter = inputVariables.iterator();iptIter.hasNext();) {
	pOut.println("INPUTS: "+iptIter.next());
      } // end of for (Iterator iptIter = inputVariables.iterator();iptIter.hasNext();iptIter.next())
      for (Iterator optIter = outputVariables.iterator();optIter.hasNext();) {
	pOut.println("OUTPUTS: "+optIter.next());
      }
    } catch (FileNotFoundException error) {
      System.out.println("Error in printing to the .vec file"+error);    
    } // end of try-catch
  }



  /**
   * Describe <code>stepThrough</code> method here.
   * This method is called iteratively to capture all the child 
   * nodes.
   * @param subChildIn a <code>Node</code> value
   */
  public static void stepThrough(Node subChildIn) {
    if (subChildIn.getNodeType() != subChildIn.TEXT_NODE) {
      //System.out.println("The sub sub child name is: "+subChildIn.getNodeName()+" its value is: "+subChildIn.getFirstChild().getNodeValue());

      //This chunk is used to get the attributes of the child nodes and 
      //add them to the linked lists.
      if (subChildIn.hasAttributes()) {
	NamedNodeMap childAttr = subChildIn.getAttributes();
	for (int count=0;count<childAttr.getLength();count++) {
	  Node Attrib = childAttr.item(count);
	  if ((Attrib.getNodeValue()).equalsIgnoreCase("in")) {
	    //System.out.println("The input variable is: "+subChildIn.getNodeValue());
	    if (!(inputVariables.contains(subChildIn.getFirstChild().getNodeValue()))) {
	      inputVariables.add(subChildIn.getFirstChild().getNodeValue());       
	    } // end of if (iptVariables.contains(subChildIn.getFirstChild().getNodeValue())
	  } // end of if ((Attrib.getNodeValue()).equalsIgnoreCase("in"))
	  if ((Attrib.getNodeValue()).equalsIgnoreCase("out")) {
	    //System.out.println("The output variable is: "+subChildIn.getNodeValue());
	    if (!(outputVariables.contains(subChildIn.getFirstChild().getNodeValue()))) {
	    outputVariables.add(subChildIn.getFirstChild().getNodeValue());
	    }
	  } // end of if ((Attrib.getNodeValue()).equalsIgnoreCase("out"))
	} // end of for (int count=0;count<childAttr.getLength();count++)
      } // end of if (subChildIn.hasAttributes())
      
      for (Node subSubChild =
	     subChildIn.getFirstChild();subSubChild!=null;subSubChild = subSubChild.getNextSibling()) {
	stepThrough(subSubChild); 
      }
    } // end of if (subChlildIn.getNodeType() != subChildIn.TEXT_NODE)
  }

}//end of class XmlToVec
