#! /usr/bin/perl

use Cwd;
use File::Copy;


$updateExpected = "false";
@ARGV = ('-') unless @ARGV;
while($ARGV = shift) {
    if($ARGV =~ /--help/ || $ARGV =~ /-help/ || $ARGV =~ /-h/) {
	print "Usage:\n";
	print " --help  print this message\n";
	print " -force_update Overwrite the expected results with the\n";
        print "                  generated results\n";
	print "  -query_update   Ask the use if the expected results should be\n";
        print "                  updated when they differ from the actual results\n";
    } elsif( $ARGV =~ /-force_update/) {
	$updateExpected = "force";
    } elsif( $ARGV =~ /-query_update/) {
	$updateExpected = "query";
    } elsif( $ARGV =~ /do_update/) {
	$updateExpected = "do";
    }
}

$curdir = cwd();
$sldl_home = "/users/ksatya/Research/Rosetta/tools";
print curdir;
#$scengen_cmd = "$curdir/bin/toScenarios";
$scengen_cmd = "$sldl_home/bin/DVTG/toScenarios";
(-e $scengen_cmd) || 
    die "Could not find samp in $scengen_cmd\n";

$logfile = STDERR;
$logLevel = 0;
$resultDir = "results/";
$expectDir = "expect/";
$passed = "PASSED";
$filed = "FAILED";
$homedir = cwd();

#############################################################################
#                 Scenario Generation for Satcom
#############################################################################
chdir($homedir);
RemoveResultsFile("test.results");
SummarizeResults("SATCOM EXAMPLE");
$testLogCount = 1;
$satcomDir = "/users/ksatya/Research/Rosetta/publications/examples/satcom";
SatcomTest($satcomDir);
#############################################################################


#############################################################################
#                 Scenario Generation for Actuator 
#############################################################################
chdir($homedir);
SummarizeResults("ACTUATOR EXAMPLE");
$testLogCount = 1;
$actuatorDir = "/users/ksatya/Research/Rosetta/publications/examples/actuator";
ActuatorTest($actuatorDir);
############################################################################


############################################################################
#                 Scenario Generation for Schmidt
############################################################################
chdir($homedir);
SummarizeResults("SCHMIDT EXAMPLE");
$testLogCount = 1;
$schmidtDir = "/users/ksatya/Research/Rosetta/publications/examples/schmidt";
SchmidtTriggerTest($schmidtDir);
############################################################################

############################################################################
#                 Scenario Generation for Spring
############################################################################
chdir($homedir);
SummarizeResults("SPRING EXAMPLE");
$testLogCount = 1;
$springDir = "/users/ksatya/Research/Rosetta/publications/examples/spring";
SpringTest($springDir);
#############################################################################


sub RemoveResultsFile {
    local($resultFile) = shift @_;

    if(-e $resultFile) {
	unlink($resultFile) || 
	    die "Unable to remove result file $resultFile";
    }
}

sub SummarizeResults {
    local($cleanDir) = shift @_;
    open(testResults,">> $homedir/test.results");
    print testResults
        "==========================================================================\n";
    print testResults
        "                                  SUMMARY OF $cleanDir\n";
    print testResults
        "==========================================================================\n";
    close(testResults);

}

#############################################################################
#                    Satcom Stuff
#############################################################################
sub SatcomTest {
    local($cleanDir) = shift @_;
    local(@files);

    $files[1] = "$satcomDir/accessorFacets.sld";
    $files[2] = "$satcomDir/carrierRecovery.sld";
    $files[3] = "$satcomDir/costasLoop.sld";

    if(!(-d "$satcomDir/scenarios")) {
	LogMsg("Creating scenarios directory\n");
	mkdir("$satcomDir/scenarios",0755) ||
	    die "Unable to create the results directory";
    }

    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i) {
	    $st = satTest($file);
	    
	    if(($st == 0) || (($st == 1) && ($updateExpected eq "force"))){
                LogMsg("Test scenarios are correctly generated\n");
	    }
	}
    }
}




sub satTest {
    local($file) = shift @_;
    local($scenName) = "Scenarios genaration";
    local($scenariosName) = "Scenarios Test ...";
    local($status) = system("$scengen_cmd $file 1>> /dev/null 2>> /dev/null");
    if($status == 0) {
	TestResults($file,$scenName,$passed,"","");
    }
    else {
	LogMsg("$failed\n");
	TestResults($file,$scenName,$failed,"Error generating scenarios",$resultFile);
    }
    $lfile = $file;
    chop($lfile);chop($lfile);chop($lfile);chop($lfile);
    $scenFile = "$lfile\_SCENARIOS\.sld";
    $expectFile = "$satcomDir/scenarios/$scenFile";
    if(-s $scenFile) {
	if($updateExpected eq "force") {
	    copy($scenFile,"$satcomDir/scenarios");
	    $compareFiles = 1;
	}else {
	    $comparefiles = CompareFiles($scenFile,$expectFile);
	}
    }else {
	$comparefiles = 2;
    }
    if($comparefiles == 0) {
	TestResults($file,$scenariosName,$passed,"","");
    }elsif ($compareFiles == 1) {
	if($updateExpected eq "force") {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is copied"]);
	}else {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is different"]);
	}
    }elsif ($comparefiles == 2){
	TestResults($file,$scenariosName,$failed,["the scenarios were
    not generated"]);
    }
    return $comparefiles;
}

##########################################################################
#                 End of Satcom Stuff
##########################################################################


##########################################################################
#                 Actuator Stuff
##########################################################################
sub ActuatorTest {
    local($cleanDir) = shift @_;
    local(@files);

    $files[1] = "$actuatorDir/actuator.sld";
    $files[2] = "$actuatorDir/cyl_pistin.sld";

    if(!(-d "$actuatorDir/scenarios")) {
	LogMsg("Creating scenarios directory\n");
	mkdir("$actuatorDir/scenarios",0755) ||
	    die "Unable to create the results directory";
    }

    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i) {
	    $st = actuatorTest($file);
	    
	    if(($st == 0) || (($st == 1) && ($updateExpected eq "force"))){
		LogMsg("Test scenarios are correctly generated\n");
	    }
	}
    }
}

sub actuatorTest {
    local($file) = shift @_;
    local($scenName) = "Scenarios genaration";
    local($scenariosName) = "Scenarios Test ...";
    local($status) = system("$scengen_cmd $file 1>> /dev/null 2>> /dev/null");
    if($status == 0) {
	TestResults($file,$scenName,$passed,"","");
    }
    else {
	LogMsg("$failed\n");
	TestResults($file,$scenName,$failed,"Error generating scenarios",$resultFile);
    }
    $lfile = $file;
    chop($lfile);chop($lfile);chop($lfile);chop($lfile);
    $scenFile = "$lfile\_SCENARIOS\.sld";
    $expectFile = "$actuatorDir/scenarios/$scenFile";
    if(-s $scenFile) {
	if($updateExpected eq "force") {
	    copy($scenFile,"$actuatorDir/scenarios");
	    $compareFiles = 1;
	}else {
	    $comparefiles = CompareFiles($scenFile,$expectFile);
	}
    }else {
	$comparefiles = 2;
    }
    if($comparefiles == 0) {
	TestResults($file,$scenariosName,$passed,"","");
    }elsif ($compareFiles == 1) {
	if($updateExpected eq "force") {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is copied"]);
	}else {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is different"]);
	}
    }elsif ($comparefiles == 2){
	TestResults($file,$scenariosName,$failed,["the scenarios were
    not generated"]);
    }
    return $comparefiles;
}
##########################################################################
#                 End of Actuator Stuff
##########################################################################



##########################################################################
#                  Schmidt Trigger stuff
##########################################################################
sub SchmidtTriggerTest {
    local($cleanDir) = shift @_;
    local(@files);

    $files[1] = "$schmidtDir/Constraints.sld";
    $files[2] = "$schmidtDir/SchmidtTriggerInteraction.sld";
    $files[3] = "$schmidtDir/schmidtTrigger.sld";

    if(!(-d "$schmidtDir/scenarios")) {
	LogMsg("Creating scenarios directory\n");
	mkdir("$schmidtDir/scenarios",0755) ||
	    die "Unable to create the results directory";
    }

    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i) {
	    $st = schmidtTest($file);
	    
	    if(($st == 0) || (($st == 1) && ($updateExpected eq "force"))){
                LogMsg("Test scenarios are correctly generated\n");
	    }
	}
    }
}

sub schmidtTest {
    local($file) = shift @_;
    local($scenName) = "Scenarios genaration";
    local($scenariosName) = "Scenarios Test ...";
    local($status) = system("$scengen_cmd $file 1>> /dev/null 2>> /dev/null");
    if($status == 0) {
	TestResults($file,$scenName,$passed,"","");
    }
    else {
	LogMsg("$failed\n");
	TestResults($file,$scenName,$failed,"Error generating scenarios",$resultFile);
    }
    $lfile = $file;
    chop($lfile);chop($lfile);chop($lfile);chop($lfile);
    $scenFile = "$lfile\_SCENARIOS\.sld";
    $expectFile = "$schmidtDir/scenarios/$scenFile";
    if(-s $scenFile) {
	if($updateExpected eq "force") {
	    copy($scenFile,"$schmidtDir/scenarios");
	    $compareFiles = 1;
	}else {
	    $comparefiles = CompareFiles($scenFile,$expectFile);
	}
    }else {
	$comparefiles = 2;
    }
    if($comparefiles == 0) {
	TestResults($file,$scenariosName,$passed,"","");
    }elsif ($compareFiles == 1) {
	if($updateExpected eq "force") {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is copied"]);
	}else {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is different"]);
	}
    }elsif ($comparefiles == 2){
	TestResults($file,$scenariosName,$failed,["the scenarios were
    not generated"]);
    }
    return $comparefiles;
}
#########################################################################
#                   End of Schmidt Trigger
#########################################################################


##########################################################################
#                  Spring stuff
##########################################################################
sub SpringTest {
    local($cleanDir) = shift @_;
    local(@files);

    $files[1] = "$springDir/springAll.sld";
    $files[2] = "$springDir/springPackage.sld";
    $files[3] = "$springDir/spring.sld";
    $files[4] = "$springDir/springSystem.sld";

    if(!(-d "$springDir/scenarios")) {
	LogMsg("Creating scenarios directory\n");
	mkdir("$springDir/scenarios",0755) ||
	    die "Unable to create the results directory";
    }

    foreach $file (@files) {
	if($file =~ /\S+\.sld$/i) {
	    $st = springTest($file);
	    
	    if(($st == 0) || (($st == 1) && ($updateExpected eq "force"))){
                LogMsg("Test scenarios are correctly generated\n");
	    }
	}
    }
}

sub springTest {
    local($file) = shift @_;
    local($scenName) = "Scenarios genaration";
    local($scenariosName) = "Scenarios Test ...";
    local($status) = system("$scengen_cmd $file 1>> /dev/null 2>> /dev/null");
    if($status == 0) {
	TestResults($file,$scenName,$passed,"","");
    }
    else {
	LogMsg("$failed\n");
	TestResults($file,$scenName,$failed,"Error generating scenarios",$resultFile);
    }
    $lfile = $file;
    chop($lfile);chop($lfile);chop($lfile);chop($lfile);
    $scenFile = "$lfile\_SCENARIOS\.sld";
    $expectFile = "$springDir/scenarios/$scenFile";
    if(-s $scenFile) {
	if($updateExpected eq "force") {
	    copy($scenFile,"$springDir/scenarios");
	    $compareFiles = 1;
	}else {
	    $comparefiles = CompareFiles($scenFile,$expectFile);
	}
    }else {
	$comparefiles = 2;
    }
    if($comparefiles == 0) {
	TestResults($file,$scenariosName,$passed,"","");
    }elsif ($compareFiles == 1) {
	if($updateExpected eq "force") {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is copied"]);
	}else {
	    TestResults($file,$scenariosName,$failed,["the scenarios
    file is different"]);
	}
    }elsif ($comparefiles == 2){
	TestResults($file,$scenariosName,$failed,["the scenarios were
    not generated"]);
    }
    return $comparefiles;
}
#########################################################################
#                   End of Spring
#########################################################################


sub CompareFiles {
    local($expectFileName) = shift @_;
    local($resultFileName) = shift @_;
    local($status) = 0;
    local($lineNum) = 0;

    # Check that files exist
    if(! -e $expectFileName){
        return -1;
    }

    open(fd,$resultFileName);
    open(fd1,$expectFileName);

    @file1 = <fd>;
    @file2 = <fd1>;

    for ($i=0;$i < @file1;$i++){
        $line1 = $line1.$file1[$i];
    }
    for ($i=0;$i < @file2;$i++){
        $line2 = $line2.$file2[$i];
    }

    $line1 =~ s/\s+//g;
    $line2 =~ s/\s+//g;
    $line1 =~ tr/A-Z/a-z/;
    $line2 =~ tr/A-Z/a-z/;
    if ($line1 == $line2){
        $status = 0;  
    }else{
        $status = 1;
    }
    return $status;
}


sub TestResults {
    open(testResults,">> $homedir/test.results");
    local($testLogCase) = shift @_;
    local($testLogExpl) = shift @_;
    local($testLogStatus) = shift @_;
    local($testLogResults) = shift @_;
    local($testLogFiles) = shift @_;
    printf testResults "%3d. %-30s\n %-30s %s\n"
                        ,$testLogCount,$testLogCase,$testLogExpl,$testLogStatus;    $testLogCount++;
     if($testLogStatus eq $failed){
         print testResults "       Explanation:\n";
         print testResults "           $testLogResults\n";
         print testResults "       See Files:\n";
         print testResults "           $testLogFiles\n";
        }
        close(testResults);
}


sub LogMsg {
    for($level = 0;$level < $logLevel; $level++){
        print $logfile "  ";
    }
    print $logfile @_[0];
}




