//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: SetType.java,v 1.1 2002/09/29 02:47:43 cvs Exp $
// Revision History:
// ----------------
// $Log: SetType.java,v $
// Revision 1.1  2002/09/29 02:47:43  cvs
// Added the directory "types" to support the different types in DVTG.
// --Murthy.
//
// Revision 1.3  2002/09/27 05:22:05  cvs
// Implemented the complete functionality of BitVectors.
// --Murthy.
//
// Revision 1.2  2002/09/08 21:50:41  cvs
// Added a new class to take care of Literals.Split the functionality of ReadTestRequirements class to make the tool more flexible.
// --Murthy.
//

package org.sldl.rosetta.tools.generatedata.types;

import org.sldl.rosetta.tools.generatedata.scanner.ParseException;
import org.sldl.rosetta.tools.generatedata.scanner.Container;
import org.sldl.rosetta.tools.generatedata.scanner.DVTGScanner;
import org.sldl.rosetta.tools.generatedata.scanner.Operand;
import org.sldl.rosetta.tools.generatedata.scanner.Operator;
import org.sldl.rosetta.tools.generatedata.values.*;


import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.SetValue;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.lang.values.RealValue;
import org.sldl.rosetta.lang.values.IntValue;

import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.BitSet;

import java.lang.Integer;
import java.lang.Byte;

/**
 * Describe class <code>SetType</code> here.
 *
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class SetType implements Type {

  private org.sldl.rosetta.ROM.SetValue initial_value;
  private org.sldl.rosetta.ROM.SetValue final_value;
  private Object  increment;
  private String input_var;
  private HashMap inputs;

  /**
   * Creates a new <code>SetType</code> instance.
   * The primary constructor.
   * @param initialIn an <code>Object</code> value
   * @param finalIn an <code>Object</code> value
   * @param incIn an <code>int</code> value
   */
  public SetType(String inputIn,Object initialIn,Object finalIn,Object incIn) {
    initial_value = (org.sldl.rosetta.ROM.SetValue)initialIn;
    final_value = (org.sldl.rosetta.ROM.SetValue)finalIn;
    increment = incIn;
    input_var = inputIn;
  }

  /**
   * Creates a new <code>SetType</code> instance.
   * The secondary constructor.
   * @param inputsMapIn a <code>HashMap</code> value
   */
  public SetType(HashMap inputsMapIn) {
    inputs = inputsMapIn;
    Set set  = inputsMapIn.entrySet();
    Iterator setIter = set.iterator();
    while (setIter.hasNext()) {
      Map.Entry entry = (Map.Entry)setIter.next();
      System.out.println("The entry class name is: "+entry.getKey().getClass().getName());
      System.out.println("The entry value is: "+entry.getKey().toString());
      input_var = (String)entry.getKey().toString();
      LinkedList values = (LinkedList)entry.getValue();
      initial_value = (SetValue)values.get(0);
      final_value = (SetValue)values.get(1);
      increment = values.get(2);
    } // end of while (setIter.hasNext())
    
  }

  /**
   * Describe <code>getInitialValue</code> method here.
   *
   * @return an <code>Object</code> value
   */
  public Object getInitialValue() {
    return (org.sldl.rosetta.ROM.SetValue)initial_value;
  }

  /**
   * Describe <code>getFinalValue</code> method here.
   *
   * @return an <code>Object</code> value
   */
  public Object getFinalValue() {
    return (org.sldl.rosetta.ROM.SetValue)final_value;
  }

  /**
   * Describe <code>getIncrement</code> method here.
   *
   * @return an <code>int</code> value
   */
  public Object getIncrement() {
    return increment;
  }

  public void setInputs(HashMap inputsIn) {
    inputs = inputsIn;
  }
  

  /**
   * Describe <code>buildInitialFinals</code> method here.
   *
   * @return a <code>LinkedList</code> value
   */
 //   public LinkedList buildInitialFinals() {
//      LinkedList initFinList = new LinkedList();
//      String initValue ="";
//      String finalValue = "";

//      for (int pos = 0; pos < initial_value.size(); pos++) {
//        initValue = initValue+initial_value.getElements().get(pos);
//        finalValue = finalValue+final_value.getElements().get(pos);
//      } // end of for (int pos = 0; pos < initial_value.size(); pos++)


   //   //Converting the values in the string format to the float values.
//      Integer initInt = new Integer(initValue);
//      Integer finalInt = new Integer(finalValue);
//      initFinList.add(initInt);
//      initFinList.add(finalInt);
//      initFinList.add(increment);
//      System.out.println("Init value: "+Byte.parseByte(initValue.toString(),2));
//      System.out.println("Final value: "+Byte.parseByte(finalValue.toString(),2));
//      System.out.println("The increment is: "+increment);
//      return initFinList;
//    }


  /**
   * Describe <code>buildVectors</code> method here.
   * This method is used to build the vectors for the 
   * SetType.
   * @param finalList a <code>LinkedList</code> value
   * @param pw a <code>PrintWriter</code> value
   * @param xmlPw a <code>PrintWriter</code> value
   * @exception FileNotFoundException if an error occurs
   */
  //This method needs to be deleted.
 //   public void buildVectors_backup(LinkedList finalList,PrintWriter pw, PrintWriter xmlPw) throws FileNotFoundException {

//      int index = 1;

//      //This method call is used to convert the initial and final
//      //values in bitvector type to the corresponding integer values.
//      LinkedList initFinalLst = this.buildInitialFinals();

//      int count = 0;//This count is used as the index value.
//      String tmpInit = (String)initFinalLst.get(count).toString();
//      String tmpFinal = (String)initFinalLst.get(count+1).toString();
//      String tmpInc = (String)initFinalLst.get(count+2).toString();
    
//      //The following lines of code generates the integer 
//      //values from the bitvectors in String.
//      int initialValue = Byte.parseByte(tmpInit,2);
//      int finalValue = Byte.parseByte(tmpFinal,2);
//      int incrementValue = (new Integer(tmpInc)).intValue();

//      //***********************************************************//
//      System.out.println("Testing the Scenarios");
//      Iterator fnlIter = finalList.iterator();
//      while (fnlIter.hasNext()) {
//        LinkedList inrLst = (LinkedList)fnlIter.next();
//        Iterator inrIter = inrLst.iterator();
//        while (inrIter.hasNext()) {
//  	System.out.println("The scenario is: "+inrIter.next());
//        } // end of while (inrIter.hasNext())
//      } // end of while (fnlIter.hasNext())
//      //***********************************************************//
    
//      xmlPw.println("<vectorslist>");
//      for (int iptVar = initialValue;iptVar <= finalValue;iptVar = iptVar+incrementValue) {

//        int flag = 1;
//        for (Iterator outerIter = finalList.iterator();outerIter.hasNext();) {
//  	LinkedList innerList = (LinkedList)outerIter.next();
	
//  	for (Iterator innerIter = innerList.iterator();innerIter.hasNext();) {
//  	  Application app = (Application)innerIter.next();
//  	  String leftOp = app.getParameters().get(0).toString();
//  	  String operator = app.getFunction().toString();

	  
//  	  //This hashmap stores all the input variables 
//  	  //and the values corresponding to them for this loop.
//  	  HashMap ipts = new HashMap();
//  	  String inptValue_tmp = Integer.toBinaryString(iptVar);
	  
//  	  //The above code returns the bitvector as a String.
//  	  //for example for 4 it returns 100.The following code is
//  	  //used to change that into a bitvector form...{1,0,0}
//  	  String inptValue = "{";
//  	  for (int cnt = 0; cnt < inptValue_tmp.length(); cnt++) {
//  	    if (cnt == inptValue_tmp.length()-1) {
//  	      inptValue = inptValue+inptValue_tmp.charAt(cnt);
//  	    } // end of if (count == inptValue_tmp.length()-1)
//  	    else {
//  	      inptValue = inptValue+inptValue_tmp.charAt(cnt)+","; 
//  	    } // end of else
//  	  } // end of for (int count = 0; count < inptValue_tmp.length(); count++)
//  	  inptValue = inptValue+"}";
	  
//  	  //Now the input_var and its associated value are stored in 
//  	  //a hashmap.
//  	  ipts.put(input_var,inptValue);
	  
//  	  //The following statements sends the expression and the 
//  	  //values associated with the input variables.
//  	  evaluateExpression(app,ipts,pw,xmlPw);	       
	  
//  	}
//        }
//      }
//      xmlPw.println("</vectorslist>");
//    }
  

  /**
   * Describe <code>buildVectors</code> method here.
   * 
   * @param finalList a <code>LinkedList</code> value
   * @param pw a <code>PrintWriter</code> value
   * @param xmlPw a <code>PrintWriter</code> value
   * @exception FileNotFoundException if an error occurs
   */
  public void buildVectors(LinkedList finalList,PrintWriter pw, PrintWriter xmlPw) throws FileNotFoundException {
   
   int xmlCount = 1;
   
   //***********************************************************//
   System.out.println("Testing the Scenarios");
   Iterator fnlIter = finalList.iterator();
   while (fnlIter.hasNext()) {
     LinkedList inrLst = (LinkedList)fnlIter.next();
     Iterator inrIter = inrLst.iterator();
     while (inrIter.hasNext()) {
       System.out.println("The scenario is: "+inrIter.next());
     } // end of while (inrIter.hasNext())
   } // end of while (fnlIter.hasNext())
   //***********************************************************//
   
   xmlPw.println("<vectorslist>");
   
   Set inputs_set = inputs.entrySet();
   Iterator inputs_set_iter = inputs_set.iterator();

   //These two are declared here because their scope will be 
   //limited if they are declared in the for loop.
   String input_key_outer = null;
   String input_key_inner = null;
   int var_outer = 0;
   int var_inner = 0;

   //This is used for the parameters in the outer loop.
   Map.Entry input_entry = (Map.Entry)inputs_set_iter.next();
   System.out.println("The key is: "+input_entry.getKey());
   input_key_outer = (String)input_entry.getKey();
   LinkedList input_value = (LinkedList)input_entry.getValue();

   
   if (inputs_set_iter.hasNext()) {
     
     //This is used for the parameters in the inner loop.
     Map.Entry input_entry_inner = (Map.Entry)inputs_set_iter.next();
     input_key_inner = (String)input_entry_inner.getKey();
     LinkedList input_value_inner = (LinkedList)input_entry_inner.getValue();
     int initial_inner = getInteger(input_value_inner.get(0));
     int final_inner = getInteger(input_value_inner.get(1));
     int increment_inner = getInteger(input_value_inner.get(2));
     
     
     //This one is the outer loop which takes care of the first operand.
     for (var_outer = getInteger(input_value.get(0));var_outer <= getInteger(input_value.get(1));var_outer = var_outer+getInteger(input_value.get(2))) {
       //if (inputs_set_iter.hasNext()) {
       
       //This is the inner loop which takes care of the second operand.
       for (var_inner  = initial_inner;var_inner <= final_inner;var_inner = var_inner+increment_inner) {
	 
	 for (Iterator outerIter = finalList.iterator();outerIter.hasNext();) {
	   LinkedList innerList = (LinkedList)outerIter.next();
	   
	   for (Iterator innerIter = innerList.iterator();innerIter.hasNext();) {
	     Application app = (Application)innerIter.next();
	     
	     //This hashmap stores all the input variables 
	     //and the values corresponding to them for this loop.
	     HashMap ipts = new HashMap();
	     ipts.put(input_key_outer,Integer.toBinaryString(var_outer));
	     ipts.put(input_key_inner,Integer.toBinaryString(var_inner));
	     
	     
	     //The following statements sends the expression and the 
	     //values associated with the input variables.
	     evaluateExpression(app,ipts,pw,xmlPw,xmlCount);	       
	     
	   }
	 }
       }
       xmlCount++;
       pw.println();
       //}
     }
   }
   else {
     for (var_outer = getInteger(input_value.get(0));var_outer <= getInteger(input_value.get(1));var_outer = var_outer+getInteger(input_value.get(2))) {
       
       for (Iterator outerIter = finalList.iterator();outerIter.hasNext();) {
	 LinkedList innerList = (LinkedList)outerIter.next();
	 
	 for (Iterator innerIter = innerList.iterator();innerIter.hasNext();) {
	   Application app = (Application)innerIter.next();
	   
	   //This hashmap stores all the input variables 
	   //and the values corresponding to them for this loop.
	   HashMap ipts = new HashMap();
	   ipts.put(input_key_outer,Integer.toBinaryString(var_outer));
	   
	   //The following statements sends the expression and the 
	   //values associated with the input variables.
	   evaluateExpression(app,ipts,pw,xmlPw,xmlCount);	
	 } // end of else
	 

       }
       xmlCount++;
     }
   }
   xmlPw.println("</vectorslist>");
  }
  
  
  
  /**
   * Describe <code>getInteger</code> method here.
   * This code is used to get the integer representation
   * of the BitVector. for example it returns the integer
   * equivlent of {0,0,1,0} as 2.
   * @param valueIn an <code>Object</code> value
   * @return an <code>int</code> value
   */
  public int getInteger(Object valueIn) {
    int final_value = 0;
    if (valueIn instanceof SetValue) {
      SetValue value = (SetValue)valueIn;
      
      //The following lines strip of the commas and braces 
      //from the bitvector.
      String initValue = "";
      for (int pos = 0; pos < value.size(); pos++) {
	initValue = initValue+value.getElements().get(pos);
      } // end of for (int pos = 0; pos < initial_value.size(); pos++)
      final_value = Byte.parseByte(initValue,2);
    } // end of if (valueIn instanceof SetValue)
    if (valueIn instanceof IntValue) {
      final_value = (new Integer(valueIn.toString())).intValue();
    } // end of if (valueIn instanceof Integer)
    return final_value;
  }
  
  /**
   * Describe <code>evaluateExpression</code> method here.
   * This method evaluates the expressions.for example 
   * all the calculations take place here.
   * @param appIn an <code>Application</code> value
   * @param ipt_var_values a <code>HashMap</code> value
   * @param text a <code>PrintWriter</code> value
   * @param xmlPwIn a <code>PrintWriter</code> value
   */
  public void evaluateExpression(Application appIn,HashMap ipt_var_values,PrintWriter text,PrintWriter xmlPwIn,int xmlCountIn) {

    String value = appIn.toString();
    System.out.println("The initial string is: "+value);

    DVTGScanner parser = new DVTGScanner(value);
    try {
      Container cont = parser.scan();       
      //cont.display();

      //This gets the operands from the container.
      Operand operand = cont.getOperands();
      LinkedList operandsList = operand.getOperands();
      System.out.println("The original operands are: "+operandsList);
      
      //This gets the operators from the container.
      Operator operator = cont.getOperators();
      LinkedList operatorsList = operator.getOperatorList();
      System.out.println("The operators size: "+operatorsList.size());

      //This case deals when there are no operators.
      if (operatorsList.size() == 0) {

	String finalValue = (String)operandsList.getFirst().toString();
	System.out.println("The final value is: "+finalValue);
	
     	Object final_value = ipt_var_values.get(finalValue);
	System.out.println("The final_value is: "+final_value.getClass().getName());
	BitSet final_bitset = Identifier.toBitSet((String)final_value);
	
	printWaves(text,ipt_var_values,(BitSet)final_bitset,operand);
	printXml(xmlPwIn,ipt_var_values,(BitSet)final_bitset,operand,xmlCountIn);
      } // end of if (operatorsList.size() == 0)

      else {

	Iterator operandsIter = operandsList.iterator();
	System.out.println("The operands are: "+operandsList);

	//The logic is similar to generating fibonacci sequence.
	//This stores the first operand which becomes the
	//left Operand.
	Object operand_left_tmp = operandsIter.next();
	BitSet left_bitset = null;

	if (operand_left_tmp instanceof Identifier) {
	  String operand_left = operand_left_tmp.toString();
	  Object left_value = ipt_var_values.get(operand_left);
	  left_bitset = Identifier.toBitSet((String)left_value);
	  System.out.println("Left Operand: "+operand_left+" value: "+left_bitset);
	} // end of if (operand_left instanceof Identifier)


	//This for loop is used to iterate through all the Operators.
	for (Iterator operatorIter = operatorsList.iterator(); operatorIter.hasNext();) {

	  //This grabs the next operand in the List of Operands 
	  //which becomes the right operand.
	  Object operand_right_tmp = operandsIter.next();
	  BitSet right_bitset = null;
	  
	  //The following code checks if the object is an Identifier.
	  if (operand_right_tmp instanceof Identifier) {
	    Identifier operand_right = (Identifier)operand_right_tmp;
	    String operand_right_str = operand_right.toString();
	    Object right_value = ipt_var_values.get(operand_right_str);
	    System.out.println("Right Operand: "+operand_right+" value: "+right_value);
	    right_bitset = Identifier.toBitSet((String)right_value);
	  }
	  
	  //The following code checks if the Object is a BitVector.
	  if (operand_right_tmp instanceof BitVector) {
	    BitVector operand_right = (BitVector)operand_right_tmp;
	    right_bitset = operand_right.toBitSet();
	    System.out.println("The right operand: "+right_bitset);
	  } // end of if (operand_right_tmp instanceof BitVector)
	  
	  System.out.println("the left bit set is: "+left_bitset);

	  //The operator is grabbed and the logical operation is
	  //performed.
	  String oper = (String)operatorIter.next();
	  if (oper.equalsIgnoreCase("and")) {
	    left_bitset.and(right_bitset);
	  } // end of if (oper.equalsIgnoreCase("and"))
	  if (oper.equalsIgnoreCase("or")) {
	    left_bitset.or(right_bitset);
	  } // end of if (oper.equalsIgnoreCase("or"))
	  System.out.println("The left op after operation: "+left_bitset);  
	}

	//To print the vectors in the wave format.
	printWaves(text,ipt_var_values,left_bitset,operand);
	
	//To print the vectors in XML format.

	printXml(xmlPwIn,ipt_var_values,left_bitset,operand,xmlCountIn);

	
      } // end of else
      
    } catch (ParseException  e) {
      System.out.println("Parse Exception: "+e);
    } // end of try-catch
  }

  public void printBitVector(PrintWriter pw,String vectorIn) {
    pw.print("{");
    for (int count = 0;count < vectorIn.length();count++) {
      if (count == (vectorIn.length() - 1)) {
	pw.print(vectorIn.charAt(count));
      } // end of if (count == (vectorIn.length() - 1))
      else {
	pw.print(vectorIn.charAt(count)+",");
      } // end of else
      
    } // end of for (int count = 0;count < vectorIn.length();count++)
    pw.print("}\t");
  }


  /**
   * Describe <code>printWaves</code> method here.
   *
   * @param textIn a <code>PrintWriter</code> value
   * @param iptsIn a <code>HashMap</code> value
   * @param outputIn a <code>BitSet</code> value
   * @param operandIn an <code>Operand</code> value
   */
  public void printWaves(PrintWriter textIn,HashMap iptsIn,BitSet outputIn,Operand operandIn) {
    System.out.println("I am into PrintWaves");
    Set iptSet = iptsIn.entrySet();
    Iterator iptSetIter = iptSet.iterator();

    while (iptSetIter.hasNext()) {
      Map.Entry iptSetEntry = (Map.Entry)iptSetIter.next();
      
      textIn.print(iptSetEntry.getKey()+" = ");
      printBitVector(textIn,(String)iptSetEntry.getValue());
    } // end of while (iptSetIter.hasNext())

    textIn.print(operandIn.getLeftOperand()+" = {");
    //Here I am taking the length of the BitVector to be 4.Should 
    //generalise it to any length.
      for (int index_print = 3; index_print >= 0; index_print--) {
	if (outputIn.get(index_print)) {
	  if (index_print == 0) {
	      textIn.print("1 ");
	  } // end of if (index_print == 0)
	  else {
	    textIn.print("1, ");	     
	  } // end of else
	} // end of if (a.get(index))
	else {
	  if (index_print == 0) {
	    textIn.print("0 ");
	  } // end of if (index_print == 0)
	  else {
	    textIn.print("0, ");	     
	  } // end of else
	} // end of else
      } // end of for (int index = 0; index < 4; index++)
      textIn.println("}");
  }
  

  /**
   * Describe <code>printXml</code> method here.
   *
   * @param xmlPw a <code>PrintWriter</code> value
   * @param iptsIn a <code>HashMap</code> value
   * @param outputIn a <code>BitSet</code> value
   * @param operandIn an <code>Operand</code> value
   */
  public void printXml(PrintWriter xmlPw,HashMap iptsIn,BitSet outputIn,Operand operandIn,int countIn) {
    xmlPw.println("<vector count = \""+countIn+" \">");
    Set iptSet = iptsIn.entrySet();
    
    Iterator iptSetIter = iptSet.iterator();
    while (iptSetIter.hasNext()) {
      Map.Entry iptSetEntry = (Map.Entry)iptSetIter.next();
      xmlPw.println("<condition>");
      xmlPw.print("\t<parameter mode = \"in\"> ");
      xmlPw.print(iptSetEntry.getKey());
      xmlPw.println(" </parameter>");
      xmlPw.print("\t<value> ");
      printBitVector(xmlPw,(String)iptSetEntry.getValue());
      xmlPw.println(" </value>");
      xmlPw.println("</condition>");
  }
    xmlPw.println("<condition>");
    xmlPw.print("\t<parameter mode=\"out\"> ");
    xmlPw.println(operandIn.getLeftOperand()+" </parameter>");
    xmlPw.print("\t<value> {");
    for (int index_print = 3; index_print >= 0; index_print--) {
	if (outputIn.get(index_print)) {
	  if (index_print == 0) {
	      xmlPw.print("1 ");
	  } // end of if (index_print == 0)
	  else {
	    xmlPw.print("1, ");	     
	  } // end of else
	} // end of if (a.get(index))
	else {
	  if (index_print == 0) {
	    xmlPw.print("0 ");
	  } // end of if (index_print == 0)
	  else {
	    xmlPw.print("0, ");	     
	  } // end of else
	} // end of else
      } // end of for (int index = 0; index < 4; index++)
      xmlPw.print("}");
      xmlPw.println(" </value>");
      xmlPw.println("</condition>");
      xmlPw.println("</vector>");
  }

}//end of class.
