//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: SymbolTable.java,v 1.1.1.1 2002/07/01 15:47:36 cvs Exp $
//
// Revision History:
// ----------------
// $Log: SymbolTable.java,v $
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.4  2002/06/06 13:46:19  kimmell
// Removed some logging output.
// -kimmell
//
// Revision 1.1.2.3  2002/05/28 21:30:28  kimmell
// Fixed a lookup problem that caused facets to ignore resolution of symbols which were in the extended domain.
// -kimmell
//
// Revision 1.1.2.2  2002/05/24 21:10:17  kimmell
// 1. Added inital code for a simpler domain import functionality.
// 2. Added a error check in the importScope method.
//
// Revision 1.1.2.1  2002/03/12 22:27:22  cvs
// Added a linker that uses the multi-dispatch visitor.
// -kimmell
//
package org.sldl.rosetta.tools.glinker;

import java.util.Map;
import java.util.Map.Entry;
import java.util.HashMap;
import java.util.Set;
import java.util.NoSuchElementException;
import java.util.Iterator;

import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.Expression;


import org.sldl.rosetta.tools.testparser.TestParser;

/**
 * SymbolTable.java
 *
 *
 * Created: Tue Feb  5 13:49:05 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

public class SymbolTable {
  
  Map symbols;
  SymbolTable parent;
  SymbolTable domain;
  String domainName;
  Linker linker;
  String name;

  
  public SymbolTable (){
    symbols = new HashMap();
    parent = null;
  }

  public SymbolTable(SymbolTable argParent) {
    symbols = new HashMap();
    parent = argParent;
  }

  public SymbolTable(SymbolTable argParent, Linker argLinker) {
    symbols = new HashMap();
    parent = argParent;
    linker = argLinker;
  }
    
 
  public void addEntry(String name, Object value) {
    SymbolTableEntry entry = (SymbolTableEntry) value;
    symbols.put(name, value);
    // System.out.println("Added " + name + " to the symboltable");
  }

  public void replaceEntry(String name, Object value) {
    symbols.put(name, value);
  }

  public void removeEntry(String name) {
    try {
      symbols.remove(name);
    } catch (NoSuchElementException e) {
    } // end of try-catch
    return;
  }

  public void importScope(SymbolTable table) {
    String symbol;
    for (Iterator keyIter = table.keySet().iterator(); keyIter.hasNext();){
      symbol = (String) keyIter.next();
      try {
	addEntry(symbol, table.lookup(symbol));
      } catch (VariableDerefException e) {
	 
      } catch (NameNotFoundException e) {
	
	 
      } // end of catch
      
    } // end of for (Iterator keyIter = table.keySet.iterator();
    // keyIter.hasNext();)
  }

  public void importScope(String name) {
    try {
      SymbolTableEntry entry = lookup(name);
      importScope(entry.getSymbolTable());
      // System.out.println("Imported scope " + name);
    } catch (NameNotFoundException e) {
      loadLibrary(name);
    } catch (VariableDerefException e) {
       
    } catch (NullPointerException e) {
      System.out.println("Could not import scope " + name);
    } // end of catch
        
  }


  public void loadLibrary(String packageName) {
    
    String rosettaPath = System.getProperty("ROSETTA_PATH");
    String sourceFile = rosettaPath + "/" + packageName + ".sld";
    System.out.println("Loading file: " + sourceFile);
    
    TestParser parser = new TestParser();
    Object root = parser.compileFile(sourceFile);
    linker.traverse(root, this);
  }


  public void setDomain(ItemReference ir) {
    setDomain(ir.getReferenceName());
  }

    
  public void setDomain(String name) {
    try {
      //      System.out.println("Setting domain of " + this.name + " to " + name);
      SymbolTableEntry entry = lookup(name);
//        if (entry == null) {
//  	System.out.println("domain " + name + " was looked up as null.");
//        } else {
//  	entry.getSymbolTable().print("");
//        } // end of else
      
      
      domain = entry.getSymbolTable();
      domainName = name;
      //      System.out.println("Found domain - Name: " + domain.getName() +
      //			 "::" + domain.getDomainName());
    } catch (NameNotFoundException e) {
      System.out.println("Could not find domain " + name);
    } catch (VariableDerefException e) {
       
    } // end of catch
  }
    

  public void setDomain(SymbolTable argDomain) {
    domain = argDomain;
  }


  public void setDomainName(String name) {
    domainName = name;
  }

  public String getName() {
    return name;
  }

  public void setName(String argName) {
    name = argName;
    // System.out.println("Set name of scope to be " + argName);
  }

  public SymbolTable getDomain() {
    return domain;
  }

  public String getDomainName() {
    return domainName;
  }


  public Set keySet() {
    return symbols.keySet();
  }

  public SymbolTableEntry getEntry(String name) {
    return (SymbolTableEntry) symbols.get(name);
  }

  public SymbolTableEntry lookup(String name) throws NameNotFoundException,
  VariableDerefException {
    String immediate;
    String rest;

    //    System.out.println("Looking up " + name + " in scope of " +
    // this.name + " with a domain " + domainName);
    // Break the name into two parts - one for the immediately
    // needed symbol, one for the remaining part (if it exists)
    int delimiter = name.indexOf(".");
    if (delimiter != -1) {
      immediate = name.substring(0,delimiter);
      rest = name.substring(delimiter + 1, name.length());
    } else {
      immediate = name;
      rest = null;
    } // end of else
    
    
   
    SymbolTableEntry entry = (SymbolTableEntry) symbols.get(immediate);

    if ((entry == null) && (domain != null)) {
      //System.out.println("Looking up in domain " + domainName);
      try {
	entry = (SymbolTableEntry) domain.lookup(name);
      } catch (NameNotFoundException e) {

      } // end of catch
      
      //System.out.println("Done looking up in domain " + domainName);
    } // end of if ((entry == null) && (domain != null))

    
    if (entry == null) {
      //      System.out.println("Did not find in current scope or in domain "
      //			 + domainName);
      if (parent == null) {
	//System.out.println("LINK ERROR: Could not find symbol " + immediate + 
	//		   " anywhere.");
	throw new NameNotFoundException(name);
      } else {

	return parent.lookup(name);
      } // end of else
    }
      
	      
    Item entryValue = entry.getValue();
    SymbolTable entrySymbolTable = entry.getSymbolTable();
    //System.out.println("Found symbol " + immediate);
    
    if (rest == null) {
      return entry;
    } else if (entrySymbolTable == null) {
      throw new VariableDerefException(name);
    } else {
      return entrySymbolTable.lookup(rest);
    } // end of else
    
  }


  public String toString() {
    Set s = symbols.entrySet();
    
    String str = "\n";
    
    for (Iterator iter = s.iterator(); iter.hasNext();){
      Map.Entry me = (Map.Entry) iter.next();
      str = str + me.getKey() + "::" + (SymbolTableEntry) me.getValue() + "\n";
    } // end of for (Iterator iter = s.iterator(); iter.hasNext();)
    
    return str;
  }

  public void print(String tabs) {
    Set s = symbols.entrySet();
    System.out.print(tabs);
    System.out.println("Name: " + name + " Domain: " + domainName);
    for (Iterator iter = s.iterator(); iter.hasNext();){
      Map.Entry me = (Map.Entry) iter.next();
      System.out.println(tabs + me.getKey() + "::" + me.getValue());
      ((SymbolTableEntry)me.getValue()).print(tabs);
    } // end of for (Iterator iter = s.iterator(); iter.hasNext();)
    
  }

  
}// SymbolTable
