//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: MultiVisitor.java,v 1.1.1.1 2002/07/01 15:47:36 cvs Exp $
//
// Revision History:
// ----------------
// $Log: MultiVisitor.java,v $
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.3  2002/05/23 20:31:37  kimmell
// Updated the logging code to allow a subclass to limit the messages by setting a logLevel variable.
// -kimmell
//
// Revision 1.1.2.2  2002/03/21 20:31:36  cvs
// Removed some logging for debugging purposes.
// -kimmell
//
// Revision 1.1.2.1  2002/03/12 22:23:50  cvs
// Added a visitor class that uses the multi-dispatch library.
// -kimmell
//
package org.sldl.rosetta.tools.multivisitor;

import fr.umlv.jmmf.reflect.*;

import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.tools.testparser.TestParser;


/**
 * MultiVisitor.java
 *
 *
 * Created: Tue Feb 26 16:13:48 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

public class MultiVisitor {

  protected MultiMethod mm1;
  protected MultiMethod mm2;
  protected MultiMethod mm3;

  public MultiVisitor() {
	initMM();
  }
 
  protected void initMM (){
    //    System.out.println("Initing multimethods: class is " +
    // getClass().getName());
    mm1 = MultiMethod.create(getClass(),"process",1);
    mm2 = MultiMethod.create(getClass(),"process",2);
    mm3 = MultiMethod.create(getClass(),"process",3);
  }
  
  /**
   * This <code>traverse</code> method performs multidispatch of the 
   * visit method with one parameter.
   *
   * @param obj an <code>Object</code> value
   * @return an <code>Object</code> value
   */
  public Object traverse(Object obj) {
    if (obj == null) {
      System.out.println("Nothing to be done for a null object");
      return obj;
    } // end of if (obj == null)
    
    // System.out.println("Traverse(" + obj + ")");
    try {
      Object retObj = mm1.invoke(this, new Object[] { obj });
      //System.out.println("Returning an instance of " +
      //		 retObj.getClass().getName());
      return retObj;
    } catch (NullPointerException e) {
      System.out.println(e);
      e.printStackTrace();
    } catch (NoSuchMethodException e) {
      System.out.println("No such Method Exception" + e);
    } catch (java.lang.reflect.InvocationTargetException e) {
      System.out.println("InvocationTargetException:" + e);
      e.printStackTrace();
    } catch (java.lang.IllegalAccessException e) {
      System.out.println("IllegalAccessException: " + e);
    } catch (fr.umlv.jmmf.reflect.MultipleMethodsException e) {
      System.out.println("MultipleMethodsException: " + e);
    }
    
    System.out.println("Returning null from traverse");
    return null;
      
  } 

  /**
   * This <code>traverse</code> method performs
   *
   * @param ob1 an <code>Object</code> value
   * @param ob2 an <code>Object</code> value
   * @return an <code>Object</code> value
   */
  public Object traverse(Object ob1, Object ob2) {
    if (ob1 == null) {
      System.out.println("Nothing to be done for a null object");
      return ob1;
    } // end of if (obj == null)

    // System.out.println("Traverse(" + ob1 + ", " + ob2 + ")");
    try {
      return mm2.invoke(this, new Object[] { ob1, ob2 });
    } catch (NoSuchMethodException e) {
      System.out.println("No such Method Exception" + e);
    } catch (java.lang.reflect.InvocationTargetException e) {
      System.out.println("InvocationTargetException:" + e);
      System.out.println("Stack trace:");
      e.printStackTrace();
    } catch (java.lang.IllegalAccessException e) {
      System.out.println("IllegalAccessException: " + e);
    } catch (fr.umlv.jmmf.reflect.MultipleMethodsException e) {
      System.out.println("MultipleMethodsException: " + e);
    }
    System.out.println("Returning null from traverse");
    return null;
  } 

  /**
   * This <code>traverse</code> method performs multi-dispatch of the
   * process method with three parameters.
   *
   * @param ob1 an <code>Object</code> value
   * @param ob2 an <code>Object</code> value
   * @param ob3 an <code>Object</code> value
   * @return an <code>Object</code> value
   */
  public Object traverse(Object ob1, Object ob2, Object ob3) {
    if (ob1 == null) {
      System.out.println("Nothing to be done for a null object");
      return ob1;
    } // end of if (obj == null)
    
    // System.out.println("Traverse(" + ob1 + ", " + ob2 + ", " + ob3 + ")");
    try {
      return mm3.invoke(this, new Object[] { ob1, ob2, ob3 });
    } catch (NoSuchMethodException e) {
      System.out.println("No such Method Exception" + e);
    } catch (java.lang.reflect.InvocationTargetException e) {
      System.out.println("InvocationTargetException:" + e);
    } catch (java.lang.IllegalAccessException e) {
      System.out.println("IllegalAccessException: " + e);
    } catch (fr.umlv.jmmf.reflect.MultipleMethodsException e) {
      System.out.println("MultipleMethodsException: " + e);
    }
    return null;
  } 

  public Object process(BaseObject obj) {
    //System.out.println("Visiting a " + obj.getClass().getName());
    //System.out.println("Value:\n" + obj + "\n\n");
    Object[] children = obj.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i]);
    } // end of for (int i = 0; i < children.length; i++)
    return obj;
  }
  

  public Object process(BaseObject obj, Object arg) {
    // System.out.println("Visiting a " + obj.getClass().getName());
    Object[] children = obj.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i], arg);
    } // end of for (int i = 0; i < children.length; i++)
    return obj;
  }

  public Object process(BaseObject obj, Object arg1, Object arg2) {
    // System.out.println("Visiting a " + obj.getClass().getName());
    Object[] children = obj.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i], arg1, arg2);
    } // end of for (int i = 0; i < children.length; i++)
    return obj;
  }

  public Object process(ItemReference obj) {
    //    log(MESSAGE, "Visiting an ItemReference:" +
    //	       obj.getReferenceName() + 
    //	       ". Defaulting to a pass-through");
    Object item = obj.getItemReferred();
    if (item == null) {
      log(MESSAGE, "Warning - itemReferred is null");
    } // end of if (item == null)
    
    return traverse(obj.getItemReferred());
  }


  // Logging functions 
  protected String toolName;
  protected  int logLevel = 0;
  private final String logLevels[] = {"DEBUG", "MESSAGE", "ERROR"};
  protected final int MESSAGE = 1;
  protected final int DEBUG = 0;
  protected final int ERROR = 2;
  protected void log(int level, String message) {
    if (level < logLevel) {
      return;
    } // end of if (level < logLevel)
    
    String s;
    s = toolName + " " + logLevels[level] + ": " + message;
    System.out.println(s);
  }

  public static void main (String[] args) {
    
    if (args.length != 1) {
      System.out.println("usage: MoMLGen [sldlfile]");
      return;
    } // end of if (args.length != 1)
    
    MultiVisitor visitor = new MultiVisitor();
    TestParser parser = new TestParser();
    Object root = parser.compileFile(args[0]);
    visitor.traverse(root);
    
  } // end of main ()
    
}// MultiVisitor
