//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// $Log: PrettyPrinter.java,v $
// Revision 1.3  2002/11/04 17:42:20  cvs
// Updated PrettyPrinter to work with the new ANTLR parser.Still running into problems with printing.
// --Murthy.
//
// Revision 1.2  2002/08/21 17:00:42  jstan
// The prettyprinter for the ROM
//
// Currently does not attempt to handle Components, Constructed Types,
// and Interactions, since they will be changing soon.
//
// Present Problems:  Elements in enumerations, partially incomplete
// function definitions, and missing parameters in let expressions.
//
// Other anomalies:
// clean10.sld -> character literal represented as "Unknown"
// test_member.sld
// test_at.sld
//
// -Stanley
//
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// Revision 1.1.1.1  2002/07/01 15:47:34  cvs
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// Rosetta tools and publications repository
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
//
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// Revision 1.1.2.1  2002/05/02 20:24:40  kimmell
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// Added the JDE project file, which includes the license and coding conventions.
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
// -kimmell
//
// $Id: PrettyPrinter.java,v 1.3 2002/11/04 17:42:20 cvs Exp $
//
// Revision History:
// ----------------
//
package org.sldl.rosetta.tools.prettyprinter;

/**
 * PrettyPrinter.java
 *
 *
 * Created: Wed Jul 17 11:24:36 2002
 *
 * @author <a href="mailto:jstanley@eecs.ku.edu">Jesse Stanley</a>
 * @version
 */
import java.util.Iterator;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.*;
import org.sldl.rosetta.tools.multivisitor.MultiVisitor;
import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;

//Packages to be imported for antlr parser.
import org.sldl.rosetta.parser.tarp.misc.*;
import org.sldl.rosetta.parser.tarp.antlr.*;

import java.io.File;
import java.io.FileReader;

public class PrettyPrinter extends MultiVisitor {
  public PrettyPrinter (){
  }

  private String INDENT = "";

  private String process(Item i){
   
    String retval = "";
  
    if (i.getValue() != null && i.getLabel() != null) {
      retval += (String) traverse(i.getValue(),i.getType(),
				  i.getLabel().toString());
    } else{
      
      String temp = "";
      if (i.getDecoration("kind") != null) {
	temp = (String) traverse(i.getDecoration("kind")) + " ";
      } // end of if (i.getDecoration("kind"))
           
      retval += INDENT + i.getLabel() + " :: " + temp;
      
      if (i.getType() != null) {
	retval += traverse(i.getType());
      } // end of if (i.getType() != null)
    }
    return retval;
  }

  private String process(Function f){
    String retval = " <*(";
    int pos;
    ItemList interfaceList = f.getInterfaceItems(); 
  
    if ((interfaceList != null) && (!interfaceList.isEmpty()))
      {
	retval += interfaceList.get(0);
	retval += "::" + interfaceList.get(0).getType();
	for (pos = 1; pos < interfaceList.size(); pos++)
	  { retval += "::" + interfaceList.get(pos) + interfaceList.get(pos).getType(); }
      }
    
    retval += ") :: " + f.getRange() + "*>";
    return retval;

  }

  public Object process(ItemReference ir){
    return ir.getReferenceName();
  }

  private String process(TypeExpression t){
    String retval;
    retval = t.getRelationship().toString() + "(";
  
    if (t.getExpression() != null) 
      retval += traverse(t.getExpression()); 
   
    return retval  + ")"; 
  }

  private String process(PrimitiveValue p, Object o, String label){
    return INDENT + label + " :: " + o.toString() + " is " + p.getPrimitiveValue();
  }
  
  private String process(LetFunction l, Object o, String label){
    String retval = "\n" + INDENT + label + ": let" 
      //I need to insert o.getsomething here to get at the terms  
      + " in " + traverse(l.getExpression()) + " end let";

    return retval;
  }

  /*   This method conflicts with ArrayValue
       MultipleMethodsException: fr.umlv.jmmf.reflect.MultipleMethodsException: mul
       tiple methods process of declaring class match
       process(org.sldl.rosetta.ROM.CompositeValue,java.lang.Object,java.lang.Str
       ing)process(org.sldl.rosetta.ROM.ArrayValue,java.lang.Object,java.lang.Object)
       
       private String process(CompositeValue c, Object o, String label)
       {
       INDENT = "  ";
       String retval;
       retval = INDENT + label + " :: type is " + c.toString() +  " size is " + c.size()
       + " values are: ";
     
       retval += traverse(c.getElements());//goes inside of
       // ExpressionList, but I can't retrieve the individual members in
       // Expressionlist or anywhere else.  Have tried many things!

       INDENT = "";
       return retval;
       
       }*/
  
  private String process(Expression e){
    String retval;
    retval = e.toString();
    return retval;
  }

  private String process(ExpressionList e){
    String retval = "";
    for (Iterator expressionParser = e.iterator(); expressionParser.hasNext();)
      retval += traverse(expressionParser.next());
    
    return retval;
  }
 
 

  private String process(IfExpression i, Object type, String label){
    String retval = "\n" + INDENT + label + ": if " + i.getIfValue() +
      " then " + i.getThenValue() + " else " + i.getElseValue() + " end if";
    return retval;
  }
  


  private String process(ArrayValue a, Object b, Object c){
    String retval = INDENT + c.toString() + "::";
    if (b != null) {
      retval += traverse(b);
    } // end of if (b != null)
    retval += " is [";
    for (Iterator arrayWalker = a.getElements().iterator(); arrayWalker.hasNext();){
      retval += traverse(arrayWalker.next());
    } // end of for (Iterator arrayWalker = a.getElements().iterator(); arrayWalker.hasNext();)
    return retval + "]";
  }

 

  private String process(Package p, Object o, String name){
    String retval = "package " + name + " is\nbegin " + p.getDomain();
    for (Iterator packIterator = p.getDeclaredItems().iterator(); packIterator.hasNext();){
      retval += traverse(packIterator.next());
    } // end of for (Iterator packIterator = p.getDeclarationItems().iterator(); packIterator.hasNext();)
    return retval + "\nend facet " + name + ";";
  }

  private String process(Function f, Object o, Object arg){
    String retval = INDENT + arg.toString() + "("; 
    INDENT = "";
    
    if (f.getInterfaceItems() != null)
      retval += traverse(f.getInterfaceItems());
   
    retval += ")::";
    retval += f.getRange(); 
    String temp = "";
    
    if (f.getExpression() != null) 
      retval += " is " + (String) traverse(f.getExpression());
    
    if (temp == null || temp == ""){
      temp = "";
      INDENT = "  ";
      return retval;
    }
    
    else {
      retval += " is " + temp;
      INDENT = "  ";
      return retval;
    } // end of else
  }

  
  private String process(Application a, Object o, String s) {
    INDENT = "  ";
    String retval = "\n" + INDENT + s + ": " + process(a);
    INDENT = "";
    return retval;
  }
  

  private String process(Application a){
    String theString = "";
    int applicationType = a.getApplicationType();

    switch (applicationType) {
	
    case Application.NORMAL:
      if (a.getParameters() != null) {
	 
	if (a.getFunction() != null) {
	  theString += traverse(a.getFunction());
	} // end of if (a.getFunction() != null)
	
	theString += "(";
	boolean first = true;
	for (Iterator paramsIter = a.getParameters().iterator();
	     paramsIter.hasNext();){
	    
	  // Add in a comma, if it is not the first parameter
	  theString = first ? theString : theString + ", ";
	  theString = theString + (traverse(paramsIter.next()));
	  first = false;
	    
	} // end of for (Iterator paramsIter = parameters.iterator();
	// paramsIter.hasNext();)
      }
      theString = theString + ")";
      break;
	
    case Application.BINARY:

      if (a.getParameters().get(0) != null) {
	theString += traverse(a.getParameters().get(0));
      } // end of if (a.getParameters().get(0))
      
      theString += " ";
      
      
      if (a.getFunction() != null) {
	theString += traverse(a.getFunction());
      } // end of if (a.getFunction() != null)
      
      theString += " ";  
      

      if (a.getParameters().get(1) != null) {
	theString += traverse(a.getParameters().get(1));
      } // end of if (a.getParameters().get(1) != null)
      	
      break;
	
    case Application.PREFIX:

      if (a.getFunction() != null) {
	theString += traverse(a.getFunction());
      } // end of if ((a.getFunction() != null)
	  
      if (a.getParameters().get(0) != null) {
	theString += traverse(a.getParameters().get(0));
      } // end of if (a.getParameters().get(0) != null)
	
      break;
	
    case Application.POSTFIX:

      if (a.getParameters().get(0) != null) {
	theString += traverse(a.getParameters().get(0));
      } // end of if (a.getParameters().get(0) != null)
      
      if (a.getFunction() != null) {
	theString += traverse(a.getFunction());
      } // end of if (a.getFunction() != null)
      
      break;
	
    } // end of switch (applicationType)

    if (a.getParens()) {
      theString = "(" + theString + ")";
    } // end of if (a.getParens())
    
    return theString;

  }
  private String process(ItemList i){
    
    String retval = "";
    boolean first = true;
    for (Iterator itemListWalker = i.iterator(); itemListWalker.hasNext();){
      retval += ( first ? "" : "; ");
      first = false;
      retval += traverse(itemListWalker.next());
    } 
    return retval;

  }

 
  private String process(Facet f, Object o, String s){
     String retval = "\nfacet " + s; 

    String temp1 = (String) traverse(f.getInterfaceItems());
    if (temp1 != "" && temp1 != null)
// end of for (Iterator facetDeclaredItems =
         retval += "(" + temp1 + ")";
    retval += " is\n";
    
    INDENT = "  ";
    
    for (Iterator facetDeclaredItems = f.getDeclaredItems().iterator(); facetDeclaredItems.hasNext();){
      String test = (String) (traverse(facetDeclaredItems.next()));
      retval += (test + ";\n");
    } 
    String temp = "";

    if (f.getExportItems() != null) {
      temp +=  (String) traverse(f.getExportItems());
    } // end of if (f.getExportItems() != null)
    
    if (temp != null && !(temp.equals("")))
      retval += INDENT + "export " + temp + ";\n";
    
    retval += "begin " + f.getDomain();

    if (f.getTermItems() != null) {
      retval += traverse(f.getTermItems());
    } // end of if (f.getTermItems() != null)
    //!
    retval += ";\nend domain " + s;
    
    INDENT = "";
    return retval;
  }  
  
  private String process(Domain d, Object o, String s){
    String retval = "\ndomain " + s; 

    String temp1 = (String) traverse(d.getInterfaceItems());
    if (temp1 != "" && temp1 != null)
      retval += "(" + temp1 + ")";
    retval += " is\n";
    
    INDENT = "  ";
    
    for (Iterator facetDeclaredItems = d.getDeclaredItems().iterator(); facetDeclaredItems.hasNext();){
      String test = (String) (traverse(facetDeclaredItems.next()));
      retval += (test + ";\n");
    } // end of for (Iterator facetDeclaredItems =
    // d.getDeclaredItems().iterator(); facetDeclaredItems.hasNext();)

    String temp = "";

    if (d.getExportItems() != null) {
      temp +=  (String) traverse(d.getExportItems());
    } // end of if (d.getExportItems() != null)
    
    if (temp != null && !(temp.equals("")))
      retval += INDENT + "export " + temp + ";\n";

    retval += "begin " + d.getDomain();
  
    if (d.getTermItems() != null) {
      retval += traverse(d.getTermItems());
    } // end of if (d.getTermItems() != null)
    
    retval += ";\nend domain " + s;
    
    INDENT = "";
    return retval;
  }  
 
  public static void main (String[] args) {
   
    if (args.length != 1) {
      System.out.println("usage: Linker filename");
      return;
    } // end of if (args.length != 1)

    try {

      PrettyPrinter PP = new PrettyPrinter();
      
      //Commented by Murthy.
      //TestParser parser = new TestParser();
      //BaseObject root = parser.compileFile(args[0]);
      
      //Added by Murthy to make the tool compatible
      //with the ANTLR parser.
      File f = new File(args[0]);
      FileReader fr = new FileReader(f);
      
      Parser myParser = new Parser();
      System.out.println("PrintROM:");
      BaseObject root = myParser.parseReader(fr,true);
      System.out.println(root.publishROM());
      System.out.println("Pretty Printer:");
      System.out.println(PP.traverse(root));
      
    } catch (Exception e) {
      System.err.println("Exception: "+e);
      e.printStackTrace(System.err);
    } // end of try-catch
    
    
  } // end of main ()
  
}// PrettyPrinter
