//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PropertyChecker.java,v 1.1.1.1 2002/07/01 15:47:36 cvs Exp $
//
// Revision History:
// ----------------
// $Log: PropertyChecker.java,v $
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.1  2002/03/12 22:26:16  cvs
// Added the framework for resolving properties.
// -kimmell
//
package org.sldl.rosetta.tools.propertychecker;

/**
 * PropertyChecker.java
 *
 *
 * Created: Mon Feb 25 15:27:15 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;

public class PropertyChecker {
  
  public static final int PROPERTY_FALSE = 0;
  public static final int PROPERTY_TRUE = 1;
  public static final int PROPERTY_UNKNOWN = 2;

  private List pcas;
  
  private List propertyTrue;
  private List propertyUnknown;
  private List propertyFalse;

  public PropertyChecker (){
    pcas = new LinkedList();
  }
  
  
  public void registerAlgorithm(PropertyCheckAlgorithm alg) {
    pcas.add(alg);
  }


  public void checkProperties(List pccs) {
    propertyTrue = new LinkedList();
    propertyFalse = new LinkedList();
    propertyUnknown = new LinkedList();
   
    int result;
    for (Iterator pccIter = pccs.iterator(); pccIter.hasNext();){
       PropertyCheckCondition cond = 
	 (PropertyCheckCondition) pccIter.next();
       
       switch (checkProperty(cond)) {
       case PROPERTY_FALSE:
	 propertyFalse.add(cond);
	 break; 
       case PROPERTY_TRUE:
	 propertyTrue.add(cond);
	 break;
       case PROPERTY_UNKNOWN:
	 propertyUnknown.add(cond);
	 break;
	 
       } // end of switch (result)
    
    }
  }


  public int checkProperty(PropertyCheckCondition cond) {
    PropertyCheckAlgorithm alg;
    int result;

    for (Iterator pcaIter = pcas.iterator(); pcaIter.hasNext();){
      alg = (PropertyCheckAlgorithm) pcaIter.next();
      
      result = alg.checkCondition(cond);
      switch (result) {
      case PROPERTY_TRUE:
      case PROPERTY_FALSE:
	return result;
	
      case PROPERTY_UNKNOWN:
	// Move on to the next algorithm
      }
	      
    } // end of for (Iterator pcaIter = pcas.iterator(); pcaIter.hasNext();)
    
    return PROPERTY_UNKNOWN;
  }
  
  
}// PropertyChecker
