//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: PVSGen.java,v 1.2 2002/07/08 20:24:05 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: PVSGen.java,v $
// Revision 1.2  2002/07/08 20:24:05  kimmell
// 1. Removed visiting of removed ROM classes.
// 2. Removed implementation of Visitor interface.
//
// This class will shift to using the MultiVisitor ROM walking strategy.
//
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.1  2002/02/08 20:05:44  cvs
// Introducing skeleton PVSGen class for PVS generation.
// --morel
//
//
//============================================================================
package org.sldl.rosetta.tools.pvsgen;

// Imports
import org.sldl.rosetta.ROM.Domain;
import org.sldl.rosetta.ROM.ConstructedType;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;
import org.sldl.rosetta.ROM.ConstructedValue;
import org.sldl.rosetta.ROM.CaseExpression;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.DataLiteral;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.IfExpression;
import org.sldl.rosetta.ROM.ArrayValue;
import org.sldl.rosetta.ROM.AtomicLiteral;
import org.sldl.rosetta.ROM.Interaction;
import org.sldl.rosetta.ROM.StringLiteral;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.LetFunction;
import org.sldl.rosetta.ROM.SetValue;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.CharacterLiteral;
import org.sldl.rosetta.ROM.CompositeValue;

import org.sldl.rosetta.ROM.Literal;
import org.sldl.rosetta.ROM.Component;
import org.sldl.rosetta.ROM.EnumerationType;
import org.sldl.rosetta.ROM.TypeExpression;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.QuantifierReference;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.ParameterItem;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.LabelList;
import org.sldl.rosetta.tools.testparser.TestParser;
import java.util.List;
import java.util.Iterator;

/** The PVS Generation Class
 * @author <a href="mailto:morel@ittc.ku.edu">Brandon Morel</a>
 * @version
 */
public class PVSGen  {

  /** Basic Constructor
   */
  public PVSGen (){
    
  }


  /** Visitation of an application
   * @param app an Application
   * @param prefix an Object
   * @return Object
   */
  public Object visitApplication(Application app, Object prefix) {
    // Get application components
    int applicationType = app.getApplicationType();
    Expression function = app.getFunction();
    ExpressionList parameters = app.getParameters();
    
    // TO BE IMPLEMENTED
    return "% Application not yet implemented";
  }


  /** Visitation of an array value
   * @param array an ArrayValue
   * @param prefix an Object
   * @return Object
   */
  public Object visitArrayValue(ArrayValue array, Object prefix) {
    // Get elements of array
    ExpressionList elements = array.getElements();

    // TO BE IMPLEMENTED
    return "% ArrayValue not yet implemented";
  }


  /** Visitation of an atomic literal
   * @param lit an AtomicLiteral
   * @param prefix an Object
   * @return Object
   */
  public Object visitAtomicLiteral(AtomicLiteral lit, Object prefix) {
    // Get atomic literal components
    String data = lit.getData();
    ItemReference marker = lit.getMarker();
    
    // TO BE IMPLEMENTED
    return "% AtomicLiteral not yet implemented";
  }


  /** Visitation of a BaseObject
   * @param param1 an BaseObject
   * @param prefix an Object
   * @return Object
   */
  public Object visitBaseObject(BaseObject param1, Object prefix) {
    // TO BE IMPLEMENTED
    return "% BaseObject not yet implemented";
  }


  /** Visitation of a CaseExpression
   * @param caseExpr a CaseExpression
   * @param prefix an Object
   * @return Object
   */
  public Object visitCaseExpression(CaseExpression caseExpr, Object
				    prefix) {
    // Get CaseExpression components
    Expression test = caseExpr.getTestExpression();
    ExpressionList options = caseExpr.getOptions();
    ExpressionList results = caseExpr.getResults();

    // TO BE IMPLEMENTED
    return "% BaseExpression not yet implemented";
  }


  /** Visitation of a CharacterLiteral
   * @param lit a CharaterLiteral
   * @param prefix an Object
   * @return Object
   */
  public Object visitCharacterLiteral(CharacterLiteral lit, Object
				      prefix) {
    // Get Character components
    Character data = lit.getData();
    
    // TO BE IMPLEMENTED
    return "% CharacterLiteral not yet implemented";
  }


  /** Visitation of a Component
   * @param component a Components
   * @param prefix an Object
   * @return Object
   */
  public Object visitComponent(Component component, Object prefix) {
    // Get Component parts
    Expression assumption = component.getAssumption();
    Expression definition = component.getDefinition();
    Expression verification = component.getVerification();

    // TO BE IMPLEMENTED
    return "% Component not yet implemented";
  }


  /** Visitation of a CompositeValue
   * @param value a CompositeValue
   * @param prefix an Object
   * @return Object
   */
  public Object visitCompositeValue(CompositeValue value, Object
				    prefix) {
    // Get CompositeValue parts
    ExpressionList elements = value.getElements();

    // TO BE IMPLEMENTED
    return "% Component not yet implemented";
  }


  /** Visitation of a ConstructedTypeConstructor
   * @param cons a ConstructedTypeConstructor
   * @param prefix an Object
   * @return Object
   */
  public Object
    visitConstructedTypeConstructor(ConstructedTypeConstructor 
				    cons, Object prefix) {
    // Get Constructor components
    Function constructors = cons.getConstructorFunction();
    Item recognizer = cons.getRecognizer();
    ItemList observers = cons.getObservers();

    // TO BE IMPLEMENTED
    return "% ConstructedTypeConstructor not yet implemented";
  }


  /** Visitation of a ConstructedType
   * @param conType a ConstructedType
   * @param prefix an Object
   * @return Object
   */
  public Object visitConstructedType(ConstructedType conType, Object
				     prefix) {

    // Get ConstructedType components
    ItemReference typeReference = conType.getTypeReference();
    ItemList constructors = conType.getConstructors();
    
    // TO BE IMPLEMENTED
    return "% ConstructedType not yet implemented";
  }


  /** Visitation of a ConstructedValue
   * @param consVal a ConstructedValue
   * @param prefix an Object
   * @return Object
   */
  public Object visitConstructedValue(ConstructedValue consVal, Object
				      prefix) {
    // Get Value components
    ItemReference constructor = consVal.getConstructor();
    ExpressionList parameters = consVal.getElements();

    // TO BE IMPLEMENTED
    return "% ConstructedValue not yet implemented";
  }


  /** Visitation of a DataLiteral
   * @param lit a DataLiteral
   * @param prefix an Object
   * @return Object
   */
  public Object visitDataLiteral(DataLiteral lit, Object prefix) {
    // Get Literal components
    Object data = lit.getData();

    // TO BE IMPLEMENTED
    return "% DataLiteral not yet implemented";
  }



  /** Visitation of a Domain
   * @param domain a Domain
   * @param prefix an Object
   * @return Object
   */
  public Object visitDomain(Domain domain, Object prefix) {
    // Get Domain components
    ExpressionList useClauses = domain.getUseClauses();
    ItemList parameters = domain.getInterfaceItems();
    ItemList declarations = domain.getDeclaredItems();
    ItemList terms = domain.getTermItems();
    ItemList exports = domain.getExportItems();


    // TO BE IMPLEMENTED
    return "% Domain not yet implemented";
  }


  /** Visitation of an EnumerationType
   * @param type an EnumerationType
   * @param prefix an Object
   * @return Object
   */
  public Object visitEnumerationType(EnumerationType type, Object
				     prefix) {
    // Get Type components
    ExpressionList elements = type.getElements();

    // TO BE IMPLEMENTED
    return "% EnumerationType not yet implemented";
  }


  /** Visitation of an Expression
   * @param expression an Expression
   * @param prefix an Object
   * @return Object
   */
  public Object visitExpression(Expression expression, Object prefix) {

    // TO BE IMPLEMENTED
    return "% Expression not yet implemented";
  }


  /** Visitation of a Facet
   * @param facet a Facet
   * @param prefix an Object
   * @return Object
   */
  public Object visitFacet(Facet facet, Object prefix) {
    // Get Facet components
    Expression domain = facet.getDomain();
    ExpressionList useClauses = facet.getUseClauses();
    ItemList parameters = facet.getInterfaceItems();
    ItemList declarations = facet.getDeclaredItems();
    ItemList terms = facet.getTermItems();
    ItemList exports = facet.getExportItems();

    // TO BE IMPLEMENTED
    return "% Facet not yet implemented";
  }



  /** Visitation of a Function
   * @param fun a Function
   * @param prefix an Object
   * @return Object
   */
  public Object visitFunction(Function fun, Object prefix) {
    // Get Function components
    ItemList parametersTypes = fun.getInterfaceItems();
    Expression returnType = fun.getRange();
    Expression body = fun.getExpression();

    // TO BE IMPLEMENTED
    return "% Function not yet implemented";
  }


  /** Visitation of an IfExpression
   * @param ifExpr an IfExpression
   * @param prefix an Object
   * @return Object
   */
  public Object visitIfExpression(IfExpression ifExpr, Object prefix) {
    // Get Expression components
    ExpressionList ifExpressions = ifExpr.getIfs();
    ExpressionList thenExpressions = ifExpr.getThens();
    Expression elseExpression = ifExpr.getElse();
    

    // TO BE IMPLEMENTED
    return "% IfExpression not yet implemented";
  }


  /** Visitation of an Interaction
   * @param interaction an Interaction
   * @param prefix an Object
   * @return Object
   */
  public Object visitInteraction(Interaction interaction, Object
				 prefix) {
    // Get Interaction components
    Expression domain = interaction.getDomain();
    ExpressionList useClauses = interaction.getUseClauses();
    ItemList parameters = interaction.getInterfaceItems();
    ItemList declarations = interaction.getDeclaredItems();
    ItemList terms = interaction.getTermItems();
    ItemList exports = interaction.getExportItems();
    
    // TO BE IMPLEMENTED
    return "% Interaction not yet implemented";
  }


  /** Visitation of an Item
   * @param item an Item
   * @param prefix an Object
   * @return Object
   */
  public Object visitItem(Item item, Object prefix) {
    // Get Item components
    Label label = item.getLabel();
    Expression type = item.getType();
    Expression value = item.getValue();
    
    // TO BE IMPLEMENTED
    return "% Item not yet implemented";
  }


  /** Visitation of an ItemReference
   * @param itemRef an ItemReference
   * @param prefix an Object
   * @return Object
   */
  public Object visitItemReference(ItemReference itemRef, Object
				   prefix) {
    // Get Reference components
    LabelList labels = itemRef.getReference();
    Item item = itemRef.getItemReferred();

    // TO BE IMPLEMENTED
    return "% ItemReference not yet implemented";
  }


  /** Visitation of a Label
   * @param lable a Lable
   * @param prefix an Object
   * @return Object
   */
  public Object visitLabel(Label label, Object prefix) {
    // Get Label components
    String labelName = label.toString();

    // TO BE IMPLEMENTED
    return "% Label not yet implemented";
  }


  /** Visitation of a LetFunction
   * @param let a LetFunction
   * @param prefix an Object
   * @return Object
   */
  public Object visitLetFunction(LetFunction let, Object prefix) {
    // Get Function components
    ItemList declarations = let.getItemList();
    Expression body = let.getExpression();

    // TO BE IMPLEMENTED
    return "% LetFunction not yet implemented";
  }


  /** Visitation of a Literal
   * @param literal a Literal
   * @param prefix an Object
   * @return Object
   */
  public Object visitLiteral(Literal literal, Object prefix) {

    // TO BE IMPLEMENTED
    return "% Literal not yet implemented";
  }


  /** Visitation of a Package
   * @param pack a Package
   * @param prefix an Object
   * @return Object
   */
  public Object visitPackage(Package pack, Object prefix) {
    // Get Package components
    Expression domain = pack.getDomain();
    ExpressionList useClauses = pack.getUseClauses();
    ItemList parameters = pack.getInterfaceItems();
    ItemList declarations = pack.getDeclaredItems();
    ItemList terms = pack.getTermItems();
    ItemList exports = pack.getExportItems();
   
    // TO BE IMPLEMENTED
    return "% Package not yet implemented";
  }


  /** Visitation of a ParameterItem
   * @param param a ParameterItem
   * @param prefix an Object
   * @return Object
   */
  public Object visitParameterItem(ParameterItem param, 
				   Object prefix) {
    // Get Item components
    Label label = param.getLabel();
    Expression type = param.getType();
    Expression value = param.getValue();
    ItemReference mode = param.getMode();
    
    // TO BE IMPLEMENTED
    return "% ParameterItem not yet implemented";
  }


  /** Visitation of a QuantifierReference
   * @param ref a QuantifierReference
   * @param prefix an Object
   * @return Object
   */
  public Object visitQuantifierReference(QuantifierReference ref,
					 Object prefix) {
    // Get Reference components
    LabelList labels = ref.getReference();
    Item item = ref.getItemReferred();
    ItemList variables = ref.getVariables();
    Expression predicate = ref.getExpression();

    // TO BE IMPLEMENTED
    return "% QuantifierReference not yet implemented";
  }


  /** Visitation of a SetValue
   * @param set a SetValue
   * @param prefix an Object
   * @return Object
   */
  public Object visitSetValue(SetValue set, Object prefix) {
    // Get Set elements
    ExpressionList elements = set.getElements();

    // TO BE IMPLEMENTED
    return "% SetValue not yet implemented";
  }


  /** Visitation of a StringLiteral
   * @param lit a StringLiteral
   * @param prefix an Object
   * @return Object
   */
  public Object visitStringLiteral(StringLiteral lit, Object prefix) {
    // Get Literal components
    String data = lit.getData();

    // TO BE IMPLEMENTED
    return "% StringLiteral not yet implemented";
  }


  /** Visitation of a TypeExpression
   * @param type a TypeExpression
   * @param prefix an Object
   * @return Object
   */
  public Object visitTypeExpression(TypeExpression type, Object
				    prefix) {
    // Get Type components
    String relationship = type.getRelationship();
    Expression expression = type.getExpression();

    // TO BE IMPLEMENTED
    return "% TypeExpression not yet implemented";
  }


  /** This is the main sphere of control
   * @param args array of String arguments
   * @return void
   */
  public static void main (String[] args) {
  
    // TO BE IMPLEMENTED
    System.out.println("% PVS GENERATION");
  } 
}
