//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: TestAntlrParser.java,v 1.2 2002/11/18 18:46:09 ksatya Exp $
//
// Revision History:
// ----------------
// $Log: TestAntlrParser.java,v $
// Revision 1.2  2002/11/18 18:46:09  ksatya
// There were some problems with the previous commit.So commiting the file again.
// --Murthy.
//
// Revision 1.1  2002/11/18 16:54:04  ksatya
// Added a new class that tests the ANTLR parser functionality.
// --Murthy.
//

package org.sldl.rosetta.tools.testparser;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.Label;

//ANTLR related classes.
import org.sldl.rosetta.parser.tarp.misc.*;
import org.sldl.rosetta.parser.tarp.antlr.*;

/**
 * Describe class <code>TestAntlrParser</code> here.
 * This class tests the ANTLR parser.
 * @author <a href="mailto:ksatya@ittc.ku.edu">Murthy Kakarlamudi</a>
 * @version 1.0
 */
public class TestAntlrParser
  {
    
    /**
     * Describe <code>addMessage</code> method here.
     * 
     * @param msg a <code>String</code> value
     */
    public void addMessage(String msg)
    { System.out.println(msg); }
    
    /**
     * Describe <code>compileFile</code> method here.
     * This method takes a filename as a String and 
     * calls a method that invokes the actual parser.
     * This method creates a FileReader for the file
     * specified as the input.
     * @param filename a <code>String</code> value
     * @return an <code>ItemList</code> value
     */
    public ItemList compileFile(String filename)
    {
      try
	{ 
	  filename = (new File(filename)).getPath();
	  File f = new File(filename);
	  //return compileFile(new ROMScanner(filename), filename); }
	  return compileFile(new FileReader(f),filename);
	} catch(IOException e)
	  { addMessage("IO of [" + filename + "] failed because of: " + e); }
      return null;
    }
    
    
    /**
     * Describe <code>compileFile</code> method here.
     * This is the actual method that invokes the parser.
     * @param fr a <code>Reader</code> value
     * @param filename a <code>String</code> value
     * @return an <code>ItemList</code> value
     */
    public ItemList compileFile(Reader fr, String filename)
    {
      Parser parser = new Parser();
      if (parser == null)
	{
	  addMessage("Scanner for [" + filename + "] is null!");
	  return null;
	}
      
      String stage = "scanning";
      try
	{
	  BaseObject root = parser.parseReader(fr,true);
	  ItemList items = ((Function)root).getInterfaceItems();
	  return items;
	}
      catch(Exception e)
	{ addMessage("Compiling of [" + filename + "] failed at " + stage + " because of: " + e); 
      e.printStackTrace();
	}
      return null;
    }
    
    
    //This method invokes ANTLR based parser.To use this uncomment this
    //and comment out the javacc based parser.
    /**
     * Describe <code>main</code> method here.
     *
     * @param args a <code>String[]</code> value
     * @exception IOException if an error occurs
     */
    public static void main(String[] args) throws IOException
    {
      TestAntlrParser tester;
      ItemList  myItemList;
      
      tester = new TestAntlrParser();
      myItemList = tester.compileFile(args[0]); 
      for (int index = 0; index < myItemList.size(); index++) {
	Item myItem = myItemList.get(index);
	if (myItem != null)
	  { System.out.println(myItem.publishROM() + ";\n\n"); }
      } // end of for (int index = 0; index < myItemList.size(); index++)
    }
}
