//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: TestParser.java,v 1.5 2002/11/18 18:46:09 ksatya Exp $
//
// Revision History:
// ----------------
// $Log: TestParser.java,v $
// Revision 1.5  2002/11/18 18:46:09  ksatya
// There were some problems with the previous commit.So commiting the file again.
// --Murthy.
//

package org.sldl.rosetta.tools.testparser;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;

import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.Label;

// Scanning
import org.sldl.rosetta.parser.scanner.ROMScanner;
import org.sldl.rosetta.parser.scanner.ParseException;

//ANTLR related classes.
import org.sldl.rosetta.parser.tarp.misc.*;
import org.sldl.rosetta.parser.tarp.antlr.*;


// Linking
//import org.sldl.rosetta.parser.linker.ROMLinker;
//import org.sldl.rosetta.parser.linker.ROMLinker.LinkerLoader;
//import org.sldl.rosetta.parser.linker.ROMSymbolTable;
//import org.sldl.rosetta.parser.linker.ROMSymbolTable.SymbolTableLoader;
//import org.sldl.rosetta.parser.linker.symboltable.BasicSymbolTable;
//import org.sldl.rosetta.parser.linker.symboltable.DefaultBasicSymbolTable;

// Typechecking
//import org.sldl.rosetta.parser.typechecker.ROMTypeChecker;
//import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.TypeCheckerMonitor;
//import org.sldl.rosetta.parser.typechecker.ROMTypeChecker.TCC;



public class TestParser
  {

    public void addMessage(String msg)
    { System.out.println(msg); }
    

    //public Item getDefaultLibrary(Item focusItem)
    //      {
    //      if (focusItem.getLabel().matches(new Label("lang")))
    //        { return null; }
    //      return compileFile("Rosetta/lang.sld");
    //      }
    
    //    public Item loadPackage(ItemReference delayedRef, ExpressionList params)
    //      {
    //        // FIXME: Need to make this do something useful.
    //        return null;
    //      }
    
    
    //    public void linkingError(String errorMessage)
    //      { addMessage("Linking Error :: " + errorMessage); }
    
    //    public BasicSymbolTable createSymbolTable(Object rootObject)
    //      { return new DefaultBasicSymbolTable(rootObject); }
    
    //      public void addTCC(TCC requirement)
    //        { typeCheckingError(requirement.toString()); }
    
    //      public void typeCheckingError(String errorMessage)
    //        { addMessage("Type Checking :: " + errorMessage); }
    



    /******* This Part invokes the JAVACC based Parser.*************/
    

    public Item regressFile(String filename)
    {
      Item myItem;
      StringReader versionTwo;
      
      myItem = compileFile(filename);
      versionTwo = new StringReader(myItem.publishROM() + ";");
      myItem = compileFile(versionTwo, filename);
      return myItem;
    }
    
    
    
    public Item compileFile(Reader source, String filename)
    { return compileFile(new ROMScanner(source, filename), filename); }
    
    public Item compileFile(String filename)
    {
      try
	{ 
	  filename = (new File(filename)).getPath();
	  return compileFile(new ROMScanner(filename), filename); }
      catch(IOException e)
	{ addMessage("IO of [" + filename + "] failed because of: " + e); }
      return null;
    }
    
    public Item compileFile(ROMScanner scanner, String filename)
    {
      if (scanner == null)
	{
	  addMessage("Scanner for [" + filename + "] is null!");
	  return null;
	}
      
      String stage = "scanning";
      try
	{
	  Item myItem;
	  //BasicSymbolTable myTable;
	  
	  myItem = scanner.packageItem();
	  //stage = "symbolTable";
	  //System.out.println("Stage: " + stage);
	  //myTable = ROMSymbolTable.checkModel(myItem, this);
	  //stage = "linking";
	  //System.out.println("Stage: " + stage);
	  //ROMLinker.link(myItem, this);
	  //stage = "typeChecking";
	  //System.out.println("Stage: " + stage);
	  //ROMTypeChecker.typeCheck(myItem, this);
	  return myItem;
	}
      catch(ParseException e)
	{ addMessage("Parsing of [" + filename + "] failed at " + stage + " because of: " + e); 
      	e.printStackTrace();
      	}
      catch(Exception e)
	{ addMessage("Compiling of [" + filename + "] failed at " + stage + " because of: " + e); }
      return null;
    }
    
    
    public static void main(String[] args) throws IOException
    {
      TestParser tester;
      Item myItem;
      
      tester = new TestParser();
      myItem = null;
      if (args.length > 1)
	{
	  if (args[0].equalsIgnoreCase("-r"))
	    { myItem = tester.regressFile(args[1]); }
	  else
	    { myItem = tester.compileFile(args[0]); }
	}
      else
	{ myItem = tester.compileFile(args[0]); }
      
      if (myItem != null)
	{ System.out.println(myItem.publishROM() + ";\n\n"); }
    }
    
    /******* End of JAVACC based Parser part.*************/
    
    
    /******* This Part invokes the ANTLR based Parser.*************/
    
    //This is ANTLR based parser. This is commented out.
    //if you want to use this uncomment this and comment out the 
    //javacc stuff to get this working.

    //  public ItemList compileFile(String filename)
//      {
//        try
//  	{ 
//  	  filename = (new File(filename)).getPath();
//  	  File f = new File(filename);
//  	  //return compileFile(new ROMScanner(filename), filename); }
//  	  return compileFile(new FileReader(f),filename);
//  	} catch(IOException e)
//  	  { addMessage("IO of [" + filename + "] failed because of: " + e); }
//        return null;
//      }
    
    
//      public ItemList compileFile(Reader fr, String filename)
//      {
//        Parser parser = new Parser();
//        if (parser == null)
//  	{
//  	  addMessage("Scanner for [" + filename + "] is null!");
//  	  return null;
//  	}
      
//        String stage = "scanning";
//        try
//  	{
//  	  BaseObject root = parser.parseReader(fr,true);
//  	  ItemList items = ((Function)root).getInterfaceItems();
//  	  return items;
//  	}
//        catch(ParseException e)
//  	{ addMessage("Parsing of [" + filename + "] failed at " + stage + " because of: " + e); 
//  	e.printStackTrace();
//  	}
//        catch(Exception e)
//  	{ addMessage("Compiling of [" + filename + "] failed at " + stage + " because of: " + e); }
//        return null;
//      }
    
    
//      //This method invokes ANTLR based parser.To use this uncomment this
//      //and comment out the javacc based parser.
//      public static void main(String[] args) throws IOException
//      {
//        TestParser tester;
//        ItemList  myItemList;
      
//        tester = new TestParser();
//        myItemList = tester.compileFile(args[0]); 
//        for (int index = 0; index < myItemList.size(); index++) {
//  	Item myItem = myItemList.get(index);
//  	if (myItem != null)
//  	  { System.out.println(myItem.publishROM() + ";\n\n"); }
//        } // end of for (int index = 0; index < myItemList.size(); index++)
//      }
    /******* End of ANTLR based Parser part.*************/
    
}



