//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: TypeConditionGen.java,v 1.2 2002/11/15 17:53:26 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: TypeConditionGen.java,v $
// Revision 1.2  2002/11/15 17:53:26  kimmell
// Removed references to FunctionType.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:38  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.2  2002/03/21 20:35:37  cvs
// Added functionality for resolving TCCs.
// -kimmell
//
// Revision 1.1.2.1  2002/03/12 22:29:40  cvs
// Added the typechecker which uses the multivisitor class and the propertycheck framework.
// -kimmell
//
package org.sldl.rosetta.tools.typechecker;

// Java Packages
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

// Tools Packages
import org.sldl.rosetta.tools.multivisitor.MultiVisitor;
import org.sldl.rosetta.tools.testparser.TestParser;
import org.sldl.rosetta.tools.glinker.Linker;
import org.sldl.rosetta.tools.propertychecker.PropertyChecker;


// ROM Packages
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.TypeExpression;

import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.ItemList;

// Primitive Packages
import org.sldl.rosetta.parser.primitive.PrimitiveType;
import org.sldl.rosetta.parser.primitive.PrimitiveValue;

import org.sldl.rosetta.lang.types.meta.FacetType;


// Typechecker Packages
import
  org.sldl.rosetta.tools.typechecker.conditions.ExistenceCondition;
import
  org.sldl.rosetta.tools.typechecker.conditions.TypeCompatabilityCondition;
import
  org.sldl.rosetta.tools.typechecker.algorithms.IdentityAlgorithm;


/**
 * TypeConditionGen.java
 *
 *
 * Created: Wed Feb 27 10:15:49 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 
 * @version
 */

public class TypeConditionGen extends MultiVisitor {

  private List conditions;
  public TypeConditionGen (){
    conditions = new LinkedList();
    toolName = "TYPE ";
  }

  public Object process(Item item) {
      //log(MESSAGE, "Traversing " + item.getLabel());
      //log(MESSAGE, "Value: " + item.getValue());

    Expression type = item.getType();
    Expression value = item.getValue();
    
    if ((type != null) && (value == null)) {
      log(MESSAGE,"(type != null) && (value == null)");
      TypeExpression formalType = (TypeExpression) traverse(type);
      ExistenceCondition cond = new
	ExistenceCondition(item.getLabel(), formalType);
      conditions.add(cond);
      return formalType;
    } else if ((type != null) && (value != null)) {
      log(MESSAGE, "(type != null) && (value != null)");
      TypeExpression formalType = (TypeExpression) traverse(type);
      TypeExpression actualType = (TypeExpression) traverse(value);
      TypeCompatabilityCondition cond = 
	new TypeCompatabilityCondition(formalType,actualType);
      conditions.add(cond);
      log(MESSAGE, cond.toString());
      return formalType;
    } else if ((type == null) && (value != null)) {
      log(MESSAGE, "(type == null) && (value != null)");
      Expression actualType = (TypeExpression) traverse(value);
      item.setType(actualType);
      return actualType;
    } else {
      log(MESSAGE, "Not generating an existence TCC. Type: " + item.getType());
      return type;
    } // end of else
  }


  public Object process(Application app) {
    log(MESSAGE, "Processing an application");
    // System.out.println("Getting the function type");
    try {

      Function fun = (Function) app.getFunction();
      
      ExpressionList fType = (ExpressionList) traverse(fun);
      
      if (fType == null) {
	log(ERROR, "fType is null");
	return null;
      } 
      
      // System.out.println("Function type returned");
      
      ExpressionList actuals = app.getParameters();
      Iterator formalIter = fType.iterator();
      Iterator actualIter = actuals.iterator();
      
      while (formalIter.hasNext() && actualIter.hasNext()){
	// System.out.println("Getting the type of the formal parameter");
	Expression formalType = (Expression)formalIter.next();
      // System.out.println("Getting the type of the actual parameter");
	Expression actualType = (Expression)actualIter.next();
	log(MESSAGE, "Creating a TCC");
	TypeExpression resolvedType = (TypeExpression)
	  traverse(actualType);
	log(MESSAGE, "Resolved Type: " + resolvedType);
	TypeCompatabilityCondition cond = 
	new TypeCompatabilityCondition(formalType,resolvedType);
	log(MESSAGE, "TCC: " + cond);
	conditions.add(cond);
      } // end of for (Iterator formalIter = formals.iterator(); formalIter.hasNext();)
      
      // Return the range
      log(MESSAGE, "Application is returning a " + fType.lastElement());
      return traverse(fType.lastElement());
    
    } catch (ClassCastException e) {
      // FIXME: This will happen when we have an application of a
      // facet, not a function.
      return null;
    } // end of catch
  }
  
    
  public Object process(Function fun) {
    log(MESSAGE, "Processing a function");
    ExpressionList type = new ExpressionList();
    Item curItem;
    Expression curType;
    ItemList parameters = fun.getInterfaceItems();
    for (Iterator paramIter = parameters.iterator();
	 paramIter.hasNext();){
      curItem = (Item) paramIter.next();
      type.add(curItem.getType());
    } // end of for (Iterator paramIter = fun.getInterfaceItems().iterator(); paramIter.hasNext();)
    
    type.add(fun.getRange());
    Function fType = new Function(type);
    log(MESSAGE, "Function type is: " + fType);
    return fType;
  }

  public Object process(Facet facet) {
    Object[] children = facet.getAllChildObjects();
    for (int i = 0; i < children.length; i++) {
      traverse(children[i]);
    } // end of for (int i = 0; i < children.length; i++)
    return new FacetType();
  }
  

  public Object process(TypeExpression expr) {
    return expr;
  }

  public Object process(PrimitiveValue val) {
    // FIXME: We need to return the type of the value, not the value itself.
    return val;
  }

  public Object process(PrimitiveType type) {
    return type.getType();
  }

  



  /* 
   * Methods for operating on conditions. These methods are not
   * involved in processing the ROM tree
   */

  private List getConditions() {
    return conditions;
  }

  public void printConditions() {
    for (Iterator condIter = conditions.iterator();
	 condIter.hasNext();){
      try {
	System.out.println(condIter.next());
      } catch (Exception e) {
	 
      } // end of catch
      
    } // end of for (Iterator condIter = conditions.iterator();
    // condIter.hasNext();)
  }


  public void resolveConditions() {
    PropertyChecker checker = new PropertyChecker();
    checker.registerAlgorithm(new IdentityAlgorithm());
    checker.checkProperties(conditions);
    
  }


  public static void main (String[] args) {
    
    if (args.length != 1) {
      System.out.println("usage: MoMLGen [sldlfile]");
      return;
    } // end of if (args.length != 1)
    
    TypeConditionGen tcgen = new TypeConditionGen();
    TestParser parser = new TestParser();

    System.out.println("Parsing...");
    BaseObject root = parser.compileFile(args[0]);

    System.out.println("Linking...");
    Linker linker = new Linker();
    linker.link(root);
    
    System.out.println("Typechecking");
    tcgen.traverse(root);
    
    tcgen.printConditions();
    
    tcgen.resolveConditions();
    
    System.out.println(root.publishROM());
    
  } // end of main ()

  
  
}// TypeConditionGen
