//============================================================================
//  Copyright (c) 2000 University of Kansas
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//
// $Id: MoMLGen.java,v 1.2 2002/07/08 20:22:54 kimmell Exp $
//
// Revision History:
// ----------------
// $Log: MoMLGen.java,v $
// Revision 1.2  2002/07/08 20:22:54  kimmell
// 1. Removed visit methods for deleted ROM elements.
// 2. Removed implementation of Visitor class.
//
// This class will shift to using the Multivisitor walking strategy.
// -kimmell
//
// Revision 1.1.1.1  2002/07/01 15:47:36  cvs
// Rosetta tools and publications repository
//
// Revision 1.1.2.2  2002/06/05 16:42:20  kimmell
// Added 2002 DAC demo tool.
// -kimmell
//
// Revision 1.1.2.1  2002/02/04 23:05:30  cvs
// Created the MoMLGen class as a proof-of-concept for the Visitor interface for accessing the ROM. The MoML generated is probably not correct.
//
// Revision 1.1.2.1  2002/01/24 19:04:08  cvs
// Added the PrettyPrinter class. Because this is pretty much a duplication of the toString() method in each ROM Class, this tool is merely a demonstration of the Visitor interface.
// Currently it just implements the Visitor interface, and takes apart each ROM element. No output is currently generated.
//
package org.sldl.rosetta.tools.momlgen;

import org.sldl.rosetta.ROM.Domain;
import org.sldl.rosetta.ROM.ConstructedType;
import org.sldl.rosetta.ROM.ConstructedType.ConstructedTypeConstructor;
import org.sldl.rosetta.ROM.ConstructedValue;
import org.sldl.rosetta.ROM.CaseExpression;
import org.sldl.rosetta.ROM.Expression;
import org.sldl.rosetta.ROM.DataLiteral;
import org.sldl.rosetta.ROM.Application;
import org.sldl.rosetta.ROM.IfExpression;
import org.sldl.rosetta.ROM.ArrayValue;
import org.sldl.rosetta.ROM.AtomicLiteral;
import org.sldl.rosetta.ROM.Interaction;
import org.sldl.rosetta.ROM.StringLiteral;
import org.sldl.rosetta.ROM.Facet;
import org.sldl.rosetta.ROM.LetFunction;
import org.sldl.rosetta.ROM.SetValue;
import org.sldl.rosetta.ROM.Item;
import org.sldl.rosetta.ROM.Function;
import org.sldl.rosetta.ROM.CharacterLiteral;
import org.sldl.rosetta.ROM.CompositeValue;
import org.sldl.rosetta.ROM.Literal;
import org.sldl.rosetta.ROM.Component;
import org.sldl.rosetta.ROM.EnumerationType;
import org.sldl.rosetta.ROM.TypeExpression;
import org.sldl.rosetta.ROM.Package;
import org.sldl.rosetta.ROM.Label;
import org.sldl.rosetta.ROM.ItemReference;
import org.sldl.rosetta.ROM.QuantifierReference;
import org.sldl.rosetta.ROM.BaseObject;
import org.sldl.rosetta.ROM.ParameterItem;
import org.sldl.rosetta.ROM.ExpressionList;
import org.sldl.rosetta.ROM.ItemList;
import org.sldl.rosetta.ROM.LabelList;

import org.sldl.rosetta.tools.testparser.TestParser;
import java.util.List;
import java.util.Iterator;

/**
 * MoMLGen.java
 *
 *
 * Created: Wed Jan 23 11:01:38 2002
 *
 * @author <a href="mailto:kimmell@ku.edu">Garrin Kimmell</a>
 * @version
 */

public class MoMLGen {
  public MoMLGen (){
    
  }
  // implementation of org.sldl.rosetta.ROM.Visitor interface

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitApplication(Application app, Object prefix)
  {
    int applicationType = app.getApplicationType();
    Expression function = app.getFunction();
    ExpressionList parameters = app.getParameters();
    
    String str = new String((String) prefix);
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    return prefix + "value=\"" + app + "\" >";
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitArrayValue(ArrayValue array, Object prefix)
  {
    ExpressionList elements = array.getElements();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    
    String str = new String((String) prefix);
    str = str  + "class=\"array.xml\">\n";
    
    for (Iterator elemIter = elements.iterator();
	 elemIter.hasNext();){
      Expression expr = (Expression) elemIter.next();
      str = str + expr.accept(this, "<property ");
      str = str + "</property>\n";
    } // end of for (Iterator elemIter = elements.iterator(); elemIter.hasNext();)
    
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitAtomicLiteral(AtomicLiteral lit, Object prefix)
  {
    String data = lit.getData();
    ItemReference marker = lit.getMarker();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    
    return ((String) prefix) + data;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitBaseObject(BaseObject param1, Object prefix)
  {
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    return (String) prefix;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitCaseExpression(CaseExpression caseExpr, Object prefix)
  {


    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    Expression test = caseExpr.getTestExpression();
    ExpressionList options = caseExpr.getOptions();
    ExpressionList results = caseExpr.getResults();

    String str = new String((String) prefix);
    return str;
  }


  /**
   *
   * @param lit <description>
   * @param indent  <description>
   * @return <description>
   */
  public Object visitCharacterLiteral(CharacterLiteral lit, Object prefix)
  {
    Character data = lit.getData();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    return prefix + "class=\"character.xml\"  value=\"" + data + "\">\n";
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitComponent(Component component, Object prefix)
  {
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    Expression assumption = component.getAssumption();
    Expression definition = component.getDefinition();
    Expression verification = component.getVerification();

    String str = new String((String) prefix);
    return null;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitCompositeValue(CompositeValue value, Object prefix)
  {
    ExpressionList elements = value.getElements();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method

    System.out.println("visitCompositValue");
    String str = new String((String) prefix);
    return null;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitConstructedTypeConstructor(ConstructedTypeConstructor cons, Object prefix)
  {
    Function constructors = cons.getConstructorFunction();
    Item recognizer = cons.getRecognizer();
    ItemList observers = cons.getObservers();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method

    System.out.println("visitConstructedTypeConstructor");
    String str = new String((String) prefix);
    
    return str;
  }

  public Object visitConstructedType(ConstructedType conType, Object
				     prefix) {
    ItemReference typeReference = conType.getTypeReference();
    ItemList constructors = conType.getConstructors();
    
    String str = new String((String) prefix);
    System.out.println("visitConstructedType");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitConstructedValue(ConstructedValue cons, Object prefix)
  {
    ItemReference constructor = cons.getConstructor();
    ExpressionList parameters = cons.getElements();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitConstructedValue");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitDataLiteral(DataLiteral lit, Object prefix)
  {
    Object data = lit.getData();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitDataLiteral");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitDeferredItem(DeferredItem item, Object prefix)
  {
    List dotNames = item.getDotNameList();
    ItemList references = item.getReferenceList();
    ItemList imports = item.getImportList();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method

    String str = new String((String) prefix);
    System.out.println("visitDeferredItem");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitDomain(Domain domain, Object prefix)
  {
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    ExpressionList useClauses = domain.getUseClauses();
    ItemList parameters = domain.getInterfaceItems();
    ItemList declarations = domain.getDeclaredItems();
    ItemList terms = domain.getTermItems();
    ItemList exports = domain.getExportItems();

    String str = new String((String) prefix);
    System.out.println("visitDomain");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitEnumerationType(EnumerationType type, Object prefix)
  {
    ExpressionList elements = type.getElements();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitEnumerationType");
    return str;
  }

  public Object visitExpression(Expression expression, Object prefix)
  {
    System.out.println("visitExpression");
    return prefix + expression.toString() + ">\n";
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitFacet(Facet facet, Object prefix)
  {
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    Expression domain = facet.getDomain();
    ExpressionList useClauses = facet.getUseClauses();
    ItemList parameters = facet.getInterfaceItems();
    ItemList declarations = facet.getDeclaredItems();
    ItemList terms = facet.getTermItems();
    ItemList exports = facet.getExportItems();

    String str = new String((String) prefix);
    String termPrefix = "<property class=\"term.xml\" ";

    System.out.println("visitFacet");
    str = str + "class=\"facet.xml\">\n";

    str = str + "<!-- parameters -->\n";
    for (Iterator paramIter = parameters.iterator(); 
	 paramIter.hasNext();){
      str = str + ((Item)paramIter.next()).accept(this, "<property ")
	+ "\n";
      
    } // end of for (Iterator paramIter = parameters.iterator(); paramIter.hasNext();)
    
    
    str = str + "<!-- declarations -->\n";

    for (Iterator declIter = declarations.iterator();
	 declIter.hasNext();){
      str = str + ((Item)declIter.next()).accept(this, "<entity ") + "</entity>\n";
    } // end of for (Iterator declIter = declarations.iterator(); declIter.hasNext();)
    
    str = str + "<!-- domain -->\n";
    if (domain != null) {
      str = str + domain.accept(this, "<property ") + "</property>\n";
    } // end of if (domain != null)
    
    str = str + "<!-- terms -->\n";
    for (Iterator termIter = terms.iterator(); termIter.hasNext();){
      str = str + ((Item)termIter.next()).accept(this, termPrefix) + "\n</property>\n";
    } // end of for (Iterator termIter = terms.iterator();
    // termIter.hasNext();)

    
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitFormula(Formula op, Object prefix)
  {
    Expression left = op.getLeftOp();
    Expression operator = op.getOperator();
    Expression right = op.getRightOp();
      
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitFormula");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitFunction(Function fun, Object prefix)
  {
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    ItemList parametersTypes = fun.getInterfaceItems();
    Expression returnType = fun.getRange();
    Expression body = fun.getExpression();

    String str = new String((String) prefix);
    if (returnType != null) {
      str = str + "class=\"" + returnType.accept(this, "") + "\">\n";
    } // end of if (returnType != null)

    str = str + "<!-- parameters -->\n";
    for (Iterator paramIter = parametersTypes.iterator();
	 paramIter.hasNext();){
      str = str + ((Item)paramIter.next()).accept(this, "<property ") + "\n";
    } // end of for (Iterator paramIter = parameterTypes.iterator(); paramIter.hasNext();)
    
    System.out.println("visitFunction");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitIfExpression(IfExpression ifExpr, Object prefix)
  {
    ExpressionList ifExpressions = ifExpr.getIfs();
    ExpressionList thenExpressions = ifExpr.getThens();
    Expression elseExpression = ifExpr.getElse();
    

    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitIfExpression");
    return prefix;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitInteraction(Interaction interaction, Object prefix)
  {
    Expression domain = interaction.getDomain();
    ExpressionList useClauses = interaction.getUseClauses();
    ItemList parameters = interaction.getInterfaceItems();
    ItemList declarations = interaction.getDeclaredItems();
    ItemList terms = interaction.getTermItems();
    ItemList exports = interaction.getExportItems();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitInteraction");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitItem(Item item, Object prefix)
  {
    Label label = item.getLabel();
    Expression type = item.getType();
    Expression value = item.getValue();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method 
    //    value.accept(this, indent);

    String str = new String((String) prefix);
    System.out.println("visitItem");

    str = str + "name=\"" + label + "\" ";
    if (value == null) {
      if (type != null) {
	str = str + "class=\"" + type + "\">";
      } // end of if (type == null)
      
      return str;
    } else {
      System.out.println("Item: value is an instance of" + value.getClass().getName());
      return value.accept(this, str); 
    }
  }


  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitItemReference(ItemReference itemRef, Object prefix)
  {
    LabelList labels = itemRef.getReference();
    Item item = itemRef.getItemReferred();
    // GMK: DEBUG
    if (labels == null) {
      System.out.println("Labels is null");
    } // end of if (labels == null)
    else {
      System.out.println("Label is null");
    } // end of if (labels == null)else

    if (item == null) {
      System.out.println("Item is null");
    } // end of if (item == null)
    else {
      System.out.println("Item is not null");
    } // end of if (item == null)else
    
    
    String str = new String((String) prefix);
   //   // TODO: implement this org.sldl.rosetta.ROM.Visitor method
//      System.out.println("visiting an item reference");
//      for (Iterator labelIter = labels.iterator(); labelIter.hasNext();){
//        str = str + ((Label)labelIter.next()).accept(this, "") + " ";
//      } // end of for (Iterator labelIter = labels.iterator(); labelIter.hasNext();)
    
    System.out.println("visitItemReference");
    return str + item.getLabel();
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitLabel(Label label, Object prefix)
  {
    String labelName = label.toString();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    System.out.println("visitLabel");
    return labelName;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitLetFunction(LetFunction let, Object prefix)
  {
    ItemList declarations = let.getItemList();
    Expression body = let.getExpression();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitLetFunction");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitLiteral(Literal literal, Object prefix)
  {
    // GMK: Literal is abstract - nothing to do.
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitLiteral");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitPackage(Package pack, Object prefix)
  {
    Expression domain = pack.getDomain();
    ExpressionList useClauses = pack.getUseClauses();
    ItemList parameters = pack.getInterfaceItems();
    ItemList declarations = pack.getDeclaredItems();
    ItemList terms = pack.getTermItems();
    ItemList exports = pack.getExportItems();
   
    String str = new String((String) prefix);
    System.out.println("visitPackage");

    str = str + "class=\"package.xml\">\n";
    
    str = str + "<property name=\"" + domain.accept(this, "") + 
      "\" class=\"domain.xml\"/>\n";

    for (Iterator declIterators = declarations.iterator(); declIterators.hasNext();){
      str = str +  ((Item) declIterators.next()).accept(this, "<entity ")
	+ "</entity>\n";
    } // end of for (Iterator declIterators = declarations.iterator(); declIterators.hasNext();)
    
    str = str + "\n</model>\n";
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method


    return str;
    
    
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitParameterItem(ParameterItem param, Object prefix)
  {
    Label label = param.getLabel();
    Expression type = param.getType();
    Expression value = param.getValue();
    ItemReference mode = param.getMode();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitParameterItem");
    str = str + "name=\"" + label.accept(this, "") + "\"" +
      " class=\"" + type.accept(this, "") + "\"/>";
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitParameterListValue(ParameterListValue paramList, Object prefix)
  {
    // GMK: No idea what this is used for.
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitParameterListValue");
    return null;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitQuantifierReference(QuantifierReference ref, Object prefix)
  {
    
    LabelList labels = ref.getReference();
    Item item = ref.getItemReferred();
    ItemList variables = ref.getVariables();
    Expression predicate = ref.getExpression();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method

    String str = new String((String) prefix);
    System.out.println("visitQuantifierReference");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitSetValue(SetValue set, Object prefix)
  {
    ExpressionList elements = set.getElements();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method

    String str = new String((String) prefix);
    System.out.println("visitSetValue");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitStringLiteral(StringLiteral lit, Object prefix)
  {
    String data = lit.getData();
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitStringLiteral");
    return str;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public Object visitTypeExpression(TypeExpression type, Object prefix)
  {
    String relationship = type.getRelationship();
    Expression expression = type.getExpression();
    
    // TODO: implement this org.sldl.rosetta.ROM.Visitor method
    String str = new String((String) prefix);
    System.out.println("visitTypeExpression");
    return str;
  }


  public static void main (String[] args) {
  
    if (args.length != 1) {
      System.out.println("usage: MoMLGen [sldlfile]");
      return;
    } // end of if (args.length != 1)
  
    MoMLGen visitor = new MoMLGen();
    TestParser parser = new TestParser();
    Item root = parser.compileFile(args[0]);
    String output = (String) root.accept(visitor, "<model ");
    System.out.println("MoML output:\n" + output);
    
  } // end of main ()
  
}// MoMLGen
